// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.4.3.
//
// Warning: This code was generated with the option '-l dolfin'
// and contains DOLFIN-specific wrappers that depend on DOLFIN.

#ifndef __FSIMOMENTUM3D_H
#define __FSIMOMENTUM3D_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_0_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_0_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_0_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_0_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_0_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_0_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_0_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_0_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_0_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_0_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_0_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_0_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_0_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_1_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_1_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_1_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_1_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_1_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_1_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_1_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_1_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_1_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_1_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_1_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_1_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_1_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_2_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_2_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_2_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_2_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_2_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_2_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_2_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_2_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_2_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_2_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_2_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_2_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_2_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_2_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_2_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_3_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_3_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_3_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_3_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_3_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_3_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_3_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_3_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_3_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_3_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_3_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_3_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_3: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_3() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_3()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[3][3] = {{0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 1, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[1];
    vertex_values[5] = dof_values[1];
    vertex_values[6] = dof_values[1];
    vertex_values[7] = dof_values[1];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[2];
    vertex_values[9] = dof_values[2];
    vertex_values[10] = dof_values[2];
    vertex_values[11] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_3_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_3_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_3_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_4: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_4() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_4()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_4();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_5: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_5() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_5()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_5();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_6_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_6_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_6_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_6_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_6_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_6_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_6_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_6_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_6_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_6_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_6_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_6_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_6: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_6() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_6()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_6_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_6_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_6_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_7_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_7_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_7_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_7_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_7_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_7_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_7_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_7_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_7_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_7_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_7_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_7_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_7: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_7() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_7()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[3][3] = {{0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 1, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[1];
    vertex_values[5] = dof_values[1];
    vertex_values[6] = dof_values[1];
    vertex_values[7] = dof_values[1];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[2];
    vertex_values[9] = dof_values[2];
    vertex_values[10] = dof_values[2];
    vertex_values[11] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_7_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_7_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_7_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_8: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_8() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_8()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_8();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_9: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_9() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_9()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_9();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_10: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_10() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_10()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_10();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DBilinearForm_finite_element_11: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_finite_element_11() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_finite_element_11()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_finite_element_11();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_0_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_0_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_0_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_0_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_0_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_0_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_0_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_0_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_0_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_0_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_0_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_0_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_0_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_1_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_1_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_1_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_1_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_1_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_1_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_1_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_1_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_1_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_1_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_1_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_1_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_1_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_2_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_2_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_2_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_2_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_2_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_2_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_2_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_2_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_2_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_2_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_2_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_2_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_2_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_2_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_2_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_3_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_3_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_3_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_3_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_3_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_3_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_3_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_3_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_3_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_3_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_3_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_3_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_3: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_3() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_3()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
    unsigned int offset = m.num_entities[3];
    dofs[1] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[2] = offset + c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[1][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[1][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[1][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[2][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[2][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[2][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_3_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_3_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_3_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_4: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_4() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_4()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_4();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_5: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_5() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_5()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_5();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_6_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_6_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_6_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_6_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_6_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_6_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_6_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_6_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_6_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_6_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_6_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_6_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_6: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_6() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_6()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_6_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_6_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_6_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_7_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_7_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_7_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_7_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_7_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_7_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_7_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_7_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_7_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_7_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_7_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_7_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_7: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_7() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_7()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
    unsigned int offset = m.num_entities[3];
    dofs[1] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[2] = offset + c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[1][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[1][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[1][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[2][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[2][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[2][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_7_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_7_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_7_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_8: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_8() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_8()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_8();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_9: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_9() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_9()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_9();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_10: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_10() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_10()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_10();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DBilinearForm_dof_map_11: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_dof_map_11() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_dof_map_11()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_dof_map_11();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class UFC_FSIMomentum3DBilinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute coefficients
    const double c7_0_0_0 = w[7][0];
    const double c9_1_0_0 = w[9][0];
    const double c7_1_1_0 = w[7][0];
    const double c6_1_2_0 = w[6][0];
    const double c0_1_3_0 = w[0][0];
    const double c0_1_3_1 = w[0][1];
    const double c0_1_3_2 = w[0][2];
    const double c0_1_3_3 = w[0][3];
    const double c0_1_3_4 = w[0][4];
    const double c0_1_3_5 = w[0][5];
    const double c0_1_3_6 = w[0][6];
    const double c0_1_3_7 = w[0][7];
    const double c0_1_3_8 = w[0][8];
    const double c0_1_3_9 = w[0][9];
    const double c0_1_3_10 = w[0][10];
    const double c0_1_3_11 = w[0][11];
    const double c9_2_0_0 = w[9][0];
    const double c7_2_1_0 = w[7][0];
    const double c6_2_2_0 = w[6][0];
    const double c4_2_3_0 = w[4][0];
    const double c4_2_3_1 = w[4][1];
    const double c4_2_3_2 = w[4][2];
    const double c4_2_3_3 = w[4][3];
    const double c4_2_3_4 = w[4][4];
    const double c4_2_3_5 = w[4][5];
    const double c4_2_3_6 = w[4][6];
    const double c4_2_3_7 = w[4][7];
    const double c4_2_3_8 = w[4][8];
    const double c4_2_3_9 = w[4][9];
    const double c4_2_3_10 = w[4][10];
    const double c4_2_3_11 = w[4][11];
    const double c9_3_0_0 = w[9][0];
    const double c8_3_1_0 = w[8][0];
    const double c9_4_0_0 = w[9][0];
    const double c8_4_1_0 = w[8][0];
    const double c9_5_0_0 = w[9][0];
    const double c8_5_1_0 = w[8][0];
    const double c9_6_0_0 = w[9][0];
    const double c2_6_1_0 = w[2][0];
    const double c1_6_2_0 = w[1][0];
    const double c1_6_2_1 = w[1][1];
    const double c1_6_2_2 = w[1][2];
    const double c1_6_3_0 = w[1][0];
    const double c1_6_3_1 = w[1][1];
    const double c1_6_3_2 = w[1][2];
    const double c9_7_0_0 = w[9][0];
    const double c2_7_1_0 = w[2][0];
    const double c5_7_2_0 = w[5][0];
    const double c5_7_2_1 = w[5][1];
    const double c5_7_2_2 = w[5][2];
    const double c5_7_3_0 = w[5][0];
    const double c5_7_3_1 = w[5][1];
    const double c5_7_3_2 = w[5][2];
    const double c9_8_0_0 = w[9][0];
    const double c2_8_1_0 = w[2][0];
    const double c1_8_2_0 = w[1][0];
    const double c1_8_2_1 = w[1][1];
    const double c1_8_2_2 = w[1][2];
    const double c5_8_3_0 = w[5][0];
    const double c5_8_3_1 = w[5][1];
    const double c5_8_3_2 = w[5][2];
    const double c9_9_0_0 = w[9][0];
    const double c2_9_1_0 = w[2][0];
    const double c5_9_2_0 = w[5][0];
    const double c5_9_2_1 = w[5][1];
    const double c5_9_2_2 = w[5][2];
    const double c1_9_3_0 = w[1][0];
    const double c1_9_3_1 = w[1][1];
    const double c1_9_3_2 = w[1][2];
    const double c9_10_0_0 = w[9][0];
    const double c3_10_1_0 = w[3][0];
    
    // Compute geometry tensors
    const double G0_0 = det*c7_0_0_0;
    const double G1_0_0_0_0_0_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_0*Jinv_00;
    const double G1_0_0_0_0_1_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_0*Jinv_10;
    const double G1_0_0_0_0_2_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_0*Jinv_20;
    const double G1_0_0_0_1_0_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_1*Jinv_00;
    const double G1_0_0_0_1_1_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_1*Jinv_10;
    const double G1_0_0_0_1_2_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_1*Jinv_20;
    const double G1_0_0_0_2_0_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_2*Jinv_00;
    const double G1_0_0_0_2_1_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_2*Jinv_10;
    const double G1_0_0_0_2_2_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_2*Jinv_20;
    const double G1_0_0_0_3_0_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_3*Jinv_00;
    const double G1_0_0_0_3_1_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_3*Jinv_10;
    const double G1_0_0_0_3_2_0 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_3*Jinv_20;
    const double G1_0_0_0_4_0_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_4*Jinv_01;
    const double G1_0_0_0_4_1_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_4*Jinv_11;
    const double G1_0_0_0_4_2_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_4*Jinv_21;
    const double G1_0_0_0_5_0_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_5*Jinv_01;
    const double G1_0_0_0_5_1_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_5*Jinv_11;
    const double G1_0_0_0_5_2_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_5*Jinv_21;
    const double G1_0_0_0_6_0_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_6*Jinv_01;
    const double G1_0_0_0_6_1_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_6*Jinv_11;
    const double G1_0_0_0_6_2_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_6*Jinv_21;
    const double G1_0_0_0_7_0_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_7*Jinv_01;
    const double G1_0_0_0_7_1_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_7*Jinv_11;
    const double G1_0_0_0_7_2_1 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_7*Jinv_21;
    const double G1_0_0_0_8_0_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_8*Jinv_02;
    const double G1_0_0_0_8_1_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_8*Jinv_12;
    const double G1_0_0_0_8_2_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_8*Jinv_22;
    const double G1_0_0_0_9_0_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_9*Jinv_02;
    const double G1_0_0_0_9_1_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_9*Jinv_12;
    const double G1_0_0_0_9_2_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_9*Jinv_22;
    const double G1_0_0_0_10_0_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_10*Jinv_02;
    const double G1_0_0_0_10_1_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_10*Jinv_12;
    const double G1_0_0_0_10_2_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_10*Jinv_22;
    const double G1_0_0_0_11_0_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_11*Jinv_02;
    const double G1_0_0_0_11_1_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_11*Jinv_12;
    const double G1_0_0_0_11_2_2 = det*c9_1_0_0*c7_1_1_0*c6_1_2_0*c0_1_3_11*Jinv_22;
    const double G2_0_0_0_0_0_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_0*Jinv_00;
    const double G2_0_0_0_0_1_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_0*Jinv_10;
    const double G2_0_0_0_0_2_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_0*Jinv_20;
    const double G2_0_0_0_1_0_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_1*Jinv_00;
    const double G2_0_0_0_1_1_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_1*Jinv_10;
    const double G2_0_0_0_1_2_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_1*Jinv_20;
    const double G2_0_0_0_2_0_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_2*Jinv_00;
    const double G2_0_0_0_2_1_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_2*Jinv_10;
    const double G2_0_0_0_2_2_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_2*Jinv_20;
    const double G2_0_0_0_3_0_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_3*Jinv_00;
    const double G2_0_0_0_3_1_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_3*Jinv_10;
    const double G2_0_0_0_3_2_0 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_3*Jinv_20;
    const double G2_0_0_0_4_0_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_4*Jinv_01;
    const double G2_0_0_0_4_1_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_4*Jinv_11;
    const double G2_0_0_0_4_2_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_4*Jinv_21;
    const double G2_0_0_0_5_0_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_5*Jinv_01;
    const double G2_0_0_0_5_1_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_5*Jinv_11;
    const double G2_0_0_0_5_2_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_5*Jinv_21;
    const double G2_0_0_0_6_0_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_6*Jinv_01;
    const double G2_0_0_0_6_1_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_6*Jinv_11;
    const double G2_0_0_0_6_2_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_6*Jinv_21;
    const double G2_0_0_0_7_0_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_7*Jinv_01;
    const double G2_0_0_0_7_1_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_7*Jinv_11;
    const double G2_0_0_0_7_2_1 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_7*Jinv_21;
    const double G2_0_0_0_8_0_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_8*Jinv_02;
    const double G2_0_0_0_8_1_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_8*Jinv_12;
    const double G2_0_0_0_8_2_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_8*Jinv_22;
    const double G2_0_0_0_9_0_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_9*Jinv_02;
    const double G2_0_0_0_9_1_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_9*Jinv_12;
    const double G2_0_0_0_9_2_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_9*Jinv_22;
    const double G2_0_0_0_10_0_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_10*Jinv_02;
    const double G2_0_0_0_10_1_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_10*Jinv_12;
    const double G2_0_0_0_10_2_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_10*Jinv_22;
    const double G2_0_0_0_11_0_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_11*Jinv_02;
    const double G2_0_0_0_11_1_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_11*Jinv_12;
    const double G2_0_0_0_11_2_2 = det*c9_2_0_0*c7_2_1_0*c6_2_2_0*c4_2_3_11*Jinv_22;
    const double G3_0_0_0_0 = det*c9_3_0_0*c8_3_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01 + Jinv_02*Jinv_02);
    const double G3_0_0_0_1 = det*c9_3_0_0*c8_3_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11 + Jinv_02*Jinv_12);
    const double G3_0_0_0_2 = det*c9_3_0_0*c8_3_1_0*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21 + Jinv_02*Jinv_22);
    const double G3_0_0_1_0 = det*c9_3_0_0*c8_3_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01 + Jinv_12*Jinv_02);
    const double G3_0_0_1_1 = det*c9_3_0_0*c8_3_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11 + Jinv_12*Jinv_12);
    const double G3_0_0_1_2 = det*c9_3_0_0*c8_3_1_0*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21 + Jinv_12*Jinv_22);
    const double G3_0_0_2_0 = det*c9_3_0_0*c8_3_1_0*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01 + Jinv_22*Jinv_02);
    const double G3_0_0_2_1 = det*c9_3_0_0*c8_3_1_0*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11 + Jinv_22*Jinv_12);
    const double G3_0_0_2_2 = det*c9_3_0_0*c8_3_1_0*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21 + Jinv_22*Jinv_22);
    const double G4_0_0_0_0_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_00);
    const double G4_0_0_0_0_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_01);
    const double G4_0_0_0_0_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_02);
    const double G4_0_0_0_0_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_00);
    const double G4_0_0_0_0_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_01);
    const double G4_0_0_0_0_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_02);
    const double G4_0_0_0_0_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_00);
    const double G4_0_0_0_0_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_01);
    const double G4_0_0_0_0_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_02);
    const double G4_0_0_0_1_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_10);
    const double G4_0_0_0_1_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_11);
    const double G4_0_0_0_1_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_12);
    const double G4_0_0_0_1_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_10);
    const double G4_0_0_0_1_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_11);
    const double G4_0_0_0_1_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_12);
    const double G4_0_0_0_1_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_10);
    const double G4_0_0_0_1_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_11);
    const double G4_0_0_0_1_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_12);
    const double G4_0_0_0_2_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_20);
    const double G4_0_0_0_2_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_21);
    const double G4_0_0_0_2_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_00*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_00*Jinv_22);
    const double G4_0_0_0_2_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_20);
    const double G4_0_0_0_2_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_21);
    const double G4_0_0_0_2_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_01*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_01*Jinv_22);
    const double G4_0_0_0_2_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_20);
    const double G4_0_0_0_2_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_21);
    const double G4_0_0_0_2_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_02*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_02*Jinv_22);
    const double G4_0_0_1_0_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_00);
    const double G4_0_0_1_0_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_01);
    const double G4_0_0_1_0_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_02);
    const double G4_0_0_1_0_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_00);
    const double G4_0_0_1_0_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_01);
    const double G4_0_0_1_0_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_02);
    const double G4_0_0_1_0_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_00);
    const double G4_0_0_1_0_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_01);
    const double G4_0_0_1_0_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_02);
    const double G4_0_0_1_1_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_10);
    const double G4_0_0_1_1_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_11);
    const double G4_0_0_1_1_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_12);
    const double G4_0_0_1_1_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_10);
    const double G4_0_0_1_1_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_11);
    const double G4_0_0_1_1_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_12);
    const double G4_0_0_1_1_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_10);
    const double G4_0_0_1_1_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_11);
    const double G4_0_0_1_1_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_12);
    const double G4_0_0_1_2_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_20);
    const double G4_0_0_1_2_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_21);
    const double G4_0_0_1_2_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_10*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_10*Jinv_22);
    const double G4_0_0_1_2_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_20);
    const double G4_0_0_1_2_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_21);
    const double G4_0_0_1_2_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_11*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_11*Jinv_22);
    const double G4_0_0_1_2_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_20);
    const double G4_0_0_1_2_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_21);
    const double G4_0_0_1_2_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_12*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_12*Jinv_22);
    const double G4_0_0_2_0_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_00);
    const double G4_0_0_2_0_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_01);
    const double G4_0_0_2_0_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_02);
    const double G4_0_0_2_0_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_00);
    const double G4_0_0_2_0_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_01);
    const double G4_0_0_2_0_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_02);
    const double G4_0_0_2_0_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_00 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_00);
    const double G4_0_0_2_0_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_01 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_01);
    const double G4_0_0_2_0_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_02 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_02);
    const double G4_0_0_2_1_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_10);
    const double G4_0_0_2_1_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_11);
    const double G4_0_0_2_1_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_12);
    const double G4_0_0_2_1_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_10);
    const double G4_0_0_2_1_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_11);
    const double G4_0_0_2_1_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_12);
    const double G4_0_0_2_1_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_10 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_10);
    const double G4_0_0_2_1_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_11 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_11);
    const double G4_0_0_2_1_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_12 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_12);
    const double G4_0_0_2_2_0_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_20);
    const double G4_0_0_2_2_0_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_21);
    const double G4_0_0_2_2_0_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_20*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_20*Jinv_22);
    const double G4_0_0_2_2_1_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_20);
    const double G4_0_0_2_2_1_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_21);
    const double G4_0_0_2_2_1_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_21*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_21*Jinv_22);
    const double G4_0_0_2_2_2_0 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_20 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_20);
    const double G4_0_0_2_2_2_1 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_21 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_21);
    const double G4_0_0_2_2_2_2 = det*(c9_4_0_0*c8_4_1_0*Jinv_22*Jinv_22 + c9_5_0_0*c8_5_1_0*Jinv_22*Jinv_22);
    const double G5_0_0_0_0_0_0_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_00*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_00*Jinv_00);
    const double G5_0_0_0_0_0_0_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_00*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_00*Jinv_10);
    const double G5_0_0_0_0_0_0_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_00*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_00*Jinv_20);
    const double G5_0_0_0_0_0_1_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_10*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_10*Jinv_00);
    const double G5_0_0_0_0_0_1_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_10*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_10*Jinv_10);
    const double G5_0_0_0_0_0_1_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_10*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_10*Jinv_20);
    const double G5_0_0_0_0_0_2_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_20*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_20*Jinv_00);
    const double G5_0_0_0_0_0_2_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_20*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_20*Jinv_10);
    const double G5_0_0_0_0_0_2_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_0*Jinv_20*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_0*Jinv_20*Jinv_20);
    const double G5_0_0_0_1_0_0_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_01*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_01*Jinv_00);
    const double G5_0_0_0_1_0_0_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_01*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_01*Jinv_10);
    const double G5_0_0_0_1_0_0_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_01*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_01*Jinv_20);
    const double G5_0_0_0_1_0_1_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_11*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_11*Jinv_00);
    const double G5_0_0_0_1_0_1_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_11*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_11*Jinv_10);
    const double G5_0_0_0_1_0_1_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_11*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_11*Jinv_20);
    const double G5_0_0_0_1_0_2_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_21*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_21*Jinv_00);
    const double G5_0_0_0_1_0_2_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_21*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_21*Jinv_10);
    const double G5_0_0_0_1_0_2_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_0*Jinv_21*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_0*Jinv_21*Jinv_20);
    const double G5_0_0_0_2_0_0_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_02*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_02*Jinv_00);
    const double G5_0_0_0_2_0_0_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_02*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_02*Jinv_10);
    const double G5_0_0_0_2_0_0_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_02*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_02*Jinv_20);
    const double G5_0_0_0_2_0_1_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_12*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_12*Jinv_00);
    const double G5_0_0_0_2_0_1_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_12*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_12*Jinv_10);
    const double G5_0_0_0_2_0_1_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_12*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_12*Jinv_20);
    const double G5_0_0_0_2_0_2_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_22*Jinv_00 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_22*Jinv_00);
    const double G5_0_0_0_2_0_2_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_22*Jinv_10 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_22*Jinv_10);
    const double G5_0_0_0_2_0_2_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_0*Jinv_22*Jinv_20 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_0*Jinv_22*Jinv_20);
    const double G5_1_0_0_0_1_0_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_00*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_00*Jinv_01);
    const double G5_1_0_0_0_1_0_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_00*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_00*Jinv_11);
    const double G5_1_0_0_0_1_0_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_00*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_00*Jinv_21);
    const double G5_1_0_0_0_1_1_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_10*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_10*Jinv_01);
    const double G5_1_0_0_0_1_1_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_10*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_10*Jinv_11);
    const double G5_1_0_0_0_1_1_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_10*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_10*Jinv_21);
    const double G5_1_0_0_0_1_2_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_20*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_20*Jinv_01);
    const double G5_1_0_0_0_1_2_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_20*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_20*Jinv_11);
    const double G5_1_0_0_0_1_2_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_1*Jinv_20*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_1*Jinv_20*Jinv_21);
    const double G5_1_0_0_1_1_0_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_01*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_01*Jinv_01);
    const double G5_1_0_0_1_1_0_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_01*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_01*Jinv_11);
    const double G5_1_0_0_1_1_0_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_01*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_01*Jinv_21);
    const double G5_1_0_0_1_1_1_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_11*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_11*Jinv_01);
    const double G5_1_0_0_1_1_1_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_11*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_11*Jinv_11);
    const double G5_1_0_0_1_1_1_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_11*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_11*Jinv_21);
    const double G5_1_0_0_1_1_2_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_21*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_21*Jinv_01);
    const double G5_1_0_0_1_1_2_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_21*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_21*Jinv_11);
    const double G5_1_0_0_1_1_2_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_1*Jinv_21*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_1*Jinv_21*Jinv_21);
    const double G5_1_0_0_2_1_0_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_02*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_02*Jinv_01);
    const double G5_1_0_0_2_1_0_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_02*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_02*Jinv_11);
    const double G5_1_0_0_2_1_0_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_02*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_02*Jinv_21);
    const double G5_1_0_0_2_1_1_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_12*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_12*Jinv_01);
    const double G5_1_0_0_2_1_1_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_12*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_12*Jinv_11);
    const double G5_1_0_0_2_1_1_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_12*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_12*Jinv_21);
    const double G5_1_0_0_2_1_2_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_22*Jinv_01 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_22*Jinv_01);
    const double G5_1_0_0_2_1_2_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_22*Jinv_11 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_22*Jinv_11);
    const double G5_1_0_0_2_1_2_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_1*Jinv_22*Jinv_21 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_1*Jinv_22*Jinv_21);
    const double G5_2_0_0_0_2_0_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_00*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_00*Jinv_02);
    const double G5_2_0_0_0_2_0_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_00*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_00*Jinv_12);
    const double G5_2_0_0_0_2_0_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_00*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_00*Jinv_22);
    const double G5_2_0_0_0_2_1_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_10*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_10*Jinv_02);
    const double G5_2_0_0_0_2_1_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_10*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_10*Jinv_12);
    const double G5_2_0_0_0_2_1_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_10*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_10*Jinv_22);
    const double G5_2_0_0_0_2_2_0_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_20*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_20*Jinv_02);
    const double G5_2_0_0_0_2_2_0_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_20*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_20*Jinv_12);
    const double G5_2_0_0_0_2_2_0_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_0*c1_6_3_2*Jinv_20*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_0*c5_7_3_2*Jinv_20*Jinv_22);
    const double G5_2_0_0_1_2_0_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_01*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_01*Jinv_02);
    const double G5_2_0_0_1_2_0_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_01*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_01*Jinv_12);
    const double G5_2_0_0_1_2_0_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_01*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_01*Jinv_22);
    const double G5_2_0_0_1_2_1_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_11*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_11*Jinv_02);
    const double G5_2_0_0_1_2_1_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_11*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_11*Jinv_12);
    const double G5_2_0_0_1_2_1_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_11*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_11*Jinv_22);
    const double G5_2_0_0_1_2_2_1_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_21*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_21*Jinv_02);
    const double G5_2_0_0_1_2_2_1_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_21*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_21*Jinv_12);
    const double G5_2_0_0_1_2_2_1_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_1*c1_6_3_2*Jinv_21*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_1*c5_7_3_2*Jinv_21*Jinv_22);
    const double G5_2_0_0_2_2_0_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_02*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_02*Jinv_02);
    const double G5_2_0_0_2_2_0_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_02*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_02*Jinv_12);
    const double G5_2_0_0_2_2_0_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_02*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_02*Jinv_22);
    const double G5_2_0_0_2_2_1_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_12*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_12*Jinv_02);
    const double G5_2_0_0_2_2_1_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_12*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_12*Jinv_12);
    const double G5_2_0_0_2_2_1_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_12*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_12*Jinv_22);
    const double G5_2_0_0_2_2_2_2_0 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_22*Jinv_02 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_22*Jinv_02);
    const double G5_2_0_0_2_2_2_2_1 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_22*Jinv_12 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_22*Jinv_12);
    const double G5_2_0_0_2_2_2_2_2 = det*(c9_6_0_0*c2_6_1_0*c1_6_2_2*c1_6_3_2*Jinv_22*Jinv_22 + c9_7_0_0*c2_7_1_0*c5_7_2_2*c5_7_3_2*Jinv_22*Jinv_22);
    const double G6_0_0_0_0_0_0_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_00*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_00*Jinv_00);
    const double G6_0_0_0_0_0_0_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_00*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_00*Jinv_10);
    const double G6_0_0_0_0_0_0_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_00*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_00*Jinv_20);
    const double G6_0_0_0_0_0_1_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_10*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_10*Jinv_00);
    const double G6_0_0_0_0_0_1_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_10*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_10*Jinv_10);
    const double G6_0_0_0_0_0_1_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_10*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_10*Jinv_20);
    const double G6_0_0_0_0_0_2_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_20*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_20*Jinv_00);
    const double G6_0_0_0_0_0_2_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_20*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_20*Jinv_10);
    const double G6_0_0_0_0_0_2_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_0*Jinv_20*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_0*Jinv_20*Jinv_20);
    const double G6_0_0_0_1_0_0_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_01*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_01*Jinv_00);
    const double G6_0_0_0_1_0_0_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_01*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_01*Jinv_10);
    const double G6_0_0_0_1_0_0_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_01*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_01*Jinv_20);
    const double G6_0_0_0_1_0_1_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_11*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_11*Jinv_00);
    const double G6_0_0_0_1_0_1_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_11*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_11*Jinv_10);
    const double G6_0_0_0_1_0_1_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_11*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_11*Jinv_20);
    const double G6_0_0_0_1_0_2_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_21*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_21*Jinv_00);
    const double G6_0_0_0_1_0_2_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_21*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_21*Jinv_10);
    const double G6_0_0_0_1_0_2_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_0*Jinv_21*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_0*Jinv_21*Jinv_20);
    const double G6_0_0_0_2_0_0_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_02*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_02*Jinv_00);
    const double G6_0_0_0_2_0_0_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_02*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_02*Jinv_10);
    const double G6_0_0_0_2_0_0_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_02*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_02*Jinv_20);
    const double G6_0_0_0_2_0_1_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_12*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_12*Jinv_00);
    const double G6_0_0_0_2_0_1_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_12*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_12*Jinv_10);
    const double G6_0_0_0_2_0_1_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_12*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_12*Jinv_20);
    const double G6_0_0_0_2_0_2_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_22*Jinv_00 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_22*Jinv_00);
    const double G6_0_0_0_2_0_2_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_22*Jinv_10 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_22*Jinv_10);
    const double G6_0_0_0_2_0_2_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_0*Jinv_22*Jinv_20 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_0*Jinv_22*Jinv_20);
    const double G6_1_0_0_0_1_0_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_00*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_00*Jinv_01);
    const double G6_1_0_0_0_1_0_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_00*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_00*Jinv_11);
    const double G6_1_0_0_0_1_0_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_00*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_00*Jinv_21);
    const double G6_1_0_0_0_1_1_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_10*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_10*Jinv_01);
    const double G6_1_0_0_0_1_1_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_10*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_10*Jinv_11);
    const double G6_1_0_0_0_1_1_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_10*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_10*Jinv_21);
    const double G6_1_0_0_0_1_2_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_20*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_20*Jinv_01);
    const double G6_1_0_0_0_1_2_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_20*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_20*Jinv_11);
    const double G6_1_0_0_0_1_2_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_1*Jinv_20*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_1*Jinv_20*Jinv_21);
    const double G6_1_0_0_1_1_0_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_01*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_01*Jinv_01);
    const double G6_1_0_0_1_1_0_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_01*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_01*Jinv_11);
    const double G6_1_0_0_1_1_0_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_01*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_01*Jinv_21);
    const double G6_1_0_0_1_1_1_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_11*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_11*Jinv_01);
    const double G6_1_0_0_1_1_1_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_11*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_11*Jinv_11);
    const double G6_1_0_0_1_1_1_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_11*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_11*Jinv_21);
    const double G6_1_0_0_1_1_2_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_21*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_21*Jinv_01);
    const double G6_1_0_0_1_1_2_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_21*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_21*Jinv_11);
    const double G6_1_0_0_1_1_2_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_1*Jinv_21*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_1*Jinv_21*Jinv_21);
    const double G6_1_0_0_2_1_0_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_02*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_02*Jinv_01);
    const double G6_1_0_0_2_1_0_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_02*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_02*Jinv_11);
    const double G6_1_0_0_2_1_0_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_02*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_02*Jinv_21);
    const double G6_1_0_0_2_1_1_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_12*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_12*Jinv_01);
    const double G6_1_0_0_2_1_1_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_12*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_12*Jinv_11);
    const double G6_1_0_0_2_1_1_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_12*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_12*Jinv_21);
    const double G6_1_0_0_2_1_2_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_22*Jinv_01 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_22*Jinv_01);
    const double G6_1_0_0_2_1_2_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_22*Jinv_11 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_22*Jinv_11);
    const double G6_1_0_0_2_1_2_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_1*Jinv_22*Jinv_21 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_1*Jinv_22*Jinv_21);
    const double G6_2_0_0_0_2_0_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_00*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_00*Jinv_02);
    const double G6_2_0_0_0_2_0_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_00*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_00*Jinv_12);
    const double G6_2_0_0_0_2_0_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_00*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_00*Jinv_22);
    const double G6_2_0_0_0_2_1_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_10*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_10*Jinv_02);
    const double G6_2_0_0_0_2_1_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_10*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_10*Jinv_12);
    const double G6_2_0_0_0_2_1_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_10*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_10*Jinv_22);
    const double G6_2_0_0_0_2_2_0_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_20*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_20*Jinv_02);
    const double G6_2_0_0_0_2_2_0_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_20*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_20*Jinv_12);
    const double G6_2_0_0_0_2_2_0_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_0*c5_8_3_2*Jinv_20*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_0*c1_9_3_2*Jinv_20*Jinv_22);
    const double G6_2_0_0_1_2_0_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_01*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_01*Jinv_02);
    const double G6_2_0_0_1_2_0_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_01*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_01*Jinv_12);
    const double G6_2_0_0_1_2_0_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_01*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_01*Jinv_22);
    const double G6_2_0_0_1_2_1_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_11*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_11*Jinv_02);
    const double G6_2_0_0_1_2_1_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_11*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_11*Jinv_12);
    const double G6_2_0_0_1_2_1_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_11*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_11*Jinv_22);
    const double G6_2_0_0_1_2_2_1_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_21*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_21*Jinv_02);
    const double G6_2_0_0_1_2_2_1_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_21*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_21*Jinv_12);
    const double G6_2_0_0_1_2_2_1_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_1*c5_8_3_2*Jinv_21*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_1*c1_9_3_2*Jinv_21*Jinv_22);
    const double G6_2_0_0_2_2_0_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_02*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_02*Jinv_02);
    const double G6_2_0_0_2_2_0_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_02*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_02*Jinv_12);
    const double G6_2_0_0_2_2_0_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_02*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_02*Jinv_22);
    const double G6_2_0_0_2_2_1_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_12*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_12*Jinv_02);
    const double G6_2_0_0_2_2_1_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_12*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_12*Jinv_12);
    const double G6_2_0_0_2_2_1_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_12*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_12*Jinv_22);
    const double G6_2_0_0_2_2_2_2_0 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_22*Jinv_02 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_22*Jinv_02);
    const double G6_2_0_0_2_2_2_2_1 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_22*Jinv_12 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_22*Jinv_12);
    const double G6_2_0_0_2_2_2_2_2 = det*(c9_8_0_0*c2_8_1_0*c1_8_2_2*c5_8_3_2*Jinv_22*Jinv_22 + c9_9_0_0*c2_9_1_0*c5_9_2_2*c1_9_3_2*Jinv_22*Jinv_22);
    const double G7_0_0_0_0_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_00;
    const double G7_0_0_0_0_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_01;
    const double G7_0_0_0_0_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_02;
    const double G7_0_0_0_0_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_10;
    const double G7_0_0_0_0_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_11;
    const double G7_0_0_0_0_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_12;
    const double G7_0_0_0_0_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_20;
    const double G7_0_0_0_0_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_21;
    const double G7_0_0_0_0_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_00*Jinv_22;
    const double G7_0_0_0_1_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_00;
    const double G7_0_0_0_1_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_01;
    const double G7_0_0_0_1_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_02;
    const double G7_0_0_0_1_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_10;
    const double G7_0_0_0_1_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_11;
    const double G7_0_0_0_1_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_12;
    const double G7_0_0_0_1_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_20;
    const double G7_0_0_0_1_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_21;
    const double G7_0_0_0_1_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_01*Jinv_22;
    const double G7_0_0_0_2_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_00;
    const double G7_0_0_0_2_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_01;
    const double G7_0_0_0_2_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_02;
    const double G7_0_0_0_2_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_10;
    const double G7_0_0_0_2_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_11;
    const double G7_0_0_0_2_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_12;
    const double G7_0_0_0_2_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_20;
    const double G7_0_0_0_2_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_21;
    const double G7_0_0_0_2_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_02*Jinv_22;
    const double G7_0_0_1_0_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_00;
    const double G7_0_0_1_0_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_01;
    const double G7_0_0_1_0_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_02;
    const double G7_0_0_1_0_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_10;
    const double G7_0_0_1_0_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_11;
    const double G7_0_0_1_0_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_12;
    const double G7_0_0_1_0_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_20;
    const double G7_0_0_1_0_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_21;
    const double G7_0_0_1_0_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_10*Jinv_22;
    const double G7_0_0_1_1_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_00;
    const double G7_0_0_1_1_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_01;
    const double G7_0_0_1_1_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_02;
    const double G7_0_0_1_1_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_10;
    const double G7_0_0_1_1_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_11;
    const double G7_0_0_1_1_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_12;
    const double G7_0_0_1_1_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_20;
    const double G7_0_0_1_1_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_21;
    const double G7_0_0_1_1_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_11*Jinv_22;
    const double G7_0_0_1_2_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_00;
    const double G7_0_0_1_2_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_01;
    const double G7_0_0_1_2_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_02;
    const double G7_0_0_1_2_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_10;
    const double G7_0_0_1_2_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_11;
    const double G7_0_0_1_2_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_12;
    const double G7_0_0_1_2_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_20;
    const double G7_0_0_1_2_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_21;
    const double G7_0_0_1_2_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_12*Jinv_22;
    const double G7_0_0_2_0_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_00;
    const double G7_0_0_2_0_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_01;
    const double G7_0_0_2_0_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_02;
    const double G7_0_0_2_0_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_10;
    const double G7_0_0_2_0_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_11;
    const double G7_0_0_2_0_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_12;
    const double G7_0_0_2_0_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_20;
    const double G7_0_0_2_0_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_21;
    const double G7_0_0_2_0_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_20*Jinv_22;
    const double G7_0_0_2_1_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_00;
    const double G7_0_0_2_1_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_01;
    const double G7_0_0_2_1_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_02;
    const double G7_0_0_2_1_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_10;
    const double G7_0_0_2_1_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_11;
    const double G7_0_0_2_1_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_12;
    const double G7_0_0_2_1_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_20;
    const double G7_0_0_2_1_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_21;
    const double G7_0_0_2_1_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_21*Jinv_22;
    const double G7_0_0_2_2_0_0 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_00;
    const double G7_0_0_2_2_0_1 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_01;
    const double G7_0_0_2_2_0_2 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_02;
    const double G7_0_0_2_2_1_0 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_10;
    const double G7_0_0_2_2_1_1 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_11;
    const double G7_0_0_2_2_1_2 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_12;
    const double G7_0_0_2_2_2_0 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_20;
    const double G7_0_0_2_2_2_1 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_21;
    const double G7_0_0_2_2_2_2 = det*c9_10_0_0*c3_10_1_0*Jinv_22*Jinv_22;
    
    // Compute element tensor
    A[0] = 0.0166666666666666*G0_0 - 0.0166666666666666*G1_0_0_0_0_0_0 - 0.0166666666666666*G1_0_0_0_0_1_0 - 0.0166666666666666*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.0166666666666666*G1_0_0_0_4_0_1 - 0.0166666666666666*G1_0_0_0_4_1_1 - 0.0166666666666666*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.0166666666666666*G1_0_0_0_8_0_2 - 0.0166666666666666*G1_0_0_0_8_1_2 - 0.0166666666666666*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.0166666666666666*G2_0_0_0_0_0_0 + 0.0166666666666666*G2_0_0_0_0_1_0 + 0.0166666666666666*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.0166666666666666*G2_0_0_0_4_0_1 + 0.0166666666666666*G2_0_0_0_4_1_1 + 0.0166666666666666*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.0166666666666666*G2_0_0_0_8_0_2 + 0.0166666666666666*G2_0_0_0_8_1_2 + 0.0166666666666666*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_0_0 + 0.0833333333333332*G3_0_0_0_1 + 0.0833333333333332*G3_0_0_0_2 + 0.0833333333333332*G3_0_0_1_0 + 0.0833333333333332*G3_0_0_1_1 + 0.0833333333333332*G3_0_0_1_2 + 0.0833333333333332*G3_0_0_2_0 + 0.0833333333333332*G3_0_0_2_1 + 0.0833333333333332*G3_0_0_2_2 + 0.0416666666666666*G4_0_0_0_0_0_0 + 0.0416666666666666*G4_0_0_0_1_0_0 + 0.0416666666666666*G4_0_0_0_2_0_0 + 0.0416666666666666*G4_0_0_1_0_0_0 + 0.0416666666666666*G4_0_0_1_1_0_0 + 0.0416666666666666*G4_0_0_1_2_0_0 + 0.0416666666666666*G4_0_0_2_0_0_0 + 0.0416666666666666*G4_0_0_2_1_0_0 + 0.0416666666666666*G4_0_0_2_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_2 + 0.0833333333333332*G5_0_0_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_0_0_1_0_1 + 0.0833333333333332*G5_0_0_0_0_0_1_0_2 + 0.0833333333333332*G5_0_0_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_2_0_1 + 0.0833333333333332*G5_0_0_0_0_0_2_0_2 + 0.0833333333333332*G5_0_0_0_1_0_0_1_0 + 0.0833333333333332*G5_0_0_0_1_0_0_1_1 + 0.0833333333333332*G5_0_0_0_1_0_0_1_2 + 0.0833333333333332*G5_0_0_0_1_0_1_1_0 + 0.0833333333333332*G5_0_0_0_1_0_1_1_1 + 0.0833333333333332*G5_0_0_0_1_0_1_1_2 + 0.0833333333333332*G5_0_0_0_1_0_2_1_0 + 0.0833333333333332*G5_0_0_0_1_0_2_1_1 + 0.0833333333333332*G5_0_0_0_1_0_2_1_2 + 0.0833333333333332*G5_0_0_0_2_0_0_2_0 + 0.0833333333333332*G5_0_0_0_2_0_0_2_1 + 0.0833333333333332*G5_0_0_0_2_0_0_2_2 + 0.0833333333333332*G5_0_0_0_2_0_1_2_0 + 0.0833333333333332*G5_0_0_0_2_0_1_2_1 + 0.0833333333333332*G5_0_0_0_2_0_1_2_2 + 0.0833333333333332*G5_0_0_0_2_0_2_2_0 + 0.0833333333333332*G5_0_0_0_2_0_2_2_1 + 0.0833333333333332*G5_0_0_0_2_0_2_2_2 + 0.0833333333333332*G5_1_0_0_0_1_0_0_0 + 0.0833333333333332*G5_1_0_0_0_1_0_0_1 + 0.0833333333333332*G5_1_0_0_0_1_0_0_2 + 0.0833333333333332*G5_1_0_0_0_1_1_0_0 + 0.0833333333333332*G5_1_0_0_0_1_1_0_1 + 0.0833333333333332*G5_1_0_0_0_1_1_0_2 + 0.0833333333333332*G5_1_0_0_0_1_2_0_0 + 0.0833333333333332*G5_1_0_0_0_1_2_0_1 + 0.0833333333333332*G5_1_0_0_0_1_2_0_2 + 0.0833333333333332*G5_1_0_0_1_1_0_1_0 + 0.0833333333333332*G5_1_0_0_1_1_0_1_1 + 0.0833333333333332*G5_1_0_0_1_1_0_1_2 + 0.0833333333333332*G5_1_0_0_1_1_1_1_0 + 0.0833333333333332*G5_1_0_0_1_1_1_1_1 + 0.0833333333333332*G5_1_0_0_1_1_1_1_2 + 0.0833333333333332*G5_1_0_0_1_1_2_1_0 + 0.0833333333333332*G5_1_0_0_1_1_2_1_1 + 0.0833333333333332*G5_1_0_0_1_1_2_1_2 + 0.0833333333333332*G5_1_0_0_2_1_0_2_0 + 0.0833333333333332*G5_1_0_0_2_1_0_2_1 + 0.0833333333333332*G5_1_0_0_2_1_0_2_2 + 0.0833333333333332*G5_1_0_0_2_1_1_2_0 + 0.0833333333333332*G5_1_0_0_2_1_1_2_1 + 0.0833333333333332*G5_1_0_0_2_1_1_2_2 + 0.0833333333333332*G5_1_0_0_2_1_2_2_0 + 0.0833333333333332*G5_1_0_0_2_1_2_2_1 + 0.0833333333333332*G5_1_0_0_2_1_2_2_2 + 0.0833333333333332*G5_2_0_0_0_2_0_0_0 + 0.0833333333333332*G5_2_0_0_0_2_0_0_1 + 0.0833333333333332*G5_2_0_0_0_2_0_0_2 + 0.0833333333333332*G5_2_0_0_0_2_1_0_0 + 0.0833333333333332*G5_2_0_0_0_2_1_0_1 + 0.0833333333333332*G5_2_0_0_0_2_1_0_2 + 0.0833333333333332*G5_2_0_0_0_2_2_0_0 + 0.0833333333333332*G5_2_0_0_0_2_2_0_1 + 0.0833333333333332*G5_2_0_0_0_2_2_0_2 + 0.0833333333333332*G5_2_0_0_1_2_0_1_0 + 0.0833333333333332*G5_2_0_0_1_2_0_1_1 + 0.0833333333333332*G5_2_0_0_1_2_0_1_2 + 0.0833333333333332*G5_2_0_0_1_2_1_1_0 + 0.0833333333333332*G5_2_0_0_1_2_1_1_1 + 0.0833333333333332*G5_2_0_0_1_2_1_1_2 + 0.0833333333333332*G5_2_0_0_1_2_2_1_0 + 0.0833333333333332*G5_2_0_0_1_2_2_1_1 + 0.0833333333333332*G5_2_0_0_1_2_2_1_2 + 0.0833333333333332*G5_2_0_0_2_2_0_2_0 + 0.0833333333333332*G5_2_0_0_2_2_0_2_1 + 0.0833333333333332*G5_2_0_0_2_2_0_2_2 + 0.0833333333333332*G5_2_0_0_2_2_1_2_0 + 0.0833333333333332*G5_2_0_0_2_2_1_2_1 + 0.0833333333333332*G5_2_0_0_2_2_1_2_2 + 0.0833333333333332*G5_2_0_0_2_2_2_2_0 + 0.0833333333333332*G5_2_0_0_2_2_2_2_1 + 0.0833333333333332*G5_2_0_0_2_2_2_2_2 - 0.0833333333333332*G6_0_0_0_0_0_0_0_0 - 0.0833333333333332*G6_0_0_0_0_0_0_0_1 - 0.0833333333333332*G6_0_0_0_0_0_0_0_2 - 0.0833333333333332*G6_0_0_0_0_0_1_0_0 - 0.0833333333333332*G6_0_0_0_0_0_1_0_1 - 0.0833333333333332*G6_0_0_0_0_0_1_0_2 - 0.0833333333333332*G6_0_0_0_0_0_2_0_0 - 0.0833333333333332*G6_0_0_0_0_0_2_0_1 - 0.0833333333333332*G6_0_0_0_0_0_2_0_2 - 0.0833333333333332*G6_0_0_0_1_0_0_1_0 - 0.0833333333333332*G6_0_0_0_1_0_0_1_1 - 0.0833333333333332*G6_0_0_0_1_0_0_1_2 - 0.0833333333333332*G6_0_0_0_1_0_1_1_0 - 0.0833333333333332*G6_0_0_0_1_0_1_1_1 - 0.0833333333333332*G6_0_0_0_1_0_1_1_2 - 0.0833333333333332*G6_0_0_0_1_0_2_1_0 - 0.0833333333333332*G6_0_0_0_1_0_2_1_1 - 0.0833333333333332*G6_0_0_0_1_0_2_1_2 - 0.0833333333333332*G6_0_0_0_2_0_0_2_0 - 0.0833333333333332*G6_0_0_0_2_0_0_2_1 - 0.0833333333333332*G6_0_0_0_2_0_0_2_2 - 0.0833333333333332*G6_0_0_0_2_0_1_2_0 - 0.0833333333333332*G6_0_0_0_2_0_1_2_1 - 0.0833333333333332*G6_0_0_0_2_0_1_2_2 - 0.0833333333333332*G6_0_0_0_2_0_2_2_0 - 0.0833333333333332*G6_0_0_0_2_0_2_2_1 - 0.0833333333333332*G6_0_0_0_2_0_2_2_2 - 0.0833333333333332*G6_1_0_0_0_1_0_0_0 - 0.0833333333333332*G6_1_0_0_0_1_0_0_1 - 0.0833333333333332*G6_1_0_0_0_1_0_0_2 - 0.0833333333333332*G6_1_0_0_0_1_1_0_0 - 0.0833333333333332*G6_1_0_0_0_1_1_0_1 - 0.0833333333333332*G6_1_0_0_0_1_1_0_2 - 0.0833333333333332*G6_1_0_0_0_1_2_0_0 - 0.0833333333333332*G6_1_0_0_0_1_2_0_1 - 0.0833333333333332*G6_1_0_0_0_1_2_0_2 - 0.0833333333333332*G6_1_0_0_1_1_0_1_0 - 0.0833333333333332*G6_1_0_0_1_1_0_1_1 - 0.0833333333333332*G6_1_0_0_1_1_0_1_2 - 0.0833333333333332*G6_1_0_0_1_1_1_1_0 - 0.0833333333333332*G6_1_0_0_1_1_1_1_1 - 0.0833333333333332*G6_1_0_0_1_1_1_1_2 - 0.0833333333333332*G6_1_0_0_1_1_2_1_0 - 0.0833333333333332*G6_1_0_0_1_1_2_1_1 - 0.0833333333333332*G6_1_0_0_1_1_2_1_2 - 0.0833333333333332*G6_1_0_0_2_1_0_2_0 - 0.0833333333333332*G6_1_0_0_2_1_0_2_1 - 0.0833333333333332*G6_1_0_0_2_1_0_2_2 - 0.0833333333333332*G6_1_0_0_2_1_1_2_0 - 0.0833333333333332*G6_1_0_0_2_1_1_2_1 - 0.0833333333333332*G6_1_0_0_2_1_1_2_2 - 0.0833333333333332*G6_1_0_0_2_1_2_2_0 - 0.0833333333333332*G6_1_0_0_2_1_2_2_1 - 0.0833333333333332*G6_1_0_0_2_1_2_2_2 - 0.0833333333333332*G6_2_0_0_0_2_0_0_0 - 0.0833333333333332*G6_2_0_0_0_2_0_0_1 - 0.0833333333333332*G6_2_0_0_0_2_0_0_2 - 0.0833333333333332*G6_2_0_0_0_2_1_0_0 - 0.0833333333333332*G6_2_0_0_0_2_1_0_1 - 0.0833333333333332*G6_2_0_0_0_2_1_0_2 - 0.0833333333333332*G6_2_0_0_0_2_2_0_0 - 0.0833333333333332*G6_2_0_0_0_2_2_0_1 - 0.0833333333333332*G6_2_0_0_0_2_2_0_2 - 0.0833333333333332*G6_2_0_0_1_2_0_1_0 - 0.0833333333333332*G6_2_0_0_1_2_0_1_1 - 0.0833333333333332*G6_2_0_0_1_2_0_1_2 - 0.0833333333333332*G6_2_0_0_1_2_1_1_0 - 0.0833333333333332*G6_2_0_0_1_2_1_1_1 - 0.0833333333333332*G6_2_0_0_1_2_1_1_2 - 0.0833333333333332*G6_2_0_0_1_2_2_1_0 - 0.0833333333333332*G6_2_0_0_1_2_2_1_1 - 0.0833333333333332*G6_2_0_0_1_2_2_1_2 - 0.0833333333333332*G6_2_0_0_2_2_0_2_0 - 0.0833333333333332*G6_2_0_0_2_2_0_2_1 - 0.0833333333333332*G6_2_0_0_2_2_0_2_2 - 0.0833333333333332*G6_2_0_0_2_2_1_2_0 - 0.0833333333333332*G6_2_0_0_2_2_1_2_1 - 0.0833333333333332*G6_2_0_0_2_2_1_2_2 - 0.0833333333333332*G6_2_0_0_2_2_2_2_0 - 0.0833333333333332*G6_2_0_0_2_2_2_2_1 - 0.0833333333333332*G6_2_0_0_2_2_2_2_2 + 0.0833333333333332*G7_0_0_0_0_0_0 + 0.0833333333333332*G7_0_0_0_0_1_0 + 0.0833333333333332*G7_0_0_0_0_2_0 + 0.0833333333333332*G7_0_0_1_0_0_0 + 0.0833333333333332*G7_0_0_1_0_1_0 + 0.0833333333333332*G7_0_0_1_0_2_0 + 0.0833333333333332*G7_0_0_2_0_0_0 + 0.0833333333333332*G7_0_0_2_0_1_0 + 0.0833333333333332*G7_0_0_2_0_2_0;
    A[1] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.0166666666666666*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.0166666666666666*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.0166666666666666*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.0166666666666666*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.0166666666666666*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 - 0.0833333333333332*G3_0_0_0_0 - 0.0833333333333332*G3_0_0_0_1 - 0.0833333333333332*G3_0_0_0_2 - 0.0416666666666666*G4_0_0_0_0_0_0 - 0.0416666666666666*G4_0_0_0_1_0_0 - 0.0416666666666666*G4_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_1_0_0_1_0 - 0.0833333333333332*G5_0_0_0_1_0_1_1_0 - 0.0833333333333332*G5_0_0_0_1_0_2_1_0 - 0.0833333333333332*G5_0_0_0_2_0_0_2_0 - 0.0833333333333332*G5_0_0_0_2_0_1_2_0 - 0.0833333333333332*G5_0_0_0_2_0_2_2_0 - 0.0833333333333332*G5_1_0_0_0_1_0_0_0 - 0.0833333333333332*G5_1_0_0_0_1_1_0_0 - 0.0833333333333332*G5_1_0_0_0_1_2_0_0 - 0.0833333333333332*G5_1_0_0_1_1_0_1_0 - 0.0833333333333332*G5_1_0_0_1_1_1_1_0 - 0.0833333333333332*G5_1_0_0_1_1_2_1_0 - 0.0833333333333332*G5_1_0_0_2_1_0_2_0 - 0.0833333333333332*G5_1_0_0_2_1_1_2_0 - 0.0833333333333332*G5_1_0_0_2_1_2_2_0 - 0.0833333333333332*G5_2_0_0_0_2_0_0_0 - 0.0833333333333332*G5_2_0_0_0_2_1_0_0 - 0.0833333333333332*G5_2_0_0_0_2_2_0_0 - 0.0833333333333332*G5_2_0_0_1_2_0_1_0 - 0.0833333333333332*G5_2_0_0_1_2_1_1_0 - 0.0833333333333332*G5_2_0_0_1_2_2_1_0 - 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G5_2_0_0_2_2_2_2_0 + 0.0833333333333332*G6_0_0_0_0_0_0_0_0 + 0.0833333333333332*G6_0_0_0_0_0_1_0_0 + 0.0833333333333332*G6_0_0_0_0_0_2_0_0 + 0.0833333333333332*G6_0_0_0_1_0_0_1_0 + 0.0833333333333332*G6_0_0_0_1_0_1_1_0 + 0.0833333333333332*G6_0_0_0_1_0_2_1_0 + 0.0833333333333332*G6_0_0_0_2_0_0_2_0 + 0.0833333333333332*G6_0_0_0_2_0_1_2_0 + 0.0833333333333332*G6_0_0_0_2_0_2_2_0 + 0.0833333333333332*G6_1_0_0_0_1_0_0_0 + 0.0833333333333332*G6_1_0_0_0_1_1_0_0 + 0.0833333333333332*G6_1_0_0_0_1_2_0_0 + 0.0833333333333332*G6_1_0_0_1_1_0_1_0 + 0.0833333333333332*G6_1_0_0_1_1_1_1_0 + 0.0833333333333332*G6_1_0_0_1_1_2_1_0 + 0.0833333333333332*G6_1_0_0_2_1_0_2_0 + 0.0833333333333332*G6_1_0_0_2_1_1_2_0 + 0.0833333333333332*G6_1_0_0_2_1_2_2_0 + 0.0833333333333332*G6_2_0_0_0_2_0_0_0 + 0.0833333333333332*G6_2_0_0_0_2_1_0_0 + 0.0833333333333332*G6_2_0_0_0_2_2_0_0 + 0.0833333333333332*G6_2_0_0_1_2_0_1_0 + 0.0833333333333332*G6_2_0_0_1_2_1_1_0 + 0.0833333333333332*G6_2_0_0_1_2_2_1_0 + 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G6_2_0_0_2_2_2_2_0 - 0.0833333333333332*G7_0_0_0_0_0_0 - 0.0833333333333332*G7_0_0_1_0_0_0 - 0.0833333333333332*G7_0_0_2_0_0_0;
    A[2] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.0166666666666666*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.0166666666666666*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.0166666666666666*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.0166666666666666*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.0166666666666666*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 - 0.0833333333333332*G3_0_0_1_0 - 0.0833333333333332*G3_0_0_1_1 - 0.0833333333333332*G3_0_0_1_2 - 0.0416666666666666*G4_0_0_1_0_0_0 - 0.0416666666666666*G4_0_0_1_1_0_0 - 0.0416666666666666*G4_0_0_1_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_1 - 0.0833333333333332*G5_0_0_0_1_0_0_1_1 - 0.0833333333333332*G5_0_0_0_1_0_1_1_1 - 0.0833333333333332*G5_0_0_0_1_0_2_1_1 - 0.0833333333333332*G5_0_0_0_2_0_0_2_1 - 0.0833333333333332*G5_0_0_0_2_0_1_2_1 - 0.0833333333333332*G5_0_0_0_2_0_2_2_1 - 0.0833333333333332*G5_1_0_0_0_1_0_0_1 - 0.0833333333333332*G5_1_0_0_0_1_1_0_1 - 0.0833333333333332*G5_1_0_0_0_1_2_0_1 - 0.0833333333333332*G5_1_0_0_1_1_0_1_1 - 0.0833333333333332*G5_1_0_0_1_1_1_1_1 - 0.0833333333333332*G5_1_0_0_1_1_2_1_1 - 0.0833333333333332*G5_1_0_0_2_1_0_2_1 - 0.0833333333333332*G5_1_0_0_2_1_1_2_1 - 0.0833333333333332*G5_1_0_0_2_1_2_2_1 - 0.0833333333333332*G5_2_0_0_0_2_0_0_1 - 0.0833333333333332*G5_2_0_0_0_2_1_0_1 - 0.0833333333333332*G5_2_0_0_0_2_2_0_1 - 0.0833333333333332*G5_2_0_0_1_2_0_1_1 - 0.0833333333333332*G5_2_0_0_1_2_1_1_1 - 0.0833333333333332*G5_2_0_0_1_2_2_1_1 - 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G5_2_0_0_2_2_2_2_1 + 0.0833333333333332*G6_0_0_0_0_0_0_0_1 + 0.0833333333333332*G6_0_0_0_0_0_1_0_1 + 0.0833333333333332*G6_0_0_0_0_0_2_0_1 + 0.0833333333333332*G6_0_0_0_1_0_0_1_1 + 0.0833333333333332*G6_0_0_0_1_0_1_1_1 + 0.0833333333333332*G6_0_0_0_1_0_2_1_1 + 0.0833333333333332*G6_0_0_0_2_0_0_2_1 + 0.0833333333333332*G6_0_0_0_2_0_1_2_1 + 0.0833333333333332*G6_0_0_0_2_0_2_2_1 + 0.0833333333333332*G6_1_0_0_0_1_0_0_1 + 0.0833333333333332*G6_1_0_0_0_1_1_0_1 + 0.0833333333333332*G6_1_0_0_0_1_2_0_1 + 0.0833333333333332*G6_1_0_0_1_1_0_1_1 + 0.0833333333333332*G6_1_0_0_1_1_1_1_1 + 0.0833333333333332*G6_1_0_0_1_1_2_1_1 + 0.0833333333333332*G6_1_0_0_2_1_0_2_1 + 0.0833333333333332*G6_1_0_0_2_1_1_2_1 + 0.0833333333333332*G6_1_0_0_2_1_2_2_1 + 0.0833333333333332*G6_2_0_0_0_2_0_0_1 + 0.0833333333333332*G6_2_0_0_0_2_1_0_1 + 0.0833333333333332*G6_2_0_0_0_2_2_0_1 + 0.0833333333333332*G6_2_0_0_1_2_0_1_1 + 0.0833333333333332*G6_2_0_0_1_2_1_1_1 + 0.0833333333333332*G6_2_0_0_1_2_2_1_1 + 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G6_2_0_0_2_2_2_2_1 - 0.0833333333333332*G7_0_0_0_0_1_0 - 0.0833333333333332*G7_0_0_1_0_1_0 - 0.0833333333333332*G7_0_0_2_0_1_0;
    A[3] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.0166666666666666*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.0166666666666666*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.0166666666666666*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.0166666666666666*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.0166666666666666*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_2_0 - 0.0833333333333332*G3_0_0_2_1 - 0.0833333333333332*G3_0_0_2_2 - 0.0416666666666666*G4_0_0_2_0_0_0 - 0.0416666666666666*G4_0_0_2_1_0_0 - 0.0416666666666666*G4_0_0_2_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_2 - 0.0833333333333332*G5_0_0_0_0_0_1_0_2 - 0.0833333333333332*G5_0_0_0_0_0_2_0_2 - 0.0833333333333332*G5_0_0_0_1_0_0_1_2 - 0.0833333333333332*G5_0_0_0_1_0_1_1_2 - 0.0833333333333332*G5_0_0_0_1_0_2_1_2 - 0.0833333333333332*G5_0_0_0_2_0_0_2_2 - 0.0833333333333332*G5_0_0_0_2_0_1_2_2 - 0.0833333333333332*G5_0_0_0_2_0_2_2_2 - 0.0833333333333332*G5_1_0_0_0_1_0_0_2 - 0.0833333333333332*G5_1_0_0_0_1_1_0_2 - 0.0833333333333332*G5_1_0_0_0_1_2_0_2 - 0.0833333333333332*G5_1_0_0_1_1_0_1_2 - 0.0833333333333332*G5_1_0_0_1_1_1_1_2 - 0.0833333333333332*G5_1_0_0_1_1_2_1_2 - 0.0833333333333332*G5_1_0_0_2_1_0_2_2 - 0.0833333333333332*G5_1_0_0_2_1_1_2_2 - 0.0833333333333332*G5_1_0_0_2_1_2_2_2 - 0.0833333333333332*G5_2_0_0_0_2_0_0_2 - 0.0833333333333332*G5_2_0_0_0_2_1_0_2 - 0.0833333333333332*G5_2_0_0_0_2_2_0_2 - 0.0833333333333332*G5_2_0_0_1_2_0_1_2 - 0.0833333333333332*G5_2_0_0_1_2_1_1_2 - 0.0833333333333332*G5_2_0_0_1_2_2_1_2 - 0.0833333333333332*G5_2_0_0_2_2_0_2_2 - 0.0833333333333332*G5_2_0_0_2_2_1_2_2 - 0.0833333333333332*G5_2_0_0_2_2_2_2_2 + 0.0833333333333332*G6_0_0_0_0_0_0_0_2 + 0.0833333333333332*G6_0_0_0_0_0_1_0_2 + 0.0833333333333332*G6_0_0_0_0_0_2_0_2 + 0.0833333333333332*G6_0_0_0_1_0_0_1_2 + 0.0833333333333332*G6_0_0_0_1_0_1_1_2 + 0.0833333333333332*G6_0_0_0_1_0_2_1_2 + 0.0833333333333332*G6_0_0_0_2_0_0_2_2 + 0.0833333333333332*G6_0_0_0_2_0_1_2_2 + 0.0833333333333332*G6_0_0_0_2_0_2_2_2 + 0.0833333333333332*G6_1_0_0_0_1_0_0_2 + 0.0833333333333332*G6_1_0_0_0_1_1_0_2 + 0.0833333333333332*G6_1_0_0_0_1_2_0_2 + 0.0833333333333332*G6_1_0_0_1_1_0_1_2 + 0.0833333333333332*G6_1_0_0_1_1_1_1_2 + 0.0833333333333332*G6_1_0_0_1_1_2_1_2 + 0.0833333333333332*G6_1_0_0_2_1_0_2_2 + 0.0833333333333332*G6_1_0_0_2_1_1_2_2 + 0.0833333333333332*G6_1_0_0_2_1_2_2_2 + 0.0833333333333332*G6_2_0_0_0_2_0_0_2 + 0.0833333333333332*G6_2_0_0_0_2_1_0_2 + 0.0833333333333332*G6_2_0_0_0_2_2_0_2 + 0.0833333333333332*G6_2_0_0_1_2_0_1_2 + 0.0833333333333332*G6_2_0_0_1_2_1_1_2 + 0.0833333333333332*G6_2_0_0_1_2_2_1_2 + 0.0833333333333332*G6_2_0_0_2_2_0_2_2 + 0.0833333333333332*G6_2_0_0_2_2_1_2_2 + 0.0833333333333332*G6_2_0_0_2_2_2_2_2 - 0.0833333333333332*G7_0_0_0_0_2_0 - 0.0833333333333332*G7_0_0_1_0_2_0 - 0.0833333333333332*G7_0_0_2_0_2_0;
    A[4] = 0.0416666666666666*G4_0_0_0_0_0_1 + 0.0416666666666666*G4_0_0_0_1_0_1 + 0.0416666666666666*G4_0_0_0_2_0_1 + 0.0416666666666666*G4_0_0_1_0_0_1 + 0.0416666666666666*G4_0_0_1_1_0_1 + 0.0416666666666666*G4_0_0_1_2_0_1 + 0.0416666666666666*G4_0_0_2_0_0_1 + 0.0416666666666666*G4_0_0_2_1_0_1 + 0.0416666666666666*G4_0_0_2_2_0_1 + 0.0833333333333332*G7_0_0_0_0_0_1 + 0.0833333333333332*G7_0_0_0_0_1_1 + 0.0833333333333332*G7_0_0_0_0_2_1 + 0.0833333333333332*G7_0_0_1_0_0_1 + 0.0833333333333332*G7_0_0_1_0_1_1 + 0.0833333333333332*G7_0_0_1_0_2_1 + 0.0833333333333332*G7_0_0_2_0_0_1 + 0.0833333333333332*G7_0_0_2_0_1_1 + 0.0833333333333332*G7_0_0_2_0_2_1;
    A[5] = -0.0416666666666666*G4_0_0_0_0_0_1 - 0.0416666666666666*G4_0_0_0_1_0_1 - 0.0416666666666666*G4_0_0_0_2_0_1 - 0.0833333333333332*G7_0_0_0_0_0_1 - 0.0833333333333332*G7_0_0_1_0_0_1 - 0.0833333333333332*G7_0_0_2_0_0_1;
    A[6] = -0.0416666666666666*G4_0_0_1_0_0_1 - 0.0416666666666666*G4_0_0_1_1_0_1 - 0.0416666666666666*G4_0_0_1_2_0_1 - 0.0833333333333332*G7_0_0_0_0_1_1 - 0.0833333333333332*G7_0_0_1_0_1_1 - 0.0833333333333332*G7_0_0_2_0_1_1;
    A[7] = -0.0416666666666666*G4_0_0_2_0_0_1 - 0.0416666666666666*G4_0_0_2_1_0_1 - 0.0416666666666666*G4_0_0_2_2_0_1 - 0.0833333333333332*G7_0_0_0_0_2_1 - 0.0833333333333332*G7_0_0_1_0_2_1 - 0.0833333333333332*G7_0_0_2_0_2_1;
    A[8] = 0.0416666666666666*G4_0_0_0_0_0_2 + 0.0416666666666666*G4_0_0_0_1_0_2 + 0.0416666666666666*G4_0_0_0_2_0_2 + 0.0416666666666666*G4_0_0_1_0_0_2 + 0.0416666666666666*G4_0_0_1_1_0_2 + 0.0416666666666666*G4_0_0_1_2_0_2 + 0.0416666666666666*G4_0_0_2_0_0_2 + 0.0416666666666666*G4_0_0_2_1_0_2 + 0.0416666666666666*G4_0_0_2_2_0_2 + 0.0833333333333332*G7_0_0_0_0_0_2 + 0.0833333333333332*G7_0_0_0_0_1_2 + 0.0833333333333332*G7_0_0_0_0_2_2 + 0.0833333333333332*G7_0_0_1_0_0_2 + 0.0833333333333332*G7_0_0_1_0_1_2 + 0.0833333333333332*G7_0_0_1_0_2_2 + 0.0833333333333332*G7_0_0_2_0_0_2 + 0.0833333333333332*G7_0_0_2_0_1_2 + 0.0833333333333332*G7_0_0_2_0_2_2;
    A[9] = -0.0416666666666666*G4_0_0_0_0_0_2 - 0.0416666666666666*G4_0_0_0_1_0_2 - 0.0416666666666666*G4_0_0_0_2_0_2 - 0.0833333333333332*G7_0_0_0_0_0_2 - 0.0833333333333332*G7_0_0_1_0_0_2 - 0.0833333333333332*G7_0_0_2_0_0_2;
    A[10] = -0.0416666666666666*G4_0_0_1_0_0_2 - 0.0416666666666666*G4_0_0_1_1_0_2 - 0.0416666666666666*G4_0_0_1_2_0_2 - 0.0833333333333332*G7_0_0_0_0_1_2 - 0.0833333333333332*G7_0_0_1_0_1_2 - 0.0833333333333332*G7_0_0_2_0_1_2;
    A[11] = -0.0416666666666666*G4_0_0_2_0_0_2 - 0.0416666666666666*G4_0_0_2_1_0_2 - 0.0416666666666666*G4_0_0_2_2_0_2 - 0.0833333333333332*G7_0_0_0_0_2_2 - 0.0833333333333332*G7_0_0_1_0_2_2 - 0.0833333333333332*G7_0_0_2_0_2_2;
    A[12] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.0166666666666666*G1_0_0_0_1_0_0 - 0.0166666666666666*G1_0_0_0_1_1_0 - 0.0166666666666666*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.0166666666666666*G1_0_0_0_5_0_1 - 0.0166666666666666*G1_0_0_0_5_1_1 - 0.0166666666666666*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.0166666666666666*G1_0_0_0_9_0_2 - 0.0166666666666666*G1_0_0_0_9_1_2 - 0.0166666666666666*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.0166666666666666*G2_0_0_0_1_0_0 + 0.0166666666666666*G2_0_0_0_1_1_0 + 0.0166666666666666*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.0166666666666666*G2_0_0_0_5_0_1 + 0.0166666666666666*G2_0_0_0_5_1_1 + 0.0166666666666666*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.0166666666666666*G2_0_0_0_9_0_2 + 0.0166666666666666*G2_0_0_0_9_1_2 + 0.0166666666666666*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_0 - 0.0833333333333332*G3_0_0_1_0 - 0.0833333333333332*G3_0_0_2_0 - 0.0416666666666666*G4_0_0_0_0_0_0 - 0.0416666666666666*G4_0_0_1_0_0_0 - 0.0416666666666666*G4_0_0_2_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_2 - 0.0833333333333332*G5_0_0_0_1_0_0_1_0 - 0.0833333333333332*G5_0_0_0_1_0_0_1_1 - 0.0833333333333332*G5_0_0_0_1_0_0_1_2 - 0.0833333333333332*G5_0_0_0_2_0_0_2_0 - 0.0833333333333332*G5_0_0_0_2_0_0_2_1 - 0.0833333333333332*G5_0_0_0_2_0_0_2_2 - 0.0833333333333332*G5_1_0_0_0_1_0_0_0 - 0.0833333333333332*G5_1_0_0_0_1_0_0_1 - 0.0833333333333332*G5_1_0_0_0_1_0_0_2 - 0.0833333333333332*G5_1_0_0_1_1_0_1_0 - 0.0833333333333332*G5_1_0_0_1_1_0_1_1 - 0.0833333333333332*G5_1_0_0_1_1_0_1_2 - 0.0833333333333332*G5_1_0_0_2_1_0_2_0 - 0.0833333333333332*G5_1_0_0_2_1_0_2_1 - 0.0833333333333332*G5_1_0_0_2_1_0_2_2 - 0.0833333333333332*G5_2_0_0_0_2_0_0_0 - 0.0833333333333332*G5_2_0_0_0_2_0_0_1 - 0.0833333333333332*G5_2_0_0_0_2_0_0_2 - 0.0833333333333332*G5_2_0_0_1_2_0_1_0 - 0.0833333333333332*G5_2_0_0_1_2_0_1_1 - 0.0833333333333332*G5_2_0_0_1_2_0_1_2 - 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G5_2_0_0_2_2_0_2_2 + 0.0833333333333332*G6_0_0_0_0_0_0_0_0 + 0.0833333333333332*G6_0_0_0_0_0_0_0_1 + 0.0833333333333332*G6_0_0_0_0_0_0_0_2 + 0.0833333333333332*G6_0_0_0_1_0_0_1_0 + 0.0833333333333332*G6_0_0_0_1_0_0_1_1 + 0.0833333333333332*G6_0_0_0_1_0_0_1_2 + 0.0833333333333332*G6_0_0_0_2_0_0_2_0 + 0.0833333333333332*G6_0_0_0_2_0_0_2_1 + 0.0833333333333332*G6_0_0_0_2_0_0_2_2 + 0.0833333333333332*G6_1_0_0_0_1_0_0_0 + 0.0833333333333332*G6_1_0_0_0_1_0_0_1 + 0.0833333333333332*G6_1_0_0_0_1_0_0_2 + 0.0833333333333332*G6_1_0_0_1_1_0_1_0 + 0.0833333333333332*G6_1_0_0_1_1_0_1_1 + 0.0833333333333332*G6_1_0_0_1_1_0_1_2 + 0.0833333333333332*G6_1_0_0_2_1_0_2_0 + 0.0833333333333332*G6_1_0_0_2_1_0_2_1 + 0.0833333333333332*G6_1_0_0_2_1_0_2_2 + 0.0833333333333332*G6_2_0_0_0_2_0_0_0 + 0.0833333333333332*G6_2_0_0_0_2_0_0_1 + 0.0833333333333332*G6_2_0_0_0_2_0_0_2 + 0.0833333333333332*G6_2_0_0_1_2_0_1_0 + 0.0833333333333332*G6_2_0_0_1_2_0_1_1 + 0.0833333333333332*G6_2_0_0_1_2_0_1_2 + 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G6_2_0_0_2_2_0_2_2 - 0.0833333333333332*G7_0_0_0_0_0_0 - 0.0833333333333332*G7_0_0_0_0_1_0 - 0.0833333333333332*G7_0_0_0_0_2_0;
    A[13] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.0166666666666666*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.0166666666666666*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.0166666666666666*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.0166666666666666*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.0166666666666666*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.0166666666666666*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_0 + 0.0416666666666666*G4_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_1_0_0_1_0 + 0.0833333333333332*G5_0_0_0_2_0_0_2_0 + 0.0833333333333332*G5_1_0_0_0_1_0_0_0 + 0.0833333333333332*G5_1_0_0_1_1_0_1_0 + 0.0833333333333332*G5_1_0_0_2_1_0_2_0 + 0.0833333333333332*G5_2_0_0_0_2_0_0_0 + 0.0833333333333332*G5_2_0_0_1_2_0_1_0 + 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G6_0_0_0_0_0_0_0_0 - 0.0833333333333332*G6_0_0_0_1_0_0_1_0 - 0.0833333333333332*G6_0_0_0_2_0_0_2_0 - 0.0833333333333332*G6_1_0_0_0_1_0_0_0 - 0.0833333333333332*G6_1_0_0_1_1_0_1_0 - 0.0833333333333332*G6_1_0_0_2_1_0_2_0 - 0.0833333333333332*G6_2_0_0_0_2_0_0_0 - 0.0833333333333332*G6_2_0_0_1_2_0_1_0 - 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G7_0_0_0_0_0_0;
    A[14] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.0166666666666666*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.0166666666666666*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.0166666666666666*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.0166666666666666*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.0166666666666666*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.0166666666666666*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_0 + 0.0416666666666666*G4_0_0_1_0_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_1 + 0.0833333333333332*G5_0_0_0_1_0_0_1_1 + 0.0833333333333332*G5_0_0_0_2_0_0_2_1 + 0.0833333333333332*G5_1_0_0_0_1_0_0_1 + 0.0833333333333332*G5_1_0_0_1_1_0_1_1 + 0.0833333333333332*G5_1_0_0_2_1_0_2_1 + 0.0833333333333332*G5_2_0_0_0_2_0_0_1 + 0.0833333333333332*G5_2_0_0_1_2_0_1_1 + 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G6_0_0_0_0_0_0_0_1 - 0.0833333333333332*G6_0_0_0_1_0_0_1_1 - 0.0833333333333332*G6_0_0_0_2_0_0_2_1 - 0.0833333333333332*G6_1_0_0_0_1_0_0_1 - 0.0833333333333332*G6_1_0_0_1_1_0_1_1 - 0.0833333333333332*G6_1_0_0_2_1_0_2_1 - 0.0833333333333332*G6_2_0_0_0_2_0_0_1 - 0.0833333333333332*G6_2_0_0_1_2_0_1_1 - 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G7_0_0_0_0_1_0;
    A[15] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.0166666666666666*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.0166666666666666*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.0166666666666666*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.0166666666666666*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.0166666666666666*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.0166666666666666*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_0 + 0.0416666666666666*G4_0_0_2_0_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_2 + 0.0833333333333332*G5_0_0_0_1_0_0_1_2 + 0.0833333333333332*G5_0_0_0_2_0_0_2_2 + 0.0833333333333332*G5_1_0_0_0_1_0_0_2 + 0.0833333333333332*G5_1_0_0_1_1_0_1_2 + 0.0833333333333332*G5_1_0_0_2_1_0_2_2 + 0.0833333333333332*G5_2_0_0_0_2_0_0_2 + 0.0833333333333332*G5_2_0_0_1_2_0_1_2 + 0.0833333333333332*G5_2_0_0_2_2_0_2_2 - 0.0833333333333332*G6_0_0_0_0_0_0_0_2 - 0.0833333333333332*G6_0_0_0_1_0_0_1_2 - 0.0833333333333332*G6_0_0_0_2_0_0_2_2 - 0.0833333333333332*G6_1_0_0_0_1_0_0_2 - 0.0833333333333332*G6_1_0_0_1_1_0_1_2 - 0.0833333333333332*G6_1_0_0_2_1_0_2_2 - 0.0833333333333332*G6_2_0_0_0_2_0_0_2 - 0.0833333333333332*G6_2_0_0_1_2_0_1_2 - 0.0833333333333332*G6_2_0_0_2_2_0_2_2 + 0.0833333333333332*G7_0_0_0_0_2_0;
    A[16] = -0.0416666666666666*G4_0_0_0_0_0_1 - 0.0416666666666666*G4_0_0_1_0_0_1 - 0.0416666666666666*G4_0_0_2_0_0_1 - 0.0833333333333332*G7_0_0_0_0_0_1 - 0.0833333333333332*G7_0_0_0_0_1_1 - 0.0833333333333332*G7_0_0_0_0_2_1;
    A[17] = 0.0416666666666666*G4_0_0_0_0_0_1 + 0.0833333333333332*G7_0_0_0_0_0_1;
    A[18] = 0.0416666666666666*G4_0_0_1_0_0_1 + 0.0833333333333332*G7_0_0_0_0_1_1;
    A[19] = 0.0416666666666666*G4_0_0_2_0_0_1 + 0.0833333333333332*G7_0_0_0_0_2_1;
    A[20] = -0.0416666666666666*G4_0_0_0_0_0_2 - 0.0416666666666666*G4_0_0_1_0_0_2 - 0.0416666666666666*G4_0_0_2_0_0_2 - 0.0833333333333332*G7_0_0_0_0_0_2 - 0.0833333333333332*G7_0_0_0_0_1_2 - 0.0833333333333332*G7_0_0_0_0_2_2;
    A[21] = 0.0416666666666666*G4_0_0_0_0_0_2 + 0.0833333333333332*G7_0_0_0_0_0_2;
    A[22] = 0.0416666666666666*G4_0_0_1_0_0_2 + 0.0833333333333332*G7_0_0_0_0_1_2;
    A[23] = 0.0416666666666666*G4_0_0_2_0_0_2 + 0.0833333333333332*G7_0_0_0_0_2_2;
    A[24] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.0166666666666666*G1_0_0_0_2_0_0 - 0.0166666666666666*G1_0_0_0_2_1_0 - 0.0166666666666666*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.0166666666666666*G1_0_0_0_6_0_1 - 0.0166666666666666*G1_0_0_0_6_1_1 - 0.0166666666666666*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.0166666666666666*G1_0_0_0_10_0_2 - 0.0166666666666666*G1_0_0_0_10_1_2 - 0.0166666666666666*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.0166666666666666*G2_0_0_0_2_0_0 + 0.0166666666666666*G2_0_0_0_2_1_0 + 0.0166666666666666*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.0166666666666666*G2_0_0_0_6_0_1 + 0.0166666666666666*G2_0_0_0_6_1_1 + 0.0166666666666666*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.0166666666666666*G2_0_0_0_10_0_2 + 0.0166666666666666*G2_0_0_0_10_1_2 + 0.0166666666666666*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_1 - 0.0833333333333332*G3_0_0_1_1 - 0.0833333333333332*G3_0_0_2_1 - 0.0416666666666666*G4_0_0_0_1_0_0 - 0.0416666666666666*G4_0_0_1_1_0_0 - 0.0416666666666666*G4_0_0_2_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_2 - 0.0833333333333332*G5_0_0_0_1_0_1_1_0 - 0.0833333333333332*G5_0_0_0_1_0_1_1_1 - 0.0833333333333332*G5_0_0_0_1_0_1_1_2 - 0.0833333333333332*G5_0_0_0_2_0_1_2_0 - 0.0833333333333332*G5_0_0_0_2_0_1_2_1 - 0.0833333333333332*G5_0_0_0_2_0_1_2_2 - 0.0833333333333332*G5_1_0_0_0_1_1_0_0 - 0.0833333333333332*G5_1_0_0_0_1_1_0_1 - 0.0833333333333332*G5_1_0_0_0_1_1_0_2 - 0.0833333333333332*G5_1_0_0_1_1_1_1_0 - 0.0833333333333332*G5_1_0_0_1_1_1_1_1 - 0.0833333333333332*G5_1_0_0_1_1_1_1_2 - 0.0833333333333332*G5_1_0_0_2_1_1_2_0 - 0.0833333333333332*G5_1_0_0_2_1_1_2_1 - 0.0833333333333332*G5_1_0_0_2_1_1_2_2 - 0.0833333333333332*G5_2_0_0_0_2_1_0_0 - 0.0833333333333332*G5_2_0_0_0_2_1_0_1 - 0.0833333333333332*G5_2_0_0_0_2_1_0_2 - 0.0833333333333332*G5_2_0_0_1_2_1_1_0 - 0.0833333333333332*G5_2_0_0_1_2_1_1_1 - 0.0833333333333332*G5_2_0_0_1_2_1_1_2 - 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G5_2_0_0_2_2_1_2_2 + 0.0833333333333332*G6_0_0_0_0_0_1_0_0 + 0.0833333333333332*G6_0_0_0_0_0_1_0_1 + 0.0833333333333332*G6_0_0_0_0_0_1_0_2 + 0.0833333333333332*G6_0_0_0_1_0_1_1_0 + 0.0833333333333332*G6_0_0_0_1_0_1_1_1 + 0.0833333333333332*G6_0_0_0_1_0_1_1_2 + 0.0833333333333332*G6_0_0_0_2_0_1_2_0 + 0.0833333333333332*G6_0_0_0_2_0_1_2_1 + 0.0833333333333332*G6_0_0_0_2_0_1_2_2 + 0.0833333333333332*G6_1_0_0_0_1_1_0_0 + 0.0833333333333332*G6_1_0_0_0_1_1_0_1 + 0.0833333333333332*G6_1_0_0_0_1_1_0_2 + 0.0833333333333332*G6_1_0_0_1_1_1_1_0 + 0.0833333333333332*G6_1_0_0_1_1_1_1_1 + 0.0833333333333332*G6_1_0_0_1_1_1_1_2 + 0.0833333333333332*G6_1_0_0_2_1_1_2_0 + 0.0833333333333332*G6_1_0_0_2_1_1_2_1 + 0.0833333333333332*G6_1_0_0_2_1_1_2_2 + 0.0833333333333332*G6_2_0_0_0_2_1_0_0 + 0.0833333333333332*G6_2_0_0_0_2_1_0_1 + 0.0833333333333332*G6_2_0_0_0_2_1_0_2 + 0.0833333333333332*G6_2_0_0_1_2_1_1_0 + 0.0833333333333332*G6_2_0_0_1_2_1_1_1 + 0.0833333333333332*G6_2_0_0_1_2_1_1_2 + 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G6_2_0_0_2_2_1_2_2 - 0.0833333333333332*G7_0_0_1_0_0_0 - 0.0833333333333332*G7_0_0_1_0_1_0 - 0.0833333333333332*G7_0_0_1_0_2_0;
    A[25] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.0166666666666666*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.0166666666666666*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.0166666666666666*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.0166666666666666*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.0166666666666666*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.0166666666666666*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_1 + 0.0416666666666666*G4_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_1_0_1_1_0 + 0.0833333333333332*G5_0_0_0_2_0_1_2_0 + 0.0833333333333332*G5_1_0_0_0_1_1_0_0 + 0.0833333333333332*G5_1_0_0_1_1_1_1_0 + 0.0833333333333332*G5_1_0_0_2_1_1_2_0 + 0.0833333333333332*G5_2_0_0_0_2_1_0_0 + 0.0833333333333332*G5_2_0_0_1_2_1_1_0 + 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G6_0_0_0_0_0_1_0_0 - 0.0833333333333332*G6_0_0_0_1_0_1_1_0 - 0.0833333333333332*G6_0_0_0_2_0_1_2_0 - 0.0833333333333332*G6_1_0_0_0_1_1_0_0 - 0.0833333333333332*G6_1_0_0_1_1_1_1_0 - 0.0833333333333332*G6_1_0_0_2_1_1_2_0 - 0.0833333333333332*G6_2_0_0_0_2_1_0_0 - 0.0833333333333332*G6_2_0_0_1_2_1_1_0 - 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G7_0_0_1_0_0_0;
    A[26] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.0166666666666666*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.0166666666666666*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.0166666666666666*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.0166666666666666*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.0166666666666666*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.0166666666666666*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_1 + 0.0416666666666666*G4_0_0_1_1_0_0 + 0.0833333333333332*G5_0_0_0_0_0_1_0_1 + 0.0833333333333332*G5_0_0_0_1_0_1_1_1 + 0.0833333333333332*G5_0_0_0_2_0_1_2_1 + 0.0833333333333332*G5_1_0_0_0_1_1_0_1 + 0.0833333333333332*G5_1_0_0_1_1_1_1_1 + 0.0833333333333332*G5_1_0_0_2_1_1_2_1 + 0.0833333333333332*G5_2_0_0_0_2_1_0_1 + 0.0833333333333332*G5_2_0_0_1_2_1_1_1 + 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G6_0_0_0_0_0_1_0_1 - 0.0833333333333332*G6_0_0_0_1_0_1_1_1 - 0.0833333333333332*G6_0_0_0_2_0_1_2_1 - 0.0833333333333332*G6_1_0_0_0_1_1_0_1 - 0.0833333333333332*G6_1_0_0_1_1_1_1_1 - 0.0833333333333332*G6_1_0_0_2_1_1_2_1 - 0.0833333333333332*G6_2_0_0_0_2_1_0_1 - 0.0833333333333332*G6_2_0_0_1_2_1_1_1 - 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G7_0_0_1_0_1_0;
    A[27] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.0166666666666666*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.0166666666666666*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.0166666666666666*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.0166666666666666*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.0166666666666666*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.0166666666666666*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_1 + 0.0416666666666666*G4_0_0_2_1_0_0 + 0.0833333333333332*G5_0_0_0_0_0_1_0_2 + 0.0833333333333332*G5_0_0_0_1_0_1_1_2 + 0.0833333333333332*G5_0_0_0_2_0_1_2_2 + 0.0833333333333332*G5_1_0_0_0_1_1_0_2 + 0.0833333333333332*G5_1_0_0_1_1_1_1_2 + 0.0833333333333332*G5_1_0_0_2_1_1_2_2 + 0.0833333333333332*G5_2_0_0_0_2_1_0_2 + 0.0833333333333332*G5_2_0_0_1_2_1_1_2 + 0.0833333333333332*G5_2_0_0_2_2_1_2_2 - 0.0833333333333332*G6_0_0_0_0_0_1_0_2 - 0.0833333333333332*G6_0_0_0_1_0_1_1_2 - 0.0833333333333332*G6_0_0_0_2_0_1_2_2 - 0.0833333333333332*G6_1_0_0_0_1_1_0_2 - 0.0833333333333332*G6_1_0_0_1_1_1_1_2 - 0.0833333333333332*G6_1_0_0_2_1_1_2_2 - 0.0833333333333332*G6_2_0_0_0_2_1_0_2 - 0.0833333333333332*G6_2_0_0_1_2_1_1_2 - 0.0833333333333332*G6_2_0_0_2_2_1_2_2 + 0.0833333333333332*G7_0_0_1_0_2_0;
    A[28] = -0.0416666666666666*G4_0_0_0_1_0_1 - 0.0416666666666666*G4_0_0_1_1_0_1 - 0.0416666666666666*G4_0_0_2_1_0_1 - 0.0833333333333332*G7_0_0_1_0_0_1 - 0.0833333333333332*G7_0_0_1_0_1_1 - 0.0833333333333332*G7_0_0_1_0_2_1;
    A[29] = 0.0416666666666666*G4_0_0_0_1_0_1 + 0.0833333333333332*G7_0_0_1_0_0_1;
    A[30] = 0.0416666666666666*G4_0_0_1_1_0_1 + 0.0833333333333332*G7_0_0_1_0_1_1;
    A[31] = 0.0416666666666666*G4_0_0_2_1_0_1 + 0.0833333333333332*G7_0_0_1_0_2_1;
    A[32] = -0.0416666666666666*G4_0_0_0_1_0_2 - 0.0416666666666666*G4_0_0_1_1_0_2 - 0.0416666666666666*G4_0_0_2_1_0_2 - 0.0833333333333332*G7_0_0_1_0_0_2 - 0.0833333333333332*G7_0_0_1_0_1_2 - 0.0833333333333332*G7_0_0_1_0_2_2;
    A[33] = 0.0416666666666666*G4_0_0_0_1_0_2 + 0.0833333333333332*G7_0_0_1_0_0_2;
    A[34] = 0.0416666666666666*G4_0_0_1_1_0_2 + 0.0833333333333332*G7_0_0_1_0_1_2;
    A[35] = 0.0416666666666666*G4_0_0_2_1_0_2 + 0.0833333333333332*G7_0_0_1_0_2_2;
    A[36] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.0166666666666666*G1_0_0_0_3_0_0 - 0.0166666666666666*G1_0_0_0_3_1_0 - 0.0166666666666666*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.0166666666666666*G1_0_0_0_7_0_1 - 0.0166666666666666*G1_0_0_0_7_1_1 - 0.0166666666666666*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.0166666666666666*G1_0_0_0_11_0_2 - 0.0166666666666666*G1_0_0_0_11_1_2 - 0.0166666666666666*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.0166666666666666*G2_0_0_0_3_0_0 + 0.0166666666666666*G2_0_0_0_3_1_0 + 0.0166666666666666*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.0166666666666666*G2_0_0_0_7_0_1 + 0.0166666666666666*G2_0_0_0_7_1_1 + 0.0166666666666666*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.0166666666666666*G2_0_0_0_11_0_2 + 0.0166666666666666*G2_0_0_0_11_1_2 + 0.0166666666666666*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_2 - 0.0833333333333332*G3_0_0_1_2 - 0.0833333333333332*G3_0_0_2_2 - 0.0416666666666666*G4_0_0_0_2_0_0 - 0.0416666666666666*G4_0_0_1_2_0_0 - 0.0416666666666666*G4_0_0_2_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_2 - 0.0833333333333332*G5_0_0_0_1_0_2_1_0 - 0.0833333333333332*G5_0_0_0_1_0_2_1_1 - 0.0833333333333332*G5_0_0_0_1_0_2_1_2 - 0.0833333333333332*G5_0_0_0_2_0_2_2_0 - 0.0833333333333332*G5_0_0_0_2_0_2_2_1 - 0.0833333333333332*G5_0_0_0_2_0_2_2_2 - 0.0833333333333332*G5_1_0_0_0_1_2_0_0 - 0.0833333333333332*G5_1_0_0_0_1_2_0_1 - 0.0833333333333332*G5_1_0_0_0_1_2_0_2 - 0.0833333333333332*G5_1_0_0_1_1_2_1_0 - 0.0833333333333332*G5_1_0_0_1_1_2_1_1 - 0.0833333333333332*G5_1_0_0_1_1_2_1_2 - 0.0833333333333332*G5_1_0_0_2_1_2_2_0 - 0.0833333333333332*G5_1_0_0_2_1_2_2_1 - 0.0833333333333332*G5_1_0_0_2_1_2_2_2 - 0.0833333333333332*G5_2_0_0_0_2_2_0_0 - 0.0833333333333332*G5_2_0_0_0_2_2_0_1 - 0.0833333333333332*G5_2_0_0_0_2_2_0_2 - 0.0833333333333332*G5_2_0_0_1_2_2_1_0 - 0.0833333333333332*G5_2_0_0_1_2_2_1_1 - 0.0833333333333332*G5_2_0_0_1_2_2_1_2 - 0.0833333333333332*G5_2_0_0_2_2_2_2_0 - 0.0833333333333332*G5_2_0_0_2_2_2_2_1 - 0.0833333333333332*G5_2_0_0_2_2_2_2_2 + 0.0833333333333332*G6_0_0_0_0_0_2_0_0 + 0.0833333333333332*G6_0_0_0_0_0_2_0_1 + 0.0833333333333332*G6_0_0_0_0_0_2_0_2 + 0.0833333333333332*G6_0_0_0_1_0_2_1_0 + 0.0833333333333332*G6_0_0_0_1_0_2_1_1 + 0.0833333333333332*G6_0_0_0_1_0_2_1_2 + 0.0833333333333332*G6_0_0_0_2_0_2_2_0 + 0.0833333333333332*G6_0_0_0_2_0_2_2_1 + 0.0833333333333332*G6_0_0_0_2_0_2_2_2 + 0.0833333333333332*G6_1_0_0_0_1_2_0_0 + 0.0833333333333332*G6_1_0_0_0_1_2_0_1 + 0.0833333333333332*G6_1_0_0_0_1_2_0_2 + 0.0833333333333332*G6_1_0_0_1_1_2_1_0 + 0.0833333333333332*G6_1_0_0_1_1_2_1_1 + 0.0833333333333332*G6_1_0_0_1_1_2_1_2 + 0.0833333333333332*G6_1_0_0_2_1_2_2_0 + 0.0833333333333332*G6_1_0_0_2_1_2_2_1 + 0.0833333333333332*G6_1_0_0_2_1_2_2_2 + 0.0833333333333332*G6_2_0_0_0_2_2_0_0 + 0.0833333333333332*G6_2_0_0_0_2_2_0_1 + 0.0833333333333332*G6_2_0_0_0_2_2_0_2 + 0.0833333333333332*G6_2_0_0_1_2_2_1_0 + 0.0833333333333332*G6_2_0_0_1_2_2_1_1 + 0.0833333333333332*G6_2_0_0_1_2_2_1_2 + 0.0833333333333332*G6_2_0_0_2_2_2_2_0 + 0.0833333333333332*G6_2_0_0_2_2_2_2_1 + 0.0833333333333332*G6_2_0_0_2_2_2_2_2 - 0.0833333333333332*G7_0_0_2_0_0_0 - 0.0833333333333332*G7_0_0_2_0_1_0 - 0.0833333333333332*G7_0_0_2_0_2_0;
    A[37] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.0166666666666666*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.0166666666666666*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.0166666666666666*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.0166666666666666*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.0166666666666666*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.0166666666666666*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_2 + 0.0416666666666666*G4_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_1_0_2_1_0 + 0.0833333333333332*G5_0_0_0_2_0_2_2_0 + 0.0833333333333332*G5_1_0_0_0_1_2_0_0 + 0.0833333333333332*G5_1_0_0_1_1_2_1_0 + 0.0833333333333332*G5_1_0_0_2_1_2_2_0 + 0.0833333333333332*G5_2_0_0_0_2_2_0_0 + 0.0833333333333332*G5_2_0_0_1_2_2_1_0 + 0.0833333333333332*G5_2_0_0_2_2_2_2_0 - 0.0833333333333332*G6_0_0_0_0_0_2_0_0 - 0.0833333333333332*G6_0_0_0_1_0_2_1_0 - 0.0833333333333332*G6_0_0_0_2_0_2_2_0 - 0.0833333333333332*G6_1_0_0_0_1_2_0_0 - 0.0833333333333332*G6_1_0_0_1_1_2_1_0 - 0.0833333333333332*G6_1_0_0_2_1_2_2_0 - 0.0833333333333332*G6_2_0_0_0_2_2_0_0 - 0.0833333333333332*G6_2_0_0_1_2_2_1_0 - 0.0833333333333332*G6_2_0_0_2_2_2_2_0 + 0.0833333333333332*G7_0_0_2_0_0_0;
    A[38] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.0166666666666666*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.0166666666666666*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.0166666666666666*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.0166666666666666*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.0166666666666666*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.0166666666666666*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_2 + 0.0416666666666666*G4_0_0_1_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_2_0_1 + 0.0833333333333332*G5_0_0_0_1_0_2_1_1 + 0.0833333333333332*G5_0_0_0_2_0_2_2_1 + 0.0833333333333332*G5_1_0_0_0_1_2_0_1 + 0.0833333333333332*G5_1_0_0_1_1_2_1_1 + 0.0833333333333332*G5_1_0_0_2_1_2_2_1 + 0.0833333333333332*G5_2_0_0_0_2_2_0_1 + 0.0833333333333332*G5_2_0_0_1_2_2_1_1 + 0.0833333333333332*G5_2_0_0_2_2_2_2_1 - 0.0833333333333332*G6_0_0_0_0_0_2_0_1 - 0.0833333333333332*G6_0_0_0_1_0_2_1_1 - 0.0833333333333332*G6_0_0_0_2_0_2_2_1 - 0.0833333333333332*G6_1_0_0_0_1_2_0_1 - 0.0833333333333332*G6_1_0_0_1_1_2_1_1 - 0.0833333333333332*G6_1_0_0_2_1_2_2_1 - 0.0833333333333332*G6_2_0_0_0_2_2_0_1 - 0.0833333333333332*G6_2_0_0_1_2_2_1_1 - 0.0833333333333332*G6_2_0_0_2_2_2_2_1 + 0.0833333333333332*G7_0_0_2_0_1_0;
    A[39] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.0166666666666666*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.0166666666666666*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.0166666666666666*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.0166666666666666*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.0166666666666666*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.0166666666666666*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_2 + 0.0416666666666666*G4_0_0_2_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_2_0_2 + 0.0833333333333332*G5_0_0_0_1_0_2_1_2 + 0.0833333333333332*G5_0_0_0_2_0_2_2_2 + 0.0833333333333332*G5_1_0_0_0_1_2_0_2 + 0.0833333333333332*G5_1_0_0_1_1_2_1_2 + 0.0833333333333332*G5_1_0_0_2_1_2_2_2 + 0.0833333333333332*G5_2_0_0_0_2_2_0_2 + 0.0833333333333332*G5_2_0_0_1_2_2_1_2 + 0.0833333333333332*G5_2_0_0_2_2_2_2_2 - 0.0833333333333332*G6_0_0_0_0_0_2_0_2 - 0.0833333333333332*G6_0_0_0_1_0_2_1_2 - 0.0833333333333332*G6_0_0_0_2_0_2_2_2 - 0.0833333333333332*G6_1_0_0_0_1_2_0_2 - 0.0833333333333332*G6_1_0_0_1_1_2_1_2 - 0.0833333333333332*G6_1_0_0_2_1_2_2_2 - 0.0833333333333332*G6_2_0_0_0_2_2_0_2 - 0.0833333333333332*G6_2_0_0_1_2_2_1_2 - 0.0833333333333332*G6_2_0_0_2_2_2_2_2 + 0.0833333333333332*G7_0_0_2_0_2_0;
    A[40] = -0.0416666666666666*G4_0_0_0_2_0_1 - 0.0416666666666666*G4_0_0_1_2_0_1 - 0.0416666666666666*G4_0_0_2_2_0_1 - 0.0833333333333332*G7_0_0_2_0_0_1 - 0.0833333333333332*G7_0_0_2_0_1_1 - 0.0833333333333332*G7_0_0_2_0_2_1;
    A[41] = 0.0416666666666666*G4_0_0_0_2_0_1 + 0.0833333333333332*G7_0_0_2_0_0_1;
    A[42] = 0.0416666666666666*G4_0_0_1_2_0_1 + 0.0833333333333332*G7_0_0_2_0_1_1;
    A[43] = 0.0416666666666666*G4_0_0_2_2_0_1 + 0.0833333333333332*G7_0_0_2_0_2_1;
    A[44] = -0.0416666666666666*G4_0_0_0_2_0_2 - 0.0416666666666666*G4_0_0_1_2_0_2 - 0.0416666666666666*G4_0_0_2_2_0_2 - 0.0833333333333332*G7_0_0_2_0_0_2 - 0.0833333333333332*G7_0_0_2_0_1_2 - 0.0833333333333332*G7_0_0_2_0_2_2;
    A[45] = 0.0416666666666666*G4_0_0_0_2_0_2 + 0.0833333333333332*G7_0_0_2_0_0_2;
    A[46] = 0.0416666666666666*G4_0_0_1_2_0_2 + 0.0833333333333332*G7_0_0_2_0_1_2;
    A[47] = 0.0416666666666666*G4_0_0_2_2_0_2 + 0.0833333333333332*G7_0_0_2_0_2_2;
    A[48] = 0.0416666666666666*G4_0_0_0_0_1_0 + 0.0416666666666666*G4_0_0_0_1_1_0 + 0.0416666666666666*G4_0_0_0_2_1_0 + 0.0416666666666666*G4_0_0_1_0_1_0 + 0.0416666666666666*G4_0_0_1_1_1_0 + 0.0416666666666666*G4_0_0_1_2_1_0 + 0.0416666666666666*G4_0_0_2_0_1_0 + 0.0416666666666666*G4_0_0_2_1_1_0 + 0.0416666666666666*G4_0_0_2_2_1_0 + 0.0833333333333332*G7_0_0_0_1_0_0 + 0.0833333333333332*G7_0_0_0_1_1_0 + 0.0833333333333332*G7_0_0_0_1_2_0 + 0.0833333333333332*G7_0_0_1_1_0_0 + 0.0833333333333332*G7_0_0_1_1_1_0 + 0.0833333333333332*G7_0_0_1_1_2_0 + 0.0833333333333332*G7_0_0_2_1_0_0 + 0.0833333333333332*G7_0_0_2_1_1_0 + 0.0833333333333332*G7_0_0_2_1_2_0;
    A[49] = -0.0416666666666666*G4_0_0_0_0_1_0 - 0.0416666666666666*G4_0_0_0_1_1_0 - 0.0416666666666666*G4_0_0_0_2_1_0 - 0.0833333333333332*G7_0_0_0_1_0_0 - 0.0833333333333332*G7_0_0_1_1_0_0 - 0.0833333333333332*G7_0_0_2_1_0_0;
    A[50] = -0.0416666666666666*G4_0_0_1_0_1_0 - 0.0416666666666666*G4_0_0_1_1_1_0 - 0.0416666666666666*G4_0_0_1_2_1_0 - 0.0833333333333332*G7_0_0_0_1_1_0 - 0.0833333333333332*G7_0_0_1_1_1_0 - 0.0833333333333332*G7_0_0_2_1_1_0;
    A[51] = -0.0416666666666666*G4_0_0_2_0_1_0 - 0.0416666666666666*G4_0_0_2_1_1_0 - 0.0416666666666666*G4_0_0_2_2_1_0 - 0.0833333333333332*G7_0_0_0_1_2_0 - 0.0833333333333332*G7_0_0_1_1_2_0 - 0.0833333333333332*G7_0_0_2_1_2_0;
    A[52] = 0.0166666666666666*G0_0 - 0.0166666666666666*G1_0_0_0_0_0_0 - 0.0166666666666666*G1_0_0_0_0_1_0 - 0.0166666666666666*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.0166666666666666*G1_0_0_0_4_0_1 - 0.0166666666666666*G1_0_0_0_4_1_1 - 0.0166666666666666*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.0166666666666666*G1_0_0_0_8_0_2 - 0.0166666666666666*G1_0_0_0_8_1_2 - 0.0166666666666666*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.0166666666666666*G2_0_0_0_0_0_0 + 0.0166666666666666*G2_0_0_0_0_1_0 + 0.0166666666666666*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.0166666666666666*G2_0_0_0_4_0_1 + 0.0166666666666666*G2_0_0_0_4_1_1 + 0.0166666666666666*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.0166666666666666*G2_0_0_0_8_0_2 + 0.0166666666666666*G2_0_0_0_8_1_2 + 0.0166666666666666*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_0_0 + 0.0833333333333332*G3_0_0_0_1 + 0.0833333333333332*G3_0_0_0_2 + 0.0833333333333332*G3_0_0_1_0 + 0.0833333333333332*G3_0_0_1_1 + 0.0833333333333332*G3_0_0_1_2 + 0.0833333333333332*G3_0_0_2_0 + 0.0833333333333332*G3_0_0_2_1 + 0.0833333333333332*G3_0_0_2_2 + 0.0416666666666666*G4_0_0_0_0_1_1 + 0.0416666666666666*G4_0_0_0_1_1_1 + 0.0416666666666666*G4_0_0_0_2_1_1 + 0.0416666666666666*G4_0_0_1_0_1_1 + 0.0416666666666666*G4_0_0_1_1_1_1 + 0.0416666666666666*G4_0_0_1_2_1_1 + 0.0416666666666666*G4_0_0_2_0_1_1 + 0.0416666666666666*G4_0_0_2_1_1_1 + 0.0416666666666666*G4_0_0_2_2_1_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_2 + 0.0833333333333332*G5_0_0_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_0_0_1_0_1 + 0.0833333333333332*G5_0_0_0_0_0_1_0_2 + 0.0833333333333332*G5_0_0_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_2_0_1 + 0.0833333333333332*G5_0_0_0_0_0_2_0_2 + 0.0833333333333332*G5_0_0_0_1_0_0_1_0 + 0.0833333333333332*G5_0_0_0_1_0_0_1_1 + 0.0833333333333332*G5_0_0_0_1_0_0_1_2 + 0.0833333333333332*G5_0_0_0_1_0_1_1_0 + 0.0833333333333332*G5_0_0_0_1_0_1_1_1 + 0.0833333333333332*G5_0_0_0_1_0_1_1_2 + 0.0833333333333332*G5_0_0_0_1_0_2_1_0 + 0.0833333333333332*G5_0_0_0_1_0_2_1_1 + 0.0833333333333332*G5_0_0_0_1_0_2_1_2 + 0.0833333333333332*G5_0_0_0_2_0_0_2_0 + 0.0833333333333332*G5_0_0_0_2_0_0_2_1 + 0.0833333333333332*G5_0_0_0_2_0_0_2_2 + 0.0833333333333332*G5_0_0_0_2_0_1_2_0 + 0.0833333333333332*G5_0_0_0_2_0_1_2_1 + 0.0833333333333332*G5_0_0_0_2_0_1_2_2 + 0.0833333333333332*G5_0_0_0_2_0_2_2_0 + 0.0833333333333332*G5_0_0_0_2_0_2_2_1 + 0.0833333333333332*G5_0_0_0_2_0_2_2_2 + 0.0833333333333332*G5_1_0_0_0_1_0_0_0 + 0.0833333333333332*G5_1_0_0_0_1_0_0_1 + 0.0833333333333332*G5_1_0_0_0_1_0_0_2 + 0.0833333333333332*G5_1_0_0_0_1_1_0_0 + 0.0833333333333332*G5_1_0_0_0_1_1_0_1 + 0.0833333333333332*G5_1_0_0_0_1_1_0_2 + 0.0833333333333332*G5_1_0_0_0_1_2_0_0 + 0.0833333333333332*G5_1_0_0_0_1_2_0_1 + 0.0833333333333332*G5_1_0_0_0_1_2_0_2 + 0.0833333333333332*G5_1_0_0_1_1_0_1_0 + 0.0833333333333332*G5_1_0_0_1_1_0_1_1 + 0.0833333333333332*G5_1_0_0_1_1_0_1_2 + 0.0833333333333332*G5_1_0_0_1_1_1_1_0 + 0.0833333333333332*G5_1_0_0_1_1_1_1_1 + 0.0833333333333332*G5_1_0_0_1_1_1_1_2 + 0.0833333333333332*G5_1_0_0_1_1_2_1_0 + 0.0833333333333332*G5_1_0_0_1_1_2_1_1 + 0.0833333333333332*G5_1_0_0_1_1_2_1_2 + 0.0833333333333332*G5_1_0_0_2_1_0_2_0 + 0.0833333333333332*G5_1_0_0_2_1_0_2_1 + 0.0833333333333332*G5_1_0_0_2_1_0_2_2 + 0.0833333333333332*G5_1_0_0_2_1_1_2_0 + 0.0833333333333332*G5_1_0_0_2_1_1_2_1 + 0.0833333333333332*G5_1_0_0_2_1_1_2_2 + 0.0833333333333332*G5_1_0_0_2_1_2_2_0 + 0.0833333333333332*G5_1_0_0_2_1_2_2_1 + 0.0833333333333332*G5_1_0_0_2_1_2_2_2 + 0.0833333333333332*G5_2_0_0_0_2_0_0_0 + 0.0833333333333332*G5_2_0_0_0_2_0_0_1 + 0.0833333333333332*G5_2_0_0_0_2_0_0_2 + 0.0833333333333332*G5_2_0_0_0_2_1_0_0 + 0.0833333333333332*G5_2_0_0_0_2_1_0_1 + 0.0833333333333332*G5_2_0_0_0_2_1_0_2 + 0.0833333333333332*G5_2_0_0_0_2_2_0_0 + 0.0833333333333332*G5_2_0_0_0_2_2_0_1 + 0.0833333333333332*G5_2_0_0_0_2_2_0_2 + 0.0833333333333332*G5_2_0_0_1_2_0_1_0 + 0.0833333333333332*G5_2_0_0_1_2_0_1_1 + 0.0833333333333332*G5_2_0_0_1_2_0_1_2 + 0.0833333333333332*G5_2_0_0_1_2_1_1_0 + 0.0833333333333332*G5_2_0_0_1_2_1_1_1 + 0.0833333333333332*G5_2_0_0_1_2_1_1_2 + 0.0833333333333332*G5_2_0_0_1_2_2_1_0 + 0.0833333333333332*G5_2_0_0_1_2_2_1_1 + 0.0833333333333332*G5_2_0_0_1_2_2_1_2 + 0.0833333333333332*G5_2_0_0_2_2_0_2_0 + 0.0833333333333332*G5_2_0_0_2_2_0_2_1 + 0.0833333333333332*G5_2_0_0_2_2_0_2_2 + 0.0833333333333332*G5_2_0_0_2_2_1_2_0 + 0.0833333333333332*G5_2_0_0_2_2_1_2_1 + 0.0833333333333332*G5_2_0_0_2_2_1_2_2 + 0.0833333333333332*G5_2_0_0_2_2_2_2_0 + 0.0833333333333332*G5_2_0_0_2_2_2_2_1 + 0.0833333333333332*G5_2_0_0_2_2_2_2_2 - 0.0833333333333332*G6_0_0_0_0_0_0_0_0 - 0.0833333333333332*G6_0_0_0_0_0_0_0_1 - 0.0833333333333332*G6_0_0_0_0_0_0_0_2 - 0.0833333333333332*G6_0_0_0_0_0_1_0_0 - 0.0833333333333332*G6_0_0_0_0_0_1_0_1 - 0.0833333333333332*G6_0_0_0_0_0_1_0_2 - 0.0833333333333332*G6_0_0_0_0_0_2_0_0 - 0.0833333333333332*G6_0_0_0_0_0_2_0_1 - 0.0833333333333332*G6_0_0_0_0_0_2_0_2 - 0.0833333333333332*G6_0_0_0_1_0_0_1_0 - 0.0833333333333332*G6_0_0_0_1_0_0_1_1 - 0.0833333333333332*G6_0_0_0_1_0_0_1_2 - 0.0833333333333332*G6_0_0_0_1_0_1_1_0 - 0.0833333333333332*G6_0_0_0_1_0_1_1_1 - 0.0833333333333332*G6_0_0_0_1_0_1_1_2 - 0.0833333333333332*G6_0_0_0_1_0_2_1_0 - 0.0833333333333332*G6_0_0_0_1_0_2_1_1 - 0.0833333333333332*G6_0_0_0_1_0_2_1_2 - 0.0833333333333332*G6_0_0_0_2_0_0_2_0 - 0.0833333333333332*G6_0_0_0_2_0_0_2_1 - 0.0833333333333332*G6_0_0_0_2_0_0_2_2 - 0.0833333333333332*G6_0_0_0_2_0_1_2_0 - 0.0833333333333332*G6_0_0_0_2_0_1_2_1 - 0.0833333333333332*G6_0_0_0_2_0_1_2_2 - 0.0833333333333332*G6_0_0_0_2_0_2_2_0 - 0.0833333333333332*G6_0_0_0_2_0_2_2_1 - 0.0833333333333332*G6_0_0_0_2_0_2_2_2 - 0.0833333333333332*G6_1_0_0_0_1_0_0_0 - 0.0833333333333332*G6_1_0_0_0_1_0_0_1 - 0.0833333333333332*G6_1_0_0_0_1_0_0_2 - 0.0833333333333332*G6_1_0_0_0_1_1_0_0 - 0.0833333333333332*G6_1_0_0_0_1_1_0_1 - 0.0833333333333332*G6_1_0_0_0_1_1_0_2 - 0.0833333333333332*G6_1_0_0_0_1_2_0_0 - 0.0833333333333332*G6_1_0_0_0_1_2_0_1 - 0.0833333333333332*G6_1_0_0_0_1_2_0_2 - 0.0833333333333332*G6_1_0_0_1_1_0_1_0 - 0.0833333333333332*G6_1_0_0_1_1_0_1_1 - 0.0833333333333332*G6_1_0_0_1_1_0_1_2 - 0.0833333333333332*G6_1_0_0_1_1_1_1_0 - 0.0833333333333332*G6_1_0_0_1_1_1_1_1 - 0.0833333333333332*G6_1_0_0_1_1_1_1_2 - 0.0833333333333332*G6_1_0_0_1_1_2_1_0 - 0.0833333333333332*G6_1_0_0_1_1_2_1_1 - 0.0833333333333332*G6_1_0_0_1_1_2_1_2 - 0.0833333333333332*G6_1_0_0_2_1_0_2_0 - 0.0833333333333332*G6_1_0_0_2_1_0_2_1 - 0.0833333333333332*G6_1_0_0_2_1_0_2_2 - 0.0833333333333332*G6_1_0_0_2_1_1_2_0 - 0.0833333333333332*G6_1_0_0_2_1_1_2_1 - 0.0833333333333332*G6_1_0_0_2_1_1_2_2 - 0.0833333333333332*G6_1_0_0_2_1_2_2_0 - 0.0833333333333332*G6_1_0_0_2_1_2_2_1 - 0.0833333333333332*G6_1_0_0_2_1_2_2_2 - 0.0833333333333332*G6_2_0_0_0_2_0_0_0 - 0.0833333333333332*G6_2_0_0_0_2_0_0_1 - 0.0833333333333332*G6_2_0_0_0_2_0_0_2 - 0.0833333333333332*G6_2_0_0_0_2_1_0_0 - 0.0833333333333332*G6_2_0_0_0_2_1_0_1 - 0.0833333333333332*G6_2_0_0_0_2_1_0_2 - 0.0833333333333332*G6_2_0_0_0_2_2_0_0 - 0.0833333333333332*G6_2_0_0_0_2_2_0_1 - 0.0833333333333332*G6_2_0_0_0_2_2_0_2 - 0.0833333333333332*G6_2_0_0_1_2_0_1_0 - 0.0833333333333332*G6_2_0_0_1_2_0_1_1 - 0.0833333333333332*G6_2_0_0_1_2_0_1_2 - 0.0833333333333332*G6_2_0_0_1_2_1_1_0 - 0.0833333333333332*G6_2_0_0_1_2_1_1_1 - 0.0833333333333332*G6_2_0_0_1_2_1_1_2 - 0.0833333333333332*G6_2_0_0_1_2_2_1_0 - 0.0833333333333332*G6_2_0_0_1_2_2_1_1 - 0.0833333333333332*G6_2_0_0_1_2_2_1_2 - 0.0833333333333332*G6_2_0_0_2_2_0_2_0 - 0.0833333333333332*G6_2_0_0_2_2_0_2_1 - 0.0833333333333332*G6_2_0_0_2_2_0_2_2 - 0.0833333333333332*G6_2_0_0_2_2_1_2_0 - 0.0833333333333332*G6_2_0_0_2_2_1_2_1 - 0.0833333333333332*G6_2_0_0_2_2_1_2_2 - 0.0833333333333332*G6_2_0_0_2_2_2_2_0 - 0.0833333333333332*G6_2_0_0_2_2_2_2_1 - 0.0833333333333332*G6_2_0_0_2_2_2_2_2 + 0.0833333333333332*G7_0_0_0_1_0_1 + 0.0833333333333332*G7_0_0_0_1_1_1 + 0.0833333333333332*G7_0_0_0_1_2_1 + 0.0833333333333332*G7_0_0_1_1_0_1 + 0.0833333333333332*G7_0_0_1_1_1_1 + 0.0833333333333332*G7_0_0_1_1_2_1 + 0.0833333333333332*G7_0_0_2_1_0_1 + 0.0833333333333332*G7_0_0_2_1_1_1 + 0.0833333333333332*G7_0_0_2_1_2_1;
    A[53] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.0166666666666666*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.0166666666666666*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.0166666666666666*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.0166666666666666*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.0166666666666666*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 - 0.0833333333333332*G3_0_0_0_0 - 0.0833333333333332*G3_0_0_0_1 - 0.0833333333333332*G3_0_0_0_2 - 0.0416666666666666*G4_0_0_0_0_1_1 - 0.0416666666666666*G4_0_0_0_1_1_1 - 0.0416666666666666*G4_0_0_0_2_1_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_1_0_0_1_0 - 0.0833333333333332*G5_0_0_0_1_0_1_1_0 - 0.0833333333333332*G5_0_0_0_1_0_2_1_0 - 0.0833333333333332*G5_0_0_0_2_0_0_2_0 - 0.0833333333333332*G5_0_0_0_2_0_1_2_0 - 0.0833333333333332*G5_0_0_0_2_0_2_2_0 - 0.0833333333333332*G5_1_0_0_0_1_0_0_0 - 0.0833333333333332*G5_1_0_0_0_1_1_0_0 - 0.0833333333333332*G5_1_0_0_0_1_2_0_0 - 0.0833333333333332*G5_1_0_0_1_1_0_1_0 - 0.0833333333333332*G5_1_0_0_1_1_1_1_0 - 0.0833333333333332*G5_1_0_0_1_1_2_1_0 - 0.0833333333333332*G5_1_0_0_2_1_0_2_0 - 0.0833333333333332*G5_1_0_0_2_1_1_2_0 - 0.0833333333333332*G5_1_0_0_2_1_2_2_0 - 0.0833333333333332*G5_2_0_0_0_2_0_0_0 - 0.0833333333333332*G5_2_0_0_0_2_1_0_0 - 0.0833333333333332*G5_2_0_0_0_2_2_0_0 - 0.0833333333333332*G5_2_0_0_1_2_0_1_0 - 0.0833333333333332*G5_2_0_0_1_2_1_1_0 - 0.0833333333333332*G5_2_0_0_1_2_2_1_0 - 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G5_2_0_0_2_2_2_2_0 + 0.0833333333333332*G6_0_0_0_0_0_0_0_0 + 0.0833333333333332*G6_0_0_0_0_0_1_0_0 + 0.0833333333333332*G6_0_0_0_0_0_2_0_0 + 0.0833333333333332*G6_0_0_0_1_0_0_1_0 + 0.0833333333333332*G6_0_0_0_1_0_1_1_0 + 0.0833333333333332*G6_0_0_0_1_0_2_1_0 + 0.0833333333333332*G6_0_0_0_2_0_0_2_0 + 0.0833333333333332*G6_0_0_0_2_0_1_2_0 + 0.0833333333333332*G6_0_0_0_2_0_2_2_0 + 0.0833333333333332*G6_1_0_0_0_1_0_0_0 + 0.0833333333333332*G6_1_0_0_0_1_1_0_0 + 0.0833333333333332*G6_1_0_0_0_1_2_0_0 + 0.0833333333333332*G6_1_0_0_1_1_0_1_0 + 0.0833333333333332*G6_1_0_0_1_1_1_1_0 + 0.0833333333333332*G6_1_0_0_1_1_2_1_0 + 0.0833333333333332*G6_1_0_0_2_1_0_2_0 + 0.0833333333333332*G6_1_0_0_2_1_1_2_0 + 0.0833333333333332*G6_1_0_0_2_1_2_2_0 + 0.0833333333333332*G6_2_0_0_0_2_0_0_0 + 0.0833333333333332*G6_2_0_0_0_2_1_0_0 + 0.0833333333333332*G6_2_0_0_0_2_2_0_0 + 0.0833333333333332*G6_2_0_0_1_2_0_1_0 + 0.0833333333333332*G6_2_0_0_1_2_1_1_0 + 0.0833333333333332*G6_2_0_0_1_2_2_1_0 + 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G6_2_0_0_2_2_2_2_0 - 0.0833333333333332*G7_0_0_0_1_0_1 - 0.0833333333333332*G7_0_0_1_1_0_1 - 0.0833333333333332*G7_0_0_2_1_0_1;
    A[54] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.0166666666666666*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.0166666666666666*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.0166666666666666*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.0166666666666666*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.0166666666666666*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 - 0.0833333333333332*G3_0_0_1_0 - 0.0833333333333332*G3_0_0_1_1 - 0.0833333333333332*G3_0_0_1_2 - 0.0416666666666666*G4_0_0_1_0_1_1 - 0.0416666666666666*G4_0_0_1_1_1_1 - 0.0416666666666666*G4_0_0_1_2_1_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_1 - 0.0833333333333332*G5_0_0_0_1_0_0_1_1 - 0.0833333333333332*G5_0_0_0_1_0_1_1_1 - 0.0833333333333332*G5_0_0_0_1_0_2_1_1 - 0.0833333333333332*G5_0_0_0_2_0_0_2_1 - 0.0833333333333332*G5_0_0_0_2_0_1_2_1 - 0.0833333333333332*G5_0_0_0_2_0_2_2_1 - 0.0833333333333332*G5_1_0_0_0_1_0_0_1 - 0.0833333333333332*G5_1_0_0_0_1_1_0_1 - 0.0833333333333332*G5_1_0_0_0_1_2_0_1 - 0.0833333333333332*G5_1_0_0_1_1_0_1_1 - 0.0833333333333332*G5_1_0_0_1_1_1_1_1 - 0.0833333333333332*G5_1_0_0_1_1_2_1_1 - 0.0833333333333332*G5_1_0_0_2_1_0_2_1 - 0.0833333333333332*G5_1_0_0_2_1_1_2_1 - 0.0833333333333332*G5_1_0_0_2_1_2_2_1 - 0.0833333333333332*G5_2_0_0_0_2_0_0_1 - 0.0833333333333332*G5_2_0_0_0_2_1_0_1 - 0.0833333333333332*G5_2_0_0_0_2_2_0_1 - 0.0833333333333332*G5_2_0_0_1_2_0_1_1 - 0.0833333333333332*G5_2_0_0_1_2_1_1_1 - 0.0833333333333332*G5_2_0_0_1_2_2_1_1 - 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G5_2_0_0_2_2_2_2_1 + 0.0833333333333332*G6_0_0_0_0_0_0_0_1 + 0.0833333333333332*G6_0_0_0_0_0_1_0_1 + 0.0833333333333332*G6_0_0_0_0_0_2_0_1 + 0.0833333333333332*G6_0_0_0_1_0_0_1_1 + 0.0833333333333332*G6_0_0_0_1_0_1_1_1 + 0.0833333333333332*G6_0_0_0_1_0_2_1_1 + 0.0833333333333332*G6_0_0_0_2_0_0_2_1 + 0.0833333333333332*G6_0_0_0_2_0_1_2_1 + 0.0833333333333332*G6_0_0_0_2_0_2_2_1 + 0.0833333333333332*G6_1_0_0_0_1_0_0_1 + 0.0833333333333332*G6_1_0_0_0_1_1_0_1 + 0.0833333333333332*G6_1_0_0_0_1_2_0_1 + 0.0833333333333332*G6_1_0_0_1_1_0_1_1 + 0.0833333333333332*G6_1_0_0_1_1_1_1_1 + 0.0833333333333332*G6_1_0_0_1_1_2_1_1 + 0.0833333333333332*G6_1_0_0_2_1_0_2_1 + 0.0833333333333332*G6_1_0_0_2_1_1_2_1 + 0.0833333333333332*G6_1_0_0_2_1_2_2_1 + 0.0833333333333332*G6_2_0_0_0_2_0_0_1 + 0.0833333333333332*G6_2_0_0_0_2_1_0_1 + 0.0833333333333332*G6_2_0_0_0_2_2_0_1 + 0.0833333333333332*G6_2_0_0_1_2_0_1_1 + 0.0833333333333332*G6_2_0_0_1_2_1_1_1 + 0.0833333333333332*G6_2_0_0_1_2_2_1_1 + 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G6_2_0_0_2_2_2_2_1 - 0.0833333333333332*G7_0_0_0_1_1_1 - 0.0833333333333332*G7_0_0_1_1_1_1 - 0.0833333333333332*G7_0_0_2_1_1_1;
    A[55] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.0166666666666666*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.0166666666666666*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.0166666666666666*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.0166666666666666*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.0166666666666666*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_2_0 - 0.0833333333333332*G3_0_0_2_1 - 0.0833333333333332*G3_0_0_2_2 - 0.0416666666666666*G4_0_0_2_0_1_1 - 0.0416666666666666*G4_0_0_2_1_1_1 - 0.0416666666666666*G4_0_0_2_2_1_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_2 - 0.0833333333333332*G5_0_0_0_0_0_1_0_2 - 0.0833333333333332*G5_0_0_0_0_0_2_0_2 - 0.0833333333333332*G5_0_0_0_1_0_0_1_2 - 0.0833333333333332*G5_0_0_0_1_0_1_1_2 - 0.0833333333333332*G5_0_0_0_1_0_2_1_2 - 0.0833333333333332*G5_0_0_0_2_0_0_2_2 - 0.0833333333333332*G5_0_0_0_2_0_1_2_2 - 0.0833333333333332*G5_0_0_0_2_0_2_2_2 - 0.0833333333333332*G5_1_0_0_0_1_0_0_2 - 0.0833333333333332*G5_1_0_0_0_1_1_0_2 - 0.0833333333333332*G5_1_0_0_0_1_2_0_2 - 0.0833333333333332*G5_1_0_0_1_1_0_1_2 - 0.0833333333333332*G5_1_0_0_1_1_1_1_2 - 0.0833333333333332*G5_1_0_0_1_1_2_1_2 - 0.0833333333333332*G5_1_0_0_2_1_0_2_2 - 0.0833333333333332*G5_1_0_0_2_1_1_2_2 - 0.0833333333333332*G5_1_0_0_2_1_2_2_2 - 0.0833333333333332*G5_2_0_0_0_2_0_0_2 - 0.0833333333333332*G5_2_0_0_0_2_1_0_2 - 0.0833333333333332*G5_2_0_0_0_2_2_0_2 - 0.0833333333333332*G5_2_0_0_1_2_0_1_2 - 0.0833333333333332*G5_2_0_0_1_2_1_1_2 - 0.0833333333333332*G5_2_0_0_1_2_2_1_2 - 0.0833333333333332*G5_2_0_0_2_2_0_2_2 - 0.0833333333333332*G5_2_0_0_2_2_1_2_2 - 0.0833333333333332*G5_2_0_0_2_2_2_2_2 + 0.0833333333333332*G6_0_0_0_0_0_0_0_2 + 0.0833333333333332*G6_0_0_0_0_0_1_0_2 + 0.0833333333333332*G6_0_0_0_0_0_2_0_2 + 0.0833333333333332*G6_0_0_0_1_0_0_1_2 + 0.0833333333333332*G6_0_0_0_1_0_1_1_2 + 0.0833333333333332*G6_0_0_0_1_0_2_1_2 + 0.0833333333333332*G6_0_0_0_2_0_0_2_2 + 0.0833333333333332*G6_0_0_0_2_0_1_2_2 + 0.0833333333333332*G6_0_0_0_2_0_2_2_2 + 0.0833333333333332*G6_1_0_0_0_1_0_0_2 + 0.0833333333333332*G6_1_0_0_0_1_1_0_2 + 0.0833333333333332*G6_1_0_0_0_1_2_0_2 + 0.0833333333333332*G6_1_0_0_1_1_0_1_2 + 0.0833333333333332*G6_1_0_0_1_1_1_1_2 + 0.0833333333333332*G6_1_0_0_1_1_2_1_2 + 0.0833333333333332*G6_1_0_0_2_1_0_2_2 + 0.0833333333333332*G6_1_0_0_2_1_1_2_2 + 0.0833333333333332*G6_1_0_0_2_1_2_2_2 + 0.0833333333333332*G6_2_0_0_0_2_0_0_2 + 0.0833333333333332*G6_2_0_0_0_2_1_0_2 + 0.0833333333333332*G6_2_0_0_0_2_2_0_2 + 0.0833333333333332*G6_2_0_0_1_2_0_1_2 + 0.0833333333333332*G6_2_0_0_1_2_1_1_2 + 0.0833333333333332*G6_2_0_0_1_2_2_1_2 + 0.0833333333333332*G6_2_0_0_2_2_0_2_2 + 0.0833333333333332*G6_2_0_0_2_2_1_2_2 + 0.0833333333333332*G6_2_0_0_2_2_2_2_2 - 0.0833333333333332*G7_0_0_0_1_2_1 - 0.0833333333333332*G7_0_0_1_1_2_1 - 0.0833333333333332*G7_0_0_2_1_2_1;
    A[56] = 0.0416666666666666*G4_0_0_0_0_1_2 + 0.0416666666666666*G4_0_0_0_1_1_2 + 0.0416666666666666*G4_0_0_0_2_1_2 + 0.0416666666666666*G4_0_0_1_0_1_2 + 0.0416666666666666*G4_0_0_1_1_1_2 + 0.0416666666666666*G4_0_0_1_2_1_2 + 0.0416666666666666*G4_0_0_2_0_1_2 + 0.0416666666666666*G4_0_0_2_1_1_2 + 0.0416666666666666*G4_0_0_2_2_1_2 + 0.0833333333333332*G7_0_0_0_1_0_2 + 0.0833333333333332*G7_0_0_0_1_1_2 + 0.0833333333333332*G7_0_0_0_1_2_2 + 0.0833333333333332*G7_0_0_1_1_0_2 + 0.0833333333333332*G7_0_0_1_1_1_2 + 0.0833333333333332*G7_0_0_1_1_2_2 + 0.0833333333333332*G7_0_0_2_1_0_2 + 0.0833333333333332*G7_0_0_2_1_1_2 + 0.0833333333333332*G7_0_0_2_1_2_2;
    A[57] = -0.0416666666666666*G4_0_0_0_0_1_2 - 0.0416666666666666*G4_0_0_0_1_1_2 - 0.0416666666666666*G4_0_0_0_2_1_2 - 0.0833333333333332*G7_0_0_0_1_0_2 - 0.0833333333333332*G7_0_0_1_1_0_2 - 0.0833333333333332*G7_0_0_2_1_0_2;
    A[58] = -0.0416666666666666*G4_0_0_1_0_1_2 - 0.0416666666666666*G4_0_0_1_1_1_2 - 0.0416666666666666*G4_0_0_1_2_1_2 - 0.0833333333333332*G7_0_0_0_1_1_2 - 0.0833333333333332*G7_0_0_1_1_1_2 - 0.0833333333333332*G7_0_0_2_1_1_2;
    A[59] = -0.0416666666666666*G4_0_0_2_0_1_2 - 0.0416666666666666*G4_0_0_2_1_1_2 - 0.0416666666666666*G4_0_0_2_2_1_2 - 0.0833333333333332*G7_0_0_0_1_2_2 - 0.0833333333333332*G7_0_0_1_1_2_2 - 0.0833333333333332*G7_0_0_2_1_2_2;
    A[60] = -0.0416666666666666*G4_0_0_0_0_1_0 - 0.0416666666666666*G4_0_0_1_0_1_0 - 0.0416666666666666*G4_0_0_2_0_1_0 - 0.0833333333333332*G7_0_0_0_1_0_0 - 0.0833333333333332*G7_0_0_0_1_1_0 - 0.0833333333333332*G7_0_0_0_1_2_0;
    A[61] = 0.0416666666666666*G4_0_0_0_0_1_0 + 0.0833333333333332*G7_0_0_0_1_0_0;
    A[62] = 0.0416666666666666*G4_0_0_1_0_1_0 + 0.0833333333333332*G7_0_0_0_1_1_0;
    A[63] = 0.0416666666666666*G4_0_0_2_0_1_0 + 0.0833333333333332*G7_0_0_0_1_2_0;
    A[64] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.0166666666666666*G1_0_0_0_1_0_0 - 0.0166666666666666*G1_0_0_0_1_1_0 - 0.0166666666666666*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.0166666666666666*G1_0_0_0_5_0_1 - 0.0166666666666666*G1_0_0_0_5_1_1 - 0.0166666666666666*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.0166666666666666*G1_0_0_0_9_0_2 - 0.0166666666666666*G1_0_0_0_9_1_2 - 0.0166666666666666*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.0166666666666666*G2_0_0_0_1_0_0 + 0.0166666666666666*G2_0_0_0_1_1_0 + 0.0166666666666666*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.0166666666666666*G2_0_0_0_5_0_1 + 0.0166666666666666*G2_0_0_0_5_1_1 + 0.0166666666666666*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.0166666666666666*G2_0_0_0_9_0_2 + 0.0166666666666666*G2_0_0_0_9_1_2 + 0.0166666666666666*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_0 - 0.0833333333333332*G3_0_0_1_0 - 0.0833333333333332*G3_0_0_2_0 - 0.0416666666666666*G4_0_0_0_0_1_1 - 0.0416666666666666*G4_0_0_1_0_1_1 - 0.0416666666666666*G4_0_0_2_0_1_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_2 - 0.0833333333333332*G5_0_0_0_1_0_0_1_0 - 0.0833333333333332*G5_0_0_0_1_0_0_1_1 - 0.0833333333333332*G5_0_0_0_1_0_0_1_2 - 0.0833333333333332*G5_0_0_0_2_0_0_2_0 - 0.0833333333333332*G5_0_0_0_2_0_0_2_1 - 0.0833333333333332*G5_0_0_0_2_0_0_2_2 - 0.0833333333333332*G5_1_0_0_0_1_0_0_0 - 0.0833333333333332*G5_1_0_0_0_1_0_0_1 - 0.0833333333333332*G5_1_0_0_0_1_0_0_2 - 0.0833333333333332*G5_1_0_0_1_1_0_1_0 - 0.0833333333333332*G5_1_0_0_1_1_0_1_1 - 0.0833333333333332*G5_1_0_0_1_1_0_1_2 - 0.0833333333333332*G5_1_0_0_2_1_0_2_0 - 0.0833333333333332*G5_1_0_0_2_1_0_2_1 - 0.0833333333333332*G5_1_0_0_2_1_0_2_2 - 0.0833333333333332*G5_2_0_0_0_2_0_0_0 - 0.0833333333333332*G5_2_0_0_0_2_0_0_1 - 0.0833333333333332*G5_2_0_0_0_2_0_0_2 - 0.0833333333333332*G5_2_0_0_1_2_0_1_0 - 0.0833333333333332*G5_2_0_0_1_2_0_1_1 - 0.0833333333333332*G5_2_0_0_1_2_0_1_2 - 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G5_2_0_0_2_2_0_2_2 + 0.0833333333333332*G6_0_0_0_0_0_0_0_0 + 0.0833333333333332*G6_0_0_0_0_0_0_0_1 + 0.0833333333333332*G6_0_0_0_0_0_0_0_2 + 0.0833333333333332*G6_0_0_0_1_0_0_1_0 + 0.0833333333333332*G6_0_0_0_1_0_0_1_1 + 0.0833333333333332*G6_0_0_0_1_0_0_1_2 + 0.0833333333333332*G6_0_0_0_2_0_0_2_0 + 0.0833333333333332*G6_0_0_0_2_0_0_2_1 + 0.0833333333333332*G6_0_0_0_2_0_0_2_2 + 0.0833333333333332*G6_1_0_0_0_1_0_0_0 + 0.0833333333333332*G6_1_0_0_0_1_0_0_1 + 0.0833333333333332*G6_1_0_0_0_1_0_0_2 + 0.0833333333333332*G6_1_0_0_1_1_0_1_0 + 0.0833333333333332*G6_1_0_0_1_1_0_1_1 + 0.0833333333333332*G6_1_0_0_1_1_0_1_2 + 0.0833333333333332*G6_1_0_0_2_1_0_2_0 + 0.0833333333333332*G6_1_0_0_2_1_0_2_1 + 0.0833333333333332*G6_1_0_0_2_1_0_2_2 + 0.0833333333333332*G6_2_0_0_0_2_0_0_0 + 0.0833333333333332*G6_2_0_0_0_2_0_0_1 + 0.0833333333333332*G6_2_0_0_0_2_0_0_2 + 0.0833333333333332*G6_2_0_0_1_2_0_1_0 + 0.0833333333333332*G6_2_0_0_1_2_0_1_1 + 0.0833333333333332*G6_2_0_0_1_2_0_1_2 + 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G6_2_0_0_2_2_0_2_2 - 0.0833333333333332*G7_0_0_0_1_0_1 - 0.0833333333333332*G7_0_0_0_1_1_1 - 0.0833333333333332*G7_0_0_0_1_2_1;
    A[65] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.0166666666666666*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.0166666666666666*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.0166666666666666*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.0166666666666666*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.0166666666666666*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.0166666666666666*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_0 + 0.0416666666666666*G4_0_0_0_0_1_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_1_0_0_1_0 + 0.0833333333333332*G5_0_0_0_2_0_0_2_0 + 0.0833333333333332*G5_1_0_0_0_1_0_0_0 + 0.0833333333333332*G5_1_0_0_1_1_0_1_0 + 0.0833333333333332*G5_1_0_0_2_1_0_2_0 + 0.0833333333333332*G5_2_0_0_0_2_0_0_0 + 0.0833333333333332*G5_2_0_0_1_2_0_1_0 + 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G6_0_0_0_0_0_0_0_0 - 0.0833333333333332*G6_0_0_0_1_0_0_1_0 - 0.0833333333333332*G6_0_0_0_2_0_0_2_0 - 0.0833333333333332*G6_1_0_0_0_1_0_0_0 - 0.0833333333333332*G6_1_0_0_1_1_0_1_0 - 0.0833333333333332*G6_1_0_0_2_1_0_2_0 - 0.0833333333333332*G6_2_0_0_0_2_0_0_0 - 0.0833333333333332*G6_2_0_0_1_2_0_1_0 - 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G7_0_0_0_1_0_1;
    A[66] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.0166666666666666*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.0166666666666666*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.0166666666666666*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.0166666666666666*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.0166666666666666*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.0166666666666666*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_0 + 0.0416666666666666*G4_0_0_1_0_1_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_1 + 0.0833333333333332*G5_0_0_0_1_0_0_1_1 + 0.0833333333333332*G5_0_0_0_2_0_0_2_1 + 0.0833333333333332*G5_1_0_0_0_1_0_0_1 + 0.0833333333333332*G5_1_0_0_1_1_0_1_1 + 0.0833333333333332*G5_1_0_0_2_1_0_2_1 + 0.0833333333333332*G5_2_0_0_0_2_0_0_1 + 0.0833333333333332*G5_2_0_0_1_2_0_1_1 + 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G6_0_0_0_0_0_0_0_1 - 0.0833333333333332*G6_0_0_0_1_0_0_1_1 - 0.0833333333333332*G6_0_0_0_2_0_0_2_1 - 0.0833333333333332*G6_1_0_0_0_1_0_0_1 - 0.0833333333333332*G6_1_0_0_1_1_0_1_1 - 0.0833333333333332*G6_1_0_0_2_1_0_2_1 - 0.0833333333333332*G6_2_0_0_0_2_0_0_1 - 0.0833333333333332*G6_2_0_0_1_2_0_1_1 - 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G7_0_0_0_1_1_1;
    A[67] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.0166666666666666*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.0166666666666666*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.0166666666666666*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.0166666666666666*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.0166666666666666*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.0166666666666666*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_0 + 0.0416666666666666*G4_0_0_2_0_1_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_2 + 0.0833333333333332*G5_0_0_0_1_0_0_1_2 + 0.0833333333333332*G5_0_0_0_2_0_0_2_2 + 0.0833333333333332*G5_1_0_0_0_1_0_0_2 + 0.0833333333333332*G5_1_0_0_1_1_0_1_2 + 0.0833333333333332*G5_1_0_0_2_1_0_2_2 + 0.0833333333333332*G5_2_0_0_0_2_0_0_2 + 0.0833333333333332*G5_2_0_0_1_2_0_1_2 + 0.0833333333333332*G5_2_0_0_2_2_0_2_2 - 0.0833333333333332*G6_0_0_0_0_0_0_0_2 - 0.0833333333333332*G6_0_0_0_1_0_0_1_2 - 0.0833333333333332*G6_0_0_0_2_0_0_2_2 - 0.0833333333333332*G6_1_0_0_0_1_0_0_2 - 0.0833333333333332*G6_1_0_0_1_1_0_1_2 - 0.0833333333333332*G6_1_0_0_2_1_0_2_2 - 0.0833333333333332*G6_2_0_0_0_2_0_0_2 - 0.0833333333333332*G6_2_0_0_1_2_0_1_2 - 0.0833333333333332*G6_2_0_0_2_2_0_2_2 + 0.0833333333333332*G7_0_0_0_1_2_1;
    A[68] = -0.0416666666666666*G4_0_0_0_0_1_2 - 0.0416666666666666*G4_0_0_1_0_1_2 - 0.0416666666666666*G4_0_0_2_0_1_2 - 0.0833333333333332*G7_0_0_0_1_0_2 - 0.0833333333333332*G7_0_0_0_1_1_2 - 0.0833333333333332*G7_0_0_0_1_2_2;
    A[69] = 0.0416666666666666*G4_0_0_0_0_1_2 + 0.0833333333333332*G7_0_0_0_1_0_2;
    A[70] = 0.0416666666666666*G4_0_0_1_0_1_2 + 0.0833333333333332*G7_0_0_0_1_1_2;
    A[71] = 0.0416666666666666*G4_0_0_2_0_1_2 + 0.0833333333333332*G7_0_0_0_1_2_2;
    A[72] = -0.0416666666666666*G4_0_0_0_1_1_0 - 0.0416666666666666*G4_0_0_1_1_1_0 - 0.0416666666666666*G4_0_0_2_1_1_0 - 0.0833333333333332*G7_0_0_1_1_0_0 - 0.0833333333333332*G7_0_0_1_1_1_0 - 0.0833333333333332*G7_0_0_1_1_2_0;
    A[73] = 0.0416666666666666*G4_0_0_0_1_1_0 + 0.0833333333333332*G7_0_0_1_1_0_0;
    A[74] = 0.0416666666666666*G4_0_0_1_1_1_0 + 0.0833333333333332*G7_0_0_1_1_1_0;
    A[75] = 0.0416666666666666*G4_0_0_2_1_1_0 + 0.0833333333333332*G7_0_0_1_1_2_0;
    A[76] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.0166666666666666*G1_0_0_0_2_0_0 - 0.0166666666666666*G1_0_0_0_2_1_0 - 0.0166666666666666*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.0166666666666666*G1_0_0_0_6_0_1 - 0.0166666666666666*G1_0_0_0_6_1_1 - 0.0166666666666666*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.0166666666666666*G1_0_0_0_10_0_2 - 0.0166666666666666*G1_0_0_0_10_1_2 - 0.0166666666666666*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.0166666666666666*G2_0_0_0_2_0_0 + 0.0166666666666666*G2_0_0_0_2_1_0 + 0.0166666666666666*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.0166666666666666*G2_0_0_0_6_0_1 + 0.0166666666666666*G2_0_0_0_6_1_1 + 0.0166666666666666*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.0166666666666666*G2_0_0_0_10_0_2 + 0.0166666666666666*G2_0_0_0_10_1_2 + 0.0166666666666666*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_1 - 0.0833333333333332*G3_0_0_1_1 - 0.0833333333333332*G3_0_0_2_1 - 0.0416666666666666*G4_0_0_0_1_1_1 - 0.0416666666666666*G4_0_0_1_1_1_1 - 0.0416666666666666*G4_0_0_2_1_1_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_2 - 0.0833333333333332*G5_0_0_0_1_0_1_1_0 - 0.0833333333333332*G5_0_0_0_1_0_1_1_1 - 0.0833333333333332*G5_0_0_0_1_0_1_1_2 - 0.0833333333333332*G5_0_0_0_2_0_1_2_0 - 0.0833333333333332*G5_0_0_0_2_0_1_2_1 - 0.0833333333333332*G5_0_0_0_2_0_1_2_2 - 0.0833333333333332*G5_1_0_0_0_1_1_0_0 - 0.0833333333333332*G5_1_0_0_0_1_1_0_1 - 0.0833333333333332*G5_1_0_0_0_1_1_0_2 - 0.0833333333333332*G5_1_0_0_1_1_1_1_0 - 0.0833333333333332*G5_1_0_0_1_1_1_1_1 - 0.0833333333333332*G5_1_0_0_1_1_1_1_2 - 0.0833333333333332*G5_1_0_0_2_1_1_2_0 - 0.0833333333333332*G5_1_0_0_2_1_1_2_1 - 0.0833333333333332*G5_1_0_0_2_1_1_2_2 - 0.0833333333333332*G5_2_0_0_0_2_1_0_0 - 0.0833333333333332*G5_2_0_0_0_2_1_0_1 - 0.0833333333333332*G5_2_0_0_0_2_1_0_2 - 0.0833333333333332*G5_2_0_0_1_2_1_1_0 - 0.0833333333333332*G5_2_0_0_1_2_1_1_1 - 0.0833333333333332*G5_2_0_0_1_2_1_1_2 - 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G5_2_0_0_2_2_1_2_2 + 0.0833333333333332*G6_0_0_0_0_0_1_0_0 + 0.0833333333333332*G6_0_0_0_0_0_1_0_1 + 0.0833333333333332*G6_0_0_0_0_0_1_0_2 + 0.0833333333333332*G6_0_0_0_1_0_1_1_0 + 0.0833333333333332*G6_0_0_0_1_0_1_1_1 + 0.0833333333333332*G6_0_0_0_1_0_1_1_2 + 0.0833333333333332*G6_0_0_0_2_0_1_2_0 + 0.0833333333333332*G6_0_0_0_2_0_1_2_1 + 0.0833333333333332*G6_0_0_0_2_0_1_2_2 + 0.0833333333333332*G6_1_0_0_0_1_1_0_0 + 0.0833333333333332*G6_1_0_0_0_1_1_0_1 + 0.0833333333333332*G6_1_0_0_0_1_1_0_2 + 0.0833333333333332*G6_1_0_0_1_1_1_1_0 + 0.0833333333333332*G6_1_0_0_1_1_1_1_1 + 0.0833333333333332*G6_1_0_0_1_1_1_1_2 + 0.0833333333333332*G6_1_0_0_2_1_1_2_0 + 0.0833333333333332*G6_1_0_0_2_1_1_2_1 + 0.0833333333333332*G6_1_0_0_2_1_1_2_2 + 0.0833333333333332*G6_2_0_0_0_2_1_0_0 + 0.0833333333333332*G6_2_0_0_0_2_1_0_1 + 0.0833333333333332*G6_2_0_0_0_2_1_0_2 + 0.0833333333333332*G6_2_0_0_1_2_1_1_0 + 0.0833333333333332*G6_2_0_0_1_2_1_1_1 + 0.0833333333333332*G6_2_0_0_1_2_1_1_2 + 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G6_2_0_0_2_2_1_2_2 - 0.0833333333333332*G7_0_0_1_1_0_1 - 0.0833333333333332*G7_0_0_1_1_1_1 - 0.0833333333333332*G7_0_0_1_1_2_1;
    A[77] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.0166666666666666*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.0166666666666666*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.0166666666666666*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.0166666666666666*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.0166666666666666*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.0166666666666666*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_1 + 0.0416666666666666*G4_0_0_0_1_1_1 + 0.0833333333333332*G5_0_0_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_1_0_1_1_0 + 0.0833333333333332*G5_0_0_0_2_0_1_2_0 + 0.0833333333333332*G5_1_0_0_0_1_1_0_0 + 0.0833333333333332*G5_1_0_0_1_1_1_1_0 + 0.0833333333333332*G5_1_0_0_2_1_1_2_0 + 0.0833333333333332*G5_2_0_0_0_2_1_0_0 + 0.0833333333333332*G5_2_0_0_1_2_1_1_0 + 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G6_0_0_0_0_0_1_0_0 - 0.0833333333333332*G6_0_0_0_1_0_1_1_0 - 0.0833333333333332*G6_0_0_0_2_0_1_2_0 - 0.0833333333333332*G6_1_0_0_0_1_1_0_0 - 0.0833333333333332*G6_1_0_0_1_1_1_1_0 - 0.0833333333333332*G6_1_0_0_2_1_1_2_0 - 0.0833333333333332*G6_2_0_0_0_2_1_0_0 - 0.0833333333333332*G6_2_0_0_1_2_1_1_0 - 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G7_0_0_1_1_0_1;
    A[78] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.0166666666666666*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.0166666666666666*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.0166666666666666*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.0166666666666666*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.0166666666666666*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.0166666666666666*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_1 + 0.0416666666666666*G4_0_0_1_1_1_1 + 0.0833333333333332*G5_0_0_0_0_0_1_0_1 + 0.0833333333333332*G5_0_0_0_1_0_1_1_1 + 0.0833333333333332*G5_0_0_0_2_0_1_2_1 + 0.0833333333333332*G5_1_0_0_0_1_1_0_1 + 0.0833333333333332*G5_1_0_0_1_1_1_1_1 + 0.0833333333333332*G5_1_0_0_2_1_1_2_1 + 0.0833333333333332*G5_2_0_0_0_2_1_0_1 + 0.0833333333333332*G5_2_0_0_1_2_1_1_1 + 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G6_0_0_0_0_0_1_0_1 - 0.0833333333333332*G6_0_0_0_1_0_1_1_1 - 0.0833333333333332*G6_0_0_0_2_0_1_2_1 - 0.0833333333333332*G6_1_0_0_0_1_1_0_1 - 0.0833333333333332*G6_1_0_0_1_1_1_1_1 - 0.0833333333333332*G6_1_0_0_2_1_1_2_1 - 0.0833333333333332*G6_2_0_0_0_2_1_0_1 - 0.0833333333333332*G6_2_0_0_1_2_1_1_1 - 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G7_0_0_1_1_1_1;
    A[79] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.0166666666666666*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.0166666666666666*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.0166666666666666*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.0166666666666666*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.0166666666666666*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.0166666666666666*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_1 + 0.0416666666666666*G4_0_0_2_1_1_1 + 0.0833333333333332*G5_0_0_0_0_0_1_0_2 + 0.0833333333333332*G5_0_0_0_1_0_1_1_2 + 0.0833333333333332*G5_0_0_0_2_0_1_2_2 + 0.0833333333333332*G5_1_0_0_0_1_1_0_2 + 0.0833333333333332*G5_1_0_0_1_1_1_1_2 + 0.0833333333333332*G5_1_0_0_2_1_1_2_2 + 0.0833333333333332*G5_2_0_0_0_2_1_0_2 + 0.0833333333333332*G5_2_0_0_1_2_1_1_2 + 0.0833333333333332*G5_2_0_0_2_2_1_2_2 - 0.0833333333333332*G6_0_0_0_0_0_1_0_2 - 0.0833333333333332*G6_0_0_0_1_0_1_1_2 - 0.0833333333333332*G6_0_0_0_2_0_1_2_2 - 0.0833333333333332*G6_1_0_0_0_1_1_0_2 - 0.0833333333333332*G6_1_0_0_1_1_1_1_2 - 0.0833333333333332*G6_1_0_0_2_1_1_2_2 - 0.0833333333333332*G6_2_0_0_0_2_1_0_2 - 0.0833333333333332*G6_2_0_0_1_2_1_1_2 - 0.0833333333333332*G6_2_0_0_2_2_1_2_2 + 0.0833333333333332*G7_0_0_1_1_2_1;
    A[80] = -0.0416666666666666*G4_0_0_0_1_1_2 - 0.0416666666666666*G4_0_0_1_1_1_2 - 0.0416666666666666*G4_0_0_2_1_1_2 - 0.0833333333333332*G7_0_0_1_1_0_2 - 0.0833333333333332*G7_0_0_1_1_1_2 - 0.0833333333333332*G7_0_0_1_1_2_2;
    A[81] = 0.0416666666666666*G4_0_0_0_1_1_2 + 0.0833333333333332*G7_0_0_1_1_0_2;
    A[82] = 0.0416666666666666*G4_0_0_1_1_1_2 + 0.0833333333333332*G7_0_0_1_1_1_2;
    A[83] = 0.0416666666666666*G4_0_0_2_1_1_2 + 0.0833333333333332*G7_0_0_1_1_2_2;
    A[84] = -0.0416666666666666*G4_0_0_0_2_1_0 - 0.0416666666666666*G4_0_0_1_2_1_0 - 0.0416666666666666*G4_0_0_2_2_1_0 - 0.0833333333333332*G7_0_0_2_1_0_0 - 0.0833333333333332*G7_0_0_2_1_1_0 - 0.0833333333333332*G7_0_0_2_1_2_0;
    A[85] = 0.0416666666666666*G4_0_0_0_2_1_0 + 0.0833333333333332*G7_0_0_2_1_0_0;
    A[86] = 0.0416666666666666*G4_0_0_1_2_1_0 + 0.0833333333333332*G7_0_0_2_1_1_0;
    A[87] = 0.0416666666666666*G4_0_0_2_2_1_0 + 0.0833333333333332*G7_0_0_2_1_2_0;
    A[88] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.0166666666666666*G1_0_0_0_3_0_0 - 0.0166666666666666*G1_0_0_0_3_1_0 - 0.0166666666666666*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.0166666666666666*G1_0_0_0_7_0_1 - 0.0166666666666666*G1_0_0_0_7_1_1 - 0.0166666666666666*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.0166666666666666*G1_0_0_0_11_0_2 - 0.0166666666666666*G1_0_0_0_11_1_2 - 0.0166666666666666*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.0166666666666666*G2_0_0_0_3_0_0 + 0.0166666666666666*G2_0_0_0_3_1_0 + 0.0166666666666666*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.0166666666666666*G2_0_0_0_7_0_1 + 0.0166666666666666*G2_0_0_0_7_1_1 + 0.0166666666666666*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.0166666666666666*G2_0_0_0_11_0_2 + 0.0166666666666666*G2_0_0_0_11_1_2 + 0.0166666666666666*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_2 - 0.0833333333333332*G3_0_0_1_2 - 0.0833333333333332*G3_0_0_2_2 - 0.0416666666666666*G4_0_0_0_2_1_1 - 0.0416666666666666*G4_0_0_1_2_1_1 - 0.0416666666666666*G4_0_0_2_2_1_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_2 - 0.0833333333333332*G5_0_0_0_1_0_2_1_0 - 0.0833333333333332*G5_0_0_0_1_0_2_1_1 - 0.0833333333333332*G5_0_0_0_1_0_2_1_2 - 0.0833333333333332*G5_0_0_0_2_0_2_2_0 - 0.0833333333333332*G5_0_0_0_2_0_2_2_1 - 0.0833333333333332*G5_0_0_0_2_0_2_2_2 - 0.0833333333333332*G5_1_0_0_0_1_2_0_0 - 0.0833333333333332*G5_1_0_0_0_1_2_0_1 - 0.0833333333333332*G5_1_0_0_0_1_2_0_2 - 0.0833333333333332*G5_1_0_0_1_1_2_1_0 - 0.0833333333333332*G5_1_0_0_1_1_2_1_1 - 0.0833333333333332*G5_1_0_0_1_1_2_1_2 - 0.0833333333333332*G5_1_0_0_2_1_2_2_0 - 0.0833333333333332*G5_1_0_0_2_1_2_2_1 - 0.0833333333333332*G5_1_0_0_2_1_2_2_2 - 0.0833333333333332*G5_2_0_0_0_2_2_0_0 - 0.0833333333333332*G5_2_0_0_0_2_2_0_1 - 0.0833333333333332*G5_2_0_0_0_2_2_0_2 - 0.0833333333333332*G5_2_0_0_1_2_2_1_0 - 0.0833333333333332*G5_2_0_0_1_2_2_1_1 - 0.0833333333333332*G5_2_0_0_1_2_2_1_2 - 0.0833333333333332*G5_2_0_0_2_2_2_2_0 - 0.0833333333333332*G5_2_0_0_2_2_2_2_1 - 0.0833333333333332*G5_2_0_0_2_2_2_2_2 + 0.0833333333333332*G6_0_0_0_0_0_2_0_0 + 0.0833333333333332*G6_0_0_0_0_0_2_0_1 + 0.0833333333333332*G6_0_0_0_0_0_2_0_2 + 0.0833333333333332*G6_0_0_0_1_0_2_1_0 + 0.0833333333333332*G6_0_0_0_1_0_2_1_1 + 0.0833333333333332*G6_0_0_0_1_0_2_1_2 + 0.0833333333333332*G6_0_0_0_2_0_2_2_0 + 0.0833333333333332*G6_0_0_0_2_0_2_2_1 + 0.0833333333333332*G6_0_0_0_2_0_2_2_2 + 0.0833333333333332*G6_1_0_0_0_1_2_0_0 + 0.0833333333333332*G6_1_0_0_0_1_2_0_1 + 0.0833333333333332*G6_1_0_0_0_1_2_0_2 + 0.0833333333333332*G6_1_0_0_1_1_2_1_0 + 0.0833333333333332*G6_1_0_0_1_1_2_1_1 + 0.0833333333333332*G6_1_0_0_1_1_2_1_2 + 0.0833333333333332*G6_1_0_0_2_1_2_2_0 + 0.0833333333333332*G6_1_0_0_2_1_2_2_1 + 0.0833333333333332*G6_1_0_0_2_1_2_2_2 + 0.0833333333333332*G6_2_0_0_0_2_2_0_0 + 0.0833333333333332*G6_2_0_0_0_2_2_0_1 + 0.0833333333333332*G6_2_0_0_0_2_2_0_2 + 0.0833333333333332*G6_2_0_0_1_2_2_1_0 + 0.0833333333333332*G6_2_0_0_1_2_2_1_1 + 0.0833333333333332*G6_2_0_0_1_2_2_1_2 + 0.0833333333333332*G6_2_0_0_2_2_2_2_0 + 0.0833333333333332*G6_2_0_0_2_2_2_2_1 + 0.0833333333333332*G6_2_0_0_2_2_2_2_2 - 0.0833333333333332*G7_0_0_2_1_0_1 - 0.0833333333333332*G7_0_0_2_1_1_1 - 0.0833333333333332*G7_0_0_2_1_2_1;
    A[89] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.0166666666666666*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.0166666666666666*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.0166666666666666*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.0166666666666666*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.0166666666666666*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.0166666666666666*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_2 + 0.0416666666666666*G4_0_0_0_2_1_1 + 0.0833333333333332*G5_0_0_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_1_0_2_1_0 + 0.0833333333333332*G5_0_0_0_2_0_2_2_0 + 0.0833333333333332*G5_1_0_0_0_1_2_0_0 + 0.0833333333333332*G5_1_0_0_1_1_2_1_0 + 0.0833333333333332*G5_1_0_0_2_1_2_2_0 + 0.0833333333333332*G5_2_0_0_0_2_2_0_0 + 0.0833333333333332*G5_2_0_0_1_2_2_1_0 + 0.0833333333333332*G5_2_0_0_2_2_2_2_0 - 0.0833333333333332*G6_0_0_0_0_0_2_0_0 - 0.0833333333333332*G6_0_0_0_1_0_2_1_0 - 0.0833333333333332*G6_0_0_0_2_0_2_2_0 - 0.0833333333333332*G6_1_0_0_0_1_2_0_0 - 0.0833333333333332*G6_1_0_0_1_1_2_1_0 - 0.0833333333333332*G6_1_0_0_2_1_2_2_0 - 0.0833333333333332*G6_2_0_0_0_2_2_0_0 - 0.0833333333333332*G6_2_0_0_1_2_2_1_0 - 0.0833333333333332*G6_2_0_0_2_2_2_2_0 + 0.0833333333333332*G7_0_0_2_1_0_1;
    A[90] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.0166666666666666*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.0166666666666666*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.0166666666666666*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.0166666666666666*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.0166666666666666*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.0166666666666666*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_2 + 0.0416666666666666*G4_0_0_1_2_1_1 + 0.0833333333333332*G5_0_0_0_0_0_2_0_1 + 0.0833333333333332*G5_0_0_0_1_0_2_1_1 + 0.0833333333333332*G5_0_0_0_2_0_2_2_1 + 0.0833333333333332*G5_1_0_0_0_1_2_0_1 + 0.0833333333333332*G5_1_0_0_1_1_2_1_1 + 0.0833333333333332*G5_1_0_0_2_1_2_2_1 + 0.0833333333333332*G5_2_0_0_0_2_2_0_1 + 0.0833333333333332*G5_2_0_0_1_2_2_1_1 + 0.0833333333333332*G5_2_0_0_2_2_2_2_1 - 0.0833333333333332*G6_0_0_0_0_0_2_0_1 - 0.0833333333333332*G6_0_0_0_1_0_2_1_1 - 0.0833333333333332*G6_0_0_0_2_0_2_2_1 - 0.0833333333333332*G6_1_0_0_0_1_2_0_1 - 0.0833333333333332*G6_1_0_0_1_1_2_1_1 - 0.0833333333333332*G6_1_0_0_2_1_2_2_1 - 0.0833333333333332*G6_2_0_0_0_2_2_0_1 - 0.0833333333333332*G6_2_0_0_1_2_2_1_1 - 0.0833333333333332*G6_2_0_0_2_2_2_2_1 + 0.0833333333333332*G7_0_0_2_1_1_1;
    A[91] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.0166666666666666*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.0166666666666666*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.0166666666666666*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.0166666666666666*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.0166666666666666*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.0166666666666666*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_2 + 0.0416666666666666*G4_0_0_2_2_1_1 + 0.0833333333333332*G5_0_0_0_0_0_2_0_2 + 0.0833333333333332*G5_0_0_0_1_0_2_1_2 + 0.0833333333333332*G5_0_0_0_2_0_2_2_2 + 0.0833333333333332*G5_1_0_0_0_1_2_0_2 + 0.0833333333333332*G5_1_0_0_1_1_2_1_2 + 0.0833333333333332*G5_1_0_0_2_1_2_2_2 + 0.0833333333333332*G5_2_0_0_0_2_2_0_2 + 0.0833333333333332*G5_2_0_0_1_2_2_1_2 + 0.0833333333333332*G5_2_0_0_2_2_2_2_2 - 0.0833333333333332*G6_0_0_0_0_0_2_0_2 - 0.0833333333333332*G6_0_0_0_1_0_2_1_2 - 0.0833333333333332*G6_0_0_0_2_0_2_2_2 - 0.0833333333333332*G6_1_0_0_0_1_2_0_2 - 0.0833333333333332*G6_1_0_0_1_1_2_1_2 - 0.0833333333333332*G6_1_0_0_2_1_2_2_2 - 0.0833333333333332*G6_2_0_0_0_2_2_0_2 - 0.0833333333333332*G6_2_0_0_1_2_2_1_2 - 0.0833333333333332*G6_2_0_0_2_2_2_2_2 + 0.0833333333333332*G7_0_0_2_1_2_1;
    A[92] = -0.0416666666666666*G4_0_0_0_2_1_2 - 0.0416666666666666*G4_0_0_1_2_1_2 - 0.0416666666666666*G4_0_0_2_2_1_2 - 0.0833333333333332*G7_0_0_2_1_0_2 - 0.0833333333333332*G7_0_0_2_1_1_2 - 0.0833333333333332*G7_0_0_2_1_2_2;
    A[93] = 0.0416666666666666*G4_0_0_0_2_1_2 + 0.0833333333333332*G7_0_0_2_1_0_2;
    A[94] = 0.0416666666666666*G4_0_0_1_2_1_2 + 0.0833333333333332*G7_0_0_2_1_1_2;
    A[95] = 0.0416666666666666*G4_0_0_2_2_1_2 + 0.0833333333333332*G7_0_0_2_1_2_2;
    A[96] = 0.0416666666666666*G4_0_0_0_0_2_0 + 0.0416666666666666*G4_0_0_0_1_2_0 + 0.0416666666666666*G4_0_0_0_2_2_0 + 0.0416666666666666*G4_0_0_1_0_2_0 + 0.0416666666666666*G4_0_0_1_1_2_0 + 0.0416666666666666*G4_0_0_1_2_2_0 + 0.0416666666666666*G4_0_0_2_0_2_0 + 0.0416666666666666*G4_0_0_2_1_2_0 + 0.0416666666666666*G4_0_0_2_2_2_0 + 0.0833333333333332*G7_0_0_0_2_0_0 + 0.0833333333333332*G7_0_0_0_2_1_0 + 0.0833333333333332*G7_0_0_0_2_2_0 + 0.0833333333333332*G7_0_0_1_2_0_0 + 0.0833333333333332*G7_0_0_1_2_1_0 + 0.0833333333333332*G7_0_0_1_2_2_0 + 0.0833333333333332*G7_0_0_2_2_0_0 + 0.0833333333333332*G7_0_0_2_2_1_0 + 0.0833333333333332*G7_0_0_2_2_2_0;
    A[97] = -0.0416666666666666*G4_0_0_0_0_2_0 - 0.0416666666666666*G4_0_0_0_1_2_0 - 0.0416666666666666*G4_0_0_0_2_2_0 - 0.0833333333333332*G7_0_0_0_2_0_0 - 0.0833333333333332*G7_0_0_1_2_0_0 - 0.0833333333333332*G7_0_0_2_2_0_0;
    A[98] = -0.0416666666666666*G4_0_0_1_0_2_0 - 0.0416666666666666*G4_0_0_1_1_2_0 - 0.0416666666666666*G4_0_0_1_2_2_0 - 0.0833333333333332*G7_0_0_0_2_1_0 - 0.0833333333333332*G7_0_0_1_2_1_0 - 0.0833333333333332*G7_0_0_2_2_1_0;
    A[99] = -0.0416666666666666*G4_0_0_2_0_2_0 - 0.0416666666666666*G4_0_0_2_1_2_0 - 0.0416666666666666*G4_0_0_2_2_2_0 - 0.0833333333333332*G7_0_0_0_2_2_0 - 0.0833333333333332*G7_0_0_1_2_2_0 - 0.0833333333333332*G7_0_0_2_2_2_0;
    A[100] = 0.0416666666666666*G4_0_0_0_0_2_1 + 0.0416666666666666*G4_0_0_0_1_2_1 + 0.0416666666666666*G4_0_0_0_2_2_1 + 0.0416666666666666*G4_0_0_1_0_2_1 + 0.0416666666666666*G4_0_0_1_1_2_1 + 0.0416666666666666*G4_0_0_1_2_2_1 + 0.0416666666666666*G4_0_0_2_0_2_1 + 0.0416666666666666*G4_0_0_2_1_2_1 + 0.0416666666666666*G4_0_0_2_2_2_1 + 0.0833333333333332*G7_0_0_0_2_0_1 + 0.0833333333333332*G7_0_0_0_2_1_1 + 0.0833333333333332*G7_0_0_0_2_2_1 + 0.0833333333333332*G7_0_0_1_2_0_1 + 0.0833333333333332*G7_0_0_1_2_1_1 + 0.0833333333333332*G7_0_0_1_2_2_1 + 0.0833333333333332*G7_0_0_2_2_0_1 + 0.0833333333333332*G7_0_0_2_2_1_1 + 0.0833333333333332*G7_0_0_2_2_2_1;
    A[101] = -0.0416666666666666*G4_0_0_0_0_2_1 - 0.0416666666666666*G4_0_0_0_1_2_1 - 0.0416666666666666*G4_0_0_0_2_2_1 - 0.0833333333333332*G7_0_0_0_2_0_1 - 0.0833333333333332*G7_0_0_1_2_0_1 - 0.0833333333333332*G7_0_0_2_2_0_1;
    A[102] = -0.0416666666666666*G4_0_0_1_0_2_1 - 0.0416666666666666*G4_0_0_1_1_2_1 - 0.0416666666666666*G4_0_0_1_2_2_1 - 0.0833333333333332*G7_0_0_0_2_1_1 - 0.0833333333333332*G7_0_0_1_2_1_1 - 0.0833333333333332*G7_0_0_2_2_1_1;
    A[103] = -0.0416666666666666*G4_0_0_2_0_2_1 - 0.0416666666666666*G4_0_0_2_1_2_1 - 0.0416666666666666*G4_0_0_2_2_2_1 - 0.0833333333333332*G7_0_0_0_2_2_1 - 0.0833333333333332*G7_0_0_1_2_2_1 - 0.0833333333333332*G7_0_0_2_2_2_1;
    A[104] = 0.0166666666666666*G0_0 - 0.0166666666666666*G1_0_0_0_0_0_0 - 0.0166666666666666*G1_0_0_0_0_1_0 - 0.0166666666666666*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.0166666666666666*G1_0_0_0_4_0_1 - 0.0166666666666666*G1_0_0_0_4_1_1 - 0.0166666666666666*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.0166666666666666*G1_0_0_0_8_0_2 - 0.0166666666666666*G1_0_0_0_8_1_2 - 0.0166666666666666*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.0166666666666666*G2_0_0_0_0_0_0 + 0.0166666666666666*G2_0_0_0_0_1_0 + 0.0166666666666666*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.0166666666666666*G2_0_0_0_4_0_1 + 0.0166666666666666*G2_0_0_0_4_1_1 + 0.0166666666666666*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.0166666666666666*G2_0_0_0_8_0_2 + 0.0166666666666666*G2_0_0_0_8_1_2 + 0.0166666666666666*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_0_0 + 0.0833333333333332*G3_0_0_0_1 + 0.0833333333333332*G3_0_0_0_2 + 0.0833333333333332*G3_0_0_1_0 + 0.0833333333333332*G3_0_0_1_1 + 0.0833333333333332*G3_0_0_1_2 + 0.0833333333333332*G3_0_0_2_0 + 0.0833333333333332*G3_0_0_2_1 + 0.0833333333333332*G3_0_0_2_2 + 0.0416666666666666*G4_0_0_0_0_2_2 + 0.0416666666666666*G4_0_0_0_1_2_2 + 0.0416666666666666*G4_0_0_0_2_2_2 + 0.0416666666666666*G4_0_0_1_0_2_2 + 0.0416666666666666*G4_0_0_1_1_2_2 + 0.0416666666666666*G4_0_0_1_2_2_2 + 0.0416666666666666*G4_0_0_2_0_2_2 + 0.0416666666666666*G4_0_0_2_1_2_2 + 0.0416666666666666*G4_0_0_2_2_2_2 + 0.0833333333333332*G5_0_0_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_0_0_0_0_1 + 0.0833333333333332*G5_0_0_0_0_0_0_0_2 + 0.0833333333333332*G5_0_0_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_0_0_1_0_1 + 0.0833333333333332*G5_0_0_0_0_0_1_0_2 + 0.0833333333333332*G5_0_0_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_0_0_2_0_1 + 0.0833333333333332*G5_0_0_0_0_0_2_0_2 + 0.0833333333333332*G5_0_0_0_1_0_0_1_0 + 0.0833333333333332*G5_0_0_0_1_0_0_1_1 + 0.0833333333333332*G5_0_0_0_1_0_0_1_2 + 0.0833333333333332*G5_0_0_0_1_0_1_1_0 + 0.0833333333333332*G5_0_0_0_1_0_1_1_1 + 0.0833333333333332*G5_0_0_0_1_0_1_1_2 + 0.0833333333333332*G5_0_0_0_1_0_2_1_0 + 0.0833333333333332*G5_0_0_0_1_0_2_1_1 + 0.0833333333333332*G5_0_0_0_1_0_2_1_2 + 0.0833333333333332*G5_0_0_0_2_0_0_2_0 + 0.0833333333333332*G5_0_0_0_2_0_0_2_1 + 0.0833333333333332*G5_0_0_0_2_0_0_2_2 + 0.0833333333333332*G5_0_0_0_2_0_1_2_0 + 0.0833333333333332*G5_0_0_0_2_0_1_2_1 + 0.0833333333333332*G5_0_0_0_2_0_1_2_2 + 0.0833333333333332*G5_0_0_0_2_0_2_2_0 + 0.0833333333333332*G5_0_0_0_2_0_2_2_1 + 0.0833333333333332*G5_0_0_0_2_0_2_2_2 + 0.0833333333333332*G5_1_0_0_0_1_0_0_0 + 0.0833333333333332*G5_1_0_0_0_1_0_0_1 + 0.0833333333333332*G5_1_0_0_0_1_0_0_2 + 0.0833333333333332*G5_1_0_0_0_1_1_0_0 + 0.0833333333333332*G5_1_0_0_0_1_1_0_1 + 0.0833333333333332*G5_1_0_0_0_1_1_0_2 + 0.0833333333333332*G5_1_0_0_0_1_2_0_0 + 0.0833333333333332*G5_1_0_0_0_1_2_0_1 + 0.0833333333333332*G5_1_0_0_0_1_2_0_2 + 0.0833333333333332*G5_1_0_0_1_1_0_1_0 + 0.0833333333333332*G5_1_0_0_1_1_0_1_1 + 0.0833333333333332*G5_1_0_0_1_1_0_1_2 + 0.0833333333333332*G5_1_0_0_1_1_1_1_0 + 0.0833333333333332*G5_1_0_0_1_1_1_1_1 + 0.0833333333333332*G5_1_0_0_1_1_1_1_2 + 0.0833333333333332*G5_1_0_0_1_1_2_1_0 + 0.0833333333333332*G5_1_0_0_1_1_2_1_1 + 0.0833333333333332*G5_1_0_0_1_1_2_1_2 + 0.0833333333333332*G5_1_0_0_2_1_0_2_0 + 0.0833333333333332*G5_1_0_0_2_1_0_2_1 + 0.0833333333333332*G5_1_0_0_2_1_0_2_2 + 0.0833333333333332*G5_1_0_0_2_1_1_2_0 + 0.0833333333333332*G5_1_0_0_2_1_1_2_1 + 0.0833333333333332*G5_1_0_0_2_1_1_2_2 + 0.0833333333333332*G5_1_0_0_2_1_2_2_0 + 0.0833333333333332*G5_1_0_0_2_1_2_2_1 + 0.0833333333333332*G5_1_0_0_2_1_2_2_2 + 0.0833333333333332*G5_2_0_0_0_2_0_0_0 + 0.0833333333333332*G5_2_0_0_0_2_0_0_1 + 0.0833333333333332*G5_2_0_0_0_2_0_0_2 + 0.0833333333333332*G5_2_0_0_0_2_1_0_0 + 0.0833333333333332*G5_2_0_0_0_2_1_0_1 + 0.0833333333333332*G5_2_0_0_0_2_1_0_2 + 0.0833333333333332*G5_2_0_0_0_2_2_0_0 + 0.0833333333333332*G5_2_0_0_0_2_2_0_1 + 0.0833333333333332*G5_2_0_0_0_2_2_0_2 + 0.0833333333333332*G5_2_0_0_1_2_0_1_0 + 0.0833333333333332*G5_2_0_0_1_2_0_1_1 + 0.0833333333333332*G5_2_0_0_1_2_0_1_2 + 0.0833333333333332*G5_2_0_0_1_2_1_1_0 + 0.0833333333333332*G5_2_0_0_1_2_1_1_1 + 0.0833333333333332*G5_2_0_0_1_2_1_1_2 + 0.0833333333333332*G5_2_0_0_1_2_2_1_0 + 0.0833333333333332*G5_2_0_0_1_2_2_1_1 + 0.0833333333333332*G5_2_0_0_1_2_2_1_2 + 0.0833333333333332*G5_2_0_0_2_2_0_2_0 + 0.0833333333333332*G5_2_0_0_2_2_0_2_1 + 0.0833333333333332*G5_2_0_0_2_2_0_2_2 + 0.0833333333333332*G5_2_0_0_2_2_1_2_0 + 0.0833333333333332*G5_2_0_0_2_2_1_2_1 + 0.0833333333333332*G5_2_0_0_2_2_1_2_2 + 0.0833333333333332*G5_2_0_0_2_2_2_2_0 + 0.0833333333333332*G5_2_0_0_2_2_2_2_1 + 0.0833333333333332*G5_2_0_0_2_2_2_2_2 - 0.0833333333333332*G6_0_0_0_0_0_0_0_0 - 0.0833333333333332*G6_0_0_0_0_0_0_0_1 - 0.0833333333333332*G6_0_0_0_0_0_0_0_2 - 0.0833333333333332*G6_0_0_0_0_0_1_0_0 - 0.0833333333333332*G6_0_0_0_0_0_1_0_1 - 0.0833333333333332*G6_0_0_0_0_0_1_0_2 - 0.0833333333333332*G6_0_0_0_0_0_2_0_0 - 0.0833333333333332*G6_0_0_0_0_0_2_0_1 - 0.0833333333333332*G6_0_0_0_0_0_2_0_2 - 0.0833333333333332*G6_0_0_0_1_0_0_1_0 - 0.0833333333333332*G6_0_0_0_1_0_0_1_1 - 0.0833333333333332*G6_0_0_0_1_0_0_1_2 - 0.0833333333333332*G6_0_0_0_1_0_1_1_0 - 0.0833333333333332*G6_0_0_0_1_0_1_1_1 - 0.0833333333333332*G6_0_0_0_1_0_1_1_2 - 0.0833333333333332*G6_0_0_0_1_0_2_1_0 - 0.0833333333333332*G6_0_0_0_1_0_2_1_1 - 0.0833333333333332*G6_0_0_0_1_0_2_1_2 - 0.0833333333333332*G6_0_0_0_2_0_0_2_0 - 0.0833333333333332*G6_0_0_0_2_0_0_2_1 - 0.0833333333333332*G6_0_0_0_2_0_0_2_2 - 0.0833333333333332*G6_0_0_0_2_0_1_2_0 - 0.0833333333333332*G6_0_0_0_2_0_1_2_1 - 0.0833333333333332*G6_0_0_0_2_0_1_2_2 - 0.0833333333333332*G6_0_0_0_2_0_2_2_0 - 0.0833333333333332*G6_0_0_0_2_0_2_2_1 - 0.0833333333333332*G6_0_0_0_2_0_2_2_2 - 0.0833333333333332*G6_1_0_0_0_1_0_0_0 - 0.0833333333333332*G6_1_0_0_0_1_0_0_1 - 0.0833333333333332*G6_1_0_0_0_1_0_0_2 - 0.0833333333333332*G6_1_0_0_0_1_1_0_0 - 0.0833333333333332*G6_1_0_0_0_1_1_0_1 - 0.0833333333333332*G6_1_0_0_0_1_1_0_2 - 0.0833333333333332*G6_1_0_0_0_1_2_0_0 - 0.0833333333333332*G6_1_0_0_0_1_2_0_1 - 0.0833333333333332*G6_1_0_0_0_1_2_0_2 - 0.0833333333333332*G6_1_0_0_1_1_0_1_0 - 0.0833333333333332*G6_1_0_0_1_1_0_1_1 - 0.0833333333333332*G6_1_0_0_1_1_0_1_2 - 0.0833333333333332*G6_1_0_0_1_1_1_1_0 - 0.0833333333333332*G6_1_0_0_1_1_1_1_1 - 0.0833333333333332*G6_1_0_0_1_1_1_1_2 - 0.0833333333333332*G6_1_0_0_1_1_2_1_0 - 0.0833333333333332*G6_1_0_0_1_1_2_1_1 - 0.0833333333333332*G6_1_0_0_1_1_2_1_2 - 0.0833333333333332*G6_1_0_0_2_1_0_2_0 - 0.0833333333333332*G6_1_0_0_2_1_0_2_1 - 0.0833333333333332*G6_1_0_0_2_1_0_2_2 - 0.0833333333333332*G6_1_0_0_2_1_1_2_0 - 0.0833333333333332*G6_1_0_0_2_1_1_2_1 - 0.0833333333333332*G6_1_0_0_2_1_1_2_2 - 0.0833333333333332*G6_1_0_0_2_1_2_2_0 - 0.0833333333333332*G6_1_0_0_2_1_2_2_1 - 0.0833333333333332*G6_1_0_0_2_1_2_2_2 - 0.0833333333333332*G6_2_0_0_0_2_0_0_0 - 0.0833333333333332*G6_2_0_0_0_2_0_0_1 - 0.0833333333333332*G6_2_0_0_0_2_0_0_2 - 0.0833333333333332*G6_2_0_0_0_2_1_0_0 - 0.0833333333333332*G6_2_0_0_0_2_1_0_1 - 0.0833333333333332*G6_2_0_0_0_2_1_0_2 - 0.0833333333333332*G6_2_0_0_0_2_2_0_0 - 0.0833333333333332*G6_2_0_0_0_2_2_0_1 - 0.0833333333333332*G6_2_0_0_0_2_2_0_2 - 0.0833333333333332*G6_2_0_0_1_2_0_1_0 - 0.0833333333333332*G6_2_0_0_1_2_0_1_1 - 0.0833333333333332*G6_2_0_0_1_2_0_1_2 - 0.0833333333333332*G6_2_0_0_1_2_1_1_0 - 0.0833333333333332*G6_2_0_0_1_2_1_1_1 - 0.0833333333333332*G6_2_0_0_1_2_1_1_2 - 0.0833333333333332*G6_2_0_0_1_2_2_1_0 - 0.0833333333333332*G6_2_0_0_1_2_2_1_1 - 0.0833333333333332*G6_2_0_0_1_2_2_1_2 - 0.0833333333333332*G6_2_0_0_2_2_0_2_0 - 0.0833333333333332*G6_2_0_0_2_2_0_2_1 - 0.0833333333333332*G6_2_0_0_2_2_0_2_2 - 0.0833333333333332*G6_2_0_0_2_2_1_2_0 - 0.0833333333333332*G6_2_0_0_2_2_1_2_1 - 0.0833333333333332*G6_2_0_0_2_2_1_2_2 - 0.0833333333333332*G6_2_0_0_2_2_2_2_0 - 0.0833333333333332*G6_2_0_0_2_2_2_2_1 - 0.0833333333333332*G6_2_0_0_2_2_2_2_2 + 0.0833333333333332*G7_0_0_0_2_0_2 + 0.0833333333333332*G7_0_0_0_2_1_2 + 0.0833333333333332*G7_0_0_0_2_2_2 + 0.0833333333333332*G7_0_0_1_2_0_2 + 0.0833333333333332*G7_0_0_1_2_1_2 + 0.0833333333333332*G7_0_0_1_2_2_2 + 0.0833333333333332*G7_0_0_2_2_0_2 + 0.0833333333333332*G7_0_0_2_2_1_2 + 0.0833333333333332*G7_0_0_2_2_2_2;
    A[105] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.0166666666666666*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.0166666666666666*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.0166666666666666*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.0166666666666666*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.0166666666666666*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 - 0.0833333333333332*G3_0_0_0_0 - 0.0833333333333332*G3_0_0_0_1 - 0.0833333333333332*G3_0_0_0_2 - 0.0416666666666666*G4_0_0_0_0_2_2 - 0.0416666666666666*G4_0_0_0_1_2_2 - 0.0416666666666666*G4_0_0_0_2_2_2 - 0.0833333333333332*G5_0_0_0_0_0_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_1_0_0_1_0 - 0.0833333333333332*G5_0_0_0_1_0_1_1_0 - 0.0833333333333332*G5_0_0_0_1_0_2_1_0 - 0.0833333333333332*G5_0_0_0_2_0_0_2_0 - 0.0833333333333332*G5_0_0_0_2_0_1_2_0 - 0.0833333333333332*G5_0_0_0_2_0_2_2_0 - 0.0833333333333332*G5_1_0_0_0_1_0_0_0 - 0.0833333333333332*G5_1_0_0_0_1_1_0_0 - 0.0833333333333332*G5_1_0_0_0_1_2_0_0 - 0.0833333333333332*G5_1_0_0_1_1_0_1_0 - 0.0833333333333332*G5_1_0_0_1_1_1_1_0 - 0.0833333333333332*G5_1_0_0_1_1_2_1_0 - 0.0833333333333332*G5_1_0_0_2_1_0_2_0 - 0.0833333333333332*G5_1_0_0_2_1_1_2_0 - 0.0833333333333332*G5_1_0_0_2_1_2_2_0 - 0.0833333333333332*G5_2_0_0_0_2_0_0_0 - 0.0833333333333332*G5_2_0_0_0_2_1_0_0 - 0.0833333333333332*G5_2_0_0_0_2_2_0_0 - 0.0833333333333332*G5_2_0_0_1_2_0_1_0 - 0.0833333333333332*G5_2_0_0_1_2_1_1_0 - 0.0833333333333332*G5_2_0_0_1_2_2_1_0 - 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G5_2_0_0_2_2_2_2_0 + 0.0833333333333332*G6_0_0_0_0_0_0_0_0 + 0.0833333333333332*G6_0_0_0_0_0_1_0_0 + 0.0833333333333332*G6_0_0_0_0_0_2_0_0 + 0.0833333333333332*G6_0_0_0_1_0_0_1_0 + 0.0833333333333332*G6_0_0_0_1_0_1_1_0 + 0.0833333333333332*G6_0_0_0_1_0_2_1_0 + 0.0833333333333332*G6_0_0_0_2_0_0_2_0 + 0.0833333333333332*G6_0_0_0_2_0_1_2_0 + 0.0833333333333332*G6_0_0_0_2_0_2_2_0 + 0.0833333333333332*G6_1_0_0_0_1_0_0_0 + 0.0833333333333332*G6_1_0_0_0_1_1_0_0 + 0.0833333333333332*G6_1_0_0_0_1_2_0_0 + 0.0833333333333332*G6_1_0_0_1_1_0_1_0 + 0.0833333333333332*G6_1_0_0_1_1_1_1_0 + 0.0833333333333332*G6_1_0_0_1_1_2_1_0 + 0.0833333333333332*G6_1_0_0_2_1_0_2_0 + 0.0833333333333332*G6_1_0_0_2_1_1_2_0 + 0.0833333333333332*G6_1_0_0_2_1_2_2_0 + 0.0833333333333332*G6_2_0_0_0_2_0_0_0 + 0.0833333333333332*G6_2_0_0_0_2_1_0_0 + 0.0833333333333332*G6_2_0_0_0_2_2_0_0 + 0.0833333333333332*G6_2_0_0_1_2_0_1_0 + 0.0833333333333332*G6_2_0_0_1_2_1_1_0 + 0.0833333333333332*G6_2_0_0_1_2_2_1_0 + 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G6_2_0_0_2_2_2_2_0 - 0.0833333333333332*G7_0_0_0_2_0_2 - 0.0833333333333332*G7_0_0_1_2_0_2 - 0.0833333333333332*G7_0_0_2_2_0_2;
    A[106] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.0166666666666666*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.0166666666666666*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.0166666666666666*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.0166666666666666*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.0166666666666666*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 - 0.0833333333333332*G3_0_0_1_0 - 0.0833333333333332*G3_0_0_1_1 - 0.0833333333333332*G3_0_0_1_2 - 0.0416666666666666*G4_0_0_1_0_2_2 - 0.0416666666666666*G4_0_0_1_1_2_2 - 0.0416666666666666*G4_0_0_1_2_2_2 - 0.0833333333333332*G5_0_0_0_0_0_0_0_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_1 - 0.0833333333333332*G5_0_0_0_1_0_0_1_1 - 0.0833333333333332*G5_0_0_0_1_0_1_1_1 - 0.0833333333333332*G5_0_0_0_1_0_2_1_1 - 0.0833333333333332*G5_0_0_0_2_0_0_2_1 - 0.0833333333333332*G5_0_0_0_2_0_1_2_1 - 0.0833333333333332*G5_0_0_0_2_0_2_2_1 - 0.0833333333333332*G5_1_0_0_0_1_0_0_1 - 0.0833333333333332*G5_1_0_0_0_1_1_0_1 - 0.0833333333333332*G5_1_0_0_0_1_2_0_1 - 0.0833333333333332*G5_1_0_0_1_1_0_1_1 - 0.0833333333333332*G5_1_0_0_1_1_1_1_1 - 0.0833333333333332*G5_1_0_0_1_1_2_1_1 - 0.0833333333333332*G5_1_0_0_2_1_0_2_1 - 0.0833333333333332*G5_1_0_0_2_1_1_2_1 - 0.0833333333333332*G5_1_0_0_2_1_2_2_1 - 0.0833333333333332*G5_2_0_0_0_2_0_0_1 - 0.0833333333333332*G5_2_0_0_0_2_1_0_1 - 0.0833333333333332*G5_2_0_0_0_2_2_0_1 - 0.0833333333333332*G5_2_0_0_1_2_0_1_1 - 0.0833333333333332*G5_2_0_0_1_2_1_1_1 - 0.0833333333333332*G5_2_0_0_1_2_2_1_1 - 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G5_2_0_0_2_2_2_2_1 + 0.0833333333333332*G6_0_0_0_0_0_0_0_1 + 0.0833333333333332*G6_0_0_0_0_0_1_0_1 + 0.0833333333333332*G6_0_0_0_0_0_2_0_1 + 0.0833333333333332*G6_0_0_0_1_0_0_1_1 + 0.0833333333333332*G6_0_0_0_1_0_1_1_1 + 0.0833333333333332*G6_0_0_0_1_0_2_1_1 + 0.0833333333333332*G6_0_0_0_2_0_0_2_1 + 0.0833333333333332*G6_0_0_0_2_0_1_2_1 + 0.0833333333333332*G6_0_0_0_2_0_2_2_1 + 0.0833333333333332*G6_1_0_0_0_1_0_0_1 + 0.0833333333333332*G6_1_0_0_0_1_1_0_1 + 0.0833333333333332*G6_1_0_0_0_1_2_0_1 + 0.0833333333333332*G6_1_0_0_1_1_0_1_1 + 0.0833333333333332*G6_1_0_0_1_1_1_1_1 + 0.0833333333333332*G6_1_0_0_1_1_2_1_1 + 0.0833333333333332*G6_1_0_0_2_1_0_2_1 + 0.0833333333333332*G6_1_0_0_2_1_1_2_1 + 0.0833333333333332*G6_1_0_0_2_1_2_2_1 + 0.0833333333333332*G6_2_0_0_0_2_0_0_1 + 0.0833333333333332*G6_2_0_0_0_2_1_0_1 + 0.0833333333333332*G6_2_0_0_0_2_2_0_1 + 0.0833333333333332*G6_2_0_0_1_2_0_1_1 + 0.0833333333333332*G6_2_0_0_1_2_1_1_1 + 0.0833333333333332*G6_2_0_0_1_2_2_1_1 + 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G6_2_0_0_2_2_2_2_1 - 0.0833333333333332*G7_0_0_0_2_1_2 - 0.0833333333333332*G7_0_0_1_2_1_2 - 0.0833333333333332*G7_0_0_2_2_1_2;
    A[107] = 0.00833333333333331*G0_0 + 0.0166666666666666*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.0166666666666666*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.0166666666666666*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.0166666666666666*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.0166666666666666*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.0166666666666666*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_2_0 - 0.0833333333333332*G3_0_0_2_1 - 0.0833333333333332*G3_0_0_2_2 - 0.0416666666666666*G4_0_0_2_0_2_2 - 0.0416666666666666*G4_0_0_2_1_2_2 - 0.0416666666666666*G4_0_0_2_2_2_2 - 0.0833333333333332*G5_0_0_0_0_0_0_0_2 - 0.0833333333333332*G5_0_0_0_0_0_1_0_2 - 0.0833333333333332*G5_0_0_0_0_0_2_0_2 - 0.0833333333333332*G5_0_0_0_1_0_0_1_2 - 0.0833333333333332*G5_0_0_0_1_0_1_1_2 - 0.0833333333333332*G5_0_0_0_1_0_2_1_2 - 0.0833333333333332*G5_0_0_0_2_0_0_2_2 - 0.0833333333333332*G5_0_0_0_2_0_1_2_2 - 0.0833333333333332*G5_0_0_0_2_0_2_2_2 - 0.0833333333333332*G5_1_0_0_0_1_0_0_2 - 0.0833333333333332*G5_1_0_0_0_1_1_0_2 - 0.0833333333333332*G5_1_0_0_0_1_2_0_2 - 0.0833333333333332*G5_1_0_0_1_1_0_1_2 - 0.0833333333333332*G5_1_0_0_1_1_1_1_2 - 0.0833333333333332*G5_1_0_0_1_1_2_1_2 - 0.0833333333333332*G5_1_0_0_2_1_0_2_2 - 0.0833333333333332*G5_1_0_0_2_1_1_2_2 - 0.0833333333333332*G5_1_0_0_2_1_2_2_2 - 0.0833333333333332*G5_2_0_0_0_2_0_0_2 - 0.0833333333333332*G5_2_0_0_0_2_1_0_2 - 0.0833333333333332*G5_2_0_0_0_2_2_0_2 - 0.0833333333333332*G5_2_0_0_1_2_0_1_2 - 0.0833333333333332*G5_2_0_0_1_2_1_1_2 - 0.0833333333333332*G5_2_0_0_1_2_2_1_2 - 0.0833333333333332*G5_2_0_0_2_2_0_2_2 - 0.0833333333333332*G5_2_0_0_2_2_1_2_2 - 0.0833333333333332*G5_2_0_0_2_2_2_2_2 + 0.0833333333333332*G6_0_0_0_0_0_0_0_2 + 0.0833333333333332*G6_0_0_0_0_0_1_0_2 + 0.0833333333333332*G6_0_0_0_0_0_2_0_2 + 0.0833333333333332*G6_0_0_0_1_0_0_1_2 + 0.0833333333333332*G6_0_0_0_1_0_1_1_2 + 0.0833333333333332*G6_0_0_0_1_0_2_1_2 + 0.0833333333333332*G6_0_0_0_2_0_0_2_2 + 0.0833333333333332*G6_0_0_0_2_0_1_2_2 + 0.0833333333333332*G6_0_0_0_2_0_2_2_2 + 0.0833333333333332*G6_1_0_0_0_1_0_0_2 + 0.0833333333333332*G6_1_0_0_0_1_1_0_2 + 0.0833333333333332*G6_1_0_0_0_1_2_0_2 + 0.0833333333333332*G6_1_0_0_1_1_0_1_2 + 0.0833333333333332*G6_1_0_0_1_1_1_1_2 + 0.0833333333333332*G6_1_0_0_1_1_2_1_2 + 0.0833333333333332*G6_1_0_0_2_1_0_2_2 + 0.0833333333333332*G6_1_0_0_2_1_1_2_2 + 0.0833333333333332*G6_1_0_0_2_1_2_2_2 + 0.0833333333333332*G6_2_0_0_0_2_0_0_2 + 0.0833333333333332*G6_2_0_0_0_2_1_0_2 + 0.0833333333333332*G6_2_0_0_0_2_2_0_2 + 0.0833333333333332*G6_2_0_0_1_2_0_1_2 + 0.0833333333333332*G6_2_0_0_1_2_1_1_2 + 0.0833333333333332*G6_2_0_0_1_2_2_1_2 + 0.0833333333333332*G6_2_0_0_2_2_0_2_2 + 0.0833333333333332*G6_2_0_0_2_2_1_2_2 + 0.0833333333333332*G6_2_0_0_2_2_2_2_2 - 0.0833333333333332*G7_0_0_0_2_2_2 - 0.0833333333333332*G7_0_0_1_2_2_2 - 0.0833333333333332*G7_0_0_2_2_2_2;
    A[108] = -0.0416666666666666*G4_0_0_0_0_2_0 - 0.0416666666666666*G4_0_0_1_0_2_0 - 0.0416666666666666*G4_0_0_2_0_2_0 - 0.0833333333333332*G7_0_0_0_2_0_0 - 0.0833333333333332*G7_0_0_0_2_1_0 - 0.0833333333333332*G7_0_0_0_2_2_0;
    A[109] = 0.0416666666666666*G4_0_0_0_0_2_0 + 0.0833333333333332*G7_0_0_0_2_0_0;
    A[110] = 0.0416666666666666*G4_0_0_1_0_2_0 + 0.0833333333333332*G7_0_0_0_2_1_0;
    A[111] = 0.0416666666666666*G4_0_0_2_0_2_0 + 0.0833333333333332*G7_0_0_0_2_2_0;
    A[112] = -0.0416666666666666*G4_0_0_0_0_2_1 - 0.0416666666666666*G4_0_0_1_0_2_1 - 0.0416666666666666*G4_0_0_2_0_2_1 - 0.0833333333333332*G7_0_0_0_2_0_1 - 0.0833333333333332*G7_0_0_0_2_1_1 - 0.0833333333333332*G7_0_0_0_2_2_1;
    A[113] = 0.0416666666666666*G4_0_0_0_0_2_1 + 0.0833333333333332*G7_0_0_0_2_0_1;
    A[114] = 0.0416666666666666*G4_0_0_1_0_2_1 + 0.0833333333333332*G7_0_0_0_2_1_1;
    A[115] = 0.0416666666666666*G4_0_0_2_0_2_1 + 0.0833333333333332*G7_0_0_0_2_2_1;
    A[116] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.0166666666666666*G1_0_0_0_1_0_0 - 0.0166666666666666*G1_0_0_0_1_1_0 - 0.0166666666666666*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.0166666666666666*G1_0_0_0_5_0_1 - 0.0166666666666666*G1_0_0_0_5_1_1 - 0.0166666666666666*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.0166666666666666*G1_0_0_0_9_0_2 - 0.0166666666666666*G1_0_0_0_9_1_2 - 0.0166666666666666*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.0166666666666666*G2_0_0_0_1_0_0 + 0.0166666666666666*G2_0_0_0_1_1_0 + 0.0166666666666666*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.0166666666666666*G2_0_0_0_5_0_1 + 0.0166666666666666*G2_0_0_0_5_1_1 + 0.0166666666666666*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.0166666666666666*G2_0_0_0_9_0_2 + 0.0166666666666666*G2_0_0_0_9_1_2 + 0.0166666666666666*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_0 - 0.0833333333333332*G3_0_0_1_0 - 0.0833333333333332*G3_0_0_2_0 - 0.0416666666666666*G4_0_0_0_0_2_2 - 0.0416666666666666*G4_0_0_1_0_2_2 - 0.0416666666666666*G4_0_0_2_0_2_2 - 0.0833333333333332*G5_0_0_0_0_0_0_0_0 - 0.0833333333333332*G5_0_0_0_0_0_0_0_1 - 0.0833333333333332*G5_0_0_0_0_0_0_0_2 - 0.0833333333333332*G5_0_0_0_1_0_0_1_0 - 0.0833333333333332*G5_0_0_0_1_0_0_1_1 - 0.0833333333333332*G5_0_0_0_1_0_0_1_2 - 0.0833333333333332*G5_0_0_0_2_0_0_2_0 - 0.0833333333333332*G5_0_0_0_2_0_0_2_1 - 0.0833333333333332*G5_0_0_0_2_0_0_2_2 - 0.0833333333333332*G5_1_0_0_0_1_0_0_0 - 0.0833333333333332*G5_1_0_0_0_1_0_0_1 - 0.0833333333333332*G5_1_0_0_0_1_0_0_2 - 0.0833333333333332*G5_1_0_0_1_1_0_1_0 - 0.0833333333333332*G5_1_0_0_1_1_0_1_1 - 0.0833333333333332*G5_1_0_0_1_1_0_1_2 - 0.0833333333333332*G5_1_0_0_2_1_0_2_0 - 0.0833333333333332*G5_1_0_0_2_1_0_2_1 - 0.0833333333333332*G5_1_0_0_2_1_0_2_2 - 0.0833333333333332*G5_2_0_0_0_2_0_0_0 - 0.0833333333333332*G5_2_0_0_0_2_0_0_1 - 0.0833333333333332*G5_2_0_0_0_2_0_0_2 - 0.0833333333333332*G5_2_0_0_1_2_0_1_0 - 0.0833333333333332*G5_2_0_0_1_2_0_1_1 - 0.0833333333333332*G5_2_0_0_1_2_0_1_2 - 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G5_2_0_0_2_2_0_2_2 + 0.0833333333333332*G6_0_0_0_0_0_0_0_0 + 0.0833333333333332*G6_0_0_0_0_0_0_0_1 + 0.0833333333333332*G6_0_0_0_0_0_0_0_2 + 0.0833333333333332*G6_0_0_0_1_0_0_1_0 + 0.0833333333333332*G6_0_0_0_1_0_0_1_1 + 0.0833333333333332*G6_0_0_0_1_0_0_1_2 + 0.0833333333333332*G6_0_0_0_2_0_0_2_0 + 0.0833333333333332*G6_0_0_0_2_0_0_2_1 + 0.0833333333333332*G6_0_0_0_2_0_0_2_2 + 0.0833333333333332*G6_1_0_0_0_1_0_0_0 + 0.0833333333333332*G6_1_0_0_0_1_0_0_1 + 0.0833333333333332*G6_1_0_0_0_1_0_0_2 + 0.0833333333333332*G6_1_0_0_1_1_0_1_0 + 0.0833333333333332*G6_1_0_0_1_1_0_1_1 + 0.0833333333333332*G6_1_0_0_1_1_0_1_2 + 0.0833333333333332*G6_1_0_0_2_1_0_2_0 + 0.0833333333333332*G6_1_0_0_2_1_0_2_1 + 0.0833333333333332*G6_1_0_0_2_1_0_2_2 + 0.0833333333333332*G6_2_0_0_0_2_0_0_0 + 0.0833333333333332*G6_2_0_0_0_2_0_0_1 + 0.0833333333333332*G6_2_0_0_0_2_0_0_2 + 0.0833333333333332*G6_2_0_0_1_2_0_1_0 + 0.0833333333333332*G6_2_0_0_1_2_0_1_1 + 0.0833333333333332*G6_2_0_0_1_2_0_1_2 + 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G6_2_0_0_2_2_0_2_2 - 0.0833333333333332*G7_0_0_0_2_0_2 - 0.0833333333333332*G7_0_0_0_2_1_2 - 0.0833333333333332*G7_0_0_0_2_2_2;
    A[117] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.0166666666666666*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.0166666666666666*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.0166666666666666*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.0166666666666666*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.0166666666666666*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.0166666666666666*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_0 + 0.0416666666666666*G4_0_0_0_0_2_2 + 0.0833333333333332*G5_0_0_0_0_0_0_0_0 + 0.0833333333333332*G5_0_0_0_1_0_0_1_0 + 0.0833333333333332*G5_0_0_0_2_0_0_2_0 + 0.0833333333333332*G5_1_0_0_0_1_0_0_0 + 0.0833333333333332*G5_1_0_0_1_1_0_1_0 + 0.0833333333333332*G5_1_0_0_2_1_0_2_0 + 0.0833333333333332*G5_2_0_0_0_2_0_0_0 + 0.0833333333333332*G5_2_0_0_1_2_0_1_0 + 0.0833333333333332*G5_2_0_0_2_2_0_2_0 - 0.0833333333333332*G6_0_0_0_0_0_0_0_0 - 0.0833333333333332*G6_0_0_0_1_0_0_1_0 - 0.0833333333333332*G6_0_0_0_2_0_0_2_0 - 0.0833333333333332*G6_1_0_0_0_1_0_0_0 - 0.0833333333333332*G6_1_0_0_1_1_0_1_0 - 0.0833333333333332*G6_1_0_0_2_1_0_2_0 - 0.0833333333333332*G6_2_0_0_0_2_0_0_0 - 0.0833333333333332*G6_2_0_0_1_2_0_1_0 - 0.0833333333333332*G6_2_0_0_2_2_0_2_0 + 0.0833333333333332*G7_0_0_0_2_0_2;
    A[118] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.0166666666666666*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.0166666666666666*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.0166666666666666*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.0166666666666666*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.0166666666666666*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.0166666666666666*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_0 + 0.0416666666666666*G4_0_0_1_0_2_2 + 0.0833333333333332*G5_0_0_0_0_0_0_0_1 + 0.0833333333333332*G5_0_0_0_1_0_0_1_1 + 0.0833333333333332*G5_0_0_0_2_0_0_2_1 + 0.0833333333333332*G5_1_0_0_0_1_0_0_1 + 0.0833333333333332*G5_1_0_0_1_1_0_1_1 + 0.0833333333333332*G5_1_0_0_2_1_0_2_1 + 0.0833333333333332*G5_2_0_0_0_2_0_0_1 + 0.0833333333333332*G5_2_0_0_1_2_0_1_1 + 0.0833333333333332*G5_2_0_0_2_2_0_2_1 - 0.0833333333333332*G6_0_0_0_0_0_0_0_1 - 0.0833333333333332*G6_0_0_0_1_0_0_1_1 - 0.0833333333333332*G6_0_0_0_2_0_0_2_1 - 0.0833333333333332*G6_1_0_0_0_1_0_0_1 - 0.0833333333333332*G6_1_0_0_1_1_0_1_1 - 0.0833333333333332*G6_1_0_0_2_1_0_2_1 - 0.0833333333333332*G6_2_0_0_0_2_0_0_1 - 0.0833333333333332*G6_2_0_0_1_2_0_1_1 - 0.0833333333333332*G6_2_0_0_2_2_0_2_1 + 0.0833333333333332*G7_0_0_0_2_1_2;
    A[119] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.0166666666666666*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.0166666666666666*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.0166666666666666*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.0166666666666666*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.0166666666666666*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.0166666666666666*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_0 + 0.0416666666666666*G4_0_0_2_0_2_2 + 0.0833333333333332*G5_0_0_0_0_0_0_0_2 + 0.0833333333333332*G5_0_0_0_1_0_0_1_2 + 0.0833333333333332*G5_0_0_0_2_0_0_2_2 + 0.0833333333333332*G5_1_0_0_0_1_0_0_2 + 0.0833333333333332*G5_1_0_0_1_1_0_1_2 + 0.0833333333333332*G5_1_0_0_2_1_0_2_2 + 0.0833333333333332*G5_2_0_0_0_2_0_0_2 + 0.0833333333333332*G5_2_0_0_1_2_0_1_2 + 0.0833333333333332*G5_2_0_0_2_2_0_2_2 - 0.0833333333333332*G6_0_0_0_0_0_0_0_2 - 0.0833333333333332*G6_0_0_0_1_0_0_1_2 - 0.0833333333333332*G6_0_0_0_2_0_0_2_2 - 0.0833333333333332*G6_1_0_0_0_1_0_0_2 - 0.0833333333333332*G6_1_0_0_1_1_0_1_2 - 0.0833333333333332*G6_1_0_0_2_1_0_2_2 - 0.0833333333333332*G6_2_0_0_0_2_0_0_2 - 0.0833333333333332*G6_2_0_0_1_2_0_1_2 - 0.0833333333333332*G6_2_0_0_2_2_0_2_2 + 0.0833333333333332*G7_0_0_0_2_2_2;
    A[120] = -0.0416666666666666*G4_0_0_0_1_2_0 - 0.0416666666666666*G4_0_0_1_1_2_0 - 0.0416666666666666*G4_0_0_2_1_2_0 - 0.0833333333333332*G7_0_0_1_2_0_0 - 0.0833333333333332*G7_0_0_1_2_1_0 - 0.0833333333333332*G7_0_0_1_2_2_0;
    A[121] = 0.0416666666666666*G4_0_0_0_1_2_0 + 0.0833333333333332*G7_0_0_1_2_0_0;
    A[122] = 0.0416666666666666*G4_0_0_1_1_2_0 + 0.0833333333333332*G7_0_0_1_2_1_0;
    A[123] = 0.0416666666666666*G4_0_0_2_1_2_0 + 0.0833333333333332*G7_0_0_1_2_2_0;
    A[124] = -0.0416666666666666*G4_0_0_0_1_2_1 - 0.0416666666666666*G4_0_0_1_1_2_1 - 0.0416666666666666*G4_0_0_2_1_2_1 - 0.0833333333333332*G7_0_0_1_2_0_1 - 0.0833333333333332*G7_0_0_1_2_1_1 - 0.0833333333333332*G7_0_0_1_2_2_1;
    A[125] = 0.0416666666666666*G4_0_0_0_1_2_1 + 0.0833333333333332*G7_0_0_1_2_0_1;
    A[126] = 0.0416666666666666*G4_0_0_1_1_2_1 + 0.0833333333333332*G7_0_0_1_2_1_1;
    A[127] = 0.0416666666666666*G4_0_0_2_1_2_1 + 0.0833333333333332*G7_0_0_1_2_2_1;
    A[128] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.0166666666666666*G1_0_0_0_2_0_0 - 0.0166666666666666*G1_0_0_0_2_1_0 - 0.0166666666666666*G1_0_0_0_2_2_0 - 0.00833333333333331*G1_0_0_0_3_0_0 - 0.00833333333333331*G1_0_0_0_3_1_0 - 0.00833333333333331*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.0166666666666666*G1_0_0_0_6_0_1 - 0.0166666666666666*G1_0_0_0_6_1_1 - 0.0166666666666666*G1_0_0_0_6_2_1 - 0.00833333333333331*G1_0_0_0_7_0_1 - 0.00833333333333331*G1_0_0_0_7_1_1 - 0.00833333333333331*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.0166666666666666*G1_0_0_0_10_0_2 - 0.0166666666666666*G1_0_0_0_10_1_2 - 0.0166666666666666*G1_0_0_0_10_2_2 - 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.0166666666666666*G2_0_0_0_2_0_0 + 0.0166666666666666*G2_0_0_0_2_1_0 + 0.0166666666666666*G2_0_0_0_2_2_0 + 0.00833333333333331*G2_0_0_0_3_0_0 + 0.00833333333333331*G2_0_0_0_3_1_0 + 0.00833333333333331*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.0166666666666666*G2_0_0_0_6_0_1 + 0.0166666666666666*G2_0_0_0_6_1_1 + 0.0166666666666666*G2_0_0_0_6_2_1 + 0.00833333333333331*G2_0_0_0_7_0_1 + 0.00833333333333331*G2_0_0_0_7_1_1 + 0.00833333333333331*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.0166666666666666*G2_0_0_0_10_0_2 + 0.0166666666666666*G2_0_0_0_10_1_2 + 0.0166666666666666*G2_0_0_0_10_2_2 + 0.00833333333333331*G2_0_0_0_11_0_2 + 0.00833333333333331*G2_0_0_0_11_1_2 + 0.00833333333333331*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_1 - 0.0833333333333332*G3_0_0_1_1 - 0.0833333333333332*G3_0_0_2_1 - 0.0416666666666666*G4_0_0_0_1_2_2 - 0.0416666666666666*G4_0_0_1_1_2_2 - 0.0416666666666666*G4_0_0_2_1_2_2 - 0.0833333333333332*G5_0_0_0_0_0_1_0_0 - 0.0833333333333332*G5_0_0_0_0_0_1_0_1 - 0.0833333333333332*G5_0_0_0_0_0_1_0_2 - 0.0833333333333332*G5_0_0_0_1_0_1_1_0 - 0.0833333333333332*G5_0_0_0_1_0_1_1_1 - 0.0833333333333332*G5_0_0_0_1_0_1_1_2 - 0.0833333333333332*G5_0_0_0_2_0_1_2_0 - 0.0833333333333332*G5_0_0_0_2_0_1_2_1 - 0.0833333333333332*G5_0_0_0_2_0_1_2_2 - 0.0833333333333332*G5_1_0_0_0_1_1_0_0 - 0.0833333333333332*G5_1_0_0_0_1_1_0_1 - 0.0833333333333332*G5_1_0_0_0_1_1_0_2 - 0.0833333333333332*G5_1_0_0_1_1_1_1_0 - 0.0833333333333332*G5_1_0_0_1_1_1_1_1 - 0.0833333333333332*G5_1_0_0_1_1_1_1_2 - 0.0833333333333332*G5_1_0_0_2_1_1_2_0 - 0.0833333333333332*G5_1_0_0_2_1_1_2_1 - 0.0833333333333332*G5_1_0_0_2_1_1_2_2 - 0.0833333333333332*G5_2_0_0_0_2_1_0_0 - 0.0833333333333332*G5_2_0_0_0_2_1_0_1 - 0.0833333333333332*G5_2_0_0_0_2_1_0_2 - 0.0833333333333332*G5_2_0_0_1_2_1_1_0 - 0.0833333333333332*G5_2_0_0_1_2_1_1_1 - 0.0833333333333332*G5_2_0_0_1_2_1_1_2 - 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G5_2_0_0_2_2_1_2_2 + 0.0833333333333332*G6_0_0_0_0_0_1_0_0 + 0.0833333333333332*G6_0_0_0_0_0_1_0_1 + 0.0833333333333332*G6_0_0_0_0_0_1_0_2 + 0.0833333333333332*G6_0_0_0_1_0_1_1_0 + 0.0833333333333332*G6_0_0_0_1_0_1_1_1 + 0.0833333333333332*G6_0_0_0_1_0_1_1_2 + 0.0833333333333332*G6_0_0_0_2_0_1_2_0 + 0.0833333333333332*G6_0_0_0_2_0_1_2_1 + 0.0833333333333332*G6_0_0_0_2_0_1_2_2 + 0.0833333333333332*G6_1_0_0_0_1_1_0_0 + 0.0833333333333332*G6_1_0_0_0_1_1_0_1 + 0.0833333333333332*G6_1_0_0_0_1_1_0_2 + 0.0833333333333332*G6_1_0_0_1_1_1_1_0 + 0.0833333333333332*G6_1_0_0_1_1_1_1_1 + 0.0833333333333332*G6_1_0_0_1_1_1_1_2 + 0.0833333333333332*G6_1_0_0_2_1_1_2_0 + 0.0833333333333332*G6_1_0_0_2_1_1_2_1 + 0.0833333333333332*G6_1_0_0_2_1_1_2_2 + 0.0833333333333332*G6_2_0_0_0_2_1_0_0 + 0.0833333333333332*G6_2_0_0_0_2_1_0_1 + 0.0833333333333332*G6_2_0_0_0_2_1_0_2 + 0.0833333333333332*G6_2_0_0_1_2_1_1_0 + 0.0833333333333332*G6_2_0_0_1_2_1_1_1 + 0.0833333333333332*G6_2_0_0_1_2_1_1_2 + 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G6_2_0_0_2_2_1_2_2 - 0.0833333333333332*G7_0_0_1_2_0_2 - 0.0833333333333332*G7_0_0_1_2_1_2 - 0.0833333333333332*G7_0_0_1_2_2_2;
    A[129] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.0166666666666666*G1_0_0_0_2_0_0 + 0.00833333333333331*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.0166666666666666*G1_0_0_0_6_0_1 + 0.00833333333333331*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.0166666666666666*G1_0_0_0_10_0_2 + 0.00833333333333331*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.0166666666666666*G2_0_0_0_2_0_0 - 0.00833333333333331*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.0166666666666666*G2_0_0_0_6_0_1 - 0.00833333333333331*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.0166666666666666*G2_0_0_0_10_0_2 - 0.00833333333333331*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_1 + 0.0416666666666666*G4_0_0_0_1_2_2 + 0.0833333333333332*G5_0_0_0_0_0_1_0_0 + 0.0833333333333332*G5_0_0_0_1_0_1_1_0 + 0.0833333333333332*G5_0_0_0_2_0_1_2_0 + 0.0833333333333332*G5_1_0_0_0_1_1_0_0 + 0.0833333333333332*G5_1_0_0_1_1_1_1_0 + 0.0833333333333332*G5_1_0_0_2_1_1_2_0 + 0.0833333333333332*G5_2_0_0_0_2_1_0_0 + 0.0833333333333332*G5_2_0_0_1_2_1_1_0 + 0.0833333333333332*G5_2_0_0_2_2_1_2_0 - 0.0833333333333332*G6_0_0_0_0_0_1_0_0 - 0.0833333333333332*G6_0_0_0_1_0_1_1_0 - 0.0833333333333332*G6_0_0_0_2_0_1_2_0 - 0.0833333333333332*G6_1_0_0_0_1_1_0_0 - 0.0833333333333332*G6_1_0_0_1_1_1_1_0 - 0.0833333333333332*G6_1_0_0_2_1_1_2_0 - 0.0833333333333332*G6_2_0_0_0_2_1_0_0 - 0.0833333333333332*G6_2_0_0_1_2_1_1_0 - 0.0833333333333332*G6_2_0_0_2_2_1_2_0 + 0.0833333333333332*G7_0_0_1_2_0_2;
    A[130] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.0166666666666666*G1_0_0_0_2_1_0 + 0.00833333333333331*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.0166666666666666*G1_0_0_0_6_1_1 + 0.00833333333333331*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.0166666666666666*G1_0_0_0_10_1_2 + 0.00833333333333331*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.0166666666666666*G2_0_0_0_2_1_0 - 0.00833333333333331*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.0166666666666666*G2_0_0_0_6_1_1 - 0.00833333333333331*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.0166666666666666*G2_0_0_0_10_1_2 - 0.00833333333333331*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_1 + 0.0416666666666666*G4_0_0_1_1_2_2 + 0.0833333333333332*G5_0_0_0_0_0_1_0_1 + 0.0833333333333332*G5_0_0_0_1_0_1_1_1 + 0.0833333333333332*G5_0_0_0_2_0_1_2_1 + 0.0833333333333332*G5_1_0_0_0_1_1_0_1 + 0.0833333333333332*G5_1_0_0_1_1_1_1_1 + 0.0833333333333332*G5_1_0_0_2_1_1_2_1 + 0.0833333333333332*G5_2_0_0_0_2_1_0_1 + 0.0833333333333332*G5_2_0_0_1_2_1_1_1 + 0.0833333333333332*G5_2_0_0_2_2_1_2_1 - 0.0833333333333332*G6_0_0_0_0_0_1_0_1 - 0.0833333333333332*G6_0_0_0_1_0_1_1_1 - 0.0833333333333332*G6_0_0_0_2_0_1_2_1 - 0.0833333333333332*G6_1_0_0_0_1_1_0_1 - 0.0833333333333332*G6_1_0_0_1_1_1_1_1 - 0.0833333333333332*G6_1_0_0_2_1_1_2_1 - 0.0833333333333332*G6_2_0_0_0_2_1_0_1 - 0.0833333333333332*G6_2_0_0_1_2_1_1_1 - 0.0833333333333332*G6_2_0_0_2_2_1_2_1 + 0.0833333333333332*G7_0_0_1_2_1_2;
    A[131] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.0166666666666666*G1_0_0_0_2_2_0 + 0.00833333333333331*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.0166666666666666*G1_0_0_0_6_2_1 + 0.00833333333333331*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.0166666666666666*G1_0_0_0_10_2_2 + 0.00833333333333331*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.0166666666666666*G2_0_0_0_2_2_0 - 0.00833333333333331*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.0166666666666666*G2_0_0_0_6_2_1 - 0.00833333333333331*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.0166666666666666*G2_0_0_0_10_2_2 - 0.00833333333333331*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_1 + 0.0416666666666666*G4_0_0_2_1_2_2 + 0.0833333333333332*G5_0_0_0_0_0_1_0_2 + 0.0833333333333332*G5_0_0_0_1_0_1_1_2 + 0.0833333333333332*G5_0_0_0_2_0_1_2_2 + 0.0833333333333332*G5_1_0_0_0_1_1_0_2 + 0.0833333333333332*G5_1_0_0_1_1_1_1_2 + 0.0833333333333332*G5_1_0_0_2_1_1_2_2 + 0.0833333333333332*G5_2_0_0_0_2_1_0_2 + 0.0833333333333332*G5_2_0_0_1_2_1_1_2 + 0.0833333333333332*G5_2_0_0_2_2_1_2_2 - 0.0833333333333332*G6_0_0_0_0_0_1_0_2 - 0.0833333333333332*G6_0_0_0_1_0_1_1_2 - 0.0833333333333332*G6_0_0_0_2_0_1_2_2 - 0.0833333333333332*G6_1_0_0_0_1_1_0_2 - 0.0833333333333332*G6_1_0_0_1_1_1_1_2 - 0.0833333333333332*G6_1_0_0_2_1_1_2_2 - 0.0833333333333332*G6_2_0_0_0_2_1_0_2 - 0.0833333333333332*G6_2_0_0_1_2_1_1_2 - 0.0833333333333332*G6_2_0_0_2_2_1_2_2 + 0.0833333333333332*G7_0_0_1_2_2_2;
    A[132] = -0.0416666666666666*G4_0_0_0_2_2_0 - 0.0416666666666666*G4_0_0_1_2_2_0 - 0.0416666666666666*G4_0_0_2_2_2_0 - 0.0833333333333332*G7_0_0_2_2_0_0 - 0.0833333333333332*G7_0_0_2_2_1_0 - 0.0833333333333332*G7_0_0_2_2_2_0;
    A[133] = 0.0416666666666666*G4_0_0_0_2_2_0 + 0.0833333333333332*G7_0_0_2_2_0_0;
    A[134] = 0.0416666666666666*G4_0_0_1_2_2_0 + 0.0833333333333332*G7_0_0_2_2_1_0;
    A[135] = 0.0416666666666666*G4_0_0_2_2_2_0 + 0.0833333333333332*G7_0_0_2_2_2_0;
    A[136] = -0.0416666666666666*G4_0_0_0_2_2_1 - 0.0416666666666666*G4_0_0_1_2_2_1 - 0.0416666666666666*G4_0_0_2_2_2_1 - 0.0833333333333332*G7_0_0_2_2_0_1 - 0.0833333333333332*G7_0_0_2_2_1_1 - 0.0833333333333332*G7_0_0_2_2_2_1;
    A[137] = 0.0416666666666666*G4_0_0_0_2_2_1 + 0.0833333333333332*G7_0_0_2_2_0_1;
    A[138] = 0.0416666666666666*G4_0_0_1_2_2_1 + 0.0833333333333332*G7_0_0_2_2_1_1;
    A[139] = 0.0416666666666666*G4_0_0_2_2_2_1 + 0.0833333333333332*G7_0_0_2_2_2_1;
    A[140] = 0.00833333333333331*G0_0 - 0.00833333333333331*G1_0_0_0_0_0_0 - 0.00833333333333331*G1_0_0_0_0_1_0 - 0.00833333333333331*G1_0_0_0_0_2_0 - 0.00833333333333331*G1_0_0_0_1_0_0 - 0.00833333333333331*G1_0_0_0_1_1_0 - 0.00833333333333331*G1_0_0_0_1_2_0 - 0.00833333333333331*G1_0_0_0_2_0_0 - 0.00833333333333331*G1_0_0_0_2_1_0 - 0.00833333333333331*G1_0_0_0_2_2_0 - 0.0166666666666666*G1_0_0_0_3_0_0 - 0.0166666666666666*G1_0_0_0_3_1_0 - 0.0166666666666666*G1_0_0_0_3_2_0 - 0.00833333333333331*G1_0_0_0_4_0_1 - 0.00833333333333331*G1_0_0_0_4_1_1 - 0.00833333333333331*G1_0_0_0_4_2_1 - 0.00833333333333331*G1_0_0_0_5_0_1 - 0.00833333333333331*G1_0_0_0_5_1_1 - 0.00833333333333331*G1_0_0_0_5_2_1 - 0.00833333333333331*G1_0_0_0_6_0_1 - 0.00833333333333331*G1_0_0_0_6_1_1 - 0.00833333333333331*G1_0_0_0_6_2_1 - 0.0166666666666666*G1_0_0_0_7_0_1 - 0.0166666666666666*G1_0_0_0_7_1_1 - 0.0166666666666666*G1_0_0_0_7_2_1 - 0.00833333333333331*G1_0_0_0_8_0_2 - 0.00833333333333331*G1_0_0_0_8_1_2 - 0.00833333333333331*G1_0_0_0_8_2_2 - 0.00833333333333331*G1_0_0_0_9_0_2 - 0.00833333333333331*G1_0_0_0_9_1_2 - 0.00833333333333331*G1_0_0_0_9_2_2 - 0.00833333333333331*G1_0_0_0_10_0_2 - 0.00833333333333331*G1_0_0_0_10_1_2 - 0.00833333333333331*G1_0_0_0_10_2_2 - 0.0166666666666666*G1_0_0_0_11_0_2 - 0.0166666666666666*G1_0_0_0_11_1_2 - 0.0166666666666666*G1_0_0_0_11_2_2 + 0.00833333333333331*G2_0_0_0_0_0_0 + 0.00833333333333331*G2_0_0_0_0_1_0 + 0.00833333333333331*G2_0_0_0_0_2_0 + 0.00833333333333331*G2_0_0_0_1_0_0 + 0.00833333333333331*G2_0_0_0_1_1_0 + 0.00833333333333331*G2_0_0_0_1_2_0 + 0.00833333333333331*G2_0_0_0_2_0_0 + 0.00833333333333331*G2_0_0_0_2_1_0 + 0.00833333333333331*G2_0_0_0_2_2_0 + 0.0166666666666666*G2_0_0_0_3_0_0 + 0.0166666666666666*G2_0_0_0_3_1_0 + 0.0166666666666666*G2_0_0_0_3_2_0 + 0.00833333333333331*G2_0_0_0_4_0_1 + 0.00833333333333331*G2_0_0_0_4_1_1 + 0.00833333333333331*G2_0_0_0_4_2_1 + 0.00833333333333331*G2_0_0_0_5_0_1 + 0.00833333333333331*G2_0_0_0_5_1_1 + 0.00833333333333331*G2_0_0_0_5_2_1 + 0.00833333333333331*G2_0_0_0_6_0_1 + 0.00833333333333331*G2_0_0_0_6_1_1 + 0.00833333333333331*G2_0_0_0_6_2_1 + 0.0166666666666666*G2_0_0_0_7_0_1 + 0.0166666666666666*G2_0_0_0_7_1_1 + 0.0166666666666666*G2_0_0_0_7_2_1 + 0.00833333333333331*G2_0_0_0_8_0_2 + 0.00833333333333331*G2_0_0_0_8_1_2 + 0.00833333333333331*G2_0_0_0_8_2_2 + 0.00833333333333331*G2_0_0_0_9_0_2 + 0.00833333333333331*G2_0_0_0_9_1_2 + 0.00833333333333331*G2_0_0_0_9_2_2 + 0.00833333333333331*G2_0_0_0_10_0_2 + 0.00833333333333331*G2_0_0_0_10_1_2 + 0.00833333333333331*G2_0_0_0_10_2_2 + 0.0166666666666666*G2_0_0_0_11_0_2 + 0.0166666666666666*G2_0_0_0_11_1_2 + 0.0166666666666666*G2_0_0_0_11_2_2 - 0.0833333333333332*G3_0_0_0_2 - 0.0833333333333332*G3_0_0_1_2 - 0.0833333333333332*G3_0_0_2_2 - 0.0416666666666666*G4_0_0_0_2_2_2 - 0.0416666666666666*G4_0_0_1_2_2_2 - 0.0416666666666666*G4_0_0_2_2_2_2 - 0.0833333333333332*G5_0_0_0_0_0_2_0_0 - 0.0833333333333332*G5_0_0_0_0_0_2_0_1 - 0.0833333333333332*G5_0_0_0_0_0_2_0_2 - 0.0833333333333332*G5_0_0_0_1_0_2_1_0 - 0.0833333333333332*G5_0_0_0_1_0_2_1_1 - 0.0833333333333332*G5_0_0_0_1_0_2_1_2 - 0.0833333333333332*G5_0_0_0_2_0_2_2_0 - 0.0833333333333332*G5_0_0_0_2_0_2_2_1 - 0.0833333333333332*G5_0_0_0_2_0_2_2_2 - 0.0833333333333332*G5_1_0_0_0_1_2_0_0 - 0.0833333333333332*G5_1_0_0_0_1_2_0_1 - 0.0833333333333332*G5_1_0_0_0_1_2_0_2 - 0.0833333333333332*G5_1_0_0_1_1_2_1_0 - 0.0833333333333332*G5_1_0_0_1_1_2_1_1 - 0.0833333333333332*G5_1_0_0_1_1_2_1_2 - 0.0833333333333332*G5_1_0_0_2_1_2_2_0 - 0.0833333333333332*G5_1_0_0_2_1_2_2_1 - 0.0833333333333332*G5_1_0_0_2_1_2_2_2 - 0.0833333333333332*G5_2_0_0_0_2_2_0_0 - 0.0833333333333332*G5_2_0_0_0_2_2_0_1 - 0.0833333333333332*G5_2_0_0_0_2_2_0_2 - 0.0833333333333332*G5_2_0_0_1_2_2_1_0 - 0.0833333333333332*G5_2_0_0_1_2_2_1_1 - 0.0833333333333332*G5_2_0_0_1_2_2_1_2 - 0.0833333333333332*G5_2_0_0_2_2_2_2_0 - 0.0833333333333332*G5_2_0_0_2_2_2_2_1 - 0.0833333333333332*G5_2_0_0_2_2_2_2_2 + 0.0833333333333332*G6_0_0_0_0_0_2_0_0 + 0.0833333333333332*G6_0_0_0_0_0_2_0_1 + 0.0833333333333332*G6_0_0_0_0_0_2_0_2 + 0.0833333333333332*G6_0_0_0_1_0_2_1_0 + 0.0833333333333332*G6_0_0_0_1_0_2_1_1 + 0.0833333333333332*G6_0_0_0_1_0_2_1_2 + 0.0833333333333332*G6_0_0_0_2_0_2_2_0 + 0.0833333333333332*G6_0_0_0_2_0_2_2_1 + 0.0833333333333332*G6_0_0_0_2_0_2_2_2 + 0.0833333333333332*G6_1_0_0_0_1_2_0_0 + 0.0833333333333332*G6_1_0_0_0_1_2_0_1 + 0.0833333333333332*G6_1_0_0_0_1_2_0_2 + 0.0833333333333332*G6_1_0_0_1_1_2_1_0 + 0.0833333333333332*G6_1_0_0_1_1_2_1_1 + 0.0833333333333332*G6_1_0_0_1_1_2_1_2 + 0.0833333333333332*G6_1_0_0_2_1_2_2_0 + 0.0833333333333332*G6_1_0_0_2_1_2_2_1 + 0.0833333333333332*G6_1_0_0_2_1_2_2_2 + 0.0833333333333332*G6_2_0_0_0_2_2_0_0 + 0.0833333333333332*G6_2_0_0_0_2_2_0_1 + 0.0833333333333332*G6_2_0_0_0_2_2_0_2 + 0.0833333333333332*G6_2_0_0_1_2_2_1_0 + 0.0833333333333332*G6_2_0_0_1_2_2_1_1 + 0.0833333333333332*G6_2_0_0_1_2_2_1_2 + 0.0833333333333332*G6_2_0_0_2_2_2_2_0 + 0.0833333333333332*G6_2_0_0_2_2_2_2_1 + 0.0833333333333332*G6_2_0_0_2_2_2_2_2 - 0.0833333333333332*G7_0_0_2_2_0_2 - 0.0833333333333332*G7_0_0_2_2_1_2 - 0.0833333333333332*G7_0_0_2_2_2_2;
    A[141] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_0_0 + 0.00833333333333331*G1_0_0_0_1_0_0 + 0.00833333333333331*G1_0_0_0_2_0_0 + 0.0166666666666666*G1_0_0_0_3_0_0 + 0.00833333333333331*G1_0_0_0_4_0_1 + 0.00833333333333331*G1_0_0_0_5_0_1 + 0.00833333333333331*G1_0_0_0_6_0_1 + 0.0166666666666666*G1_0_0_0_7_0_1 + 0.00833333333333331*G1_0_0_0_8_0_2 + 0.00833333333333331*G1_0_0_0_9_0_2 + 0.00833333333333331*G1_0_0_0_10_0_2 + 0.0166666666666666*G1_0_0_0_11_0_2 - 0.00833333333333331*G2_0_0_0_0_0_0 - 0.00833333333333331*G2_0_0_0_1_0_0 - 0.00833333333333331*G2_0_0_0_2_0_0 - 0.0166666666666666*G2_0_0_0_3_0_0 - 0.00833333333333331*G2_0_0_0_4_0_1 - 0.00833333333333331*G2_0_0_0_5_0_1 - 0.00833333333333331*G2_0_0_0_6_0_1 - 0.0166666666666666*G2_0_0_0_7_0_1 - 0.00833333333333331*G2_0_0_0_8_0_2 - 0.00833333333333331*G2_0_0_0_9_0_2 - 0.00833333333333331*G2_0_0_0_10_0_2 - 0.0166666666666666*G2_0_0_0_11_0_2 + 0.0833333333333332*G3_0_0_0_2 + 0.0416666666666666*G4_0_0_0_2_2_2 + 0.0833333333333332*G5_0_0_0_0_0_2_0_0 + 0.0833333333333332*G5_0_0_0_1_0_2_1_0 + 0.0833333333333332*G5_0_0_0_2_0_2_2_0 + 0.0833333333333332*G5_1_0_0_0_1_2_0_0 + 0.0833333333333332*G5_1_0_0_1_1_2_1_0 + 0.0833333333333332*G5_1_0_0_2_1_2_2_0 + 0.0833333333333332*G5_2_0_0_0_2_2_0_0 + 0.0833333333333332*G5_2_0_0_1_2_2_1_0 + 0.0833333333333332*G5_2_0_0_2_2_2_2_0 - 0.0833333333333332*G6_0_0_0_0_0_2_0_0 - 0.0833333333333332*G6_0_0_0_1_0_2_1_0 - 0.0833333333333332*G6_0_0_0_2_0_2_2_0 - 0.0833333333333332*G6_1_0_0_0_1_2_0_0 - 0.0833333333333332*G6_1_0_0_1_1_2_1_0 - 0.0833333333333332*G6_1_0_0_2_1_2_2_0 - 0.0833333333333332*G6_2_0_0_0_2_2_0_0 - 0.0833333333333332*G6_2_0_0_1_2_2_1_0 - 0.0833333333333332*G6_2_0_0_2_2_2_2_0 + 0.0833333333333332*G7_0_0_2_2_0_2;
    A[142] = 0.00833333333333331*G0_0 + 0.00833333333333331*G1_0_0_0_0_1_0 + 0.00833333333333331*G1_0_0_0_1_1_0 + 0.00833333333333331*G1_0_0_0_2_1_0 + 0.0166666666666666*G1_0_0_0_3_1_0 + 0.00833333333333331*G1_0_0_0_4_1_1 + 0.00833333333333331*G1_0_0_0_5_1_1 + 0.00833333333333331*G1_0_0_0_6_1_1 + 0.0166666666666666*G1_0_0_0_7_1_1 + 0.00833333333333331*G1_0_0_0_8_1_2 + 0.00833333333333331*G1_0_0_0_9_1_2 + 0.00833333333333331*G1_0_0_0_10_1_2 + 0.0166666666666666*G1_0_0_0_11_1_2 - 0.00833333333333331*G2_0_0_0_0_1_0 - 0.00833333333333331*G2_0_0_0_1_1_0 - 0.00833333333333331*G2_0_0_0_2_1_0 - 0.0166666666666666*G2_0_0_0_3_1_0 - 0.00833333333333331*G2_0_0_0_4_1_1 - 0.00833333333333331*G2_0_0_0_5_1_1 - 0.00833333333333331*G2_0_0_0_6_1_1 - 0.0166666666666666*G2_0_0_0_7_1_1 - 0.00833333333333331*G2_0_0_0_8_1_2 - 0.00833333333333331*G2_0_0_0_9_1_2 - 0.00833333333333331*G2_0_0_0_10_1_2 - 0.0166666666666666*G2_0_0_0_11_1_2 + 0.0833333333333332*G3_0_0_1_2 + 0.0416666666666666*G4_0_0_1_2_2_2 + 0.0833333333333332*G5_0_0_0_0_0_2_0_1 + 0.0833333333333332*G5_0_0_0_1_0_2_1_1 + 0.0833333333333332*G5_0_0_0_2_0_2_2_1 + 0.0833333333333332*G5_1_0_0_0_1_2_0_1 + 0.0833333333333332*G5_1_0_0_1_1_2_1_1 + 0.0833333333333332*G5_1_0_0_2_1_2_2_1 + 0.0833333333333332*G5_2_0_0_0_2_2_0_1 + 0.0833333333333332*G5_2_0_0_1_2_2_1_1 + 0.0833333333333332*G5_2_0_0_2_2_2_2_1 - 0.0833333333333332*G6_0_0_0_0_0_2_0_1 - 0.0833333333333332*G6_0_0_0_1_0_2_1_1 - 0.0833333333333332*G6_0_0_0_2_0_2_2_1 - 0.0833333333333332*G6_1_0_0_0_1_2_0_1 - 0.0833333333333332*G6_1_0_0_1_1_2_1_1 - 0.0833333333333332*G6_1_0_0_2_1_2_2_1 - 0.0833333333333332*G6_2_0_0_0_2_2_0_1 - 0.0833333333333332*G6_2_0_0_1_2_2_1_1 - 0.0833333333333332*G6_2_0_0_2_2_2_2_1 + 0.0833333333333332*G7_0_0_2_2_1_2;
    A[143] = 0.0166666666666666*G0_0 + 0.00833333333333331*G1_0_0_0_0_2_0 + 0.00833333333333331*G1_0_0_0_1_2_0 + 0.00833333333333331*G1_0_0_0_2_2_0 + 0.0166666666666666*G1_0_0_0_3_2_0 + 0.00833333333333331*G1_0_0_0_4_2_1 + 0.00833333333333331*G1_0_0_0_5_2_1 + 0.00833333333333331*G1_0_0_0_6_2_1 + 0.0166666666666666*G1_0_0_0_7_2_1 + 0.00833333333333331*G1_0_0_0_8_2_2 + 0.00833333333333331*G1_0_0_0_9_2_2 + 0.00833333333333331*G1_0_0_0_10_2_2 + 0.0166666666666666*G1_0_0_0_11_2_2 - 0.00833333333333331*G2_0_0_0_0_2_0 - 0.00833333333333331*G2_0_0_0_1_2_0 - 0.00833333333333331*G2_0_0_0_2_2_0 - 0.0166666666666666*G2_0_0_0_3_2_0 - 0.00833333333333331*G2_0_0_0_4_2_1 - 0.00833333333333331*G2_0_0_0_5_2_1 - 0.00833333333333331*G2_0_0_0_6_2_1 - 0.0166666666666666*G2_0_0_0_7_2_1 - 0.00833333333333331*G2_0_0_0_8_2_2 - 0.00833333333333331*G2_0_0_0_9_2_2 - 0.00833333333333331*G2_0_0_0_10_2_2 - 0.0166666666666666*G2_0_0_0_11_2_2 + 0.0833333333333332*G3_0_0_2_2 + 0.0416666666666666*G4_0_0_2_2_2_2 + 0.0833333333333332*G5_0_0_0_0_0_2_0_2 + 0.0833333333333332*G5_0_0_0_1_0_2_1_2 + 0.0833333333333332*G5_0_0_0_2_0_2_2_2 + 0.0833333333333332*G5_1_0_0_0_1_2_0_2 + 0.0833333333333332*G5_1_0_0_1_1_2_1_2 + 0.0833333333333332*G5_1_0_0_2_1_2_2_2 + 0.0833333333333332*G5_2_0_0_0_2_2_0_2 + 0.0833333333333332*G5_2_0_0_1_2_2_1_2 + 0.0833333333333332*G5_2_0_0_2_2_2_2_2 - 0.0833333333333332*G6_0_0_0_0_0_2_0_2 - 0.0833333333333332*G6_0_0_0_1_0_2_1_2 - 0.0833333333333332*G6_0_0_0_2_0_2_2_2 - 0.0833333333333332*G6_1_0_0_0_1_2_0_2 - 0.0833333333333332*G6_1_0_0_1_1_2_1_2 - 0.0833333333333332*G6_1_0_0_2_1_2_2_2 - 0.0833333333333332*G6_2_0_0_0_2_2_0_2 - 0.0833333333333332*G6_2_0_0_1_2_2_1_2 - 0.0833333333333332*G6_2_0_0_2_2_2_2_2 + 0.0833333333333332*G7_0_0_2_2_2_2;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class UFC_FSIMomentum3DBilinearForm: public ufc::form
{
public:

  /// Constructor
  UFC_FSIMomentum3DBilinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DBilinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w7_a0 | va0*vi1[b0]*vi0[b0]*dX(0) + w9_a0w7_a1w6_a2w0_a3(dXa4/dxa5) | va0*va1*va2*va3[a5]*((d/dXa4)vi1[b0])*vi0[b0]*dX(0) + -w9_a0w7_a1w6_a2w4_a3(dXa4/dxa5) | va0*va1*va2*va3[a5]*((d/dXa4)vi1[b0])*vi0[b0]*dX(0) + 0.5w9_a0w8_a1(dXa2/dxb0)(dXa3/dxb0) | va0*va1*((d/dXa2)vi1[b0])*((d/dXa3)vi0[b0])*dX(0) + 0.25w9_a0w8_a1(dXa2/dxa4)(dXa3/dxa5) | va0*va1*((d/dXa2)vi1[a5])*((d/dXa3)vi0[a4])*dX(0) + 0.25w9_a0w8_a1(dXa2/dxa5)(dXa3/dxa4) | va0*va1*((d/dXa2)vi1[a4])*((d/dXa3)vi0[a5])*dX(0) + 0.5w9_a1w2_a2w1_a3w1_a4(dXa5/dxa6)(dXa7/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa7)vi1[b0])*dX(0) + -0.5w9_a1w2_a2w1_a3w5_a4(dXa5/dxa6)(dXa7/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa7)vi1[b0])*dX(0) + -0.5w9_a1w2_a2w5_a3w1_a4(dXa5/dxa6)(dXa7/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa7)vi1[b0])*dX(0) + 0.5w9_a1w2_a2w5_a3w5_a4(dXa5/dxa6)(dXa7/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa7)vi1[b0])*dX(0) + 0.5w9_a0w3_a1(dXa2/dxa3)(dXa4/dxa5) | va0*va1*((d/dXa2)vi0[a3])*((d/dXa4)vi1[a5])*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 10;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_2();
      break;
    case 3:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_3();
      break;
    case 4:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_4();
      break;
    case 5:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_5();
      break;
    case 6:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_6();
      break;
    case 7:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_7();
      break;
    case 8:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_8();
      break;
    case 9:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_9();
      break;
    case 10:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_10();
      break;
    case 11:
      return new UFC_FSIMomentum3DBilinearForm_finite_element_11();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_2();
      break;
    case 3:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_3();
      break;
    case 4:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_4();
      break;
    case 5:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_5();
      break;
    case 6:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_6();
      break;
    case 7:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_7();
      break;
    case 8:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_8();
      break;
    case 9:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_9();
      break;
    case 10:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_10();
      break;
    case 11:
      return new UFC_FSIMomentum3DBilinearForm_dof_map_11();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new UFC_FSIMomentum3DBilinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_0_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_0_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_0_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_0_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_0_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_0_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_0_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_0_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_0_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_0_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_0_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_0_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_0_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_1_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_1_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_1_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_1_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_1_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_1_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_1_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_1_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_1_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_1_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[3][3] = {{0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 1, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[1];
    vertex_values[5] = dof_values[1];
    vertex_values[6] = dof_values[1];
    vertex_values[7] = dof_values[1];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[2];
    vertex_values[9] = dof_values[2];
    vertex_values[10] = dof_values[2];
    vertex_values[11] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_1_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_1_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_1_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_2_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_2_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_2_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_2_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_2_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_2_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_2_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_2_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_2_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_2_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_2_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_2_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_2_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_2_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_2_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_3_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_3_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_3_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_3_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_3_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_3_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_3_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_3_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_3_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_3_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_3_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_3_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_3: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_3() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_3()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 12;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 11)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
      {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
      {0.288675134594813, 0, 0, 0.223606797749979}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532033676, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {5.47722557505166, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766016838, 0, 0, 0},
      {1.82574185835055, 0, 0, 0},
      {5.16397779494322, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[12][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[12] = {0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[4];
    vertex_values[5] = dof_values[5];
    vertex_values[6] = dof_values[6];
    vertex_values[7] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[8];
    vertex_values[9] = dof_values[9];
    vertex_values[10] = dof_values[10];
    vertex_values[11] = dof_values[11];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_3_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_3_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_3_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_4: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_4() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_4()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.288675134594813, -0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0.182574185835055, -0.105409255338946, -0.074535599249993},
    {0.288675134594813, 0, 0.210818510677892, -0.074535599249993},
    {0.288675134594813, 0, 0, 0.223606797749979}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532033676, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {5.47722557505166, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0},
    {1.82574185835055, 0, 0, 0},
    {5.16397779494322, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[4][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
    
    // Components for each dof
    static unsigned int components[4] = {0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_4();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_5: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_5() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_5()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_5();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_6: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_6() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_6()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_6();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_7_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_7_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_7_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_7_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_7_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_7_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_7_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_7_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_7_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_7_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_7_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_7_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_7: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_7() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_7()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[3];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[3][3] = {{0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 1, 2};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[1];
    vertex_values[5] = dof_values[1];
    vertex_values[6] = dof_values[1];
    vertex_values[7] = dof_values[1];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[2];
    vertex_values[9] = dof_values[2];
    vertex_values[10] = dof_values[2];
    vertex_values[11] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_7_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_7_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_7_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_8: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_8() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_8()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_8();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_9: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_9() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_9()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_9();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_1();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_2: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_2();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_3: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_3() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_3()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_3();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_4: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_4() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_4()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_4();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_5: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_5() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_5()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_5();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_6: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_6() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_6()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_6();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_7: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_7() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_7()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_7();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10_8: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10_8() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10_8()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_10_8();
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_10: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_10() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_10()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 9;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 9;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    values[3] = 0;
    values[4] = 0;
    values[5] = 0;
    values[6] = 0;
    values[7] = 0;
    values[8] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0;
    }
    
    if (3 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 3;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[3] = coeff0_0*basisvalue0;
    }
    
    if (4 <= i && i <= 4)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[4] = coeff0_0*basisvalue0;
    }
    
    if (5 <= i && i <= 5)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 5;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[5] = coeff0_0*basisvalue0;
    }
    
    if (6 <= i && i <= 6)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 6;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[6] = coeff0_0*basisvalue0;
    }
    
    if (7 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 7;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[7] = coeff0_0*basisvalue0;
    }
    
    if (8 <= i && i <= 8)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[8] = coeff0_0*basisvalue0;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 9*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (1 <= i && i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (2 <= i && i <= 2)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 2;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (3 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 3;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[3*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 4)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[4*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (5 <= i && i <= 5)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 5;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[5*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (6 <= i && i <= 6)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 6;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[6*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (7 <= i && i <= 7)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 7;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[7*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (8 <= i && i <= 8)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 8;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_z_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.15470053837925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      const static double dmats2[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[8*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[9];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[9][3] = {{0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}, {0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[9] = {0, 1, 2, 3, 4, 5, 6, 7, 8};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[1];
    vertex_values[5] = dof_values[1];
    vertex_values[6] = dof_values[1];
    vertex_values[7] = dof_values[1];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[2];
    vertex_values[9] = dof_values[2];
    vertex_values[10] = dof_values[2];
    vertex_values[11] = dof_values[2];
    // Evaluate at vertices and use affine mapping
    vertex_values[12] = dof_values[3];
    vertex_values[13] = dof_values[3];
    vertex_values[14] = dof_values[3];
    vertex_values[15] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[16] = dof_values[4];
    vertex_values[17] = dof_values[4];
    vertex_values[18] = dof_values[4];
    vertex_values[19] = dof_values[4];
    // Evaluate at vertices and use affine mapping
    vertex_values[20] = dof_values[5];
    vertex_values[21] = dof_values[5];
    vertex_values[22] = dof_values[5];
    vertex_values[23] = dof_values[5];
    // Evaluate at vertices and use affine mapping
    vertex_values[24] = dof_values[6];
    vertex_values[25] = dof_values[6];
    vertex_values[26] = dof_values[6];
    vertex_values[27] = dof_values[6];
    // Evaluate at vertices and use affine mapping
    vertex_values[28] = dof_values[7];
    vertex_values[29] = dof_values[7];
    vertex_values[30] = dof_values[7];
    vertex_values[31] = dof_values[7];
    // Evaluate at vertices and use affine mapping
    vertex_values[32] = dof_values[8];
    vertex_values[33] = dof_values[8];
    vertex_values[34] = dof_values[8];
    vertex_values[35] = dof_values[8];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 9;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_2();
      break;
    case 3:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_3();
      break;
    case 4:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_4();
      break;
    case 5:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_5();
      break;
    case 6:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_6();
      break;
    case 7:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_7();
      break;
    case 8:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10_8();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_FSIMomentum3DLinearForm_finite_element_11: public ufc::finite_element
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_finite_element_11() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_finite_element_11()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_z_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.15470053837925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    const static double dmats2[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[1][3] = {{0.25, 0.25, 0.25}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    vertex_values[3] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_finite_element_11();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_0_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_0_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_0_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_0_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_0_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_0_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_0_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_0_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_0_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_0_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_0_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_0_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_0_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_1_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_1_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_1_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_1_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_1_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_1_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_1_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_1_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_1_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_1_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
    unsigned int offset = m.num_entities[3];
    dofs[1] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[2] = offset + c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[1][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[1][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[1][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[2][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[2][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[2][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_1_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_1_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_1_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_2_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_2_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_2_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_2_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_2_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_2_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_2_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_2_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_2_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_2_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_2_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_2_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_2_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_2_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_2_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_3_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_3_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_3_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_3_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_3_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_3_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_3_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_3_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_3_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_3_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_3_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_3_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_3: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_3() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_3()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron, Lagrange finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 12;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 9;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[0][0];
    dofs[5] = offset + c.entity_indices[0][1];
    dofs[6] = offset + c.entity_indices[0][2];
    dofs[7] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[8] = offset + c.entity_indices[0][0];
    dofs[9] = offset + c.entity_indices[0][1];
    dofs[10] = offset + c.entity_indices[0][2];
    dofs[11] = offset + c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 9;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 6;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 10;
      dofs[8] = 11;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 7;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 11;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 8;
      dofs[7] = 9;
      dofs[8] = 10;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = x[0][0];
    coordinates[4][1] = x[0][1];
    coordinates[4][2] = x[0][2];
    coordinates[5][0] = x[1][0];
    coordinates[5][1] = x[1][1];
    coordinates[5][2] = x[1][2];
    coordinates[6][0] = x[2][0];
    coordinates[6][1] = x[2][1];
    coordinates[6][2] = x[2][2];
    coordinates[7][0] = x[3][0];
    coordinates[7][1] = x[3][1];
    coordinates[7][2] = x[3][2];
    coordinates[8][0] = x[0][0];
    coordinates[8][1] = x[0][1];
    coordinates[8][2] = x[0][2];
    coordinates[9][0] = x[1][0];
    coordinates[9][1] = x[1][1];
    coordinates[9][2] = x[1][2];
    coordinates[10][0] = x[2][0];
    coordinates[10][1] = x[2][1];
    coordinates[10][2] = x[2][2];
    coordinates[11][0] = x[3][0];
    coordinates[11][1] = x[3][1];
    coordinates[11][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_3_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_3_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_3_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_4: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_4() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_4()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_4();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_5: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_5() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_5()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_5();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_6: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_6() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_6()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_6();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_7_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_7_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_7_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_7_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_7_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_7_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_7_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_7_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_7_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_7_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_7_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_7_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_7: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_7() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_7()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
    unsigned int offset = m.num_entities[3];
    dofs[1] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[2] = offset + c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[1][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[1][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[1][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[2][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[2][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[2][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_7_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_7_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_7_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_8: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_8() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_8()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_8();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_9: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_9() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_9()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_9();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_3: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_3() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_3()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_3();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_4: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_4() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_4()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_4();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_5: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_5() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_5()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_5();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_6: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_6() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_6()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_6();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_7: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_7() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_7()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_7();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10_8: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10_8() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10_8()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_10_8();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_10: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_10() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_10()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron, Discontinuous Lagrange finite element of degree 0 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 9*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 9;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
    unsigned int offset = m.num_entities[3];
    dofs[1] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[2] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[3] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[4] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[5] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[6] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[7] = offset + c.entity_indices[3][0];
    offset = offset + m.num_entities[3];
    dofs[8] = offset + c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[1][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[1][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[1][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[2][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[2][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[2][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[3][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[3][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[3][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[4][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[4][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[4][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[5][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[5][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[5][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[6][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[6][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[6][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[7][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[7][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[7][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
    coordinates[8][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[8][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[8][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 9;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_2();
      break;
    case 3:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_3();
      break;
    case 4:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_4();
      break;
    case 5:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_5();
      break;
    case 6:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_6();
      break;
    case 7:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_7();
      break;
    case 8:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10_8();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_FSIMomentum3DLinearForm_dof_map_11: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_dof_map_11() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_dof_map_11()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[3][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    case 3:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0] + 0.25*x[3][0];
    coordinates[0][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1] + 0.25*x[3][1];
    coordinates[0][2] = 0.25*x[0][2] + 0.25*x[1][2] + 0.25*x[2][2] + 0.25*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_dof_map_11();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class UFC_FSIMomentum3DLinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute coefficients
    const double c8_0_0_0 = w[8][0];
    const double c1_0_1_0 = w[1][0];
    const double c1_0_1_1 = w[1][1];
    const double c1_0_1_2 = w[1][2];
    const double c1_0_1_3 = w[1][3];
    const double c1_0_1_4 = w[1][4];
    const double c1_0_1_5 = w[1][5];
    const double c1_0_1_6 = w[1][6];
    const double c1_0_1_7 = w[1][7];
    const double c1_0_1_8 = w[1][8];
    const double c1_0_1_9 = w[1][9];
    const double c1_0_1_10 = w[1][10];
    const double c1_0_1_11 = w[1][11];
    const double c10_1_0_0 = w[10][0];
    const double c2_1_1_0 = w[2][0];
    const double c2_1_1_1 = w[2][1];
    const double c2_1_1_2 = w[2][2];
    const double c2_1_1_3 = w[2][3];
    const double c2_1_1_4 = w[2][4];
    const double c2_1_1_5 = w[2][5];
    const double c2_1_1_6 = w[2][6];
    const double c2_1_1_7 = w[2][7];
    const double c2_1_1_8 = w[2][8];
    const double c2_1_1_9 = w[2][9];
    const double c2_1_1_10 = w[2][10];
    const double c2_1_1_11 = w[2][11];
    const double c10_2_0_0 = w[10][0];
    const double c7_2_1_0 = w[7][0];
    const double c9_2_2_0 = w[9][0];
    const double c9_2_2_1 = w[9][1];
    const double c9_2_2_2 = w[9][2];
    const double c10_3_0_0 = w[10][0];
    const double c7_3_1_0 = w[7][0];
    const double c9_3_2_3 = w[9][3];
    const double c10_4_0_0 = w[10][0];
    const double c7_4_1_0 = w[7][0];
    const double c9_4_2_6 = w[9][6];
    const double c10_5_0_0 = w[10][0];
    const double c7_5_1_0 = w[7][0];
    const double c9_5_2_4 = w[9][4];
    const double c10_6_0_0 = w[10][0];
    const double c7_6_1_0 = w[7][0];
    const double c9_6_2_7 = w[9][7];
    const double c10_7_0_0 = w[10][0];
    const double c7_7_1_0 = w[7][0];
    const double c9_7_2_5 = w[9][5];
    const double c10_8_0_0 = w[10][0];
    const double c7_8_1_0 = w[7][0];
    const double c9_8_2_8 = w[9][8];
    const double c10_9_0_0 = w[10][0];
    const double c9_9_1_0 = w[9][0];
    const double c9_9_1_1 = w[9][1];
    const double c9_9_1_2 = w[9][2];
    const double c10_10_0_0 = w[10][0];
    const double c9_10_1_3 = w[9][3];
    const double c10_11_0_0 = w[10][0];
    const double c9_11_1_6 = w[9][6];
    const double c10_12_0_0 = w[10][0];
    const double c9_12_1_4 = w[9][4];
    const double c10_13_0_0 = w[10][0];
    const double c9_13_1_7 = w[9][7];
    const double c10_14_0_0 = w[10][0];
    const double c9_14_1_5 = w[9][5];
    const double c10_15_0_0 = w[10][0];
    const double c9_15_1_8 = w[9][8];
    const double c10_16_0_0 = w[10][0];
    const double c3_16_1_0 = w[3][0];
    const double c3_16_1_1 = w[3][1];
    const double c3_16_1_2 = w[3][2];
    const double c3_16_1_3 = w[3][3];
    const double c10_17_0_0 = w[10][0];
    const double c4_17_1_0 = w[4][0];
    const double c0_17_2_0 = w[0][0];
    const double c0_17_2_1 = w[0][1];
    const double c0_17_2_2 = w[0][2];
    const double c2_17_3_0 = w[2][0];
    const double c2_17_3_1 = w[2][1];
    const double c2_17_3_2 = w[2][2];
    const double c2_17_3_3 = w[2][3];
    const double c2_17_3_4 = w[2][4];
    const double c2_17_3_5 = w[2][5];
    const double c2_17_3_6 = w[2][6];
    const double c2_17_3_7 = w[2][7];
    const double c2_17_3_8 = w[2][8];
    const double c2_17_3_9 = w[2][9];
    const double c2_17_3_10 = w[2][10];
    const double c2_17_3_11 = w[2][11];
    const double c10_18_0_0 = w[10][0];
    const double c4_18_1_0 = w[4][0];
    const double c6_18_2_0 = w[6][0];
    const double c6_18_2_1 = w[6][1];
    const double c6_18_2_2 = w[6][2];
    const double c2_18_3_0 = w[2][0];
    const double c2_18_3_1 = w[2][1];
    const double c2_18_3_2 = w[2][2];
    const double c2_18_3_3 = w[2][3];
    const double c2_18_3_4 = w[2][4];
    const double c2_18_3_5 = w[2][5];
    const double c2_18_3_6 = w[2][6];
    const double c2_18_3_7 = w[2][7];
    const double c2_18_3_8 = w[2][8];
    const double c2_18_3_9 = w[2][9];
    const double c2_18_3_10 = w[2][10];
    const double c2_18_3_11 = w[2][11];
    const double c10_19_0_0 = w[10][0];
    const double c4_19_1_0 = w[4][0];
    const double c0_19_2_0 = w[0][0];
    const double c0_19_2_1 = w[0][1];
    const double c0_19_2_2 = w[0][2];
    const double c0_19_3_0 = w[0][0];
    const double c0_19_3_1 = w[0][1];
    const double c0_19_3_2 = w[0][2];
    const double c1_19_4_0 = w[1][0];
    const double c1_19_4_1 = w[1][1];
    const double c1_19_4_2 = w[1][2];
    const double c1_19_4_3 = w[1][3];
    const double c1_19_4_4 = w[1][4];
    const double c1_19_4_5 = w[1][5];
    const double c1_19_4_6 = w[1][6];
    const double c1_19_4_7 = w[1][7];
    const double c1_19_4_8 = w[1][8];
    const double c1_19_4_9 = w[1][9];
    const double c1_19_4_10 = w[1][10];
    const double c1_19_4_11 = w[1][11];
    const double c10_20_0_0 = w[10][0];
    const double c4_20_1_0 = w[4][0];
    const double c6_20_2_0 = w[6][0];
    const double c6_20_2_1 = w[6][1];
    const double c6_20_2_2 = w[6][2];
    const double c6_20_3_0 = w[6][0];
    const double c6_20_3_1 = w[6][1];
    const double c6_20_3_2 = w[6][2];
    const double c1_20_4_0 = w[1][0];
    const double c1_20_4_1 = w[1][1];
    const double c1_20_4_2 = w[1][2];
    const double c1_20_4_3 = w[1][3];
    const double c1_20_4_4 = w[1][4];
    const double c1_20_4_5 = w[1][5];
    const double c1_20_4_6 = w[1][6];
    const double c1_20_4_7 = w[1][7];
    const double c1_20_4_8 = w[1][8];
    const double c1_20_4_9 = w[1][9];
    const double c1_20_4_10 = w[1][10];
    const double c1_20_4_11 = w[1][11];
    const double c10_21_0_0 = w[10][0];
    const double c4_21_1_0 = w[4][0];
    const double c0_21_2_0 = w[0][0];
    const double c0_21_2_1 = w[0][1];
    const double c0_21_2_2 = w[0][2];
    const double c6_21_3_0 = w[6][0];
    const double c6_21_3_1 = w[6][1];
    const double c6_21_3_2 = w[6][2];
    const double c1_21_4_0 = w[1][0];
    const double c1_21_4_1 = w[1][1];
    const double c1_21_4_2 = w[1][2];
    const double c1_21_4_3 = w[1][3];
    const double c1_21_4_4 = w[1][4];
    const double c1_21_4_5 = w[1][5];
    const double c1_21_4_6 = w[1][6];
    const double c1_21_4_7 = w[1][7];
    const double c1_21_4_8 = w[1][8];
    const double c1_21_4_9 = w[1][9];
    const double c1_21_4_10 = w[1][10];
    const double c1_21_4_11 = w[1][11];
    const double c10_22_0_0 = w[10][0];
    const double c4_22_1_0 = w[4][0];
    const double c6_22_2_0 = w[6][0];
    const double c6_22_2_1 = w[6][1];
    const double c6_22_2_2 = w[6][2];
    const double c0_22_3_0 = w[0][0];
    const double c0_22_3_1 = w[0][1];
    const double c0_22_3_2 = w[0][2];
    const double c1_22_4_0 = w[1][0];
    const double c1_22_4_1 = w[1][1];
    const double c1_22_4_2 = w[1][2];
    const double c1_22_4_3 = w[1][3];
    const double c1_22_4_4 = w[1][4];
    const double c1_22_4_5 = w[1][5];
    const double c1_22_4_6 = w[1][6];
    const double c1_22_4_7 = w[1][7];
    const double c1_22_4_8 = w[1][8];
    const double c1_22_4_9 = w[1][9];
    const double c1_22_4_10 = w[1][10];
    const double c1_22_4_11 = w[1][11];
    const double c10_23_0_0 = w[10][0];
    const double c4_23_1_0 = w[4][0];
    const double c0_23_2_0 = w[0][0];
    const double c0_23_2_1 = w[0][1];
    const double c0_23_2_2 = w[0][2];
    const double c3_23_3_0 = w[3][0];
    const double c3_23_3_1 = w[3][1];
    const double c3_23_3_2 = w[3][2];
    const double c3_23_3_3 = w[3][3];
    const double c10_24_0_0 = w[10][0];
    const double c4_24_1_0 = w[4][0];
    const double c6_24_2_0 = w[6][0];
    const double c6_24_2_1 = w[6][1];
    const double c6_24_2_2 = w[6][2];
    const double c3_24_3_0 = w[3][0];
    const double c3_24_3_1 = w[3][1];
    const double c3_24_3_2 = w[3][2];
    const double c3_24_3_3 = w[3][3];
    const double c10_25_0_0 = w[10][0];
    const double c5_25_1_0 = w[5][0];
    const double c1_25_2_0 = w[1][0];
    const double c1_25_2_1 = w[1][1];
    const double c1_25_2_2 = w[1][2];
    const double c1_25_2_3 = w[1][3];
    const double c1_25_2_4 = w[1][4];
    const double c1_25_2_5 = w[1][5];
    const double c1_25_2_6 = w[1][6];
    const double c1_25_2_7 = w[1][7];
    const double c1_25_2_8 = w[1][8];
    const double c1_25_2_9 = w[1][9];
    const double c1_25_2_10 = w[1][10];
    const double c1_25_2_11 = w[1][11];
    
    // Compute geometry tensors
    const double G0_0_0 = det*c8_0_0_0*c1_0_1_0;
    const double G0_0_1 = det*c8_0_0_0*c1_0_1_1;
    const double G0_0_2 = det*c8_0_0_0*c1_0_1_2;
    const double G0_0_3 = det*c8_0_0_0*c1_0_1_3;
    const double G0_0_4 = det*c8_0_0_0*c1_0_1_4;
    const double G0_0_5 = det*c8_0_0_0*c1_0_1_5;
    const double G0_0_6 = det*c8_0_0_0*c1_0_1_6;
    const double G0_0_7 = det*c8_0_0_0*c1_0_1_7;
    const double G0_0_8 = det*c8_0_0_0*c1_0_1_8;
    const double G0_0_9 = det*c8_0_0_0*c1_0_1_9;
    const double G0_0_10 = det*c8_0_0_0*c1_0_1_10;
    const double G0_0_11 = det*c8_0_0_0*c1_0_1_11;
    const double G1_0_0 = det*c10_1_0_0*c2_1_1_0;
    const double G1_0_1 = det*c10_1_0_0*c2_1_1_1;
    const double G1_0_2 = det*c10_1_0_0*c2_1_1_2;
    const double G1_0_3 = det*c10_1_0_0*c2_1_1_3;
    const double G1_0_4 = det*c10_1_0_0*c2_1_1_4;
    const double G1_0_5 = det*c10_1_0_0*c2_1_1_5;
    const double G1_0_6 = det*c10_1_0_0*c2_1_1_6;
    const double G1_0_7 = det*c10_1_0_0*c2_1_1_7;
    const double G1_0_8 = det*c10_1_0_0*c2_1_1_8;
    const double G1_0_9 = det*c10_1_0_0*c2_1_1_9;
    const double G1_0_10 = det*c10_1_0_0*c2_1_1_10;
    const double G1_0_11 = det*c10_1_0_0*c2_1_1_11;
    const double G2_0_0_0_0 = det*c10_2_0_0*c7_2_1_0*c9_2_2_0*Jinv_00;
    const double G2_0_0_0_1 = det*c10_2_0_0*c7_2_1_0*c9_2_2_0*Jinv_10;
    const double G2_0_0_0_2 = det*c10_2_0_0*c7_2_1_0*c9_2_2_0*Jinv_20;
    const double G2_0_0_1_0 = det*c10_2_0_0*c7_2_1_0*c9_2_2_1*Jinv_00;
    const double G2_0_0_1_1 = det*c10_2_0_0*c7_2_1_0*c9_2_2_1*Jinv_10;
    const double G2_0_0_1_2 = det*c10_2_0_0*c7_2_1_0*c9_2_2_1*Jinv_20;
    const double G2_0_0_2_0 = det*c10_2_0_0*c7_2_1_0*c9_2_2_2*Jinv_00;
    const double G2_0_0_2_1 = det*c10_2_0_0*c7_2_1_0*c9_2_2_2*Jinv_10;
    const double G2_0_0_2_2 = det*c10_2_0_0*c7_2_1_0*c9_2_2_2*Jinv_20;
    const double G3_0_0_3_0 = det*c10_3_0_0*c7_3_1_0*c9_3_2_3*Jinv_01;
    const double G3_0_0_3_1 = det*c10_3_0_0*c7_3_1_0*c9_3_2_3*Jinv_11;
    const double G3_0_0_3_2 = det*c10_3_0_0*c7_3_1_0*c9_3_2_3*Jinv_21;
    const double G4_0_0_6_0 = det*c10_4_0_0*c7_4_1_0*c9_4_2_6*Jinv_02;
    const double G4_0_0_6_1 = det*c10_4_0_0*c7_4_1_0*c9_4_2_6*Jinv_12;
    const double G4_0_0_6_2 = det*c10_4_0_0*c7_4_1_0*c9_4_2_6*Jinv_22;
    const double G5_0_0_4_0 = det*c10_5_0_0*c7_5_1_0*c9_5_2_4*Jinv_01;
    const double G5_0_0_4_1 = det*c10_5_0_0*c7_5_1_0*c9_5_2_4*Jinv_11;
    const double G5_0_0_4_2 = det*c10_5_0_0*c7_5_1_0*c9_5_2_4*Jinv_21;
    const double G6_0_0_7_0 = det*c10_6_0_0*c7_6_1_0*c9_6_2_7*Jinv_02;
    const double G6_0_0_7_1 = det*c10_6_0_0*c7_6_1_0*c9_6_2_7*Jinv_12;
    const double G6_0_0_7_2 = det*c10_6_0_0*c7_6_1_0*c9_6_2_7*Jinv_22;
    const double G7_0_0_5_0 = det*c10_7_0_0*c7_7_1_0*c9_7_2_5*Jinv_01;
    const double G7_0_0_5_1 = det*c10_7_0_0*c7_7_1_0*c9_7_2_5*Jinv_11;
    const double G7_0_0_5_2 = det*c10_7_0_0*c7_7_1_0*c9_7_2_5*Jinv_21;
    const double G8_0_0_8_0 = det*c10_8_0_0*c7_8_1_0*c9_8_2_8*Jinv_02;
    const double G8_0_0_8_1 = det*c10_8_0_0*c7_8_1_0*c9_8_2_8*Jinv_12;
    const double G8_0_0_8_2 = det*c10_8_0_0*c7_8_1_0*c9_8_2_8*Jinv_22;
    const double G9_0_0_0 = det*c10_9_0_0*c9_9_1_0*Jinv_00;
    const double G9_0_0_1 = det*c10_9_0_0*c9_9_1_0*Jinv_10;
    const double G9_0_0_2 = det*c10_9_0_0*c9_9_1_0*Jinv_20;
    const double G9_0_1_0 = det*c10_9_0_0*c9_9_1_1*Jinv_00;
    const double G9_0_1_1 = det*c10_9_0_0*c9_9_1_1*Jinv_10;
    const double G9_0_1_2 = det*c10_9_0_0*c9_9_1_1*Jinv_20;
    const double G9_0_2_0 = det*c10_9_0_0*c9_9_1_2*Jinv_00;
    const double G9_0_2_1 = det*c10_9_0_0*c9_9_1_2*Jinv_10;
    const double G9_0_2_2 = det*c10_9_0_0*c9_9_1_2*Jinv_20;
    const double G10_0_3_0 = det*c10_10_0_0*c9_10_1_3*Jinv_01;
    const double G10_0_3_1 = det*c10_10_0_0*c9_10_1_3*Jinv_11;
    const double G10_0_3_2 = det*c10_10_0_0*c9_10_1_3*Jinv_21;
    const double G11_0_6_0 = det*c10_11_0_0*c9_11_1_6*Jinv_02;
    const double G11_0_6_1 = det*c10_11_0_0*c9_11_1_6*Jinv_12;
    const double G11_0_6_2 = det*c10_11_0_0*c9_11_1_6*Jinv_22;
    const double G12_0_4_0 = det*c10_12_0_0*c9_12_1_4*Jinv_01;
    const double G12_0_4_1 = det*c10_12_0_0*c9_12_1_4*Jinv_11;
    const double G12_0_4_2 = det*c10_12_0_0*c9_12_1_4*Jinv_21;
    const double G13_0_7_0 = det*c10_13_0_0*c9_13_1_7*Jinv_02;
    const double G13_0_7_1 = det*c10_13_0_0*c9_13_1_7*Jinv_12;
    const double G13_0_7_2 = det*c10_13_0_0*c9_13_1_7*Jinv_22;
    const double G14_0_5_0 = det*c10_14_0_0*c9_14_1_5*Jinv_01;
    const double G14_0_5_1 = det*c10_14_0_0*c9_14_1_5*Jinv_11;
    const double G14_0_5_2 = det*c10_14_0_0*c9_14_1_5*Jinv_21;
    const double G15_0_8_0 = det*c10_15_0_0*c9_15_1_8*Jinv_02;
    const double G15_0_8_1 = det*c10_15_0_0*c9_15_1_8*Jinv_12;
    const double G15_0_8_2 = det*c10_15_0_0*c9_15_1_8*Jinv_22;
    const double G16_0_0_0_0 = det*c10_16_0_0*c3_16_1_0*Jinv_00;
    const double G16_0_0_0_1 = det*c10_16_0_0*c3_16_1_0*Jinv_01;
    const double G16_0_0_0_2 = det*c10_16_0_0*c3_16_1_0*Jinv_02;
    const double G16_0_0_1_0 = det*c10_16_0_0*c3_16_1_0*Jinv_10;
    const double G16_0_0_1_1 = det*c10_16_0_0*c3_16_1_0*Jinv_11;
    const double G16_0_0_1_2 = det*c10_16_0_0*c3_16_1_0*Jinv_12;
    const double G16_0_0_2_0 = det*c10_16_0_0*c3_16_1_0*Jinv_20;
    const double G16_0_0_2_1 = det*c10_16_0_0*c3_16_1_0*Jinv_21;
    const double G16_0_0_2_2 = det*c10_16_0_0*c3_16_1_0*Jinv_22;
    const double G16_0_1_0_0 = det*c10_16_0_0*c3_16_1_1*Jinv_00;
    const double G16_0_1_0_1 = det*c10_16_0_0*c3_16_1_1*Jinv_01;
    const double G16_0_1_0_2 = det*c10_16_0_0*c3_16_1_1*Jinv_02;
    const double G16_0_1_1_0 = det*c10_16_0_0*c3_16_1_1*Jinv_10;
    const double G16_0_1_1_1 = det*c10_16_0_0*c3_16_1_1*Jinv_11;
    const double G16_0_1_1_2 = det*c10_16_0_0*c3_16_1_1*Jinv_12;
    const double G16_0_1_2_0 = det*c10_16_0_0*c3_16_1_1*Jinv_20;
    const double G16_0_1_2_1 = det*c10_16_0_0*c3_16_1_1*Jinv_21;
    const double G16_0_1_2_2 = det*c10_16_0_0*c3_16_1_1*Jinv_22;
    const double G16_0_2_0_0 = det*c10_16_0_0*c3_16_1_2*Jinv_00;
    const double G16_0_2_0_1 = det*c10_16_0_0*c3_16_1_2*Jinv_01;
    const double G16_0_2_0_2 = det*c10_16_0_0*c3_16_1_2*Jinv_02;
    const double G16_0_2_1_0 = det*c10_16_0_0*c3_16_1_2*Jinv_10;
    const double G16_0_2_1_1 = det*c10_16_0_0*c3_16_1_2*Jinv_11;
    const double G16_0_2_1_2 = det*c10_16_0_0*c3_16_1_2*Jinv_12;
    const double G16_0_2_2_0 = det*c10_16_0_0*c3_16_1_2*Jinv_20;
    const double G16_0_2_2_1 = det*c10_16_0_0*c3_16_1_2*Jinv_21;
    const double G16_0_2_2_2 = det*c10_16_0_0*c3_16_1_2*Jinv_22;
    const double G16_0_3_0_0 = det*c10_16_0_0*c3_16_1_3*Jinv_00;
    const double G16_0_3_0_1 = det*c10_16_0_0*c3_16_1_3*Jinv_01;
    const double G16_0_3_0_2 = det*c10_16_0_0*c3_16_1_3*Jinv_02;
    const double G16_0_3_1_0 = det*c10_16_0_0*c3_16_1_3*Jinv_10;
    const double G16_0_3_1_1 = det*c10_16_0_0*c3_16_1_3*Jinv_11;
    const double G16_0_3_1_2 = det*c10_16_0_0*c3_16_1_3*Jinv_12;
    const double G16_0_3_2_0 = det*c10_16_0_0*c3_16_1_3*Jinv_20;
    const double G16_0_3_2_1 = det*c10_16_0_0*c3_16_1_3*Jinv_21;
    const double G16_0_3_2_2 = det*c10_16_0_0*c3_16_1_3*Jinv_22;
    const double G17_0_0_0_0_0_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_0*Jinv_00;
    const double G17_0_0_0_0_0_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_1*Jinv_00;
    const double G17_0_0_0_0_0_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_2*Jinv_00;
    const double G17_0_0_0_0_0_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_3*Jinv_00;
    const double G17_0_0_0_0_0_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_4*Jinv_00;
    const double G17_0_0_0_0_0_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_5*Jinv_00;
    const double G17_0_0_0_0_0_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_6*Jinv_00;
    const double G17_0_0_0_0_0_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_7*Jinv_00;
    const double G17_0_0_0_0_0_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_8*Jinv_00;
    const double G17_0_0_0_0_0_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_9*Jinv_00;
    const double G17_0_0_0_0_0_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_10*Jinv_00;
    const double G17_0_0_0_0_0_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_11*Jinv_00;
    const double G17_0_0_0_1_0_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_0*Jinv_10;
    const double G17_0_0_0_1_0_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_1*Jinv_10;
    const double G17_0_0_0_1_0_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_2*Jinv_10;
    const double G17_0_0_0_1_0_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_3*Jinv_10;
    const double G17_0_0_0_1_0_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_4*Jinv_10;
    const double G17_0_0_0_1_0_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_5*Jinv_10;
    const double G17_0_0_0_1_0_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_6*Jinv_10;
    const double G17_0_0_0_1_0_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_7*Jinv_10;
    const double G17_0_0_0_1_0_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_8*Jinv_10;
    const double G17_0_0_0_1_0_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_9*Jinv_10;
    const double G17_0_0_0_1_0_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_10*Jinv_10;
    const double G17_0_0_0_1_0_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_11*Jinv_10;
    const double G17_0_0_0_2_0_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_0*Jinv_20;
    const double G17_0_0_0_2_0_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_1*Jinv_20;
    const double G17_0_0_0_2_0_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_2*Jinv_20;
    const double G17_0_0_0_2_0_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_3*Jinv_20;
    const double G17_0_0_0_2_0_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_4*Jinv_20;
    const double G17_0_0_0_2_0_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_5*Jinv_20;
    const double G17_0_0_0_2_0_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_6*Jinv_20;
    const double G17_0_0_0_2_0_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_7*Jinv_20;
    const double G17_0_0_0_2_0_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_8*Jinv_20;
    const double G17_0_0_0_2_0_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_9*Jinv_20;
    const double G17_0_0_0_2_0_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_10*Jinv_20;
    const double G17_0_0_0_2_0_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_0*c2_17_3_11*Jinv_20;
    const double G17_0_0_1_0_1_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_0*Jinv_01;
    const double G17_0_0_1_0_1_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_1*Jinv_01;
    const double G17_0_0_1_0_1_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_2*Jinv_01;
    const double G17_0_0_1_0_1_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_3*Jinv_01;
    const double G17_0_0_1_0_1_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_4*Jinv_01;
    const double G17_0_0_1_0_1_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_5*Jinv_01;
    const double G17_0_0_1_0_1_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_6*Jinv_01;
    const double G17_0_0_1_0_1_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_7*Jinv_01;
    const double G17_0_0_1_0_1_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_8*Jinv_01;
    const double G17_0_0_1_0_1_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_9*Jinv_01;
    const double G17_0_0_1_0_1_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_10*Jinv_01;
    const double G17_0_0_1_0_1_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_11*Jinv_01;
    const double G17_0_0_1_1_1_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_0*Jinv_11;
    const double G17_0_0_1_1_1_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_1*Jinv_11;
    const double G17_0_0_1_1_1_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_2*Jinv_11;
    const double G17_0_0_1_1_1_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_3*Jinv_11;
    const double G17_0_0_1_1_1_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_4*Jinv_11;
    const double G17_0_0_1_1_1_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_5*Jinv_11;
    const double G17_0_0_1_1_1_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_6*Jinv_11;
    const double G17_0_0_1_1_1_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_7*Jinv_11;
    const double G17_0_0_1_1_1_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_8*Jinv_11;
    const double G17_0_0_1_1_1_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_9*Jinv_11;
    const double G17_0_0_1_1_1_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_10*Jinv_11;
    const double G17_0_0_1_1_1_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_11*Jinv_11;
    const double G17_0_0_1_2_1_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_0*Jinv_21;
    const double G17_0_0_1_2_1_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_1*Jinv_21;
    const double G17_0_0_1_2_1_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_2*Jinv_21;
    const double G17_0_0_1_2_1_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_3*Jinv_21;
    const double G17_0_0_1_2_1_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_4*Jinv_21;
    const double G17_0_0_1_2_1_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_5*Jinv_21;
    const double G17_0_0_1_2_1_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_6*Jinv_21;
    const double G17_0_0_1_2_1_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_7*Jinv_21;
    const double G17_0_0_1_2_1_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_8*Jinv_21;
    const double G17_0_0_1_2_1_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_9*Jinv_21;
    const double G17_0_0_1_2_1_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_10*Jinv_21;
    const double G17_0_0_1_2_1_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_1*c2_17_3_11*Jinv_21;
    const double G17_0_0_2_0_2_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_0*Jinv_02;
    const double G17_0_0_2_0_2_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_1*Jinv_02;
    const double G17_0_0_2_0_2_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_2*Jinv_02;
    const double G17_0_0_2_0_2_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_3*Jinv_02;
    const double G17_0_0_2_0_2_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_4*Jinv_02;
    const double G17_0_0_2_0_2_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_5*Jinv_02;
    const double G17_0_0_2_0_2_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_6*Jinv_02;
    const double G17_0_0_2_0_2_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_7*Jinv_02;
    const double G17_0_0_2_0_2_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_8*Jinv_02;
    const double G17_0_0_2_0_2_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_9*Jinv_02;
    const double G17_0_0_2_0_2_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_10*Jinv_02;
    const double G17_0_0_2_0_2_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_11*Jinv_02;
    const double G17_0_0_2_1_2_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_0*Jinv_12;
    const double G17_0_0_2_1_2_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_1*Jinv_12;
    const double G17_0_0_2_1_2_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_2*Jinv_12;
    const double G17_0_0_2_1_2_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_3*Jinv_12;
    const double G17_0_0_2_1_2_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_4*Jinv_12;
    const double G17_0_0_2_1_2_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_5*Jinv_12;
    const double G17_0_0_2_1_2_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_6*Jinv_12;
    const double G17_0_0_2_1_2_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_7*Jinv_12;
    const double G17_0_0_2_1_2_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_8*Jinv_12;
    const double G17_0_0_2_1_2_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_9*Jinv_12;
    const double G17_0_0_2_1_2_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_10*Jinv_12;
    const double G17_0_0_2_1_2_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_11*Jinv_12;
    const double G17_0_0_2_2_2_0 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_0*Jinv_22;
    const double G17_0_0_2_2_2_1 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_1*Jinv_22;
    const double G17_0_0_2_2_2_2 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_2*Jinv_22;
    const double G17_0_0_2_2_2_3 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_3*Jinv_22;
    const double G17_0_0_2_2_2_4 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_4*Jinv_22;
    const double G17_0_0_2_2_2_5 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_5*Jinv_22;
    const double G17_0_0_2_2_2_6 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_6*Jinv_22;
    const double G17_0_0_2_2_2_7 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_7*Jinv_22;
    const double G17_0_0_2_2_2_8 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_8*Jinv_22;
    const double G17_0_0_2_2_2_9 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_9*Jinv_22;
    const double G17_0_0_2_2_2_10 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_10*Jinv_22;
    const double G17_0_0_2_2_2_11 = det*c10_17_0_0*c4_17_1_0*c0_17_2_2*c2_17_3_11*Jinv_22;
    const double G18_0_0_0_0_0_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_0*Jinv_00;
    const double G18_0_0_0_0_0_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_1*Jinv_00;
    const double G18_0_0_0_0_0_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_2*Jinv_00;
    const double G18_0_0_0_0_0_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_3*Jinv_00;
    const double G18_0_0_0_0_0_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_4*Jinv_00;
    const double G18_0_0_0_0_0_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_5*Jinv_00;
    const double G18_0_0_0_0_0_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_6*Jinv_00;
    const double G18_0_0_0_0_0_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_7*Jinv_00;
    const double G18_0_0_0_0_0_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_8*Jinv_00;
    const double G18_0_0_0_0_0_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_9*Jinv_00;
    const double G18_0_0_0_0_0_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_10*Jinv_00;
    const double G18_0_0_0_0_0_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_11*Jinv_00;
    const double G18_0_0_0_1_0_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_0*Jinv_10;
    const double G18_0_0_0_1_0_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_1*Jinv_10;
    const double G18_0_0_0_1_0_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_2*Jinv_10;
    const double G18_0_0_0_1_0_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_3*Jinv_10;
    const double G18_0_0_0_1_0_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_4*Jinv_10;
    const double G18_0_0_0_1_0_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_5*Jinv_10;
    const double G18_0_0_0_1_0_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_6*Jinv_10;
    const double G18_0_0_0_1_0_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_7*Jinv_10;
    const double G18_0_0_0_1_0_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_8*Jinv_10;
    const double G18_0_0_0_1_0_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_9*Jinv_10;
    const double G18_0_0_0_1_0_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_10*Jinv_10;
    const double G18_0_0_0_1_0_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_11*Jinv_10;
    const double G18_0_0_0_2_0_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_0*Jinv_20;
    const double G18_0_0_0_2_0_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_1*Jinv_20;
    const double G18_0_0_0_2_0_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_2*Jinv_20;
    const double G18_0_0_0_2_0_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_3*Jinv_20;
    const double G18_0_0_0_2_0_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_4*Jinv_20;
    const double G18_0_0_0_2_0_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_5*Jinv_20;
    const double G18_0_0_0_2_0_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_6*Jinv_20;
    const double G18_0_0_0_2_0_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_7*Jinv_20;
    const double G18_0_0_0_2_0_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_8*Jinv_20;
    const double G18_0_0_0_2_0_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_9*Jinv_20;
    const double G18_0_0_0_2_0_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_10*Jinv_20;
    const double G18_0_0_0_2_0_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_0*c2_18_3_11*Jinv_20;
    const double G18_0_0_1_0_1_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_0*Jinv_01;
    const double G18_0_0_1_0_1_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_1*Jinv_01;
    const double G18_0_0_1_0_1_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_2*Jinv_01;
    const double G18_0_0_1_0_1_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_3*Jinv_01;
    const double G18_0_0_1_0_1_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_4*Jinv_01;
    const double G18_0_0_1_0_1_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_5*Jinv_01;
    const double G18_0_0_1_0_1_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_6*Jinv_01;
    const double G18_0_0_1_0_1_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_7*Jinv_01;
    const double G18_0_0_1_0_1_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_8*Jinv_01;
    const double G18_0_0_1_0_1_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_9*Jinv_01;
    const double G18_0_0_1_0_1_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_10*Jinv_01;
    const double G18_0_0_1_0_1_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_11*Jinv_01;
    const double G18_0_0_1_1_1_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_0*Jinv_11;
    const double G18_0_0_1_1_1_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_1*Jinv_11;
    const double G18_0_0_1_1_1_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_2*Jinv_11;
    const double G18_0_0_1_1_1_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_3*Jinv_11;
    const double G18_0_0_1_1_1_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_4*Jinv_11;
    const double G18_0_0_1_1_1_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_5*Jinv_11;
    const double G18_0_0_1_1_1_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_6*Jinv_11;
    const double G18_0_0_1_1_1_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_7*Jinv_11;
    const double G18_0_0_1_1_1_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_8*Jinv_11;
    const double G18_0_0_1_1_1_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_9*Jinv_11;
    const double G18_0_0_1_1_1_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_10*Jinv_11;
    const double G18_0_0_1_1_1_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_11*Jinv_11;
    const double G18_0_0_1_2_1_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_0*Jinv_21;
    const double G18_0_0_1_2_1_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_1*Jinv_21;
    const double G18_0_0_1_2_1_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_2*Jinv_21;
    const double G18_0_0_1_2_1_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_3*Jinv_21;
    const double G18_0_0_1_2_1_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_4*Jinv_21;
    const double G18_0_0_1_2_1_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_5*Jinv_21;
    const double G18_0_0_1_2_1_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_6*Jinv_21;
    const double G18_0_0_1_2_1_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_7*Jinv_21;
    const double G18_0_0_1_2_1_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_8*Jinv_21;
    const double G18_0_0_1_2_1_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_9*Jinv_21;
    const double G18_0_0_1_2_1_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_10*Jinv_21;
    const double G18_0_0_1_2_1_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_1*c2_18_3_11*Jinv_21;
    const double G18_0_0_2_0_2_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_0*Jinv_02;
    const double G18_0_0_2_0_2_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_1*Jinv_02;
    const double G18_0_0_2_0_2_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_2*Jinv_02;
    const double G18_0_0_2_0_2_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_3*Jinv_02;
    const double G18_0_0_2_0_2_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_4*Jinv_02;
    const double G18_0_0_2_0_2_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_5*Jinv_02;
    const double G18_0_0_2_0_2_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_6*Jinv_02;
    const double G18_0_0_2_0_2_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_7*Jinv_02;
    const double G18_0_0_2_0_2_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_8*Jinv_02;
    const double G18_0_0_2_0_2_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_9*Jinv_02;
    const double G18_0_0_2_0_2_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_10*Jinv_02;
    const double G18_0_0_2_0_2_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_11*Jinv_02;
    const double G18_0_0_2_1_2_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_0*Jinv_12;
    const double G18_0_0_2_1_2_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_1*Jinv_12;
    const double G18_0_0_2_1_2_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_2*Jinv_12;
    const double G18_0_0_2_1_2_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_3*Jinv_12;
    const double G18_0_0_2_1_2_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_4*Jinv_12;
    const double G18_0_0_2_1_2_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_5*Jinv_12;
    const double G18_0_0_2_1_2_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_6*Jinv_12;
    const double G18_0_0_2_1_2_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_7*Jinv_12;
    const double G18_0_0_2_1_2_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_8*Jinv_12;
    const double G18_0_0_2_1_2_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_9*Jinv_12;
    const double G18_0_0_2_1_2_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_10*Jinv_12;
    const double G18_0_0_2_1_2_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_11*Jinv_12;
    const double G18_0_0_2_2_2_0 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_0*Jinv_22;
    const double G18_0_0_2_2_2_1 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_1*Jinv_22;
    const double G18_0_0_2_2_2_2 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_2*Jinv_22;
    const double G18_0_0_2_2_2_3 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_3*Jinv_22;
    const double G18_0_0_2_2_2_4 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_4*Jinv_22;
    const double G18_0_0_2_2_2_5 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_5*Jinv_22;
    const double G18_0_0_2_2_2_6 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_6*Jinv_22;
    const double G18_0_0_2_2_2_7 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_7*Jinv_22;
    const double G18_0_0_2_2_2_8 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_8*Jinv_22;
    const double G18_0_0_2_2_2_9 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_9*Jinv_22;
    const double G18_0_0_2_2_2_10 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_10*Jinv_22;
    const double G18_0_0_2_2_2_11 = det*c10_18_0_0*c4_18_1_0*c6_18_2_2*c2_18_3_11*Jinv_22;
    const double G19_0_0_0_0_0_0_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_00*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_00*Jinv_00);
    const double G19_0_0_0_0_0_0_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_00*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_00*Jinv_10);
    const double G19_0_0_0_0_0_0_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_00*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_00*Jinv_20);
    const double G19_0_0_0_0_0_0_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_1*Jinv_00*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_1*Jinv_00*Jinv_00);
    const double G19_0_0_0_0_0_0_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_2*Jinv_00*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_2*Jinv_00*Jinv_10);
    const double G19_0_0_0_0_0_0_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_3*Jinv_00*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_3*Jinv_00*Jinv_20);
    const double G19_0_0_0_0_0_0_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_00*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_00*Jinv_00);
    const double G19_0_0_0_0_0_0_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_00*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_00*Jinv_10);
    const double G19_0_0_0_0_0_0_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_00*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_00*Jinv_20);
    const double G19_0_0_0_0_0_0_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_5*Jinv_00*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_5*Jinv_00*Jinv_00);
    const double G19_0_0_0_0_0_0_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_6*Jinv_00*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_6*Jinv_00*Jinv_10);
    const double G19_0_0_0_0_0_0_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_7*Jinv_00*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_7*Jinv_00*Jinv_20);
    const double G19_0_0_0_0_0_0_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_00*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_00*Jinv_00);
    const double G19_0_0_0_0_0_0_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_00*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_00*Jinv_10);
    const double G19_0_0_0_0_0_0_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_00*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_00*Jinv_20);
    const double G19_0_0_0_0_0_0_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_9*Jinv_00*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_9*Jinv_00*Jinv_00);
    const double G19_0_0_0_0_0_0_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_10*Jinv_00*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_10*Jinv_00*Jinv_10);
    const double G19_0_0_0_0_0_0_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_11*Jinv_00*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_11*Jinv_00*Jinv_20);
    const double G19_0_0_0_0_0_1_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_10*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_10*Jinv_00);
    const double G19_0_0_0_0_0_1_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_10*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_10*Jinv_10);
    const double G19_0_0_0_0_0_1_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_10*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_10*Jinv_20);
    const double G19_0_0_0_0_0_1_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_1*Jinv_10*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_1*Jinv_10*Jinv_00);
    const double G19_0_0_0_0_0_1_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_2*Jinv_10*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_2*Jinv_10*Jinv_10);
    const double G19_0_0_0_0_0_1_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_3*Jinv_10*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_3*Jinv_10*Jinv_20);
    const double G19_0_0_0_0_0_1_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_10*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_10*Jinv_00);
    const double G19_0_0_0_0_0_1_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_10*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_10*Jinv_10);
    const double G19_0_0_0_0_0_1_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_10*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_10*Jinv_20);
    const double G19_0_0_0_0_0_1_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_5*Jinv_10*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_5*Jinv_10*Jinv_00);
    const double G19_0_0_0_0_0_1_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_6*Jinv_10*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_6*Jinv_10*Jinv_10);
    const double G19_0_0_0_0_0_1_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_7*Jinv_10*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_7*Jinv_10*Jinv_20);
    const double G19_0_0_0_0_0_1_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_10*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_10*Jinv_00);
    const double G19_0_0_0_0_0_1_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_10*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_10*Jinv_10);
    const double G19_0_0_0_0_0_1_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_10*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_10*Jinv_20);
    const double G19_0_0_0_0_0_1_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_9*Jinv_10*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_9*Jinv_10*Jinv_00);
    const double G19_0_0_0_0_0_1_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_10*Jinv_10*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_10*Jinv_10*Jinv_10);
    const double G19_0_0_0_0_0_1_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_11*Jinv_10*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_11*Jinv_10*Jinv_20);
    const double G19_0_0_0_0_0_2_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_20*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_20*Jinv_00);
    const double G19_0_0_0_0_0_2_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_20*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_20*Jinv_10);
    const double G19_0_0_0_0_0_2_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_0*Jinv_20*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_0*Jinv_20*Jinv_20);
    const double G19_0_0_0_0_0_2_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_1*Jinv_20*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_1*Jinv_20*Jinv_00);
    const double G19_0_0_0_0_0_2_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_2*Jinv_20*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_2*Jinv_20*Jinv_10);
    const double G19_0_0_0_0_0_2_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_3*Jinv_20*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_3*Jinv_20*Jinv_20);
    const double G19_0_0_0_0_0_2_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_20*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_20*Jinv_00);
    const double G19_0_0_0_0_0_2_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_20*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_20*Jinv_10);
    const double G19_0_0_0_0_0_2_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_4*Jinv_20*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_4*Jinv_20*Jinv_20);
    const double G19_0_0_0_0_0_2_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_5*Jinv_20*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_5*Jinv_20*Jinv_00);
    const double G19_0_0_0_0_0_2_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_6*Jinv_20*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_6*Jinv_20*Jinv_10);
    const double G19_0_0_0_0_0_2_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_7*Jinv_20*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_7*Jinv_20*Jinv_20);
    const double G19_0_0_0_0_0_2_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_20*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_20*Jinv_00);
    const double G19_0_0_0_0_0_2_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_20*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_20*Jinv_10);
    const double G19_0_0_0_0_0_2_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_8*Jinv_20*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_8*Jinv_20*Jinv_20);
    const double G19_0_0_0_0_0_2_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_9*Jinv_20*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_9*Jinv_20*Jinv_00);
    const double G19_0_0_0_0_0_2_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_10*Jinv_20*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_10*Jinv_20*Jinv_10);
    const double G19_0_0_0_0_0_2_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_0*c1_19_4_11*Jinv_20*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_0*c1_20_4_11*Jinv_20*Jinv_20);
    const double G19_0_0_0_1_0_0_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_01*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_01*Jinv_00);
    const double G19_0_0_0_1_0_0_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_01*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_01*Jinv_10);
    const double G19_0_0_0_1_0_0_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_01*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_01*Jinv_20);
    const double G19_0_0_0_1_0_0_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_1*Jinv_01*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_1*Jinv_01*Jinv_00);
    const double G19_0_0_0_1_0_0_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_2*Jinv_01*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_2*Jinv_01*Jinv_10);
    const double G19_0_0_0_1_0_0_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_3*Jinv_01*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_3*Jinv_01*Jinv_20);
    const double G19_0_0_0_1_0_0_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_01*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_01*Jinv_00);
    const double G19_0_0_0_1_0_0_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_01*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_01*Jinv_10);
    const double G19_0_0_0_1_0_0_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_01*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_01*Jinv_20);
    const double G19_0_0_0_1_0_0_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_5*Jinv_01*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_5*Jinv_01*Jinv_00);
    const double G19_0_0_0_1_0_0_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_6*Jinv_01*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_6*Jinv_01*Jinv_10);
    const double G19_0_0_0_1_0_0_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_7*Jinv_01*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_7*Jinv_01*Jinv_20);
    const double G19_0_0_0_1_0_0_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_01*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_01*Jinv_00);
    const double G19_0_0_0_1_0_0_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_01*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_01*Jinv_10);
    const double G19_0_0_0_1_0_0_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_01*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_01*Jinv_20);
    const double G19_0_0_0_1_0_0_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_9*Jinv_01*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_9*Jinv_01*Jinv_00);
    const double G19_0_0_0_1_0_0_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_10*Jinv_01*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_10*Jinv_01*Jinv_10);
    const double G19_0_0_0_1_0_0_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_11*Jinv_01*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_11*Jinv_01*Jinv_20);
    const double G19_0_0_0_1_0_1_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_11*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_11*Jinv_00);
    const double G19_0_0_0_1_0_1_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_11*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_11*Jinv_10);
    const double G19_0_0_0_1_0_1_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_11*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_11*Jinv_20);
    const double G19_0_0_0_1_0_1_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_1*Jinv_11*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_1*Jinv_11*Jinv_00);
    const double G19_0_0_0_1_0_1_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_2*Jinv_11*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_2*Jinv_11*Jinv_10);
    const double G19_0_0_0_1_0_1_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_3*Jinv_11*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_3*Jinv_11*Jinv_20);
    const double G19_0_0_0_1_0_1_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_11*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_11*Jinv_00);
    const double G19_0_0_0_1_0_1_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_11*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_11*Jinv_10);
    const double G19_0_0_0_1_0_1_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_11*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_11*Jinv_20);
    const double G19_0_0_0_1_0_1_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_5*Jinv_11*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_5*Jinv_11*Jinv_00);
    const double G19_0_0_0_1_0_1_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_6*Jinv_11*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_6*Jinv_11*Jinv_10);
    const double G19_0_0_0_1_0_1_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_7*Jinv_11*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_7*Jinv_11*Jinv_20);
    const double G19_0_0_0_1_0_1_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_11*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_11*Jinv_00);
    const double G19_0_0_0_1_0_1_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_11*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_11*Jinv_10);
    const double G19_0_0_0_1_0_1_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_11*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_11*Jinv_20);
    const double G19_0_0_0_1_0_1_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_9*Jinv_11*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_9*Jinv_11*Jinv_00);
    const double G19_0_0_0_1_0_1_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_10*Jinv_11*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_10*Jinv_11*Jinv_10);
    const double G19_0_0_0_1_0_1_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_11*Jinv_11*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_11*Jinv_11*Jinv_20);
    const double G19_0_0_0_1_0_2_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_21*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_21*Jinv_00);
    const double G19_0_0_0_1_0_2_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_21*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_21*Jinv_10);
    const double G19_0_0_0_1_0_2_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_0*Jinv_21*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_0*Jinv_21*Jinv_20);
    const double G19_0_0_0_1_0_2_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_1*Jinv_21*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_1*Jinv_21*Jinv_00);
    const double G19_0_0_0_1_0_2_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_2*Jinv_21*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_2*Jinv_21*Jinv_10);
    const double G19_0_0_0_1_0_2_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_3*Jinv_21*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_3*Jinv_21*Jinv_20);
    const double G19_0_0_0_1_0_2_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_21*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_21*Jinv_00);
    const double G19_0_0_0_1_0_2_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_21*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_21*Jinv_10);
    const double G19_0_0_0_1_0_2_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_4*Jinv_21*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_4*Jinv_21*Jinv_20);
    const double G19_0_0_0_1_0_2_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_5*Jinv_21*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_5*Jinv_21*Jinv_00);
    const double G19_0_0_0_1_0_2_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_6*Jinv_21*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_6*Jinv_21*Jinv_10);
    const double G19_0_0_0_1_0_2_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_7*Jinv_21*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_7*Jinv_21*Jinv_20);
    const double G19_0_0_0_1_0_2_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_21*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_21*Jinv_00);
    const double G19_0_0_0_1_0_2_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_21*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_21*Jinv_10);
    const double G19_0_0_0_1_0_2_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_8*Jinv_21*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_8*Jinv_21*Jinv_20);
    const double G19_0_0_0_1_0_2_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_9*Jinv_21*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_9*Jinv_21*Jinv_00);
    const double G19_0_0_0_1_0_2_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_10*Jinv_21*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_10*Jinv_21*Jinv_10);
    const double G19_0_0_0_1_0_2_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_0*c1_19_4_11*Jinv_21*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_0*c1_20_4_11*Jinv_21*Jinv_20);
    const double G19_0_0_0_2_0_0_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_02*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_02*Jinv_00);
    const double G19_0_0_0_2_0_0_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_02*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_02*Jinv_10);
    const double G19_0_0_0_2_0_0_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_02*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_02*Jinv_20);
    const double G19_0_0_0_2_0_0_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_1*Jinv_02*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_1*Jinv_02*Jinv_00);
    const double G19_0_0_0_2_0_0_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_2*Jinv_02*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_2*Jinv_02*Jinv_10);
    const double G19_0_0_0_2_0_0_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_3*Jinv_02*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_3*Jinv_02*Jinv_20);
    const double G19_0_0_0_2_0_0_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_02*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_02*Jinv_00);
    const double G19_0_0_0_2_0_0_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_02*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_02*Jinv_10);
    const double G19_0_0_0_2_0_0_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_02*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_02*Jinv_20);
    const double G19_0_0_0_2_0_0_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_5*Jinv_02*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_5*Jinv_02*Jinv_00);
    const double G19_0_0_0_2_0_0_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_6*Jinv_02*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_6*Jinv_02*Jinv_10);
    const double G19_0_0_0_2_0_0_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_7*Jinv_02*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_7*Jinv_02*Jinv_20);
    const double G19_0_0_0_2_0_0_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_02*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_02*Jinv_00);
    const double G19_0_0_0_2_0_0_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_02*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_02*Jinv_10);
    const double G19_0_0_0_2_0_0_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_02*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_02*Jinv_20);
    const double G19_0_0_0_2_0_0_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_9*Jinv_02*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_9*Jinv_02*Jinv_00);
    const double G19_0_0_0_2_0_0_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_10*Jinv_02*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_10*Jinv_02*Jinv_10);
    const double G19_0_0_0_2_0_0_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_11*Jinv_02*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_11*Jinv_02*Jinv_20);
    const double G19_0_0_0_2_0_1_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_12*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_12*Jinv_00);
    const double G19_0_0_0_2_0_1_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_12*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_12*Jinv_10);
    const double G19_0_0_0_2_0_1_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_12*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_12*Jinv_20);
    const double G19_0_0_0_2_0_1_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_1*Jinv_12*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_1*Jinv_12*Jinv_00);
    const double G19_0_0_0_2_0_1_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_2*Jinv_12*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_2*Jinv_12*Jinv_10);
    const double G19_0_0_0_2_0_1_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_3*Jinv_12*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_3*Jinv_12*Jinv_20);
    const double G19_0_0_0_2_0_1_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_12*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_12*Jinv_00);
    const double G19_0_0_0_2_0_1_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_12*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_12*Jinv_10);
    const double G19_0_0_0_2_0_1_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_12*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_12*Jinv_20);
    const double G19_0_0_0_2_0_1_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_5*Jinv_12*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_5*Jinv_12*Jinv_00);
    const double G19_0_0_0_2_0_1_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_6*Jinv_12*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_6*Jinv_12*Jinv_10);
    const double G19_0_0_0_2_0_1_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_7*Jinv_12*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_7*Jinv_12*Jinv_20);
    const double G19_0_0_0_2_0_1_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_12*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_12*Jinv_00);
    const double G19_0_0_0_2_0_1_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_12*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_12*Jinv_10);
    const double G19_0_0_0_2_0_1_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_12*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_12*Jinv_20);
    const double G19_0_0_0_2_0_1_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_9*Jinv_12*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_9*Jinv_12*Jinv_00);
    const double G19_0_0_0_2_0_1_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_10*Jinv_12*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_10*Jinv_12*Jinv_10);
    const double G19_0_0_0_2_0_1_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_11*Jinv_12*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_11*Jinv_12*Jinv_20);
    const double G19_0_0_0_2_0_2_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_22*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_22*Jinv_00);
    const double G19_0_0_0_2_0_2_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_22*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_22*Jinv_10);
    const double G19_0_0_0_2_0_2_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_0*Jinv_22*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_0*Jinv_22*Jinv_20);
    const double G19_0_0_0_2_0_2_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_1*Jinv_22*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_1*Jinv_22*Jinv_00);
    const double G19_0_0_0_2_0_2_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_2*Jinv_22*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_2*Jinv_22*Jinv_10);
    const double G19_0_0_0_2_0_2_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_3*Jinv_22*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_3*Jinv_22*Jinv_20);
    const double G19_0_0_0_2_0_2_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_22*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_22*Jinv_00);
    const double G19_0_0_0_2_0_2_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_22*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_22*Jinv_10);
    const double G19_0_0_0_2_0_2_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_4*Jinv_22*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_4*Jinv_22*Jinv_20);
    const double G19_0_0_0_2_0_2_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_5*Jinv_22*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_5*Jinv_22*Jinv_00);
    const double G19_0_0_0_2_0_2_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_6*Jinv_22*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_6*Jinv_22*Jinv_10);
    const double G19_0_0_0_2_0_2_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_7*Jinv_22*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_7*Jinv_22*Jinv_20);
    const double G19_0_0_0_2_0_2_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_22*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_22*Jinv_00);
    const double G19_0_0_0_2_0_2_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_22*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_22*Jinv_10);
    const double G19_0_0_0_2_0_2_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_8*Jinv_22*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_8*Jinv_22*Jinv_20);
    const double G19_0_0_0_2_0_2_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_9*Jinv_22*Jinv_00 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_9*Jinv_22*Jinv_00);
    const double G19_0_0_0_2_0_2_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_10*Jinv_22*Jinv_10 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_10*Jinv_22*Jinv_10);
    const double G19_0_0_0_2_0_2_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_0*c1_19_4_11*Jinv_22*Jinv_20 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_0*c1_20_4_11*Jinv_22*Jinv_20);
    const double G19_1_0_0_0_1_0_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_00*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_00*Jinv_01);
    const double G19_1_0_0_0_1_0_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_00*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_00*Jinv_11);
    const double G19_1_0_0_0_1_0_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_00*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_00*Jinv_21);
    const double G19_1_0_0_0_1_0_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_1*Jinv_00*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_1*Jinv_00*Jinv_01);
    const double G19_1_0_0_0_1_0_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_2*Jinv_00*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_2*Jinv_00*Jinv_11);
    const double G19_1_0_0_0_1_0_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_3*Jinv_00*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_3*Jinv_00*Jinv_21);
    const double G19_1_0_0_0_1_0_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_00*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_00*Jinv_01);
    const double G19_1_0_0_0_1_0_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_00*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_00*Jinv_11);
    const double G19_1_0_0_0_1_0_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_00*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_00*Jinv_21);
    const double G19_1_0_0_0_1_0_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_5*Jinv_00*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_5*Jinv_00*Jinv_01);
    const double G19_1_0_0_0_1_0_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_6*Jinv_00*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_6*Jinv_00*Jinv_11);
    const double G19_1_0_0_0_1_0_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_7*Jinv_00*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_7*Jinv_00*Jinv_21);
    const double G19_1_0_0_0_1_0_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_00*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_00*Jinv_01);
    const double G19_1_0_0_0_1_0_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_00*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_00*Jinv_11);
    const double G19_1_0_0_0_1_0_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_00*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_00*Jinv_21);
    const double G19_1_0_0_0_1_0_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_9*Jinv_00*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_9*Jinv_00*Jinv_01);
    const double G19_1_0_0_0_1_0_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_10*Jinv_00*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_10*Jinv_00*Jinv_11);
    const double G19_1_0_0_0_1_0_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_11*Jinv_00*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_11*Jinv_00*Jinv_21);
    const double G19_1_0_0_0_1_1_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_10*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_10*Jinv_01);
    const double G19_1_0_0_0_1_1_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_10*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_10*Jinv_11);
    const double G19_1_0_0_0_1_1_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_10*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_10*Jinv_21);
    const double G19_1_0_0_0_1_1_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_1*Jinv_10*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_1*Jinv_10*Jinv_01);
    const double G19_1_0_0_0_1_1_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_2*Jinv_10*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_2*Jinv_10*Jinv_11);
    const double G19_1_0_0_0_1_1_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_3*Jinv_10*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_3*Jinv_10*Jinv_21);
    const double G19_1_0_0_0_1_1_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_10*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_10*Jinv_01);
    const double G19_1_0_0_0_1_1_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_10*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_10*Jinv_11);
    const double G19_1_0_0_0_1_1_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_10*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_10*Jinv_21);
    const double G19_1_0_0_0_1_1_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_5*Jinv_10*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_5*Jinv_10*Jinv_01);
    const double G19_1_0_0_0_1_1_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_6*Jinv_10*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_6*Jinv_10*Jinv_11);
    const double G19_1_0_0_0_1_1_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_7*Jinv_10*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_7*Jinv_10*Jinv_21);
    const double G19_1_0_0_0_1_1_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_10*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_10*Jinv_01);
    const double G19_1_0_0_0_1_1_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_10*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_10*Jinv_11);
    const double G19_1_0_0_0_1_1_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_10*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_10*Jinv_21);
    const double G19_1_0_0_0_1_1_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_9*Jinv_10*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_9*Jinv_10*Jinv_01);
    const double G19_1_0_0_0_1_1_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_10*Jinv_10*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_10*Jinv_10*Jinv_11);
    const double G19_1_0_0_0_1_1_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_11*Jinv_10*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_11*Jinv_10*Jinv_21);
    const double G19_1_0_0_0_1_2_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_20*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_20*Jinv_01);
    const double G19_1_0_0_0_1_2_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_20*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_20*Jinv_11);
    const double G19_1_0_0_0_1_2_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_0*Jinv_20*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_0*Jinv_20*Jinv_21);
    const double G19_1_0_0_0_1_2_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_1*Jinv_20*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_1*Jinv_20*Jinv_01);
    const double G19_1_0_0_0_1_2_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_2*Jinv_20*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_2*Jinv_20*Jinv_11);
    const double G19_1_0_0_0_1_2_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_3*Jinv_20*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_3*Jinv_20*Jinv_21);
    const double G19_1_0_0_0_1_2_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_20*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_20*Jinv_01);
    const double G19_1_0_0_0_1_2_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_20*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_20*Jinv_11);
    const double G19_1_0_0_0_1_2_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_4*Jinv_20*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_4*Jinv_20*Jinv_21);
    const double G19_1_0_0_0_1_2_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_5*Jinv_20*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_5*Jinv_20*Jinv_01);
    const double G19_1_0_0_0_1_2_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_6*Jinv_20*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_6*Jinv_20*Jinv_11);
    const double G19_1_0_0_0_1_2_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_7*Jinv_20*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_7*Jinv_20*Jinv_21);
    const double G19_1_0_0_0_1_2_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_20*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_20*Jinv_01);
    const double G19_1_0_0_0_1_2_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_20*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_20*Jinv_11);
    const double G19_1_0_0_0_1_2_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_8*Jinv_20*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_8*Jinv_20*Jinv_21);
    const double G19_1_0_0_0_1_2_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_9*Jinv_20*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_9*Jinv_20*Jinv_01);
    const double G19_1_0_0_0_1_2_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_10*Jinv_20*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_10*Jinv_20*Jinv_11);
    const double G19_1_0_0_0_1_2_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_1*c1_19_4_11*Jinv_20*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_1*c1_20_4_11*Jinv_20*Jinv_21);
    const double G19_1_0_0_1_1_0_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_01*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_01*Jinv_01);
    const double G19_1_0_0_1_1_0_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_01*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_01*Jinv_11);
    const double G19_1_0_0_1_1_0_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_01*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_01*Jinv_21);
    const double G19_1_0_0_1_1_0_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_1*Jinv_01*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_1*Jinv_01*Jinv_01);
    const double G19_1_0_0_1_1_0_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_2*Jinv_01*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_2*Jinv_01*Jinv_11);
    const double G19_1_0_0_1_1_0_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_3*Jinv_01*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_3*Jinv_01*Jinv_21);
    const double G19_1_0_0_1_1_0_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_01*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_01*Jinv_01);
    const double G19_1_0_0_1_1_0_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_01*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_01*Jinv_11);
    const double G19_1_0_0_1_1_0_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_01*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_01*Jinv_21);
    const double G19_1_0_0_1_1_0_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_5*Jinv_01*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_5*Jinv_01*Jinv_01);
    const double G19_1_0_0_1_1_0_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_6*Jinv_01*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_6*Jinv_01*Jinv_11);
    const double G19_1_0_0_1_1_0_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_7*Jinv_01*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_7*Jinv_01*Jinv_21);
    const double G19_1_0_0_1_1_0_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_01*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_01*Jinv_01);
    const double G19_1_0_0_1_1_0_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_01*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_01*Jinv_11);
    const double G19_1_0_0_1_1_0_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_01*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_01*Jinv_21);
    const double G19_1_0_0_1_1_0_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_9*Jinv_01*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_9*Jinv_01*Jinv_01);
    const double G19_1_0_0_1_1_0_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_10*Jinv_01*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_10*Jinv_01*Jinv_11);
    const double G19_1_0_0_1_1_0_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_11*Jinv_01*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_11*Jinv_01*Jinv_21);
    const double G19_1_0_0_1_1_1_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_11*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_11*Jinv_01);
    const double G19_1_0_0_1_1_1_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_11*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_11*Jinv_11);
    const double G19_1_0_0_1_1_1_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_11*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_11*Jinv_21);
    const double G19_1_0_0_1_1_1_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_1*Jinv_11*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_1*Jinv_11*Jinv_01);
    const double G19_1_0_0_1_1_1_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_2*Jinv_11*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_2*Jinv_11*Jinv_11);
    const double G19_1_0_0_1_1_1_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_3*Jinv_11*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_3*Jinv_11*Jinv_21);
    const double G19_1_0_0_1_1_1_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_11*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_11*Jinv_01);
    const double G19_1_0_0_1_1_1_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_11*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_11*Jinv_11);
    const double G19_1_0_0_1_1_1_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_11*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_11*Jinv_21);
    const double G19_1_0_0_1_1_1_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_5*Jinv_11*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_5*Jinv_11*Jinv_01);
    const double G19_1_0_0_1_1_1_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_6*Jinv_11*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_6*Jinv_11*Jinv_11);
    const double G19_1_0_0_1_1_1_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_7*Jinv_11*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_7*Jinv_11*Jinv_21);
    const double G19_1_0_0_1_1_1_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_11*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_11*Jinv_01);
    const double G19_1_0_0_1_1_1_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_11*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_11*Jinv_11);
    const double G19_1_0_0_1_1_1_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_11*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_11*Jinv_21);
    const double G19_1_0_0_1_1_1_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_9*Jinv_11*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_9*Jinv_11*Jinv_01);
    const double G19_1_0_0_1_1_1_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_10*Jinv_11*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_10*Jinv_11*Jinv_11);
    const double G19_1_0_0_1_1_1_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_11*Jinv_11*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_11*Jinv_11*Jinv_21);
    const double G19_1_0_0_1_1_2_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_21*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_21*Jinv_01);
    const double G19_1_0_0_1_1_2_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_21*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_21*Jinv_11);
    const double G19_1_0_0_1_1_2_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_0*Jinv_21*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_0*Jinv_21*Jinv_21);
    const double G19_1_0_0_1_1_2_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_1*Jinv_21*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_1*Jinv_21*Jinv_01);
    const double G19_1_0_0_1_1_2_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_2*Jinv_21*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_2*Jinv_21*Jinv_11);
    const double G19_1_0_0_1_1_2_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_3*Jinv_21*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_3*Jinv_21*Jinv_21);
    const double G19_1_0_0_1_1_2_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_21*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_21*Jinv_01);
    const double G19_1_0_0_1_1_2_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_21*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_21*Jinv_11);
    const double G19_1_0_0_1_1_2_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_4*Jinv_21*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_4*Jinv_21*Jinv_21);
    const double G19_1_0_0_1_1_2_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_5*Jinv_21*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_5*Jinv_21*Jinv_01);
    const double G19_1_0_0_1_1_2_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_6*Jinv_21*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_6*Jinv_21*Jinv_11);
    const double G19_1_0_0_1_1_2_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_7*Jinv_21*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_7*Jinv_21*Jinv_21);
    const double G19_1_0_0_1_1_2_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_21*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_21*Jinv_01);
    const double G19_1_0_0_1_1_2_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_21*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_21*Jinv_11);
    const double G19_1_0_0_1_1_2_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_8*Jinv_21*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_8*Jinv_21*Jinv_21);
    const double G19_1_0_0_1_1_2_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_9*Jinv_21*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_9*Jinv_21*Jinv_01);
    const double G19_1_0_0_1_1_2_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_10*Jinv_21*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_10*Jinv_21*Jinv_11);
    const double G19_1_0_0_1_1_2_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_1*c1_19_4_11*Jinv_21*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_1*c1_20_4_11*Jinv_21*Jinv_21);
    const double G19_1_0_0_2_1_0_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_02*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_02*Jinv_01);
    const double G19_1_0_0_2_1_0_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_02*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_02*Jinv_11);
    const double G19_1_0_0_2_1_0_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_02*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_02*Jinv_21);
    const double G19_1_0_0_2_1_0_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_1*Jinv_02*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_1*Jinv_02*Jinv_01);
    const double G19_1_0_0_2_1_0_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_2*Jinv_02*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_2*Jinv_02*Jinv_11);
    const double G19_1_0_0_2_1_0_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_3*Jinv_02*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_3*Jinv_02*Jinv_21);
    const double G19_1_0_0_2_1_0_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_02*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_02*Jinv_01);
    const double G19_1_0_0_2_1_0_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_02*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_02*Jinv_11);
    const double G19_1_0_0_2_1_0_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_02*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_02*Jinv_21);
    const double G19_1_0_0_2_1_0_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_5*Jinv_02*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_5*Jinv_02*Jinv_01);
    const double G19_1_0_0_2_1_0_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_6*Jinv_02*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_6*Jinv_02*Jinv_11);
    const double G19_1_0_0_2_1_0_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_7*Jinv_02*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_7*Jinv_02*Jinv_21);
    const double G19_1_0_0_2_1_0_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_02*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_02*Jinv_01);
    const double G19_1_0_0_2_1_0_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_02*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_02*Jinv_11);
    const double G19_1_0_0_2_1_0_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_02*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_02*Jinv_21);
    const double G19_1_0_0_2_1_0_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_9*Jinv_02*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_9*Jinv_02*Jinv_01);
    const double G19_1_0_0_2_1_0_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_10*Jinv_02*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_10*Jinv_02*Jinv_11);
    const double G19_1_0_0_2_1_0_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_11*Jinv_02*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_11*Jinv_02*Jinv_21);
    const double G19_1_0_0_2_1_1_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_12*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_12*Jinv_01);
    const double G19_1_0_0_2_1_1_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_12*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_12*Jinv_11);
    const double G19_1_0_0_2_1_1_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_12*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_12*Jinv_21);
    const double G19_1_0_0_2_1_1_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_1*Jinv_12*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_1*Jinv_12*Jinv_01);
    const double G19_1_0_0_2_1_1_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_2*Jinv_12*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_2*Jinv_12*Jinv_11);
    const double G19_1_0_0_2_1_1_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_3*Jinv_12*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_3*Jinv_12*Jinv_21);
    const double G19_1_0_0_2_1_1_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_12*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_12*Jinv_01);
    const double G19_1_0_0_2_1_1_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_12*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_12*Jinv_11);
    const double G19_1_0_0_2_1_1_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_12*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_12*Jinv_21);
    const double G19_1_0_0_2_1_1_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_5*Jinv_12*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_5*Jinv_12*Jinv_01);
    const double G19_1_0_0_2_1_1_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_6*Jinv_12*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_6*Jinv_12*Jinv_11);
    const double G19_1_0_0_2_1_1_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_7*Jinv_12*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_7*Jinv_12*Jinv_21);
    const double G19_1_0_0_2_1_1_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_12*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_12*Jinv_01);
    const double G19_1_0_0_2_1_1_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_12*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_12*Jinv_11);
    const double G19_1_0_0_2_1_1_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_12*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_12*Jinv_21);
    const double G19_1_0_0_2_1_1_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_9*Jinv_12*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_9*Jinv_12*Jinv_01);
    const double G19_1_0_0_2_1_1_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_10*Jinv_12*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_10*Jinv_12*Jinv_11);
    const double G19_1_0_0_2_1_1_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_11*Jinv_12*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_11*Jinv_12*Jinv_21);
    const double G19_1_0_0_2_1_2_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_22*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_22*Jinv_01);
    const double G19_1_0_0_2_1_2_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_22*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_22*Jinv_11);
    const double G19_1_0_0_2_1_2_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_0*Jinv_22*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_0*Jinv_22*Jinv_21);
    const double G19_1_0_0_2_1_2_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_1*Jinv_22*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_1*Jinv_22*Jinv_01);
    const double G19_1_0_0_2_1_2_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_2*Jinv_22*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_2*Jinv_22*Jinv_11);
    const double G19_1_0_0_2_1_2_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_3*Jinv_22*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_3*Jinv_22*Jinv_21);
    const double G19_1_0_0_2_1_2_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_22*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_22*Jinv_01);
    const double G19_1_0_0_2_1_2_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_22*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_22*Jinv_11);
    const double G19_1_0_0_2_1_2_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_4*Jinv_22*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_4*Jinv_22*Jinv_21);
    const double G19_1_0_0_2_1_2_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_5*Jinv_22*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_5*Jinv_22*Jinv_01);
    const double G19_1_0_0_2_1_2_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_6*Jinv_22*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_6*Jinv_22*Jinv_11);
    const double G19_1_0_0_2_1_2_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_7*Jinv_22*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_7*Jinv_22*Jinv_21);
    const double G19_1_0_0_2_1_2_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_22*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_22*Jinv_01);
    const double G19_1_0_0_2_1_2_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_22*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_22*Jinv_11);
    const double G19_1_0_0_2_1_2_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_8*Jinv_22*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_8*Jinv_22*Jinv_21);
    const double G19_1_0_0_2_1_2_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_9*Jinv_22*Jinv_01 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_9*Jinv_22*Jinv_01);
    const double G19_1_0_0_2_1_2_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_10*Jinv_22*Jinv_11 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_10*Jinv_22*Jinv_11);
    const double G19_1_0_0_2_1_2_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_1*c1_19_4_11*Jinv_22*Jinv_21 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_1*c1_20_4_11*Jinv_22*Jinv_21);
    const double G19_2_0_0_0_2_0_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_00*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_00*Jinv_02);
    const double G19_2_0_0_0_2_0_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_00*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_00*Jinv_12);
    const double G19_2_0_0_0_2_0_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_00*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_00*Jinv_22);
    const double G19_2_0_0_0_2_0_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_1*Jinv_00*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_1*Jinv_00*Jinv_02);
    const double G19_2_0_0_0_2_0_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_2*Jinv_00*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_2*Jinv_00*Jinv_12);
    const double G19_2_0_0_0_2_0_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_3*Jinv_00*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_3*Jinv_00*Jinv_22);
    const double G19_2_0_0_0_2_0_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_00*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_00*Jinv_02);
    const double G19_2_0_0_0_2_0_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_00*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_00*Jinv_12);
    const double G19_2_0_0_0_2_0_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_00*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_00*Jinv_22);
    const double G19_2_0_0_0_2_0_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_5*Jinv_00*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_5*Jinv_00*Jinv_02);
    const double G19_2_0_0_0_2_0_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_6*Jinv_00*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_6*Jinv_00*Jinv_12);
    const double G19_2_0_0_0_2_0_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_7*Jinv_00*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_7*Jinv_00*Jinv_22);
    const double G19_2_0_0_0_2_0_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_00*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_00*Jinv_02);
    const double G19_2_0_0_0_2_0_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_00*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_00*Jinv_12);
    const double G19_2_0_0_0_2_0_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_00*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_00*Jinv_22);
    const double G19_2_0_0_0_2_0_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_9*Jinv_00*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_9*Jinv_00*Jinv_02);
    const double G19_2_0_0_0_2_0_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_10*Jinv_00*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_10*Jinv_00*Jinv_12);
    const double G19_2_0_0_0_2_0_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_11*Jinv_00*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_11*Jinv_00*Jinv_22);
    const double G19_2_0_0_0_2_1_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_10*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_10*Jinv_02);
    const double G19_2_0_0_0_2_1_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_10*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_10*Jinv_12);
    const double G19_2_0_0_0_2_1_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_10*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_10*Jinv_22);
    const double G19_2_0_0_0_2_1_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_1*Jinv_10*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_1*Jinv_10*Jinv_02);
    const double G19_2_0_0_0_2_1_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_2*Jinv_10*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_2*Jinv_10*Jinv_12);
    const double G19_2_0_0_0_2_1_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_3*Jinv_10*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_3*Jinv_10*Jinv_22);
    const double G19_2_0_0_0_2_1_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_10*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_10*Jinv_02);
    const double G19_2_0_0_0_2_1_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_10*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_10*Jinv_12);
    const double G19_2_0_0_0_2_1_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_10*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_10*Jinv_22);
    const double G19_2_0_0_0_2_1_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_5*Jinv_10*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_5*Jinv_10*Jinv_02);
    const double G19_2_0_0_0_2_1_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_6*Jinv_10*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_6*Jinv_10*Jinv_12);
    const double G19_2_0_0_0_2_1_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_7*Jinv_10*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_7*Jinv_10*Jinv_22);
    const double G19_2_0_0_0_2_1_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_10*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_10*Jinv_02);
    const double G19_2_0_0_0_2_1_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_10*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_10*Jinv_12);
    const double G19_2_0_0_0_2_1_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_10*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_10*Jinv_22);
    const double G19_2_0_0_0_2_1_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_9*Jinv_10*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_9*Jinv_10*Jinv_02);
    const double G19_2_0_0_0_2_1_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_10*Jinv_10*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_10*Jinv_10*Jinv_12);
    const double G19_2_0_0_0_2_1_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_11*Jinv_10*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_11*Jinv_10*Jinv_22);
    const double G19_2_0_0_0_2_2_0_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_20*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_20*Jinv_02);
    const double G19_2_0_0_0_2_2_0_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_20*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_20*Jinv_12);
    const double G19_2_0_0_0_2_2_0_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_0*Jinv_20*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_0*Jinv_20*Jinv_22);
    const double G19_2_0_0_0_2_2_0_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_1*Jinv_20*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_1*Jinv_20*Jinv_02);
    const double G19_2_0_0_0_2_2_0_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_2*Jinv_20*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_2*Jinv_20*Jinv_12);
    const double G19_2_0_0_0_2_2_0_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_3*Jinv_20*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_3*Jinv_20*Jinv_22);
    const double G19_2_0_0_0_2_2_0_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_20*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_20*Jinv_02);
    const double G19_2_0_0_0_2_2_0_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_20*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_20*Jinv_12);
    const double G19_2_0_0_0_2_2_0_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_4*Jinv_20*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_4*Jinv_20*Jinv_22);
    const double G19_2_0_0_0_2_2_0_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_5*Jinv_20*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_5*Jinv_20*Jinv_02);
    const double G19_2_0_0_0_2_2_0_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_6*Jinv_20*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_6*Jinv_20*Jinv_12);
    const double G19_2_0_0_0_2_2_0_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_7*Jinv_20*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_7*Jinv_20*Jinv_22);
    const double G19_2_0_0_0_2_2_0_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_20*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_20*Jinv_02);
    const double G19_2_0_0_0_2_2_0_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_20*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_20*Jinv_12);
    const double G19_2_0_0_0_2_2_0_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_8*Jinv_20*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_8*Jinv_20*Jinv_22);
    const double G19_2_0_0_0_2_2_0_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_9*Jinv_20*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_9*Jinv_20*Jinv_02);
    const double G19_2_0_0_0_2_2_0_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_10*Jinv_20*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_10*Jinv_20*Jinv_12);
    const double G19_2_0_0_0_2_2_0_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_0*c0_19_3_2*c1_19_4_11*Jinv_20*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_0*c6_20_3_2*c1_20_4_11*Jinv_20*Jinv_22);
    const double G19_2_0_0_1_2_0_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_01*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_01*Jinv_02);
    const double G19_2_0_0_1_2_0_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_01*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_01*Jinv_12);
    const double G19_2_0_0_1_2_0_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_01*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_01*Jinv_22);
    const double G19_2_0_0_1_2_0_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_1*Jinv_01*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_1*Jinv_01*Jinv_02);
    const double G19_2_0_0_1_2_0_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_2*Jinv_01*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_2*Jinv_01*Jinv_12);
    const double G19_2_0_0_1_2_0_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_3*Jinv_01*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_3*Jinv_01*Jinv_22);
    const double G19_2_0_0_1_2_0_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_01*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_01*Jinv_02);
    const double G19_2_0_0_1_2_0_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_01*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_01*Jinv_12);
    const double G19_2_0_0_1_2_0_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_01*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_01*Jinv_22);
    const double G19_2_0_0_1_2_0_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_5*Jinv_01*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_5*Jinv_01*Jinv_02);
    const double G19_2_0_0_1_2_0_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_6*Jinv_01*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_6*Jinv_01*Jinv_12);
    const double G19_2_0_0_1_2_0_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_7*Jinv_01*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_7*Jinv_01*Jinv_22);
    const double G19_2_0_0_1_2_0_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_01*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_01*Jinv_02);
    const double G19_2_0_0_1_2_0_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_01*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_01*Jinv_12);
    const double G19_2_0_0_1_2_0_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_01*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_01*Jinv_22);
    const double G19_2_0_0_1_2_0_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_9*Jinv_01*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_9*Jinv_01*Jinv_02);
    const double G19_2_0_0_1_2_0_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_10*Jinv_01*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_10*Jinv_01*Jinv_12);
    const double G19_2_0_0_1_2_0_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_11*Jinv_01*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_11*Jinv_01*Jinv_22);
    const double G19_2_0_0_1_2_1_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_11*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_11*Jinv_02);
    const double G19_2_0_0_1_2_1_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_11*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_11*Jinv_12);
    const double G19_2_0_0_1_2_1_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_11*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_11*Jinv_22);
    const double G19_2_0_0_1_2_1_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_1*Jinv_11*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_1*Jinv_11*Jinv_02);
    const double G19_2_0_0_1_2_1_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_2*Jinv_11*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_2*Jinv_11*Jinv_12);
    const double G19_2_0_0_1_2_1_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_3*Jinv_11*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_3*Jinv_11*Jinv_22);
    const double G19_2_0_0_1_2_1_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_11*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_11*Jinv_02);
    const double G19_2_0_0_1_2_1_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_11*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_11*Jinv_12);
    const double G19_2_0_0_1_2_1_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_11*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_11*Jinv_22);
    const double G19_2_0_0_1_2_1_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_5*Jinv_11*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_5*Jinv_11*Jinv_02);
    const double G19_2_0_0_1_2_1_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_6*Jinv_11*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_6*Jinv_11*Jinv_12);
    const double G19_2_0_0_1_2_1_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_7*Jinv_11*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_7*Jinv_11*Jinv_22);
    const double G19_2_0_0_1_2_1_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_11*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_11*Jinv_02);
    const double G19_2_0_0_1_2_1_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_11*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_11*Jinv_12);
    const double G19_2_0_0_1_2_1_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_11*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_11*Jinv_22);
    const double G19_2_0_0_1_2_1_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_9*Jinv_11*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_9*Jinv_11*Jinv_02);
    const double G19_2_0_0_1_2_1_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_10*Jinv_11*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_10*Jinv_11*Jinv_12);
    const double G19_2_0_0_1_2_1_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_11*Jinv_11*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_11*Jinv_11*Jinv_22);
    const double G19_2_0_0_1_2_2_1_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_21*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_21*Jinv_02);
    const double G19_2_0_0_1_2_2_1_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_21*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_21*Jinv_12);
    const double G19_2_0_0_1_2_2_1_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_0*Jinv_21*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_0*Jinv_21*Jinv_22);
    const double G19_2_0_0_1_2_2_1_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_1*Jinv_21*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_1*Jinv_21*Jinv_02);
    const double G19_2_0_0_1_2_2_1_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_2*Jinv_21*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_2*Jinv_21*Jinv_12);
    const double G19_2_0_0_1_2_2_1_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_3*Jinv_21*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_3*Jinv_21*Jinv_22);
    const double G19_2_0_0_1_2_2_1_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_21*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_21*Jinv_02);
    const double G19_2_0_0_1_2_2_1_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_21*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_21*Jinv_12);
    const double G19_2_0_0_1_2_2_1_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_4*Jinv_21*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_4*Jinv_21*Jinv_22);
    const double G19_2_0_0_1_2_2_1_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_5*Jinv_21*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_5*Jinv_21*Jinv_02);
    const double G19_2_0_0_1_2_2_1_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_6*Jinv_21*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_6*Jinv_21*Jinv_12);
    const double G19_2_0_0_1_2_2_1_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_7*Jinv_21*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_7*Jinv_21*Jinv_22);
    const double G19_2_0_0_1_2_2_1_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_21*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_21*Jinv_02);
    const double G19_2_0_0_1_2_2_1_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_21*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_21*Jinv_12);
    const double G19_2_0_0_1_2_2_1_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_8*Jinv_21*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_8*Jinv_21*Jinv_22);
    const double G19_2_0_0_1_2_2_1_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_9*Jinv_21*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_9*Jinv_21*Jinv_02);
    const double G19_2_0_0_1_2_2_1_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_10*Jinv_21*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_10*Jinv_21*Jinv_12);
    const double G19_2_0_0_1_2_2_1_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_1*c0_19_3_2*c1_19_4_11*Jinv_21*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_1*c6_20_3_2*c1_20_4_11*Jinv_21*Jinv_22);
    const double G19_2_0_0_2_2_0_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_02*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_02*Jinv_02);
    const double G19_2_0_0_2_2_0_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_02*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_02*Jinv_12);
    const double G19_2_0_0_2_2_0_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_02*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_02*Jinv_22);
    const double G19_2_0_0_2_2_0_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_1*Jinv_02*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_1*Jinv_02*Jinv_02);
    const double G19_2_0_0_2_2_0_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_2*Jinv_02*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_2*Jinv_02*Jinv_12);
    const double G19_2_0_0_2_2_0_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_3*Jinv_02*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_3*Jinv_02*Jinv_22);
    const double G19_2_0_0_2_2_0_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_02*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_02*Jinv_02);
    const double G19_2_0_0_2_2_0_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_02*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_02*Jinv_12);
    const double G19_2_0_0_2_2_0_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_02*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_02*Jinv_22);
    const double G19_2_0_0_2_2_0_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_5*Jinv_02*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_5*Jinv_02*Jinv_02);
    const double G19_2_0_0_2_2_0_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_6*Jinv_02*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_6*Jinv_02*Jinv_12);
    const double G19_2_0_0_2_2_0_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_7*Jinv_02*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_7*Jinv_02*Jinv_22);
    const double G19_2_0_0_2_2_0_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_02*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_02*Jinv_02);
    const double G19_2_0_0_2_2_0_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_02*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_02*Jinv_12);
    const double G19_2_0_0_2_2_0_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_02*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_02*Jinv_22);
    const double G19_2_0_0_2_2_0_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_9*Jinv_02*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_9*Jinv_02*Jinv_02);
    const double G19_2_0_0_2_2_0_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_10*Jinv_02*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_10*Jinv_02*Jinv_12);
    const double G19_2_0_0_2_2_0_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_11*Jinv_02*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_11*Jinv_02*Jinv_22);
    const double G19_2_0_0_2_2_1_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_12*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_12*Jinv_02);
    const double G19_2_0_0_2_2_1_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_12*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_12*Jinv_12);
    const double G19_2_0_0_2_2_1_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_12*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_12*Jinv_22);
    const double G19_2_0_0_2_2_1_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_1*Jinv_12*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_1*Jinv_12*Jinv_02);
    const double G19_2_0_0_2_2_1_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_2*Jinv_12*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_2*Jinv_12*Jinv_12);
    const double G19_2_0_0_2_2_1_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_3*Jinv_12*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_3*Jinv_12*Jinv_22);
    const double G19_2_0_0_2_2_1_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_12*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_12*Jinv_02);
    const double G19_2_0_0_2_2_1_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_12*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_12*Jinv_12);
    const double G19_2_0_0_2_2_1_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_12*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_12*Jinv_22);
    const double G19_2_0_0_2_2_1_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_5*Jinv_12*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_5*Jinv_12*Jinv_02);
    const double G19_2_0_0_2_2_1_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_6*Jinv_12*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_6*Jinv_12*Jinv_12);
    const double G19_2_0_0_2_2_1_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_7*Jinv_12*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_7*Jinv_12*Jinv_22);
    const double G19_2_0_0_2_2_1_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_12*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_12*Jinv_02);
    const double G19_2_0_0_2_2_1_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_12*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_12*Jinv_12);
    const double G19_2_0_0_2_2_1_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_12*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_12*Jinv_22);
    const double G19_2_0_0_2_2_1_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_9*Jinv_12*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_9*Jinv_12*Jinv_02);
    const double G19_2_0_0_2_2_1_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_10*Jinv_12*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_10*Jinv_12*Jinv_12);
    const double G19_2_0_0_2_2_1_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_11*Jinv_12*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_11*Jinv_12*Jinv_22);
    const double G19_2_0_0_2_2_2_2_0_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_22*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_22*Jinv_02);
    const double G19_2_0_0_2_2_2_2_0_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_22*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_22*Jinv_12);
    const double G19_2_0_0_2_2_2_2_0_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_0*Jinv_22*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_0*Jinv_22*Jinv_22);
    const double G19_2_0_0_2_2_2_2_1_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_1*Jinv_22*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_1*Jinv_22*Jinv_02);
    const double G19_2_0_0_2_2_2_2_2_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_2*Jinv_22*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_2*Jinv_22*Jinv_12);
    const double G19_2_0_0_2_2_2_2_3_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_3*Jinv_22*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_3*Jinv_22*Jinv_22);
    const double G19_2_0_0_2_2_2_2_4_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_22*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_22*Jinv_02);
    const double G19_2_0_0_2_2_2_2_4_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_22*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_22*Jinv_12);
    const double G19_2_0_0_2_2_2_2_4_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_4*Jinv_22*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_4*Jinv_22*Jinv_22);
    const double G19_2_0_0_2_2_2_2_5_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_5*Jinv_22*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_5*Jinv_22*Jinv_02);
    const double G19_2_0_0_2_2_2_2_6_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_6*Jinv_22*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_6*Jinv_22*Jinv_12);
    const double G19_2_0_0_2_2_2_2_7_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_7*Jinv_22*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_7*Jinv_22*Jinv_22);
    const double G19_2_0_0_2_2_2_2_8_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_22*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_22*Jinv_02);
    const double G19_2_0_0_2_2_2_2_8_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_22*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_22*Jinv_12);
    const double G19_2_0_0_2_2_2_2_8_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_8*Jinv_22*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_8*Jinv_22*Jinv_22);
    const double G19_2_0_0_2_2_2_2_9_0 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_9*Jinv_22*Jinv_02 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_9*Jinv_22*Jinv_02);
    const double G19_2_0_0_2_2_2_2_10_1 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_10*Jinv_22*Jinv_12 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_10*Jinv_22*Jinv_12);
    const double G19_2_0_0_2_2_2_2_11_2 = det*(c10_19_0_0*c4_19_1_0*c0_19_2_2*c0_19_3_2*c1_19_4_11*Jinv_22*Jinv_22 + c10_20_0_0*c4_20_1_0*c6_20_2_2*c6_20_3_2*c1_20_4_11*Jinv_22*Jinv_22);
    const double G20_0_0_0_0_0_0_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_00*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_00*Jinv_00);
    const double G20_0_0_0_0_0_0_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_00*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_00*Jinv_10);
    const double G20_0_0_0_0_0_0_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_00*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_00*Jinv_20);
    const double G20_0_0_0_0_0_0_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_1*Jinv_00*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_1*Jinv_00*Jinv_00);
    const double G20_0_0_0_0_0_0_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_2*Jinv_00*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_2*Jinv_00*Jinv_10);
    const double G20_0_0_0_0_0_0_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_3*Jinv_00*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_3*Jinv_00*Jinv_20);
    const double G20_0_0_0_0_0_0_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_00*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_00*Jinv_00);
    const double G20_0_0_0_0_0_0_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_00*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_00*Jinv_10);
    const double G20_0_0_0_0_0_0_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_00*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_00*Jinv_20);
    const double G20_0_0_0_0_0_0_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_5*Jinv_00*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_5*Jinv_00*Jinv_00);
    const double G20_0_0_0_0_0_0_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_6*Jinv_00*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_6*Jinv_00*Jinv_10);
    const double G20_0_0_0_0_0_0_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_7*Jinv_00*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_7*Jinv_00*Jinv_20);
    const double G20_0_0_0_0_0_0_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_00*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_00*Jinv_00);
    const double G20_0_0_0_0_0_0_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_00*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_00*Jinv_10);
    const double G20_0_0_0_0_0_0_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_00*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_00*Jinv_20);
    const double G20_0_0_0_0_0_0_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_9*Jinv_00*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_9*Jinv_00*Jinv_00);
    const double G20_0_0_0_0_0_0_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_10*Jinv_00*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_10*Jinv_00*Jinv_10);
    const double G20_0_0_0_0_0_0_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_11*Jinv_00*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_11*Jinv_00*Jinv_20);
    const double G20_0_0_0_0_0_1_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_10*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_10*Jinv_00);
    const double G20_0_0_0_0_0_1_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_10*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_10*Jinv_10);
    const double G20_0_0_0_0_0_1_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_10*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_10*Jinv_20);
    const double G20_0_0_0_0_0_1_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_1*Jinv_10*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_1*Jinv_10*Jinv_00);
    const double G20_0_0_0_0_0_1_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_2*Jinv_10*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_2*Jinv_10*Jinv_10);
    const double G20_0_0_0_0_0_1_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_3*Jinv_10*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_3*Jinv_10*Jinv_20);
    const double G20_0_0_0_0_0_1_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_10*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_10*Jinv_00);
    const double G20_0_0_0_0_0_1_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_10*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_10*Jinv_10);
    const double G20_0_0_0_0_0_1_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_10*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_10*Jinv_20);
    const double G20_0_0_0_0_0_1_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_5*Jinv_10*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_5*Jinv_10*Jinv_00);
    const double G20_0_0_0_0_0_1_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_6*Jinv_10*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_6*Jinv_10*Jinv_10);
    const double G20_0_0_0_0_0_1_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_7*Jinv_10*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_7*Jinv_10*Jinv_20);
    const double G20_0_0_0_0_0_1_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_10*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_10*Jinv_00);
    const double G20_0_0_0_0_0_1_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_10*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_10*Jinv_10);
    const double G20_0_0_0_0_0_1_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_10*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_10*Jinv_20);
    const double G20_0_0_0_0_0_1_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_9*Jinv_10*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_9*Jinv_10*Jinv_00);
    const double G20_0_0_0_0_0_1_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_10*Jinv_10*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_10*Jinv_10*Jinv_10);
    const double G20_0_0_0_0_0_1_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_11*Jinv_10*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_11*Jinv_10*Jinv_20);
    const double G20_0_0_0_0_0_2_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_20*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_20*Jinv_00);
    const double G20_0_0_0_0_0_2_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_20*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_20*Jinv_10);
    const double G20_0_0_0_0_0_2_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_0*Jinv_20*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_0*Jinv_20*Jinv_20);
    const double G20_0_0_0_0_0_2_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_1*Jinv_20*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_1*Jinv_20*Jinv_00);
    const double G20_0_0_0_0_0_2_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_2*Jinv_20*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_2*Jinv_20*Jinv_10);
    const double G20_0_0_0_0_0_2_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_3*Jinv_20*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_3*Jinv_20*Jinv_20);
    const double G20_0_0_0_0_0_2_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_20*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_20*Jinv_00);
    const double G20_0_0_0_0_0_2_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_20*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_20*Jinv_10);
    const double G20_0_0_0_0_0_2_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_4*Jinv_20*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_4*Jinv_20*Jinv_20);
    const double G20_0_0_0_0_0_2_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_5*Jinv_20*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_5*Jinv_20*Jinv_00);
    const double G20_0_0_0_0_0_2_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_6*Jinv_20*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_6*Jinv_20*Jinv_10);
    const double G20_0_0_0_0_0_2_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_7*Jinv_20*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_7*Jinv_20*Jinv_20);
    const double G20_0_0_0_0_0_2_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_20*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_20*Jinv_00);
    const double G20_0_0_0_0_0_2_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_20*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_20*Jinv_10);
    const double G20_0_0_0_0_0_2_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_8*Jinv_20*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_8*Jinv_20*Jinv_20);
    const double G20_0_0_0_0_0_2_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_9*Jinv_20*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_9*Jinv_20*Jinv_00);
    const double G20_0_0_0_0_0_2_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_10*Jinv_20*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_10*Jinv_20*Jinv_10);
    const double G20_0_0_0_0_0_2_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_0*c1_21_4_11*Jinv_20*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_0*c1_22_4_11*Jinv_20*Jinv_20);
    const double G20_0_0_0_1_0_0_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_01*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_01*Jinv_00);
    const double G20_0_0_0_1_0_0_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_01*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_01*Jinv_10);
    const double G20_0_0_0_1_0_0_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_01*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_01*Jinv_20);
    const double G20_0_0_0_1_0_0_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_1*Jinv_01*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_1*Jinv_01*Jinv_00);
    const double G20_0_0_0_1_0_0_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_2*Jinv_01*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_2*Jinv_01*Jinv_10);
    const double G20_0_0_0_1_0_0_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_3*Jinv_01*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_3*Jinv_01*Jinv_20);
    const double G20_0_0_0_1_0_0_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_01*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_01*Jinv_00);
    const double G20_0_0_0_1_0_0_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_01*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_01*Jinv_10);
    const double G20_0_0_0_1_0_0_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_01*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_01*Jinv_20);
    const double G20_0_0_0_1_0_0_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_5*Jinv_01*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_5*Jinv_01*Jinv_00);
    const double G20_0_0_0_1_0_0_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_6*Jinv_01*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_6*Jinv_01*Jinv_10);
    const double G20_0_0_0_1_0_0_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_7*Jinv_01*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_7*Jinv_01*Jinv_20);
    const double G20_0_0_0_1_0_0_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_01*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_01*Jinv_00);
    const double G20_0_0_0_1_0_0_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_01*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_01*Jinv_10);
    const double G20_0_0_0_1_0_0_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_01*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_01*Jinv_20);
    const double G20_0_0_0_1_0_0_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_9*Jinv_01*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_9*Jinv_01*Jinv_00);
    const double G20_0_0_0_1_0_0_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_10*Jinv_01*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_10*Jinv_01*Jinv_10);
    const double G20_0_0_0_1_0_0_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_11*Jinv_01*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_11*Jinv_01*Jinv_20);
    const double G20_0_0_0_1_0_1_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_11*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_11*Jinv_00);
    const double G20_0_0_0_1_0_1_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_11*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_11*Jinv_10);
    const double G20_0_0_0_1_0_1_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_11*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_11*Jinv_20);
    const double G20_0_0_0_1_0_1_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_1*Jinv_11*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_1*Jinv_11*Jinv_00);
    const double G20_0_0_0_1_0_1_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_2*Jinv_11*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_2*Jinv_11*Jinv_10);
    const double G20_0_0_0_1_0_1_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_3*Jinv_11*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_3*Jinv_11*Jinv_20);
    const double G20_0_0_0_1_0_1_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_11*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_11*Jinv_00);
    const double G20_0_0_0_1_0_1_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_11*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_11*Jinv_10);
    const double G20_0_0_0_1_0_1_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_11*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_11*Jinv_20);
    const double G20_0_0_0_1_0_1_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_5*Jinv_11*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_5*Jinv_11*Jinv_00);
    const double G20_0_0_0_1_0_1_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_6*Jinv_11*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_6*Jinv_11*Jinv_10);
    const double G20_0_0_0_1_0_1_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_7*Jinv_11*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_7*Jinv_11*Jinv_20);
    const double G20_0_0_0_1_0_1_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_11*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_11*Jinv_00);
    const double G20_0_0_0_1_0_1_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_11*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_11*Jinv_10);
    const double G20_0_0_0_1_0_1_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_11*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_11*Jinv_20);
    const double G20_0_0_0_1_0_1_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_9*Jinv_11*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_9*Jinv_11*Jinv_00);
    const double G20_0_0_0_1_0_1_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_10*Jinv_11*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_10*Jinv_11*Jinv_10);
    const double G20_0_0_0_1_0_1_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_11*Jinv_11*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_11*Jinv_11*Jinv_20);
    const double G20_0_0_0_1_0_2_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_21*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_21*Jinv_00);
    const double G20_0_0_0_1_0_2_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_21*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_21*Jinv_10);
    const double G20_0_0_0_1_0_2_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_0*Jinv_21*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_0*Jinv_21*Jinv_20);
    const double G20_0_0_0_1_0_2_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_1*Jinv_21*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_1*Jinv_21*Jinv_00);
    const double G20_0_0_0_1_0_2_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_2*Jinv_21*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_2*Jinv_21*Jinv_10);
    const double G20_0_0_0_1_0_2_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_3*Jinv_21*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_3*Jinv_21*Jinv_20);
    const double G20_0_0_0_1_0_2_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_21*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_21*Jinv_00);
    const double G20_0_0_0_1_0_2_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_21*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_21*Jinv_10);
    const double G20_0_0_0_1_0_2_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_4*Jinv_21*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_4*Jinv_21*Jinv_20);
    const double G20_0_0_0_1_0_2_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_5*Jinv_21*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_5*Jinv_21*Jinv_00);
    const double G20_0_0_0_1_0_2_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_6*Jinv_21*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_6*Jinv_21*Jinv_10);
    const double G20_0_0_0_1_0_2_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_7*Jinv_21*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_7*Jinv_21*Jinv_20);
    const double G20_0_0_0_1_0_2_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_21*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_21*Jinv_00);
    const double G20_0_0_0_1_0_2_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_21*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_21*Jinv_10);
    const double G20_0_0_0_1_0_2_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_8*Jinv_21*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_8*Jinv_21*Jinv_20);
    const double G20_0_0_0_1_0_2_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_9*Jinv_21*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_9*Jinv_21*Jinv_00);
    const double G20_0_0_0_1_0_2_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_10*Jinv_21*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_10*Jinv_21*Jinv_10);
    const double G20_0_0_0_1_0_2_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_0*c1_21_4_11*Jinv_21*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_0*c1_22_4_11*Jinv_21*Jinv_20);
    const double G20_0_0_0_2_0_0_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_02*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_02*Jinv_00);
    const double G20_0_0_0_2_0_0_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_02*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_02*Jinv_10);
    const double G20_0_0_0_2_0_0_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_02*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_02*Jinv_20);
    const double G20_0_0_0_2_0_0_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_1*Jinv_02*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_1*Jinv_02*Jinv_00);
    const double G20_0_0_0_2_0_0_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_2*Jinv_02*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_2*Jinv_02*Jinv_10);
    const double G20_0_0_0_2_0_0_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_3*Jinv_02*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_3*Jinv_02*Jinv_20);
    const double G20_0_0_0_2_0_0_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_02*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_02*Jinv_00);
    const double G20_0_0_0_2_0_0_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_02*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_02*Jinv_10);
    const double G20_0_0_0_2_0_0_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_02*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_02*Jinv_20);
    const double G20_0_0_0_2_0_0_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_5*Jinv_02*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_5*Jinv_02*Jinv_00);
    const double G20_0_0_0_2_0_0_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_6*Jinv_02*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_6*Jinv_02*Jinv_10);
    const double G20_0_0_0_2_0_0_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_7*Jinv_02*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_7*Jinv_02*Jinv_20);
    const double G20_0_0_0_2_0_0_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_02*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_02*Jinv_00);
    const double G20_0_0_0_2_0_0_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_02*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_02*Jinv_10);
    const double G20_0_0_0_2_0_0_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_02*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_02*Jinv_20);
    const double G20_0_0_0_2_0_0_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_9*Jinv_02*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_9*Jinv_02*Jinv_00);
    const double G20_0_0_0_2_0_0_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_10*Jinv_02*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_10*Jinv_02*Jinv_10);
    const double G20_0_0_0_2_0_0_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_11*Jinv_02*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_11*Jinv_02*Jinv_20);
    const double G20_0_0_0_2_0_1_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_12*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_12*Jinv_00);
    const double G20_0_0_0_2_0_1_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_12*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_12*Jinv_10);
    const double G20_0_0_0_2_0_1_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_12*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_12*Jinv_20);
    const double G20_0_0_0_2_0_1_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_1*Jinv_12*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_1*Jinv_12*Jinv_00);
    const double G20_0_0_0_2_0_1_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_2*Jinv_12*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_2*Jinv_12*Jinv_10);
    const double G20_0_0_0_2_0_1_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_3*Jinv_12*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_3*Jinv_12*Jinv_20);
    const double G20_0_0_0_2_0_1_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_12*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_12*Jinv_00);
    const double G20_0_0_0_2_0_1_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_12*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_12*Jinv_10);
    const double G20_0_0_0_2_0_1_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_12*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_12*Jinv_20);
    const double G20_0_0_0_2_0_1_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_5*Jinv_12*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_5*Jinv_12*Jinv_00);
    const double G20_0_0_0_2_0_1_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_6*Jinv_12*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_6*Jinv_12*Jinv_10);
    const double G20_0_0_0_2_0_1_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_7*Jinv_12*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_7*Jinv_12*Jinv_20);
    const double G20_0_0_0_2_0_1_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_12*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_12*Jinv_00);
    const double G20_0_0_0_2_0_1_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_12*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_12*Jinv_10);
    const double G20_0_0_0_2_0_1_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_12*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_12*Jinv_20);
    const double G20_0_0_0_2_0_1_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_9*Jinv_12*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_9*Jinv_12*Jinv_00);
    const double G20_0_0_0_2_0_1_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_10*Jinv_12*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_10*Jinv_12*Jinv_10);
    const double G20_0_0_0_2_0_1_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_11*Jinv_12*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_11*Jinv_12*Jinv_20);
    const double G20_0_0_0_2_0_2_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_22*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_22*Jinv_00);
    const double G20_0_0_0_2_0_2_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_22*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_22*Jinv_10);
    const double G20_0_0_0_2_0_2_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_0*Jinv_22*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_0*Jinv_22*Jinv_20);
    const double G20_0_0_0_2_0_2_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_1*Jinv_22*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_1*Jinv_22*Jinv_00);
    const double G20_0_0_0_2_0_2_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_2*Jinv_22*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_2*Jinv_22*Jinv_10);
    const double G20_0_0_0_2_0_2_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_3*Jinv_22*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_3*Jinv_22*Jinv_20);
    const double G20_0_0_0_2_0_2_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_22*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_22*Jinv_00);
    const double G20_0_0_0_2_0_2_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_22*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_22*Jinv_10);
    const double G20_0_0_0_2_0_2_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_4*Jinv_22*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_4*Jinv_22*Jinv_20);
    const double G20_0_0_0_2_0_2_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_5*Jinv_22*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_5*Jinv_22*Jinv_00);
    const double G20_0_0_0_2_0_2_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_6*Jinv_22*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_6*Jinv_22*Jinv_10);
    const double G20_0_0_0_2_0_2_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_7*Jinv_22*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_7*Jinv_22*Jinv_20);
    const double G20_0_0_0_2_0_2_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_22*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_22*Jinv_00);
    const double G20_0_0_0_2_0_2_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_22*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_22*Jinv_10);
    const double G20_0_0_0_2_0_2_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_8*Jinv_22*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_8*Jinv_22*Jinv_20);
    const double G20_0_0_0_2_0_2_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_9*Jinv_22*Jinv_00 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_9*Jinv_22*Jinv_00);
    const double G20_0_0_0_2_0_2_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_10*Jinv_22*Jinv_10 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_10*Jinv_22*Jinv_10);
    const double G20_0_0_0_2_0_2_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_0*c1_21_4_11*Jinv_22*Jinv_20 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_0*c1_22_4_11*Jinv_22*Jinv_20);
    const double G20_1_0_0_0_1_0_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_00*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_00*Jinv_01);
    const double G20_1_0_0_0_1_0_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_00*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_00*Jinv_11);
    const double G20_1_0_0_0_1_0_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_00*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_00*Jinv_21);
    const double G20_1_0_0_0_1_0_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_1*Jinv_00*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_1*Jinv_00*Jinv_01);
    const double G20_1_0_0_0_1_0_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_2*Jinv_00*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_2*Jinv_00*Jinv_11);
    const double G20_1_0_0_0_1_0_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_3*Jinv_00*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_3*Jinv_00*Jinv_21);
    const double G20_1_0_0_0_1_0_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_00*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_00*Jinv_01);
    const double G20_1_0_0_0_1_0_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_00*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_00*Jinv_11);
    const double G20_1_0_0_0_1_0_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_00*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_00*Jinv_21);
    const double G20_1_0_0_0_1_0_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_5*Jinv_00*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_5*Jinv_00*Jinv_01);
    const double G20_1_0_0_0_1_0_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_6*Jinv_00*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_6*Jinv_00*Jinv_11);
    const double G20_1_0_0_0_1_0_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_7*Jinv_00*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_7*Jinv_00*Jinv_21);
    const double G20_1_0_0_0_1_0_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_00*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_00*Jinv_01);
    const double G20_1_0_0_0_1_0_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_00*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_00*Jinv_11);
    const double G20_1_0_0_0_1_0_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_00*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_00*Jinv_21);
    const double G20_1_0_0_0_1_0_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_9*Jinv_00*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_9*Jinv_00*Jinv_01);
    const double G20_1_0_0_0_1_0_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_10*Jinv_00*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_10*Jinv_00*Jinv_11);
    const double G20_1_0_0_0_1_0_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_11*Jinv_00*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_11*Jinv_00*Jinv_21);
    const double G20_1_0_0_0_1_1_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_10*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_10*Jinv_01);
    const double G20_1_0_0_0_1_1_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_10*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_10*Jinv_11);
    const double G20_1_0_0_0_1_1_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_10*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_10*Jinv_21);
    const double G20_1_0_0_0_1_1_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_1*Jinv_10*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_1*Jinv_10*Jinv_01);
    const double G20_1_0_0_0_1_1_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_2*Jinv_10*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_2*Jinv_10*Jinv_11);
    const double G20_1_0_0_0_1_1_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_3*Jinv_10*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_3*Jinv_10*Jinv_21);
    const double G20_1_0_0_0_1_1_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_10*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_10*Jinv_01);
    const double G20_1_0_0_0_1_1_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_10*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_10*Jinv_11);
    const double G20_1_0_0_0_1_1_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_10*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_10*Jinv_21);
    const double G20_1_0_0_0_1_1_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_5*Jinv_10*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_5*Jinv_10*Jinv_01);
    const double G20_1_0_0_0_1_1_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_6*Jinv_10*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_6*Jinv_10*Jinv_11);
    const double G20_1_0_0_0_1_1_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_7*Jinv_10*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_7*Jinv_10*Jinv_21);
    const double G20_1_0_0_0_1_1_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_10*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_10*Jinv_01);
    const double G20_1_0_0_0_1_1_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_10*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_10*Jinv_11);
    const double G20_1_0_0_0_1_1_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_10*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_10*Jinv_21);
    const double G20_1_0_0_0_1_1_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_9*Jinv_10*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_9*Jinv_10*Jinv_01);
    const double G20_1_0_0_0_1_1_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_10*Jinv_10*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_10*Jinv_10*Jinv_11);
    const double G20_1_0_0_0_1_1_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_11*Jinv_10*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_11*Jinv_10*Jinv_21);
    const double G20_1_0_0_0_1_2_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_20*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_20*Jinv_01);
    const double G20_1_0_0_0_1_2_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_20*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_20*Jinv_11);
    const double G20_1_0_0_0_1_2_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_0*Jinv_20*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_0*Jinv_20*Jinv_21);
    const double G20_1_0_0_0_1_2_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_1*Jinv_20*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_1*Jinv_20*Jinv_01);
    const double G20_1_0_0_0_1_2_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_2*Jinv_20*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_2*Jinv_20*Jinv_11);
    const double G20_1_0_0_0_1_2_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_3*Jinv_20*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_3*Jinv_20*Jinv_21);
    const double G20_1_0_0_0_1_2_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_20*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_20*Jinv_01);
    const double G20_1_0_0_0_1_2_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_20*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_20*Jinv_11);
    const double G20_1_0_0_0_1_2_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_4*Jinv_20*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_4*Jinv_20*Jinv_21);
    const double G20_1_0_0_0_1_2_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_5*Jinv_20*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_5*Jinv_20*Jinv_01);
    const double G20_1_0_0_0_1_2_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_6*Jinv_20*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_6*Jinv_20*Jinv_11);
    const double G20_1_0_0_0_1_2_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_7*Jinv_20*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_7*Jinv_20*Jinv_21);
    const double G20_1_0_0_0_1_2_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_20*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_20*Jinv_01);
    const double G20_1_0_0_0_1_2_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_20*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_20*Jinv_11);
    const double G20_1_0_0_0_1_2_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_8*Jinv_20*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_8*Jinv_20*Jinv_21);
    const double G20_1_0_0_0_1_2_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_9*Jinv_20*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_9*Jinv_20*Jinv_01);
    const double G20_1_0_0_0_1_2_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_10*Jinv_20*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_10*Jinv_20*Jinv_11);
    const double G20_1_0_0_0_1_2_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_1*c1_21_4_11*Jinv_20*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_1*c1_22_4_11*Jinv_20*Jinv_21);
    const double G20_1_0_0_1_1_0_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_01*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_01*Jinv_01);
    const double G20_1_0_0_1_1_0_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_01*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_01*Jinv_11);
    const double G20_1_0_0_1_1_0_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_01*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_01*Jinv_21);
    const double G20_1_0_0_1_1_0_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_1*Jinv_01*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_1*Jinv_01*Jinv_01);
    const double G20_1_0_0_1_1_0_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_2*Jinv_01*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_2*Jinv_01*Jinv_11);
    const double G20_1_0_0_1_1_0_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_3*Jinv_01*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_3*Jinv_01*Jinv_21);
    const double G20_1_0_0_1_1_0_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_01*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_01*Jinv_01);
    const double G20_1_0_0_1_1_0_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_01*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_01*Jinv_11);
    const double G20_1_0_0_1_1_0_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_01*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_01*Jinv_21);
    const double G20_1_0_0_1_1_0_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_5*Jinv_01*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_5*Jinv_01*Jinv_01);
    const double G20_1_0_0_1_1_0_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_6*Jinv_01*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_6*Jinv_01*Jinv_11);
    const double G20_1_0_0_1_1_0_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_7*Jinv_01*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_7*Jinv_01*Jinv_21);
    const double G20_1_0_0_1_1_0_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_01*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_01*Jinv_01);
    const double G20_1_0_0_1_1_0_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_01*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_01*Jinv_11);
    const double G20_1_0_0_1_1_0_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_01*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_01*Jinv_21);
    const double G20_1_0_0_1_1_0_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_9*Jinv_01*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_9*Jinv_01*Jinv_01);
    const double G20_1_0_0_1_1_0_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_10*Jinv_01*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_10*Jinv_01*Jinv_11);
    const double G20_1_0_0_1_1_0_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_11*Jinv_01*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_11*Jinv_01*Jinv_21);
    const double G20_1_0_0_1_1_1_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_11*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_11*Jinv_01);
    const double G20_1_0_0_1_1_1_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_11*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_11*Jinv_11);
    const double G20_1_0_0_1_1_1_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_11*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_11*Jinv_21);
    const double G20_1_0_0_1_1_1_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_1*Jinv_11*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_1*Jinv_11*Jinv_01);
    const double G20_1_0_0_1_1_1_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_2*Jinv_11*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_2*Jinv_11*Jinv_11);
    const double G20_1_0_0_1_1_1_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_3*Jinv_11*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_3*Jinv_11*Jinv_21);
    const double G20_1_0_0_1_1_1_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_11*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_11*Jinv_01);
    const double G20_1_0_0_1_1_1_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_11*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_11*Jinv_11);
    const double G20_1_0_0_1_1_1_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_11*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_11*Jinv_21);
    const double G20_1_0_0_1_1_1_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_5*Jinv_11*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_5*Jinv_11*Jinv_01);
    const double G20_1_0_0_1_1_1_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_6*Jinv_11*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_6*Jinv_11*Jinv_11);
    const double G20_1_0_0_1_1_1_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_7*Jinv_11*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_7*Jinv_11*Jinv_21);
    const double G20_1_0_0_1_1_1_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_11*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_11*Jinv_01);
    const double G20_1_0_0_1_1_1_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_11*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_11*Jinv_11);
    const double G20_1_0_0_1_1_1_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_11*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_11*Jinv_21);
    const double G20_1_0_0_1_1_1_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_9*Jinv_11*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_9*Jinv_11*Jinv_01);
    const double G20_1_0_0_1_1_1_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_10*Jinv_11*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_10*Jinv_11*Jinv_11);
    const double G20_1_0_0_1_1_1_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_11*Jinv_11*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_11*Jinv_11*Jinv_21);
    const double G20_1_0_0_1_1_2_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_21*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_21*Jinv_01);
    const double G20_1_0_0_1_1_2_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_21*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_21*Jinv_11);
    const double G20_1_0_0_1_1_2_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_0*Jinv_21*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_0*Jinv_21*Jinv_21);
    const double G20_1_0_0_1_1_2_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_1*Jinv_21*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_1*Jinv_21*Jinv_01);
    const double G20_1_0_0_1_1_2_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_2*Jinv_21*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_2*Jinv_21*Jinv_11);
    const double G20_1_0_0_1_1_2_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_3*Jinv_21*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_3*Jinv_21*Jinv_21);
    const double G20_1_0_0_1_1_2_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_21*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_21*Jinv_01);
    const double G20_1_0_0_1_1_2_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_21*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_21*Jinv_11);
    const double G20_1_0_0_1_1_2_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_4*Jinv_21*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_4*Jinv_21*Jinv_21);
    const double G20_1_0_0_1_1_2_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_5*Jinv_21*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_5*Jinv_21*Jinv_01);
    const double G20_1_0_0_1_1_2_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_6*Jinv_21*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_6*Jinv_21*Jinv_11);
    const double G20_1_0_0_1_1_2_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_7*Jinv_21*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_7*Jinv_21*Jinv_21);
    const double G20_1_0_0_1_1_2_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_21*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_21*Jinv_01);
    const double G20_1_0_0_1_1_2_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_21*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_21*Jinv_11);
    const double G20_1_0_0_1_1_2_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_8*Jinv_21*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_8*Jinv_21*Jinv_21);
    const double G20_1_0_0_1_1_2_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_9*Jinv_21*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_9*Jinv_21*Jinv_01);
    const double G20_1_0_0_1_1_2_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_10*Jinv_21*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_10*Jinv_21*Jinv_11);
    const double G20_1_0_0_1_1_2_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_1*c1_21_4_11*Jinv_21*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_1*c1_22_4_11*Jinv_21*Jinv_21);
    const double G20_1_0_0_2_1_0_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_02*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_02*Jinv_01);
    const double G20_1_0_0_2_1_0_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_02*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_02*Jinv_11);
    const double G20_1_0_0_2_1_0_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_02*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_02*Jinv_21);
    const double G20_1_0_0_2_1_0_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_1*Jinv_02*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_1*Jinv_02*Jinv_01);
    const double G20_1_0_0_2_1_0_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_2*Jinv_02*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_2*Jinv_02*Jinv_11);
    const double G20_1_0_0_2_1_0_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_3*Jinv_02*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_3*Jinv_02*Jinv_21);
    const double G20_1_0_0_2_1_0_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_02*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_02*Jinv_01);
    const double G20_1_0_0_2_1_0_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_02*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_02*Jinv_11);
    const double G20_1_0_0_2_1_0_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_02*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_02*Jinv_21);
    const double G20_1_0_0_2_1_0_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_5*Jinv_02*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_5*Jinv_02*Jinv_01);
    const double G20_1_0_0_2_1_0_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_6*Jinv_02*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_6*Jinv_02*Jinv_11);
    const double G20_1_0_0_2_1_0_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_7*Jinv_02*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_7*Jinv_02*Jinv_21);
    const double G20_1_0_0_2_1_0_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_02*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_02*Jinv_01);
    const double G20_1_0_0_2_1_0_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_02*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_02*Jinv_11);
    const double G20_1_0_0_2_1_0_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_02*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_02*Jinv_21);
    const double G20_1_0_0_2_1_0_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_9*Jinv_02*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_9*Jinv_02*Jinv_01);
    const double G20_1_0_0_2_1_0_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_10*Jinv_02*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_10*Jinv_02*Jinv_11);
    const double G20_1_0_0_2_1_0_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_11*Jinv_02*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_11*Jinv_02*Jinv_21);
    const double G20_1_0_0_2_1_1_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_12*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_12*Jinv_01);
    const double G20_1_0_0_2_1_1_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_12*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_12*Jinv_11);
    const double G20_1_0_0_2_1_1_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_12*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_12*Jinv_21);
    const double G20_1_0_0_2_1_1_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_1*Jinv_12*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_1*Jinv_12*Jinv_01);
    const double G20_1_0_0_2_1_1_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_2*Jinv_12*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_2*Jinv_12*Jinv_11);
    const double G20_1_0_0_2_1_1_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_3*Jinv_12*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_3*Jinv_12*Jinv_21);
    const double G20_1_0_0_2_1_1_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_12*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_12*Jinv_01);
    const double G20_1_0_0_2_1_1_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_12*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_12*Jinv_11);
    const double G20_1_0_0_2_1_1_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_12*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_12*Jinv_21);
    const double G20_1_0_0_2_1_1_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_5*Jinv_12*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_5*Jinv_12*Jinv_01);
    const double G20_1_0_0_2_1_1_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_6*Jinv_12*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_6*Jinv_12*Jinv_11);
    const double G20_1_0_0_2_1_1_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_7*Jinv_12*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_7*Jinv_12*Jinv_21);
    const double G20_1_0_0_2_1_1_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_12*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_12*Jinv_01);
    const double G20_1_0_0_2_1_1_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_12*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_12*Jinv_11);
    const double G20_1_0_0_2_1_1_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_12*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_12*Jinv_21);
    const double G20_1_0_0_2_1_1_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_9*Jinv_12*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_9*Jinv_12*Jinv_01);
    const double G20_1_0_0_2_1_1_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_10*Jinv_12*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_10*Jinv_12*Jinv_11);
    const double G20_1_0_0_2_1_1_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_11*Jinv_12*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_11*Jinv_12*Jinv_21);
    const double G20_1_0_0_2_1_2_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_22*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_22*Jinv_01);
    const double G20_1_0_0_2_1_2_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_22*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_22*Jinv_11);
    const double G20_1_0_0_2_1_2_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_0*Jinv_22*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_0*Jinv_22*Jinv_21);
    const double G20_1_0_0_2_1_2_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_1*Jinv_22*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_1*Jinv_22*Jinv_01);
    const double G20_1_0_0_2_1_2_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_2*Jinv_22*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_2*Jinv_22*Jinv_11);
    const double G20_1_0_0_2_1_2_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_3*Jinv_22*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_3*Jinv_22*Jinv_21);
    const double G20_1_0_0_2_1_2_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_22*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_22*Jinv_01);
    const double G20_1_0_0_2_1_2_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_22*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_22*Jinv_11);
    const double G20_1_0_0_2_1_2_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_4*Jinv_22*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_4*Jinv_22*Jinv_21);
    const double G20_1_0_0_2_1_2_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_5*Jinv_22*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_5*Jinv_22*Jinv_01);
    const double G20_1_0_0_2_1_2_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_6*Jinv_22*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_6*Jinv_22*Jinv_11);
    const double G20_1_0_0_2_1_2_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_7*Jinv_22*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_7*Jinv_22*Jinv_21);
    const double G20_1_0_0_2_1_2_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_22*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_22*Jinv_01);
    const double G20_1_0_0_2_1_2_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_22*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_22*Jinv_11);
    const double G20_1_0_0_2_1_2_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_8*Jinv_22*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_8*Jinv_22*Jinv_21);
    const double G20_1_0_0_2_1_2_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_9*Jinv_22*Jinv_01 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_9*Jinv_22*Jinv_01);
    const double G20_1_0_0_2_1_2_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_10*Jinv_22*Jinv_11 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_10*Jinv_22*Jinv_11);
    const double G20_1_0_0_2_1_2_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_1*c1_21_4_11*Jinv_22*Jinv_21 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_1*c1_22_4_11*Jinv_22*Jinv_21);
    const double G20_2_0_0_0_2_0_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_00*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_00*Jinv_02);
    const double G20_2_0_0_0_2_0_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_00*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_00*Jinv_12);
    const double G20_2_0_0_0_2_0_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_00*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_00*Jinv_22);
    const double G20_2_0_0_0_2_0_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_1*Jinv_00*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_1*Jinv_00*Jinv_02);
    const double G20_2_0_0_0_2_0_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_2*Jinv_00*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_2*Jinv_00*Jinv_12);
    const double G20_2_0_0_0_2_0_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_3*Jinv_00*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_3*Jinv_00*Jinv_22);
    const double G20_2_0_0_0_2_0_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_00*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_00*Jinv_02);
    const double G20_2_0_0_0_2_0_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_00*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_00*Jinv_12);
    const double G20_2_0_0_0_2_0_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_00*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_00*Jinv_22);
    const double G20_2_0_0_0_2_0_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_5*Jinv_00*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_5*Jinv_00*Jinv_02);
    const double G20_2_0_0_0_2_0_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_6*Jinv_00*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_6*Jinv_00*Jinv_12);
    const double G20_2_0_0_0_2_0_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_7*Jinv_00*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_7*Jinv_00*Jinv_22);
    const double G20_2_0_0_0_2_0_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_00*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_00*Jinv_02);
    const double G20_2_0_0_0_2_0_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_00*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_00*Jinv_12);
    const double G20_2_0_0_0_2_0_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_00*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_00*Jinv_22);
    const double G20_2_0_0_0_2_0_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_9*Jinv_00*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_9*Jinv_00*Jinv_02);
    const double G20_2_0_0_0_2_0_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_10*Jinv_00*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_10*Jinv_00*Jinv_12);
    const double G20_2_0_0_0_2_0_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_11*Jinv_00*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_11*Jinv_00*Jinv_22);
    const double G20_2_0_0_0_2_1_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_10*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_10*Jinv_02);
    const double G20_2_0_0_0_2_1_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_10*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_10*Jinv_12);
    const double G20_2_0_0_0_2_1_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_10*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_10*Jinv_22);
    const double G20_2_0_0_0_2_1_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_1*Jinv_10*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_1*Jinv_10*Jinv_02);
    const double G20_2_0_0_0_2_1_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_2*Jinv_10*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_2*Jinv_10*Jinv_12);
    const double G20_2_0_0_0_2_1_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_3*Jinv_10*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_3*Jinv_10*Jinv_22);
    const double G20_2_0_0_0_2_1_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_10*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_10*Jinv_02);
    const double G20_2_0_0_0_2_1_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_10*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_10*Jinv_12);
    const double G20_2_0_0_0_2_1_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_10*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_10*Jinv_22);
    const double G20_2_0_0_0_2_1_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_5*Jinv_10*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_5*Jinv_10*Jinv_02);
    const double G20_2_0_0_0_2_1_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_6*Jinv_10*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_6*Jinv_10*Jinv_12);
    const double G20_2_0_0_0_2_1_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_7*Jinv_10*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_7*Jinv_10*Jinv_22);
    const double G20_2_0_0_0_2_1_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_10*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_10*Jinv_02);
    const double G20_2_0_0_0_2_1_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_10*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_10*Jinv_12);
    const double G20_2_0_0_0_2_1_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_10*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_10*Jinv_22);
    const double G20_2_0_0_0_2_1_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_9*Jinv_10*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_9*Jinv_10*Jinv_02);
    const double G20_2_0_0_0_2_1_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_10*Jinv_10*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_10*Jinv_10*Jinv_12);
    const double G20_2_0_0_0_2_1_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_11*Jinv_10*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_11*Jinv_10*Jinv_22);
    const double G20_2_0_0_0_2_2_0_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_20*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_20*Jinv_02);
    const double G20_2_0_0_0_2_2_0_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_20*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_20*Jinv_12);
    const double G20_2_0_0_0_2_2_0_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_0*Jinv_20*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_0*Jinv_20*Jinv_22);
    const double G20_2_0_0_0_2_2_0_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_1*Jinv_20*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_1*Jinv_20*Jinv_02);
    const double G20_2_0_0_0_2_2_0_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_2*Jinv_20*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_2*Jinv_20*Jinv_12);
    const double G20_2_0_0_0_2_2_0_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_3*Jinv_20*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_3*Jinv_20*Jinv_22);
    const double G20_2_0_0_0_2_2_0_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_20*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_20*Jinv_02);
    const double G20_2_0_0_0_2_2_0_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_20*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_20*Jinv_12);
    const double G20_2_0_0_0_2_2_0_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_4*Jinv_20*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_4*Jinv_20*Jinv_22);
    const double G20_2_0_0_0_2_2_0_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_5*Jinv_20*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_5*Jinv_20*Jinv_02);
    const double G20_2_0_0_0_2_2_0_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_6*Jinv_20*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_6*Jinv_20*Jinv_12);
    const double G20_2_0_0_0_2_2_0_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_7*Jinv_20*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_7*Jinv_20*Jinv_22);
    const double G20_2_0_0_0_2_2_0_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_20*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_20*Jinv_02);
    const double G20_2_0_0_0_2_2_0_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_20*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_20*Jinv_12);
    const double G20_2_0_0_0_2_2_0_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_8*Jinv_20*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_8*Jinv_20*Jinv_22);
    const double G20_2_0_0_0_2_2_0_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_9*Jinv_20*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_9*Jinv_20*Jinv_02);
    const double G20_2_0_0_0_2_2_0_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_10*Jinv_20*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_10*Jinv_20*Jinv_12);
    const double G20_2_0_0_0_2_2_0_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_0*c6_21_3_2*c1_21_4_11*Jinv_20*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_0*c0_22_3_2*c1_22_4_11*Jinv_20*Jinv_22);
    const double G20_2_0_0_1_2_0_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_01*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_01*Jinv_02);
    const double G20_2_0_0_1_2_0_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_01*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_01*Jinv_12);
    const double G20_2_0_0_1_2_0_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_01*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_01*Jinv_22);
    const double G20_2_0_0_1_2_0_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_1*Jinv_01*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_1*Jinv_01*Jinv_02);
    const double G20_2_0_0_1_2_0_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_2*Jinv_01*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_2*Jinv_01*Jinv_12);
    const double G20_2_0_0_1_2_0_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_3*Jinv_01*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_3*Jinv_01*Jinv_22);
    const double G20_2_0_0_1_2_0_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_01*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_01*Jinv_02);
    const double G20_2_0_0_1_2_0_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_01*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_01*Jinv_12);
    const double G20_2_0_0_1_2_0_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_01*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_01*Jinv_22);
    const double G20_2_0_0_1_2_0_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_5*Jinv_01*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_5*Jinv_01*Jinv_02);
    const double G20_2_0_0_1_2_0_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_6*Jinv_01*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_6*Jinv_01*Jinv_12);
    const double G20_2_0_0_1_2_0_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_7*Jinv_01*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_7*Jinv_01*Jinv_22);
    const double G20_2_0_0_1_2_0_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_01*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_01*Jinv_02);
    const double G20_2_0_0_1_2_0_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_01*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_01*Jinv_12);
    const double G20_2_0_0_1_2_0_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_01*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_01*Jinv_22);
    const double G20_2_0_0_1_2_0_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_9*Jinv_01*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_9*Jinv_01*Jinv_02);
    const double G20_2_0_0_1_2_0_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_10*Jinv_01*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_10*Jinv_01*Jinv_12);
    const double G20_2_0_0_1_2_0_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_11*Jinv_01*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_11*Jinv_01*Jinv_22);
    const double G20_2_0_0_1_2_1_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_11*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_11*Jinv_02);
    const double G20_2_0_0_1_2_1_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_11*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_11*Jinv_12);
    const double G20_2_0_0_1_2_1_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_11*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_11*Jinv_22);
    const double G20_2_0_0_1_2_1_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_1*Jinv_11*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_1*Jinv_11*Jinv_02);
    const double G20_2_0_0_1_2_1_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_2*Jinv_11*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_2*Jinv_11*Jinv_12);
    const double G20_2_0_0_1_2_1_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_3*Jinv_11*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_3*Jinv_11*Jinv_22);
    const double G20_2_0_0_1_2_1_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_11*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_11*Jinv_02);
    const double G20_2_0_0_1_2_1_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_11*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_11*Jinv_12);
    const double G20_2_0_0_1_2_1_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_11*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_11*Jinv_22);
    const double G20_2_0_0_1_2_1_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_5*Jinv_11*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_5*Jinv_11*Jinv_02);
    const double G20_2_0_0_1_2_1_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_6*Jinv_11*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_6*Jinv_11*Jinv_12);
    const double G20_2_0_0_1_2_1_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_7*Jinv_11*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_7*Jinv_11*Jinv_22);
    const double G20_2_0_0_1_2_1_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_11*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_11*Jinv_02);
    const double G20_2_0_0_1_2_1_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_11*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_11*Jinv_12);
    const double G20_2_0_0_1_2_1_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_11*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_11*Jinv_22);
    const double G20_2_0_0_1_2_1_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_9*Jinv_11*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_9*Jinv_11*Jinv_02);
    const double G20_2_0_0_1_2_1_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_10*Jinv_11*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_10*Jinv_11*Jinv_12);
    const double G20_2_0_0_1_2_1_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_11*Jinv_11*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_11*Jinv_11*Jinv_22);
    const double G20_2_0_0_1_2_2_1_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_21*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_21*Jinv_02);
    const double G20_2_0_0_1_2_2_1_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_21*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_21*Jinv_12);
    const double G20_2_0_0_1_2_2_1_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_0*Jinv_21*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_0*Jinv_21*Jinv_22);
    const double G20_2_0_0_1_2_2_1_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_1*Jinv_21*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_1*Jinv_21*Jinv_02);
    const double G20_2_0_0_1_2_2_1_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_2*Jinv_21*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_2*Jinv_21*Jinv_12);
    const double G20_2_0_0_1_2_2_1_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_3*Jinv_21*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_3*Jinv_21*Jinv_22);
    const double G20_2_0_0_1_2_2_1_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_21*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_21*Jinv_02);
    const double G20_2_0_0_1_2_2_1_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_21*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_21*Jinv_12);
    const double G20_2_0_0_1_2_2_1_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_4*Jinv_21*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_4*Jinv_21*Jinv_22);
    const double G20_2_0_0_1_2_2_1_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_5*Jinv_21*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_5*Jinv_21*Jinv_02);
    const double G20_2_0_0_1_2_2_1_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_6*Jinv_21*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_6*Jinv_21*Jinv_12);
    const double G20_2_0_0_1_2_2_1_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_7*Jinv_21*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_7*Jinv_21*Jinv_22);
    const double G20_2_0_0_1_2_2_1_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_21*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_21*Jinv_02);
    const double G20_2_0_0_1_2_2_1_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_21*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_21*Jinv_12);
    const double G20_2_0_0_1_2_2_1_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_8*Jinv_21*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_8*Jinv_21*Jinv_22);
    const double G20_2_0_0_1_2_2_1_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_9*Jinv_21*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_9*Jinv_21*Jinv_02);
    const double G20_2_0_0_1_2_2_1_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_10*Jinv_21*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_10*Jinv_21*Jinv_12);
    const double G20_2_0_0_1_2_2_1_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_1*c6_21_3_2*c1_21_4_11*Jinv_21*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_1*c0_22_3_2*c1_22_4_11*Jinv_21*Jinv_22);
    const double G20_2_0_0_2_2_0_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_02*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_02*Jinv_02);
    const double G20_2_0_0_2_2_0_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_02*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_02*Jinv_12);
    const double G20_2_0_0_2_2_0_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_02*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_02*Jinv_22);
    const double G20_2_0_0_2_2_0_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_1*Jinv_02*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_1*Jinv_02*Jinv_02);
    const double G20_2_0_0_2_2_0_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_2*Jinv_02*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_2*Jinv_02*Jinv_12);
    const double G20_2_0_0_2_2_0_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_3*Jinv_02*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_3*Jinv_02*Jinv_22);
    const double G20_2_0_0_2_2_0_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_02*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_02*Jinv_02);
    const double G20_2_0_0_2_2_0_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_02*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_02*Jinv_12);
    const double G20_2_0_0_2_2_0_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_02*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_02*Jinv_22);
    const double G20_2_0_0_2_2_0_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_5*Jinv_02*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_5*Jinv_02*Jinv_02);
    const double G20_2_0_0_2_2_0_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_6*Jinv_02*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_6*Jinv_02*Jinv_12);
    const double G20_2_0_0_2_2_0_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_7*Jinv_02*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_7*Jinv_02*Jinv_22);
    const double G20_2_0_0_2_2_0_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_02*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_02*Jinv_02);
    const double G20_2_0_0_2_2_0_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_02*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_02*Jinv_12);
    const double G20_2_0_0_2_2_0_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_02*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_02*Jinv_22);
    const double G20_2_0_0_2_2_0_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_9*Jinv_02*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_9*Jinv_02*Jinv_02);
    const double G20_2_0_0_2_2_0_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_10*Jinv_02*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_10*Jinv_02*Jinv_12);
    const double G20_2_0_0_2_2_0_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_11*Jinv_02*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_11*Jinv_02*Jinv_22);
    const double G20_2_0_0_2_2_1_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_12*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_12*Jinv_02);
    const double G20_2_0_0_2_2_1_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_12*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_12*Jinv_12);
    const double G20_2_0_0_2_2_1_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_12*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_12*Jinv_22);
    const double G20_2_0_0_2_2_1_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_1*Jinv_12*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_1*Jinv_12*Jinv_02);
    const double G20_2_0_0_2_2_1_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_2*Jinv_12*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_2*Jinv_12*Jinv_12);
    const double G20_2_0_0_2_2_1_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_3*Jinv_12*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_3*Jinv_12*Jinv_22);
    const double G20_2_0_0_2_2_1_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_12*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_12*Jinv_02);
    const double G20_2_0_0_2_2_1_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_12*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_12*Jinv_12);
    const double G20_2_0_0_2_2_1_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_12*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_12*Jinv_22);
    const double G20_2_0_0_2_2_1_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_5*Jinv_12*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_5*Jinv_12*Jinv_02);
    const double G20_2_0_0_2_2_1_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_6*Jinv_12*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_6*Jinv_12*Jinv_12);
    const double G20_2_0_0_2_2_1_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_7*Jinv_12*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_7*Jinv_12*Jinv_22);
    const double G20_2_0_0_2_2_1_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_12*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_12*Jinv_02);
    const double G20_2_0_0_2_2_1_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_12*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_12*Jinv_12);
    const double G20_2_0_0_2_2_1_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_12*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_12*Jinv_22);
    const double G20_2_0_0_2_2_1_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_9*Jinv_12*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_9*Jinv_12*Jinv_02);
    const double G20_2_0_0_2_2_1_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_10*Jinv_12*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_10*Jinv_12*Jinv_12);
    const double G20_2_0_0_2_2_1_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_11*Jinv_12*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_11*Jinv_12*Jinv_22);
    const double G20_2_0_0_2_2_2_2_0_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_22*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_22*Jinv_02);
    const double G20_2_0_0_2_2_2_2_0_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_22*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_22*Jinv_12);
    const double G20_2_0_0_2_2_2_2_0_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_0*Jinv_22*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_0*Jinv_22*Jinv_22);
    const double G20_2_0_0_2_2_2_2_1_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_1*Jinv_22*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_1*Jinv_22*Jinv_02);
    const double G20_2_0_0_2_2_2_2_2_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_2*Jinv_22*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_2*Jinv_22*Jinv_12);
    const double G20_2_0_0_2_2_2_2_3_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_3*Jinv_22*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_3*Jinv_22*Jinv_22);
    const double G20_2_0_0_2_2_2_2_4_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_22*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_22*Jinv_02);
    const double G20_2_0_0_2_2_2_2_4_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_22*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_22*Jinv_12);
    const double G20_2_0_0_2_2_2_2_4_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_4*Jinv_22*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_4*Jinv_22*Jinv_22);
    const double G20_2_0_0_2_2_2_2_5_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_5*Jinv_22*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_5*Jinv_22*Jinv_02);
    const double G20_2_0_0_2_2_2_2_6_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_6*Jinv_22*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_6*Jinv_22*Jinv_12);
    const double G20_2_0_0_2_2_2_2_7_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_7*Jinv_22*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_7*Jinv_22*Jinv_22);
    const double G20_2_0_0_2_2_2_2_8_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_22*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_22*Jinv_02);
    const double G20_2_0_0_2_2_2_2_8_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_22*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_22*Jinv_12);
    const double G20_2_0_0_2_2_2_2_8_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_8*Jinv_22*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_8*Jinv_22*Jinv_22);
    const double G20_2_0_0_2_2_2_2_9_0 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_9*Jinv_22*Jinv_02 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_9*Jinv_22*Jinv_02);
    const double G20_2_0_0_2_2_2_2_10_1 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_10*Jinv_22*Jinv_12 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_10*Jinv_22*Jinv_12);
    const double G20_2_0_0_2_2_2_2_11_2 = det*(c10_21_0_0*c4_21_1_0*c0_21_2_2*c6_21_3_2*c1_21_4_11*Jinv_22*Jinv_22 + c10_22_0_0*c4_22_1_0*c6_22_2_2*c0_22_3_2*c1_22_4_11*Jinv_22*Jinv_22);
    const double G21_0_0_0_0_0_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_00;
    const double G21_0_0_0_0_0_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_10;
    const double G21_0_0_0_0_0_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_20;
    const double G21_0_0_0_0_0_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_00*Jinv_00;
    const double G21_0_0_0_0_0_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_00*Jinv_10;
    const double G21_0_0_0_0_0_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_00*Jinv_20;
    const double G21_0_0_0_0_1_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_00;
    const double G21_0_0_0_0_1_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_10;
    const double G21_0_0_0_0_1_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_20;
    const double G21_0_0_0_0_1_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_10*Jinv_00;
    const double G21_0_0_0_0_1_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_10*Jinv_10;
    const double G21_0_0_0_0_1_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_10*Jinv_20;
    const double G21_0_0_0_0_2_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_00;
    const double G21_0_0_0_0_2_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_10;
    const double G21_0_0_0_0_2_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_20;
    const double G21_0_0_0_0_2_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_20*Jinv_00;
    const double G21_0_0_0_0_2_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_20*Jinv_10;
    const double G21_0_0_0_0_2_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_20*Jinv_20;
    const double G21_0_0_0_1_0_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_00;
    const double G21_0_0_0_1_0_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_10;
    const double G21_0_0_0_1_0_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_20;
    const double G21_0_0_0_1_0_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_01*Jinv_00;
    const double G21_0_0_0_1_0_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_01*Jinv_10;
    const double G21_0_0_0_1_0_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_01*Jinv_20;
    const double G21_0_0_0_1_1_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_00;
    const double G21_0_0_0_1_1_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_10;
    const double G21_0_0_0_1_1_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_20;
    const double G21_0_0_0_1_1_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_11*Jinv_00;
    const double G21_0_0_0_1_1_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_11*Jinv_10;
    const double G21_0_0_0_1_1_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_11*Jinv_20;
    const double G21_0_0_0_1_2_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_00;
    const double G21_0_0_0_1_2_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_10;
    const double G21_0_0_0_1_2_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_20;
    const double G21_0_0_0_1_2_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_21*Jinv_00;
    const double G21_0_0_0_1_2_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_21*Jinv_10;
    const double G21_0_0_0_1_2_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_21*Jinv_20;
    const double G21_0_0_0_2_0_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_00;
    const double G21_0_0_0_2_0_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_10;
    const double G21_0_0_0_2_0_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_20;
    const double G21_0_0_0_2_0_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_02*Jinv_00;
    const double G21_0_0_0_2_0_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_02*Jinv_10;
    const double G21_0_0_0_2_0_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_02*Jinv_20;
    const double G21_0_0_0_2_1_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_00;
    const double G21_0_0_0_2_1_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_10;
    const double G21_0_0_0_2_1_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_20;
    const double G21_0_0_0_2_1_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_12*Jinv_00;
    const double G21_0_0_0_2_1_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_12*Jinv_10;
    const double G21_0_0_0_2_1_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_12*Jinv_20;
    const double G21_0_0_0_2_2_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_00;
    const double G21_0_0_0_2_2_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_10;
    const double G21_0_0_0_2_2_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_20;
    const double G21_0_0_0_2_2_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_22*Jinv_00;
    const double G21_0_0_0_2_2_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_22*Jinv_10;
    const double G21_0_0_0_2_2_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_22*Jinv_20;
    const double G21_1_0_0_0_0_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_01;
    const double G21_1_0_0_0_0_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_11;
    const double G21_1_0_0_0_0_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_21;
    const double G21_1_0_0_0_0_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_00*Jinv_01;
    const double G21_1_0_0_0_0_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_00*Jinv_11;
    const double G21_1_0_0_0_0_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_00*Jinv_21;
    const double G21_1_0_0_0_1_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_01;
    const double G21_1_0_0_0_1_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_11;
    const double G21_1_0_0_0_1_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_21;
    const double G21_1_0_0_0_1_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_10*Jinv_01;
    const double G21_1_0_0_0_1_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_10*Jinv_11;
    const double G21_1_0_0_0_1_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_10*Jinv_21;
    const double G21_1_0_0_0_2_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_01;
    const double G21_1_0_0_0_2_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_11;
    const double G21_1_0_0_0_2_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_21;
    const double G21_1_0_0_0_2_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_20*Jinv_01;
    const double G21_1_0_0_0_2_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_20*Jinv_11;
    const double G21_1_0_0_0_2_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_20*Jinv_21;
    const double G21_1_0_0_1_0_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_01;
    const double G21_1_0_0_1_0_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_11;
    const double G21_1_0_0_1_0_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_21;
    const double G21_1_0_0_1_0_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_01*Jinv_01;
    const double G21_1_0_0_1_0_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_01*Jinv_11;
    const double G21_1_0_0_1_0_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_01*Jinv_21;
    const double G21_1_0_0_1_1_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_01;
    const double G21_1_0_0_1_1_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_11;
    const double G21_1_0_0_1_1_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_21;
    const double G21_1_0_0_1_1_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_11*Jinv_01;
    const double G21_1_0_0_1_1_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_11*Jinv_11;
    const double G21_1_0_0_1_1_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_11*Jinv_21;
    const double G21_1_0_0_1_2_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_01;
    const double G21_1_0_0_1_2_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_11;
    const double G21_1_0_0_1_2_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_21;
    const double G21_1_0_0_1_2_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_21*Jinv_01;
    const double G21_1_0_0_1_2_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_21*Jinv_11;
    const double G21_1_0_0_1_2_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_21*Jinv_21;
    const double G21_1_0_0_2_0_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_01;
    const double G21_1_0_0_2_0_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_11;
    const double G21_1_0_0_2_0_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_21;
    const double G21_1_0_0_2_0_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_02*Jinv_01;
    const double G21_1_0_0_2_0_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_02*Jinv_11;
    const double G21_1_0_0_2_0_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_02*Jinv_21;
    const double G21_1_0_0_2_1_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_01;
    const double G21_1_0_0_2_1_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_11;
    const double G21_1_0_0_2_1_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_21;
    const double G21_1_0_0_2_1_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_12*Jinv_01;
    const double G21_1_0_0_2_1_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_12*Jinv_11;
    const double G21_1_0_0_2_1_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_12*Jinv_21;
    const double G21_1_0_0_2_2_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_01;
    const double G21_1_0_0_2_2_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_11;
    const double G21_1_0_0_2_2_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_21;
    const double G21_1_0_0_2_2_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_22*Jinv_01;
    const double G21_1_0_0_2_2_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_22*Jinv_11;
    const double G21_1_0_0_2_2_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_22*Jinv_21;
    const double G21_2_0_0_0_0_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_02;
    const double G21_2_0_0_0_0_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_12;
    const double G21_2_0_0_0_0_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_00*Jinv_22;
    const double G21_2_0_0_0_0_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_00*Jinv_02;
    const double G21_2_0_0_0_0_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_00*Jinv_12;
    const double G21_2_0_0_0_0_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_00*Jinv_22;
    const double G21_2_0_0_0_1_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_02;
    const double G21_2_0_0_0_1_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_12;
    const double G21_2_0_0_0_1_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_10*Jinv_22;
    const double G21_2_0_0_0_1_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_10*Jinv_02;
    const double G21_2_0_0_0_1_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_10*Jinv_12;
    const double G21_2_0_0_0_1_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_10*Jinv_22;
    const double G21_2_0_0_0_2_0_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_02;
    const double G21_2_0_0_0_2_0_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_12;
    const double G21_2_0_0_0_2_0_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_0*Jinv_20*Jinv_22;
    const double G21_2_0_0_0_2_0_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_1*Jinv_20*Jinv_02;
    const double G21_2_0_0_0_2_0_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_2*Jinv_20*Jinv_12;
    const double G21_2_0_0_0_2_0_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_0*c3_23_3_3*Jinv_20*Jinv_22;
    const double G21_2_0_0_1_0_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_02;
    const double G21_2_0_0_1_0_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_12;
    const double G21_2_0_0_1_0_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_01*Jinv_22;
    const double G21_2_0_0_1_0_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_01*Jinv_02;
    const double G21_2_0_0_1_0_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_01*Jinv_12;
    const double G21_2_0_0_1_0_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_01*Jinv_22;
    const double G21_2_0_0_1_1_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_02;
    const double G21_2_0_0_1_1_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_12;
    const double G21_2_0_0_1_1_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_11*Jinv_22;
    const double G21_2_0_0_1_1_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_11*Jinv_02;
    const double G21_2_0_0_1_1_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_11*Jinv_12;
    const double G21_2_0_0_1_1_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_11*Jinv_22;
    const double G21_2_0_0_1_2_1_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_02;
    const double G21_2_0_0_1_2_1_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_12;
    const double G21_2_0_0_1_2_1_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_0*Jinv_21*Jinv_22;
    const double G21_2_0_0_1_2_1_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_1*Jinv_21*Jinv_02;
    const double G21_2_0_0_1_2_1_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_2*Jinv_21*Jinv_12;
    const double G21_2_0_0_1_2_1_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_1*c3_23_3_3*Jinv_21*Jinv_22;
    const double G21_2_0_0_2_0_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_02;
    const double G21_2_0_0_2_0_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_12;
    const double G21_2_0_0_2_0_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_02*Jinv_22;
    const double G21_2_0_0_2_0_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_02*Jinv_02;
    const double G21_2_0_0_2_0_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_02*Jinv_12;
    const double G21_2_0_0_2_0_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_02*Jinv_22;
    const double G21_2_0_0_2_1_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_02;
    const double G21_2_0_0_2_1_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_12;
    const double G21_2_0_0_2_1_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_12*Jinv_22;
    const double G21_2_0_0_2_1_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_12*Jinv_02;
    const double G21_2_0_0_2_1_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_12*Jinv_12;
    const double G21_2_0_0_2_1_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_12*Jinv_22;
    const double G21_2_0_0_2_2_2_0_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_02;
    const double G21_2_0_0_2_2_2_0_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_12;
    const double G21_2_0_0_2_2_2_0_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_0*Jinv_22*Jinv_22;
    const double G21_2_0_0_2_2_2_1_0 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_1*Jinv_22*Jinv_02;
    const double G21_2_0_0_2_2_2_2_1 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_2*Jinv_22*Jinv_12;
    const double G21_2_0_0_2_2_2_3_2 = det*c10_23_0_0*c4_23_1_0*c0_23_2_2*c3_23_3_3*Jinv_22*Jinv_22;
    const double G22_0_0_0_0_0_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_00;
    const double G22_0_0_0_0_0_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_10;
    const double G22_0_0_0_0_0_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_20;
    const double G22_0_0_0_0_0_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_00*Jinv_00;
    const double G22_0_0_0_0_0_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_00*Jinv_10;
    const double G22_0_0_0_0_0_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_00*Jinv_20;
    const double G22_0_0_0_0_1_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_00;
    const double G22_0_0_0_0_1_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_10;
    const double G22_0_0_0_0_1_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_20;
    const double G22_0_0_0_0_1_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_10*Jinv_00;
    const double G22_0_0_0_0_1_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_10*Jinv_10;
    const double G22_0_0_0_0_1_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_10*Jinv_20;
    const double G22_0_0_0_0_2_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_00;
    const double G22_0_0_0_0_2_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_10;
    const double G22_0_0_0_0_2_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_20;
    const double G22_0_0_0_0_2_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_20*Jinv_00;
    const double G22_0_0_0_0_2_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_20*Jinv_10;
    const double G22_0_0_0_0_2_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_20*Jinv_20;
    const double G22_0_0_0_1_0_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_00;
    const double G22_0_0_0_1_0_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_10;
    const double G22_0_0_0_1_0_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_20;
    const double G22_0_0_0_1_0_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_01*Jinv_00;
    const double G22_0_0_0_1_0_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_01*Jinv_10;
    const double G22_0_0_0_1_0_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_01*Jinv_20;
    const double G22_0_0_0_1_1_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_00;
    const double G22_0_0_0_1_1_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_10;
    const double G22_0_0_0_1_1_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_20;
    const double G22_0_0_0_1_1_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_11*Jinv_00;
    const double G22_0_0_0_1_1_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_11*Jinv_10;
    const double G22_0_0_0_1_1_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_11*Jinv_20;
    const double G22_0_0_0_1_2_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_00;
    const double G22_0_0_0_1_2_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_10;
    const double G22_0_0_0_1_2_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_20;
    const double G22_0_0_0_1_2_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_21*Jinv_00;
    const double G22_0_0_0_1_2_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_21*Jinv_10;
    const double G22_0_0_0_1_2_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_21*Jinv_20;
    const double G22_0_0_0_2_0_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_00;
    const double G22_0_0_0_2_0_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_10;
    const double G22_0_0_0_2_0_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_20;
    const double G22_0_0_0_2_0_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_02*Jinv_00;
    const double G22_0_0_0_2_0_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_02*Jinv_10;
    const double G22_0_0_0_2_0_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_02*Jinv_20;
    const double G22_0_0_0_2_1_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_00;
    const double G22_0_0_0_2_1_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_10;
    const double G22_0_0_0_2_1_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_20;
    const double G22_0_0_0_2_1_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_12*Jinv_00;
    const double G22_0_0_0_2_1_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_12*Jinv_10;
    const double G22_0_0_0_2_1_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_12*Jinv_20;
    const double G22_0_0_0_2_2_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_00;
    const double G22_0_0_0_2_2_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_10;
    const double G22_0_0_0_2_2_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_20;
    const double G22_0_0_0_2_2_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_22*Jinv_00;
    const double G22_0_0_0_2_2_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_22*Jinv_10;
    const double G22_0_0_0_2_2_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_22*Jinv_20;
    const double G22_1_0_0_0_0_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_01;
    const double G22_1_0_0_0_0_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_11;
    const double G22_1_0_0_0_0_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_21;
    const double G22_1_0_0_0_0_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_00*Jinv_01;
    const double G22_1_0_0_0_0_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_00*Jinv_11;
    const double G22_1_0_0_0_0_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_00*Jinv_21;
    const double G22_1_0_0_0_1_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_01;
    const double G22_1_0_0_0_1_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_11;
    const double G22_1_0_0_0_1_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_21;
    const double G22_1_0_0_0_1_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_10*Jinv_01;
    const double G22_1_0_0_0_1_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_10*Jinv_11;
    const double G22_1_0_0_0_1_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_10*Jinv_21;
    const double G22_1_0_0_0_2_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_01;
    const double G22_1_0_0_0_2_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_11;
    const double G22_1_0_0_0_2_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_21;
    const double G22_1_0_0_0_2_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_20*Jinv_01;
    const double G22_1_0_0_0_2_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_20*Jinv_11;
    const double G22_1_0_0_0_2_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_20*Jinv_21;
    const double G22_1_0_0_1_0_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_01;
    const double G22_1_0_0_1_0_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_11;
    const double G22_1_0_0_1_0_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_21;
    const double G22_1_0_0_1_0_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_01*Jinv_01;
    const double G22_1_0_0_1_0_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_01*Jinv_11;
    const double G22_1_0_0_1_0_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_01*Jinv_21;
    const double G22_1_0_0_1_1_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_01;
    const double G22_1_0_0_1_1_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_11;
    const double G22_1_0_0_1_1_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_21;
    const double G22_1_0_0_1_1_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_11*Jinv_01;
    const double G22_1_0_0_1_1_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_11*Jinv_11;
    const double G22_1_0_0_1_1_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_11*Jinv_21;
    const double G22_1_0_0_1_2_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_01;
    const double G22_1_0_0_1_2_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_11;
    const double G22_1_0_0_1_2_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_21;
    const double G22_1_0_0_1_2_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_21*Jinv_01;
    const double G22_1_0_0_1_2_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_21*Jinv_11;
    const double G22_1_0_0_1_2_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_21*Jinv_21;
    const double G22_1_0_0_2_0_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_01;
    const double G22_1_0_0_2_0_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_11;
    const double G22_1_0_0_2_0_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_21;
    const double G22_1_0_0_2_0_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_02*Jinv_01;
    const double G22_1_0_0_2_0_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_02*Jinv_11;
    const double G22_1_0_0_2_0_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_02*Jinv_21;
    const double G22_1_0_0_2_1_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_01;
    const double G22_1_0_0_2_1_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_11;
    const double G22_1_0_0_2_1_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_21;
    const double G22_1_0_0_2_1_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_12*Jinv_01;
    const double G22_1_0_0_2_1_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_12*Jinv_11;
    const double G22_1_0_0_2_1_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_12*Jinv_21;
    const double G22_1_0_0_2_2_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_01;
    const double G22_1_0_0_2_2_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_11;
    const double G22_1_0_0_2_2_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_21;
    const double G22_1_0_0_2_2_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_22*Jinv_01;
    const double G22_1_0_0_2_2_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_22*Jinv_11;
    const double G22_1_0_0_2_2_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_22*Jinv_21;
    const double G22_2_0_0_0_0_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_02;
    const double G22_2_0_0_0_0_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_12;
    const double G22_2_0_0_0_0_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_00*Jinv_22;
    const double G22_2_0_0_0_0_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_00*Jinv_02;
    const double G22_2_0_0_0_0_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_00*Jinv_12;
    const double G22_2_0_0_0_0_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_00*Jinv_22;
    const double G22_2_0_0_0_1_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_02;
    const double G22_2_0_0_0_1_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_12;
    const double G22_2_0_0_0_1_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_10*Jinv_22;
    const double G22_2_0_0_0_1_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_10*Jinv_02;
    const double G22_2_0_0_0_1_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_10*Jinv_12;
    const double G22_2_0_0_0_1_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_10*Jinv_22;
    const double G22_2_0_0_0_2_0_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_02;
    const double G22_2_0_0_0_2_0_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_12;
    const double G22_2_0_0_0_2_0_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_0*Jinv_20*Jinv_22;
    const double G22_2_0_0_0_2_0_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_1*Jinv_20*Jinv_02;
    const double G22_2_0_0_0_2_0_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_2*Jinv_20*Jinv_12;
    const double G22_2_0_0_0_2_0_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_0*c3_24_3_3*Jinv_20*Jinv_22;
    const double G22_2_0_0_1_0_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_02;
    const double G22_2_0_0_1_0_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_12;
    const double G22_2_0_0_1_0_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_01*Jinv_22;
    const double G22_2_0_0_1_0_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_01*Jinv_02;
    const double G22_2_0_0_1_0_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_01*Jinv_12;
    const double G22_2_0_0_1_0_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_01*Jinv_22;
    const double G22_2_0_0_1_1_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_02;
    const double G22_2_0_0_1_1_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_12;
    const double G22_2_0_0_1_1_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_11*Jinv_22;
    const double G22_2_0_0_1_1_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_11*Jinv_02;
    const double G22_2_0_0_1_1_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_11*Jinv_12;
    const double G22_2_0_0_1_1_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_11*Jinv_22;
    const double G22_2_0_0_1_2_1_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_02;
    const double G22_2_0_0_1_2_1_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_12;
    const double G22_2_0_0_1_2_1_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_0*Jinv_21*Jinv_22;
    const double G22_2_0_0_1_2_1_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_1*Jinv_21*Jinv_02;
    const double G22_2_0_0_1_2_1_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_2*Jinv_21*Jinv_12;
    const double G22_2_0_0_1_2_1_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_1*c3_24_3_3*Jinv_21*Jinv_22;
    const double G22_2_0_0_2_0_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_02;
    const double G22_2_0_0_2_0_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_12;
    const double G22_2_0_0_2_0_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_02*Jinv_22;
    const double G22_2_0_0_2_0_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_02*Jinv_02;
    const double G22_2_0_0_2_0_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_02*Jinv_12;
    const double G22_2_0_0_2_0_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_02*Jinv_22;
    const double G22_2_0_0_2_1_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_02;
    const double G22_2_0_0_2_1_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_12;
    const double G22_2_0_0_2_1_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_12*Jinv_22;
    const double G22_2_0_0_2_1_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_12*Jinv_02;
    const double G22_2_0_0_2_1_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_12*Jinv_12;
    const double G22_2_0_0_2_1_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_12*Jinv_22;
    const double G22_2_0_0_2_2_2_0_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_02;
    const double G22_2_0_0_2_2_2_0_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_12;
    const double G22_2_0_0_2_2_2_0_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_0*Jinv_22*Jinv_22;
    const double G22_2_0_0_2_2_2_1_0 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_1*Jinv_22*Jinv_02;
    const double G22_2_0_0_2_2_2_2_1 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_2*Jinv_22*Jinv_12;
    const double G22_2_0_0_2_2_2_3_2 = det*c10_24_0_0*c4_24_1_0*c6_24_2_2*c3_24_3_3*Jinv_22*Jinv_22;
    const double G23_0_0_0_0_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_00*Jinv_00;
    const double G23_0_0_0_0_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_00*Jinv_10;
    const double G23_0_0_0_0_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_00*Jinv_20;
    const double G23_0_0_0_0_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_00*Jinv_00;
    const double G23_0_0_0_0_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_00*Jinv_10;
    const double G23_0_0_0_0_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_00*Jinv_20;
    const double G23_0_0_0_0_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_00*Jinv_01;
    const double G23_0_0_0_0_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_00*Jinv_11;
    const double G23_0_0_0_0_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_00*Jinv_21;
    const double G23_0_0_0_0_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_00*Jinv_01;
    const double G23_0_0_0_0_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_00*Jinv_11;
    const double G23_0_0_0_0_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_00*Jinv_21;
    const double G23_0_0_0_0_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_00*Jinv_02;
    const double G23_0_0_0_0_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_00*Jinv_12;
    const double G23_0_0_0_0_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_00*Jinv_22;
    const double G23_0_0_0_0_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_00*Jinv_02;
    const double G23_0_0_0_0_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_00*Jinv_12;
    const double G23_0_0_0_0_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_00*Jinv_22;
    const double G23_0_0_0_1_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_01*Jinv_00;
    const double G23_0_0_0_1_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_01*Jinv_10;
    const double G23_0_0_0_1_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_01*Jinv_20;
    const double G23_0_0_0_1_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_01*Jinv_00;
    const double G23_0_0_0_1_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_01*Jinv_10;
    const double G23_0_0_0_1_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_01*Jinv_20;
    const double G23_0_0_0_1_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_01*Jinv_01;
    const double G23_0_0_0_1_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_01*Jinv_11;
    const double G23_0_0_0_1_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_01*Jinv_21;
    const double G23_0_0_0_1_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_01*Jinv_01;
    const double G23_0_0_0_1_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_01*Jinv_11;
    const double G23_0_0_0_1_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_01*Jinv_21;
    const double G23_0_0_0_1_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_01*Jinv_02;
    const double G23_0_0_0_1_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_01*Jinv_12;
    const double G23_0_0_0_1_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_01*Jinv_22;
    const double G23_0_0_0_1_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_01*Jinv_02;
    const double G23_0_0_0_1_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_01*Jinv_12;
    const double G23_0_0_0_1_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_01*Jinv_22;
    const double G23_0_0_0_2_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_02*Jinv_00;
    const double G23_0_0_0_2_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_02*Jinv_10;
    const double G23_0_0_0_2_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_02*Jinv_20;
    const double G23_0_0_0_2_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_02*Jinv_00;
    const double G23_0_0_0_2_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_02*Jinv_10;
    const double G23_0_0_0_2_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_02*Jinv_20;
    const double G23_0_0_0_2_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_02*Jinv_01;
    const double G23_0_0_0_2_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_02*Jinv_11;
    const double G23_0_0_0_2_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_02*Jinv_21;
    const double G23_0_0_0_2_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_02*Jinv_01;
    const double G23_0_0_0_2_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_02*Jinv_11;
    const double G23_0_0_0_2_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_02*Jinv_21;
    const double G23_0_0_0_2_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_02*Jinv_02;
    const double G23_0_0_0_2_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_02*Jinv_12;
    const double G23_0_0_0_2_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_02*Jinv_22;
    const double G23_0_0_0_2_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_02*Jinv_02;
    const double G23_0_0_0_2_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_02*Jinv_12;
    const double G23_0_0_0_2_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_02*Jinv_22;
    const double G23_0_0_1_0_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_10*Jinv_00;
    const double G23_0_0_1_0_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_10*Jinv_10;
    const double G23_0_0_1_0_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_10*Jinv_20;
    const double G23_0_0_1_0_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_10*Jinv_00;
    const double G23_0_0_1_0_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_10*Jinv_10;
    const double G23_0_0_1_0_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_10*Jinv_20;
    const double G23_0_0_1_0_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_10*Jinv_01;
    const double G23_0_0_1_0_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_10*Jinv_11;
    const double G23_0_0_1_0_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_10*Jinv_21;
    const double G23_0_0_1_0_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_10*Jinv_01;
    const double G23_0_0_1_0_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_10*Jinv_11;
    const double G23_0_0_1_0_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_10*Jinv_21;
    const double G23_0_0_1_0_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_10*Jinv_02;
    const double G23_0_0_1_0_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_10*Jinv_12;
    const double G23_0_0_1_0_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_10*Jinv_22;
    const double G23_0_0_1_0_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_10*Jinv_02;
    const double G23_0_0_1_0_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_10*Jinv_12;
    const double G23_0_0_1_0_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_10*Jinv_22;
    const double G23_0_0_1_1_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_11*Jinv_00;
    const double G23_0_0_1_1_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_11*Jinv_10;
    const double G23_0_0_1_1_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_11*Jinv_20;
    const double G23_0_0_1_1_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_11*Jinv_00;
    const double G23_0_0_1_1_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_11*Jinv_10;
    const double G23_0_0_1_1_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_11*Jinv_20;
    const double G23_0_0_1_1_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_11*Jinv_01;
    const double G23_0_0_1_1_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_11*Jinv_11;
    const double G23_0_0_1_1_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_11*Jinv_21;
    const double G23_0_0_1_1_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_11*Jinv_01;
    const double G23_0_0_1_1_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_11*Jinv_11;
    const double G23_0_0_1_1_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_11*Jinv_21;
    const double G23_0_0_1_1_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_11*Jinv_02;
    const double G23_0_0_1_1_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_11*Jinv_12;
    const double G23_0_0_1_1_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_11*Jinv_22;
    const double G23_0_0_1_1_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_11*Jinv_02;
    const double G23_0_0_1_1_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_11*Jinv_12;
    const double G23_0_0_1_1_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_11*Jinv_22;
    const double G23_0_0_1_2_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_12*Jinv_00;
    const double G23_0_0_1_2_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_12*Jinv_10;
    const double G23_0_0_1_2_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_12*Jinv_20;
    const double G23_0_0_1_2_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_12*Jinv_00;
    const double G23_0_0_1_2_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_12*Jinv_10;
    const double G23_0_0_1_2_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_12*Jinv_20;
    const double G23_0_0_1_2_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_12*Jinv_01;
    const double G23_0_0_1_2_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_12*Jinv_11;
    const double G23_0_0_1_2_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_12*Jinv_21;
    const double G23_0_0_1_2_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_12*Jinv_01;
    const double G23_0_0_1_2_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_12*Jinv_11;
    const double G23_0_0_1_2_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_12*Jinv_21;
    const double G23_0_0_1_2_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_12*Jinv_02;
    const double G23_0_0_1_2_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_12*Jinv_12;
    const double G23_0_0_1_2_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_12*Jinv_22;
    const double G23_0_0_1_2_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_12*Jinv_02;
    const double G23_0_0_1_2_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_12*Jinv_12;
    const double G23_0_0_1_2_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_12*Jinv_22;
    const double G23_0_0_2_0_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_20*Jinv_00;
    const double G23_0_0_2_0_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_20*Jinv_10;
    const double G23_0_0_2_0_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_20*Jinv_20;
    const double G23_0_0_2_0_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_20*Jinv_00;
    const double G23_0_0_2_0_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_20*Jinv_10;
    const double G23_0_0_2_0_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_20*Jinv_20;
    const double G23_0_0_2_0_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_20*Jinv_01;
    const double G23_0_0_2_0_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_20*Jinv_11;
    const double G23_0_0_2_0_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_20*Jinv_21;
    const double G23_0_0_2_0_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_20*Jinv_01;
    const double G23_0_0_2_0_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_20*Jinv_11;
    const double G23_0_0_2_0_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_20*Jinv_21;
    const double G23_0_0_2_0_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_20*Jinv_02;
    const double G23_0_0_2_0_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_20*Jinv_12;
    const double G23_0_0_2_0_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_20*Jinv_22;
    const double G23_0_0_2_0_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_20*Jinv_02;
    const double G23_0_0_2_0_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_20*Jinv_12;
    const double G23_0_0_2_0_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_20*Jinv_22;
    const double G23_0_0_2_1_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_21*Jinv_00;
    const double G23_0_0_2_1_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_21*Jinv_10;
    const double G23_0_0_2_1_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_21*Jinv_20;
    const double G23_0_0_2_1_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_21*Jinv_00;
    const double G23_0_0_2_1_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_21*Jinv_10;
    const double G23_0_0_2_1_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_21*Jinv_20;
    const double G23_0_0_2_1_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_21*Jinv_01;
    const double G23_0_0_2_1_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_21*Jinv_11;
    const double G23_0_0_2_1_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_21*Jinv_21;
    const double G23_0_0_2_1_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_21*Jinv_01;
    const double G23_0_0_2_1_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_21*Jinv_11;
    const double G23_0_0_2_1_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_21*Jinv_21;
    const double G23_0_0_2_1_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_21*Jinv_02;
    const double G23_0_0_2_1_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_21*Jinv_12;
    const double G23_0_0_2_1_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_21*Jinv_22;
    const double G23_0_0_2_1_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_21*Jinv_02;
    const double G23_0_0_2_1_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_21*Jinv_12;
    const double G23_0_0_2_1_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_21*Jinv_22;
    const double G23_0_0_2_2_0_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_22*Jinv_00;
    const double G23_0_0_2_2_0_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_22*Jinv_10;
    const double G23_0_0_2_2_0_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_0*Jinv_22*Jinv_20;
    const double G23_0_0_2_2_1_0_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_1*Jinv_22*Jinv_00;
    const double G23_0_0_2_2_2_1_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_2*Jinv_22*Jinv_10;
    const double G23_0_0_2_2_3_2_0 = det*c10_25_0_0*c5_25_1_0*c1_25_2_3*Jinv_22*Jinv_20;
    const double G23_0_0_2_2_4_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_22*Jinv_01;
    const double G23_0_0_2_2_4_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_22*Jinv_11;
    const double G23_0_0_2_2_4_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_4*Jinv_22*Jinv_21;
    const double G23_0_0_2_2_5_0_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_5*Jinv_22*Jinv_01;
    const double G23_0_0_2_2_6_1_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_6*Jinv_22*Jinv_11;
    const double G23_0_0_2_2_7_2_1 = det*c10_25_0_0*c5_25_1_0*c1_25_2_7*Jinv_22*Jinv_21;
    const double G23_0_0_2_2_8_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_22*Jinv_02;
    const double G23_0_0_2_2_8_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_22*Jinv_12;
    const double G23_0_0_2_2_8_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_8*Jinv_22*Jinv_22;
    const double G23_0_0_2_2_9_0_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_9*Jinv_22*Jinv_02;
    const double G23_0_0_2_2_10_1_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_10*Jinv_22*Jinv_12;
    const double G23_0_0_2_2_11_2_2 = det*c10_25_0_0*c5_25_1_0*c1_25_2_11*Jinv_22*Jinv_22;
    
    // Compute element tensor
    A[0] = 0.0166666666666666*G0_0_0 + 0.00833333333333331*G0_0_1 + 0.00833333333333331*G0_0_2 + 0.00833333333333331*G0_0_3 + 0.0166666666666666*G1_0_0 + 0.00833333333333331*G1_0_1 + 0.00833333333333331*G1_0_2 + 0.00833333333333331*G1_0_3 - 0.166666666666666*G2_0_0_0_0 - 0.166666666666666*G2_0_0_0_1 - 0.166666666666666*G2_0_0_0_2 - 0.166666666666666*G3_0_0_3_0 - 0.166666666666666*G3_0_0_3_1 - 0.166666666666666*G3_0_0_3_2 - 0.166666666666666*G4_0_0_6_0 - 0.166666666666666*G4_0_0_6_1 - 0.166666666666666*G4_0_0_6_2 + 0.166666666666666*G9_0_0_0 + 0.166666666666666*G9_0_0_1 + 0.166666666666666*G9_0_0_2 + 0.166666666666666*G10_0_3_0 + 0.166666666666666*G10_0_3_1 + 0.166666666666666*G10_0_3_2 + 0.166666666666666*G11_0_6_0 + 0.166666666666666*G11_0_6_1 + 0.166666666666666*G11_0_6_2 - 0.0416666666666666*G16_0_0_0_0 - 0.0416666666666666*G16_0_0_1_0 - 0.0416666666666666*G16_0_0_2_0 - 0.0416666666666666*G16_0_1_0_0 - 0.0416666666666666*G16_0_1_1_0 - 0.0416666666666666*G16_0_1_2_0 - 0.0416666666666666*G16_0_2_0_0 - 0.0416666666666666*G16_0_2_1_0 - 0.0416666666666666*G16_0_2_2_0 - 0.0416666666666666*G16_0_3_0_0 - 0.0416666666666666*G16_0_3_1_0 - 0.0416666666666666*G16_0_3_2_0 - 0.0416666666666666*G17_0_0_0_0_0_0 - 0.0416666666666666*G17_0_0_0_0_0_1 - 0.0416666666666666*G17_0_0_0_0_0_2 - 0.0416666666666666*G17_0_0_0_0_0_3 - 0.0416666666666666*G17_0_0_0_1_0_0 - 0.0416666666666666*G17_0_0_0_1_0_1 - 0.0416666666666666*G17_0_0_0_1_0_2 - 0.0416666666666666*G17_0_0_0_1_0_3 - 0.0416666666666666*G17_0_0_0_2_0_0 - 0.0416666666666666*G17_0_0_0_2_0_1 - 0.0416666666666666*G17_0_0_0_2_0_2 - 0.0416666666666666*G17_0_0_0_2_0_3 - 0.0416666666666666*G17_0_0_1_0_1_0 - 0.0416666666666666*G17_0_0_1_0_1_1 - 0.0416666666666666*G17_0_0_1_0_1_2 - 0.0416666666666666*G17_0_0_1_0_1_3 - 0.0416666666666666*G17_0_0_1_1_1_0 - 0.0416666666666666*G17_0_0_1_1_1_1 - 0.0416666666666666*G17_0_0_1_1_1_2 - 0.0416666666666666*G17_0_0_1_1_1_3 - 0.0416666666666666*G17_0_0_1_2_1_0 - 0.0416666666666666*G17_0_0_1_2_1_1 - 0.0416666666666666*G17_0_0_1_2_1_2 - 0.0416666666666666*G17_0_0_1_2_1_3 - 0.0416666666666666*G17_0_0_2_0_2_0 - 0.0416666666666666*G17_0_0_2_0_2_1 - 0.0416666666666666*G17_0_0_2_0_2_2 - 0.0416666666666666*G17_0_0_2_0_2_3 - 0.0416666666666666*G17_0_0_2_1_2_0 - 0.0416666666666666*G17_0_0_2_1_2_1 - 0.0416666666666666*G17_0_0_2_1_2_2 - 0.0416666666666666*G17_0_0_2_1_2_3 - 0.0416666666666666*G17_0_0_2_2_2_0 - 0.0416666666666666*G17_0_0_2_2_2_1 - 0.0416666666666666*G17_0_0_2_2_2_2 - 0.0416666666666666*G17_0_0_2_2_2_3 + 0.0416666666666666*G18_0_0_0_0_0_0 + 0.0416666666666666*G18_0_0_0_0_0_1 + 0.0416666666666666*G18_0_0_0_0_0_2 + 0.0416666666666666*G18_0_0_0_0_0_3 + 0.0416666666666666*G18_0_0_0_1_0_0 + 0.0416666666666666*G18_0_0_0_1_0_1 + 0.0416666666666666*G18_0_0_0_1_0_2 + 0.0416666666666666*G18_0_0_0_1_0_3 + 0.0416666666666666*G18_0_0_0_2_0_0 + 0.0416666666666666*G18_0_0_0_2_0_1 + 0.0416666666666666*G18_0_0_0_2_0_2 + 0.0416666666666666*G18_0_0_0_2_0_3 + 0.0416666666666666*G18_0_0_1_0_1_0 + 0.0416666666666666*G18_0_0_1_0_1_1 + 0.0416666666666666*G18_0_0_1_0_1_2 + 0.0416666666666666*G18_0_0_1_0_1_3 + 0.0416666666666666*G18_0_0_1_1_1_0 + 0.0416666666666666*G18_0_0_1_1_1_1 + 0.0416666666666666*G18_0_0_1_1_1_2 + 0.0416666666666666*G18_0_0_1_1_1_3 + 0.0416666666666666*G18_0_0_1_2_1_0 + 0.0416666666666666*G18_0_0_1_2_1_1 + 0.0416666666666666*G18_0_0_1_2_1_2 + 0.0416666666666666*G18_0_0_1_2_1_3 + 0.0416666666666666*G18_0_0_2_0_2_0 + 0.0416666666666666*G18_0_0_2_0_2_1 + 0.0416666666666666*G18_0_0_2_0_2_2 + 0.0416666666666666*G18_0_0_2_0_2_3 + 0.0416666666666666*G18_0_0_2_1_2_0 + 0.0416666666666666*G18_0_0_2_1_2_1 + 0.0416666666666666*G18_0_0_2_1_2_2 + 0.0416666666666666*G18_0_0_2_1_2_3 + 0.0416666666666666*G18_0_0_2_2_2_0 + 0.0416666666666666*G18_0_0_2_2_2_1 + 0.0416666666666666*G18_0_0_2_2_2_2 + 0.0416666666666666*G18_0_0_2_2_2_3 - 0.0833333333333332*G19_0_0_0_0_0_0_0_0_0 - 0.0833333333333332*G19_0_0_0_0_0_0_0_0_1 - 0.0833333333333332*G19_0_0_0_0_0_0_0_0_2 + 0.0833333333333332*G19_0_0_0_0_0_0_0_1_0 + 0.0833333333333332*G19_0_0_0_0_0_0_0_2_1 + 0.0833333333333332*G19_0_0_0_0_0_0_0_3_2 - 0.0833333333333332*G19_0_0_0_0_0_1_0_0_0 - 0.0833333333333332*G19_0_0_0_0_0_1_0_0_1 - 0.0833333333333332*G19_0_0_0_0_0_1_0_0_2 + 0.0833333333333332*G19_0_0_0_0_0_1_0_1_0 + 0.0833333333333332*G19_0_0_0_0_0_1_0_2_1 + 0.0833333333333332*G19_0_0_0_0_0_1_0_3_2 - 0.0833333333333332*G19_0_0_0_0_0_2_0_0_0 - 0.0833333333333332*G19_0_0_0_0_0_2_0_0_1 - 0.0833333333333332*G19_0_0_0_0_0_2_0_0_2 + 0.0833333333333332*G19_0_0_0_0_0_2_0_1_0 + 0.0833333333333332*G19_0_0_0_0_0_2_0_2_1 + 0.0833333333333332*G19_0_0_0_0_0_2_0_3_2 - 0.0833333333333332*G19_0_0_0_1_0_0_1_0_0 - 0.0833333333333332*G19_0_0_0_1_0_0_1_0_1 - 0.0833333333333332*G19_0_0_0_1_0_0_1_0_2 + 0.0833333333333332*G19_0_0_0_1_0_0_1_1_0 + 0.0833333333333332*G19_0_0_0_1_0_0_1_2_1 + 0.0833333333333332*G19_0_0_0_1_0_0_1_3_2 - 0.0833333333333332*G19_0_0_0_1_0_1_1_0_0 - 0.0833333333333332*G19_0_0_0_1_0_1_1_0_1 - 0.0833333333333332*G19_0_0_0_1_0_1_1_0_2 + 0.0833333333333332*G19_0_0_0_1_0_1_1_1_0 + 0.0833333333333332*G19_0_0_0_1_0_1_1_2_1 + 0.0833333333333332*G19_0_0_0_1_0_1_1_3_2 - 0.0833333333333332*G19_0_0_0_1_0_2_1_0_0 - 0.0833333333333332*G19_0_0_0_1_0_2_1_0_1 - 0.0833333333333332*G19_0_0_0_1_0_2_1_0_2 + 0.0833333333333332*G19_0_0_0_1_0_2_1_1_0 + 0.0833333333333332*G19_0_0_0_1_0_2_1_2_1 + 0.0833333333333332*G19_0_0_0_1_0_2_1_3_2 - 0.0833333333333332*G19_0_0_0_2_0_0_2_0_0 - 0.0833333333333332*G19_0_0_0_2_0_0_2_0_1 - 0.0833333333333332*G19_0_0_0_2_0_0_2_0_2 + 0.0833333333333332*G19_0_0_0_2_0_0_2_1_0 + 0.0833333333333332*G19_0_0_0_2_0_0_2_2_1 + 0.0833333333333332*G19_0_0_0_2_0_0_2_3_2 - 0.0833333333333332*G19_0_0_0_2_0_1_2_0_0 - 0.0833333333333332*G19_0_0_0_2_0_1_2_0_1 - 0.0833333333333332*G19_0_0_0_2_0_1_2_0_2 + 0.0833333333333332*G19_0_0_0_2_0_1_2_1_0 + 0.0833333333333332*G19_0_0_0_2_0_1_2_2_1 + 0.0833333333333332*G19_0_0_0_2_0_1_2_3_2 - 0.0833333333333332*G19_0_0_0_2_0_2_2_0_0 - 0.0833333333333332*G19_0_0_0_2_0_2_2_0_1 - 0.0833333333333332*G19_0_0_0_2_0_2_2_0_2 + 0.0833333333333332*G19_0_0_0_2_0_2_2_1_0 + 0.0833333333333332*G19_0_0_0_2_0_2_2_2_1 + 0.0833333333333332*G19_0_0_0_2_0_2_2_3_2 - 0.0833333333333332*G19_1_0_0_0_1_0_0_0_0 - 0.0833333333333332*G19_1_0_0_0_1_0_0_0_1 - 0.0833333333333332*G19_1_0_0_0_1_0_0_0_2 + 0.0833333333333332*G19_1_0_0_0_1_0_0_1_0 + 0.0833333333333332*G19_1_0_0_0_1_0_0_2_1 + 0.0833333333333332*G19_1_0_0_0_1_0_0_3_2 - 0.0833333333333332*G19_1_0_0_0_1_1_0_0_0 - 0.0833333333333332*G19_1_0_0_0_1_1_0_0_1 - 0.0833333333333332*G19_1_0_0_0_1_1_0_0_2 + 0.0833333333333332*G19_1_0_0_0_1_1_0_1_0 + 0.0833333333333332*G19_1_0_0_0_1_1_0_2_1 + 0.0833333333333332*G19_1_0_0_0_1_1_0_3_2 - 0.0833333333333332*G19_1_0_0_0_1_2_0_0_0 - 0.0833333333333332*G19_1_0_0_0_1_2_0_0_1 - 0.0833333333333332*G19_1_0_0_0_1_2_0_0_2 + 0.0833333333333332*G19_1_0_0_0_1_2_0_1_0 + 0.0833333333333332*G19_1_0_0_0_1_2_0_2_1 + 0.0833333333333332*G19_1_0_0_0_1_2_0_3_2 - 0.0833333333333332*G19_1_0_0_1_1_0_1_0_0 - 0.0833333333333332*G19_1_0_0_1_1_0_1_0_1 - 0.0833333333333332*G19_1_0_0_1_1_0_1_0_2 + 0.0833333333333332*G19_1_0_0_1_1_0_1_1_0 + 0.0833333333333332*G19_1_0_0_1_1_0_1_2_1 + 0.0833333333333332*G19_1_0_0_1_1_0_1_3_2 - 0.0833333333333332*G19_1_0_0_1_1_1_1_0_0 - 0.0833333333333332*G19_1_0_0_1_1_1_1_0_1 - 0.0833333333333332*G19_1_0_0_1_1_1_1_0_2 + 0.0833333333333332*G19_1_0_0_1_1_1_1_1_0 + 0.0833333333333332*G19_1_0_0_1_1_1_1_2_1 + 0.0833333333333332*G19_1_0_0_1_1_1_1_3_2 - 0.0833333333333332*G19_1_0_0_1_1_2_1_0_0 - 0.0833333333333332*G19_1_0_0_1_1_2_1_0_1 - 0.0833333333333332*G19_1_0_0_1_1_2_1_0_2 + 0.0833333333333332*G19_1_0_0_1_1_2_1_1_0 + 0.0833333333333332*G19_1_0_0_1_1_2_1_2_1 + 0.0833333333333332*G19_1_0_0_1_1_2_1_3_2 - 0.0833333333333332*G19_1_0_0_2_1_0_2_0_0 - 0.0833333333333332*G19_1_0_0_2_1_0_2_0_1 - 0.0833333333333332*G19_1_0_0_2_1_0_2_0_2 + 0.0833333333333332*G19_1_0_0_2_1_0_2_1_0 + 0.0833333333333332*G19_1_0_0_2_1_0_2_2_1 + 0.0833333333333332*G19_1_0_0_2_1_0_2_3_2 - 0.0833333333333332*G19_1_0_0_2_1_1_2_0_0 - 0.0833333333333332*G19_1_0_0_2_1_1_2_0_1 - 0.0833333333333332*G19_1_0_0_2_1_1_2_0_2 + 0.0833333333333332*G19_1_0_0_2_1_1_2_1_0 + 0.0833333333333332*G19_1_0_0_2_1_1_2_2_1 + 0.0833333333333332*G19_1_0_0_2_1_1_2_3_2 - 0.0833333333333332*G19_1_0_0_2_1_2_2_0_0 - 0.0833333333333332*G19_1_0_0_2_1_2_2_0_1 - 0.0833333333333332*G19_1_0_0_2_1_2_2_0_2 + 0.0833333333333332*G19_1_0_0_2_1_2_2_1_0 + 0.0833333333333332*G19_1_0_0_2_1_2_2_2_1 + 0.0833333333333332*G19_1_0_0_2_1_2_2_3_2 - 0.0833333333333332*G19_2_0_0_0_2_0_0_0_0 - 0.0833333333333332*G19_2_0_0_0_2_0_0_0_1 - 0.0833333333333332*G19_2_0_0_0_2_0_0_0_2 + 0.0833333333333332*G19_2_0_0_0_2_0_0_1_0 + 0.0833333333333332*G19_2_0_0_0_2_0_0_2_1 + 0.0833333333333332*G19_2_0_0_0_2_0_0_3_2 - 0.0833333333333332*G19_2_0_0_0_2_1_0_0_0 - 0.0833333333333332*G19_2_0_0_0_2_1_0_0_1 - 0.0833333333333332*G19_2_0_0_0_2_1_0_0_2 + 0.0833333333333332*G19_2_0_0_0_2_1_0_1_0 + 0.0833333333333332*G19_2_0_0_0_2_1_0_2_1 + 0.0833333333333332*G19_2_0_0_0_2_1_0_3_2 - 0.0833333333333332*G19_2_0_0_0_2_2_0_0_0 - 0.0833333333333332*G19_2_0_0_0_2_2_0_0_1 - 0.0833333333333332*G19_2_0_0_0_2_2_0_0_2 + 0.0833333333333332*G19_2_0_0_0_2_2_0_1_0 + 0.0833333333333332*G19_2_0_0_0_2_2_0_2_1 + 0.0833333333333332*G19_2_0_0_0_2_2_0_3_2 - 0.0833333333333332*G19_2_0_0_1_2_0_1_0_0 - 0.0833333333333332*G19_2_0_0_1_2_0_1_0_1 - 0.0833333333333332*G19_2_0_0_1_2_0_1_0_2 + 0.0833333333333332*G19_2_0_0_1_2_0_1_1_0 + 0.0833333333333332*G19_2_0_0_1_2_0_1_2_1 + 0.0833333333333332*G19_2_0_0_1_2_0_1_3_2 - 0.0833333333333332*G19_2_0_0_1_2_1_1_0_0 - 0.0833333333333332*G19_2_0_0_1_2_1_1_0_1 - 0.0833333333333332*G19_2_0_0_1_2_1_1_0_2 + 0.0833333333333332*G19_2_0_0_1_2_1_1_1_0 + 0.0833333333333332*G19_2_0_0_1_2_1_1_2_1 + 0.0833333333333332*G19_2_0_0_1_2_1_1_3_2 - 0.0833333333333332*G19_2_0_0_1_2_2_1_0_0 - 0.0833333333333332*G19_2_0_0_1_2_2_1_0_1 - 0.0833333333333332*G19_2_0_0_1_2_2_1_0_2 + 0.0833333333333332*G19_2_0_0_1_2_2_1_1_0 + 0.0833333333333332*G19_2_0_0_1_2_2_1_2_1 + 0.0833333333333332*G19_2_0_0_1_2_2_1_3_2 - 0.0833333333333332*G19_2_0_0_2_2_0_2_0_0 - 0.0833333333333332*G19_2_0_0_2_2_0_2_0_1 - 0.0833333333333332*G19_2_0_0_2_2_0_2_0_2 + 0.0833333333333332*G19_2_0_0_2_2_0_2_1_0 + 0.0833333333333332*G19_2_0_0_2_2_0_2_2_1 + 0.0833333333333332*G19_2_0_0_2_2_0_2_3_2 - 0.0833333333333332*G19_2_0_0_2_2_1_2_0_0 - 0.0833333333333332*G19_2_0_0_2_2_1_2_0_1 - 0.0833333333333332*G19_2_0_0_2_2_1_2_0_2 + 0.0833333333333332*G19_2_0_0_2_2_1_2_1_0 + 0.0833333333333332*G19_2_0_0_2_2_1_2_2_1 + 0.0833333333333332*G19_2_0_0_2_2_1_2_3_2 - 0.0833333333333332*G19_2_0_0_2_2_2_2_0_0 - 0.0833333333333332*G19_2_0_0_2_2_2_2_0_1 - 0.0833333333333332*G19_2_0_0_2_2_2_2_0_2 + 0.0833333333333332*G19_2_0_0_2_2_2_2_1_0 + 0.0833333333333332*G19_2_0_0_2_2_2_2_2_1 + 0.0833333333333332*G19_2_0_0_2_2_2_2_3_2 + 0.0833333333333332*G20_0_0_0_0_0_0_0_0_0 + 0.0833333333333332*G20_0_0_0_0_0_0_0_0_1 + 0.0833333333333332*G20_0_0_0_0_0_0_0_0_2 - 0.0833333333333332*G20_0_0_0_0_0_0_0_1_0 - 0.0833333333333332*G20_0_0_0_0_0_0_0_2_1 - 0.0833333333333332*G20_0_0_0_0_0_0_0_3_2 + 0.0833333333333332*G20_0_0_0_0_0_1_0_0_0 + 0.0833333333333332*G20_0_0_0_0_0_1_0_0_1 + 0.0833333333333332*G20_0_0_0_0_0_1_0_0_2 - 0.0833333333333332*G20_0_0_0_0_0_1_0_1_0 - 0.0833333333333332*G20_0_0_0_0_0_1_0_2_1 - 0.0833333333333332*G20_0_0_0_0_0_1_0_3_2 + 0.0833333333333332*G20_0_0_0_0_0_2_0_0_0 + 0.0833333333333332*G20_0_0_0_0_0_2_0_0_1 + 0.0833333333333332*G20_0_0_0_0_0_2_0_0_2 - 0.0833333333333332*G20_0_0_0_0_0_2_0_1_0 - 0.0833333333333332*G20_0_0_0_0_0_2_0_2_1 - 0.0833333333333332*G20_0_0_0_0_0_2_0_3_2 + 0.0833333333333332*G20_0_0_0_1_0_0_1_0_0 + 0.0833333333333332*G20_0_0_0_1_0_0_1_0_1 + 0.0833333333333332*G20_0_0_0_1_0_0_1_0_2 - 0.0833333333333332*G20_0_0_0_1_0_0_1_1_0 - 0.0833333333333332*G20_0_0_0_1_0_0_1_2_1 - 0.0833333333333332*G20_0_0_0_1_0_0_1_3_2 + 0.0833333333333332*G20_0_0_0_1_0_1_1_0_0 + 0.0833333333333332*G20_0_0_0_1_0_1_1_0_1 + 0.0833333333333332*G20_0_0_0_1_0_1_1_0_2 - 0.0833333333333332*G20_0_0_0_1_0_1_1_1_0 - 0.0833333333333332*G20_0_0_0_1_0_1_1_2_1 - 0.0833333333333332*G20_0_0_0_1_0_1_1_3_2 + 0.0833333333333332*G20_0_0_0_1_0_2_1_0_0 + 0.0833333333333332*G20_0_0_0_1_0_2_1_0_1 + 0.0833333333333332*G20_0_0_0_1_0_2_1_0_2 - 0.0833333333333332*G20_0_0_0_1_0_2_1_1_0 - 0.0833333333333332*G20_0_0_0_1_0_2_1_2_1 - 0.0833333333333332*G20_0_0_0_1_0_2_1_3_2 + 0.0833333333333332*G20_0_0_0_2_0_0_2_0_0 + 0.0833333333333332*G20_0_0_0_2_0_0_2_0_1 + 0.0833333333333332*G20_0_0_0_2_0_0_2_0_2 - 0.0833333333333332*G20_0_0_0_2_0_0_2_1_0 - 0.0833333333333332*G20_0_0_0_2_0_0_2_2_1 - 0.0833333333333332*G20_0_0_0_2_0_0_2_3_2 + 0.0833333333333332*G20_0_0_0_2_0_1_2_0_0 + 0.0833333333333332*G20_0_0_0_2_0_1_2_0_1 + 0.0833333333333332*G20_0_0_0_2_0_1_2_0_2 - 0.0833333333333332*G20_0_0_0_2_0_1_2_1_0 - 0.0833333333333332*G20_0_0_0_2_0_1_2_2_1 - 0.0833333333333332*G20_0_0_0_2_0_1_2_3_2 + 0.0833333333333332*G20_0_0_0_2_0_2_2_0_0 + 0.0833333333333332*G20_0_0_0_2_0_2_2_0_1 + 0.0833333333333332*G20_0_0_0_2_0_2_2_0_2 - 0.0833333333333332*G20_0_0_0_2_0_2_2_1_0 - 0.0833333333333332*G20_0_0_0_2_0_2_2_2_1 - 0.0833333333333332*G20_0_0_0_2_0_2_2_3_2 + 0.0833333333333332*G20_1_0_0_0_1_0_0_0_0 + 0.0833333333333332*G20_1_0_0_0_1_0_0_0_1 + 0.0833333333333332*G20_1_0_0_0_1_0_0_0_2 - 0.0833333333333332*G20_1_0_0_0_1_0_0_1_0 - 0.0833333333333332*G20_1_0_0_0_1_0_0_2_1 - 0.0833333333333332*G20_1_0_0_0_1_0_0_3_2 + 0.0833333333333332*G20_1_0_0_0_1_1_0_0_0 + 0.0833333333333332*G20_1_0_0_0_1_1_0_0_1 + 0.0833333333333332*G20_1_0_0_0_1_1_0_0_2 - 0.0833333333333332*G20_1_0_0_0_1_1_0_1_0 - 0.0833333333333332*G20_1_0_0_0_1_1_0_2_1 - 0.0833333333333332*G20_1_0_0_0_1_1_0_3_2 + 0.0833333333333332*G20_1_0_0_0_1_2_0_0_0 + 0.0833333333333332*G20_1_0_0_0_1_2_0_0_1 + 0.0833333333333332*G20_1_0_0_0_1_2_0_0_2 - 0.0833333333333332*G20_1_0_0_0_1_2_0_1_0 - 0.0833333333333332*G20_1_0_0_0_1_2_0_2_1 - 0.0833333333333332*G20_1_0_0_0_1_2_0_3_2 + 0.0833333333333332*G20_1_0_0_1_1_0_1_0_0 + 0.0833333333333332*G20_1_0_0_1_1_0_1_0_1 + 0.0833333333333332*G20_1_0_0_1_1_0_1_0_2 - 0.0833333333333332*G20_1_0_0_1_1_0_1_1_0 - 0.0833333333333332*G20_1_0_0_1_1_0_1_2_1 - 0.0833333333333332*G20_1_0_0_1_1_0_1_3_2 + 0.0833333333333332*G20_1_0_0_1_1_1_1_0_0 + 0.0833333333333332*G20_1_0_0_1_1_1_1_0_1 + 0.0833333333333332*G20_1_0_0_1_1_1_1_0_2 - 0.0833333333333332*G20_1_0_0_1_1_1_1_1_0 - 0.0833333333333332*G20_1_0_0_1_1_1_1_2_1 - 0.0833333333333332*G20_1_0_0_1_1_1_1_3_2 + 0.0833333333333332*G20_1_0_0_1_1_2_1_0_0 + 0.0833333333333332*G20_1_0_0_1_1_2_1_0_1 + 0.0833333333333332*G20_1_0_0_1_1_2_1_0_2 - 0.0833333333333332*G20_1_0_0_1_1_2_1_1_0 - 0.0833333333333332*G20_1_0_0_1_1_2_1_2_1 - 0.0833333333333332*G20_1_0_0_1_1_2_1_3_2 + 0.0833333333333332*G20_1_0_0_2_1_0_2_0_0 + 0.0833333333333332*G20_1_0_0_2_1_0_2_0_1 + 0.0833333333333332*G20_1_0_0_2_1_0_2_0_2 - 0.0833333333333332*G20_1_0_0_2_1_0_2_1_0 - 0.0833333333333332*G20_1_0_0_2_1_0_2_2_1 - 0.0833333333333332*G20_1_0_0_2_1_0_2_3_2 + 0.0833333333333332*G20_1_0_0_2_1_1_2_0_0 + 0.0833333333333332*G20_1_0_0_2_1_1_2_0_1 + 0.0833333333333332*G20_1_0_0_2_1_1_2_0_2 - 0.0833333333333332*G20_1_0_0_2_1_1_2_1_0 - 0.0833333333333332*G20_1_0_0_2_1_1_2_2_1 - 0.0833333333333332*G20_1_0_0_2_1_1_2_3_2 + 0.0833333333333332*G20_1_0_0_2_1_2_2_0_0 + 0.0833333333333332*G20_1_0_0_2_1_2_2_0_1 + 0.0833333333333332*G20_1_0_0_2_1_2_2_0_2 - 0.0833333333333332*G20_1_0_0_2_1_2_2_1_0 - 0.0833333333333332*G20_1_0_0_2_1_2_2_2_1 - 0.0833333333333332*G20_1_0_0_2_1_2_2_3_2 + 0.0833333333333332*G20_2_0_0_0_2_0_0_0_0 + 0.0833333333333332*G20_2_0_0_0_2_0_0_0_1 + 0.0833333333333332*G20_2_0_0_0_2_0_0_0_2 - 0.0833333333333332*G20_2_0_0_0_2_0_0_1_0 - 0.0833333333333332*G20_2_0_0_0_2_0_0_2_1 - 0.0833333333333332*G20_2_0_0_0_2_0_0_3_2 + 0.0833333333333332*G20_2_0_0_0_2_1_0_0_0 + 0.0833333333333332*G20_2_0_0_0_2_1_0_0_1 + 0.0833333333333332*G20_2_0_0_0_2_1_0_0_2 - 0.0833333333333332*G20_2_0_0_0_2_1_0_1_0 - 0.0833333333333332*G20_2_0_0_0_2_1_0_2_1 - 0.0833333333333332*G20_2_0_0_0_2_1_0_3_2 + 0.0833333333333332*G20_2_0_0_0_2_2_0_0_0 + 0.0833333333333332*G20_2_0_0_0_2_2_0_0_1 + 0.0833333333333332*G20_2_0_0_0_2_2_0_0_2 - 0.0833333333333332*G20_2_0_0_0_2_2_0_1_0 - 0.0833333333333332*G20_2_0_0_0_2_2_0_2_1 - 0.0833333333333332*G20_2_0_0_0_2_2_0_3_2 + 0.0833333333333332*G20_2_0_0_1_2_0_1_0_0 + 0.0833333333333332*G20_2_0_0_1_2_0_1_0_1 + 0.0833333333333332*G20_2_0_0_1_2_0_1_0_2 - 0.0833333333333332*G20_2_0_0_1_2_0_1_1_0 - 0.0833333333333332*G20_2_0_0_1_2_0_1_2_1 - 0.0833333333333332*G20_2_0_0_1_2_0_1_3_2 + 0.0833333333333332*G20_2_0_0_1_2_1_1_0_0 + 0.0833333333333332*G20_2_0_0_1_2_1_1_0_1 + 0.0833333333333332*G20_2_0_0_1_2_1_1_0_2 - 0.0833333333333332*G20_2_0_0_1_2_1_1_1_0 - 0.0833333333333332*G20_2_0_0_1_2_1_1_2_1 - 0.0833333333333332*G20_2_0_0_1_2_1_1_3_2 + 0.0833333333333332*G20_2_0_0_1_2_2_1_0_0 + 0.0833333333333332*G20_2_0_0_1_2_2_1_0_1 + 0.0833333333333332*G20_2_0_0_1_2_2_1_0_2 - 0.0833333333333332*G20_2_0_0_1_2_2_1_1_0 - 0.0833333333333332*G20_2_0_0_1_2_2_1_2_1 - 0.0833333333333332*G20_2_0_0_1_2_2_1_3_2 + 0.0833333333333332*G20_2_0_0_2_2_0_2_0_0 + 0.0833333333333332*G20_2_0_0_2_2_0_2_0_1 + 0.0833333333333332*G20_2_0_0_2_2_0_2_0_2 - 0.0833333333333332*G20_2_0_0_2_2_0_2_1_0 - 0.0833333333333332*G20_2_0_0_2_2_0_2_2_1 - 0.0833333333333332*G20_2_0_0_2_2_0_2_3_2 + 0.0833333333333332*G20_2_0_0_2_2_1_2_0_0 + 0.0833333333333332*G20_2_0_0_2_2_1_2_0_1 + 0.0833333333333332*G20_2_0_0_2_2_1_2_0_2 - 0.0833333333333332*G20_2_0_0_2_2_1_2_1_0 - 0.0833333333333332*G20_2_0_0_2_2_1_2_2_1 - 0.0833333333333332*G20_2_0_0_2_2_1_2_3_2 + 0.0833333333333332*G20_2_0_0_2_2_2_2_0_0 + 0.0833333333333332*G20_2_0_0_2_2_2_2_0_1 + 0.0833333333333332*G20_2_0_0_2_2_2_2_0_2 - 0.0833333333333332*G20_2_0_0_2_2_2_2_1_0 - 0.0833333333333332*G20_2_0_0_2_2_2_2_2_1 - 0.0833333333333332*G20_2_0_0_2_2_2_2_3_2 - 0.166666666666666*G21_0_0_0_0_0_0_0_0 - 0.166666666666666*G21_0_0_0_0_0_0_0_1 - 0.166666666666666*G21_0_0_0_0_0_0_0_2 + 0.166666666666666*G21_0_0_0_0_0_0_1_0 + 0.166666666666666*G21_0_0_0_0_0_0_2_1 + 0.166666666666666*G21_0_0_0_0_0_0_3_2 - 0.166666666666666*G21_0_0_0_0_1_0_0_0 - 0.166666666666666*G21_0_0_0_0_1_0_0_1 - 0.166666666666666*G21_0_0_0_0_1_0_0_2 + 0.166666666666666*G21_0_0_0_0_1_0_1_0 + 0.166666666666666*G21_0_0_0_0_1_0_2_1 + 0.166666666666666*G21_0_0_0_0_1_0_3_2 - 0.166666666666666*G21_0_0_0_0_2_0_0_0 - 0.166666666666666*G21_0_0_0_0_2_0_0_1 - 0.166666666666666*G21_0_0_0_0_2_0_0_2 + 0.166666666666666*G21_0_0_0_0_2_0_1_0 + 0.166666666666666*G21_0_0_0_0_2_0_2_1 + 0.166666666666666*G21_0_0_0_0_2_0_3_2 - 0.166666666666666*G21_0_0_0_1_0_1_0_0 - 0.166666666666666*G21_0_0_0_1_0_1_0_1 - 0.166666666666666*G21_0_0_0_1_0_1_0_2 + 0.166666666666666*G21_0_0_0_1_0_1_1_0 + 0.166666666666666*G21_0_0_0_1_0_1_2_1 + 0.166666666666666*G21_0_0_0_1_0_1_3_2 - 0.166666666666666*G21_0_0_0_1_1_1_0_0 - 0.166666666666666*G21_0_0_0_1_1_1_0_1 - 0.166666666666666*G21_0_0_0_1_1_1_0_2 + 0.166666666666666*G21_0_0_0_1_1_1_1_0 + 0.166666666666666*G21_0_0_0_1_1_1_2_1 + 0.166666666666666*G21_0_0_0_1_1_1_3_2 - 0.166666666666666*G21_0_0_0_1_2_1_0_0 - 0.166666666666666*G21_0_0_0_1_2_1_0_1 - 0.166666666666666*G21_0_0_0_1_2_1_0_2 + 0.166666666666666*G21_0_0_0_1_2_1_1_0 + 0.166666666666666*G21_0_0_0_1_2_1_2_1 + 0.166666666666666*G21_0_0_0_1_2_1_3_2 - 0.166666666666666*G21_0_0_0_2_0_2_0_0 - 0.166666666666666*G21_0_0_0_2_0_2_0_1 - 0.166666666666666*G21_0_0_0_2_0_2_0_2 + 0.166666666666666*G21_0_0_0_2_0_2_1_0 + 0.166666666666666*G21_0_0_0_2_0_2_2_1 + 0.166666666666666*G21_0_0_0_2_0_2_3_2 - 0.166666666666666*G21_0_0_0_2_1_2_0_0 - 0.166666666666666*G21_0_0_0_2_1_2_0_1 - 0.166666666666666*G21_0_0_0_2_1_2_0_2 + 0.166666666666666*G21_0_0_0_2_1_2_1_0 + 0.166666666666666*G21_0_0_0_2_1_2_2_1 + 0.166666666666666*G21_0_0_0_2_1_2_3_2 - 0.166666666666666*G21_0_0_0_2_2_2_0_0 - 0.166666666666666*G21_0_0_0_2_2_2_0_1 - 0.166666666666666*G21_0_0_0_2_2_2_0_2 + 0.166666666666666*G21_0_0_0_2_2_2_1_0 + 0.166666666666666*G21_0_0_0_2_2_2_2_1 + 0.166666666666666*G21_0_0_0_2_2_2_3_2 + 0.166666666666666*G22_0_0_0_0_0_0_0_0 + 0.166666666666666*G22_0_0_0_0_0_0_0_1 + 0.166666666666666*G22_0_0_0_0_0_0_0_2 - 0.166666666666666*G22_0_0_0_0_0_0_1_0 - 0.166666666666666*G22_0_0_0_0_0_0_2_1 - 0.166666666666666*G22_0_0_0_0_0_0_3_2 + 0.166666666666666*G22_0_0_0_0_1_0_0_0 + 0.166666666666666*G22_0_0_0_0_1_0_0_1 + 0.166666666666666*G22_0_0_0_0_1_0_0_2 - 0.166666666666666*G22_0_0_0_0_1_0_1_0 - 0.166666666666666*G22_0_0_0_0_1_0_2_1 - 0.166666666666666*G22_0_0_0_0_1_0_3_2 + 0.166666666666666*G22_0_0_0_0_2_0_0_0 + 0.166666666666666*G22_0_0_0_0_2_0_0_1 + 0.166666666666666*G22_0_0_0_0_2_0_0_2 - 0.166666666666666*G22_0_0_0_0_2_0_1_0 - 0.166666666666666*G22_0_0_0_0_2_0_2_1 - 0.166666666666666*G22_0_0_0_0_2_0_3_2 + 0.166666666666666*G22_0_0_0_1_0_1_0_0 + 0.166666666666666*G22_0_0_0_1_0_1_0_1 + 0.166666666666666*G22_0_0_0_1_0_1_0_2 - 0.166666666666666*G22_0_0_0_1_0_1_1_0 - 0.166666666666666*G22_0_0_0_1_0_1_2_1 - 0.166666666666666*G22_0_0_0_1_0_1_3_2 + 0.166666666666666*G22_0_0_0_1_1_1_0_0 + 0.166666666666666*G22_0_0_0_1_1_1_0_1 + 0.166666666666666*G22_0_0_0_1_1_1_0_2 - 0.166666666666666*G22_0_0_0_1_1_1_1_0 - 0.166666666666666*G22_0_0_0_1_1_1_2_1 - 0.166666666666666*G22_0_0_0_1_1_1_3_2 + 0.166666666666666*G22_0_0_0_1_2_1_0_0 + 0.166666666666666*G22_0_0_0_1_2_1_0_1 + 0.166666666666666*G22_0_0_0_1_2_1_0_2 - 0.166666666666666*G22_0_0_0_1_2_1_1_0 - 0.166666666666666*G22_0_0_0_1_2_1_2_1 - 0.166666666666666*G22_0_0_0_1_2_1_3_2 + 0.166666666666666*G22_0_0_0_2_0_2_0_0 + 0.166666666666666*G22_0_0_0_2_0_2_0_1 + 0.166666666666666*G22_0_0_0_2_0_2_0_2 - 0.166666666666666*G22_0_0_0_2_0_2_1_0 - 0.166666666666666*G22_0_0_0_2_0_2_2_1 - 0.166666666666666*G22_0_0_0_2_0_2_3_2 + 0.166666666666666*G22_0_0_0_2_1_2_0_0 + 0.166666666666666*G22_0_0_0_2_1_2_0_1 + 0.166666666666666*G22_0_0_0_2_1_2_0_2 - 0.166666666666666*G22_0_0_0_2_1_2_1_0 - 0.166666666666666*G22_0_0_0_2_1_2_2_1 - 0.166666666666666*G22_0_0_0_2_1_2_3_2 + 0.166666666666666*G22_0_0_0_2_2_2_0_0 + 0.166666666666666*G22_0_0_0_2_2_2_0_1 + 0.166666666666666*G22_0_0_0_2_2_2_0_2 - 0.166666666666666*G22_0_0_0_2_2_2_1_0 - 0.166666666666666*G22_0_0_0_2_2_2_2_1 - 0.166666666666666*G22_0_0_0_2_2_2_3_2 - 0.0833333333333332*G23_0_0_0_0_0_0_0 - 0.0833333333333332*G23_0_0_0_0_0_1_0 - 0.0833333333333332*G23_0_0_0_0_0_2_0 + 0.0833333333333332*G23_0_0_0_0_1_0_0 + 0.0833333333333332*G23_0_0_0_0_2_1_0 + 0.0833333333333332*G23_0_0_0_0_3_2_0 - 0.0833333333333332*G23_0_0_0_0_4_0_1 - 0.0833333333333332*G23_0_0_0_0_4_1_1 - 0.0833333333333332*G23_0_0_0_0_4_2_1 + 0.0833333333333332*G23_0_0_0_0_5_0_1 + 0.0833333333333332*G23_0_0_0_0_6_1_1 + 0.0833333333333332*G23_0_0_0_0_7_2_1 - 0.0833333333333332*G23_0_0_0_0_8_0_2 - 0.0833333333333332*G23_0_0_0_0_8_1_2 - 0.0833333333333332*G23_0_0_0_0_8_2_2 + 0.0833333333333332*G23_0_0_0_0_9_0_2 + 0.0833333333333332*G23_0_0_0_0_10_1_2 + 0.0833333333333332*G23_0_0_0_0_11_2_2 - 0.0833333333333332*G23_0_0_1_0_0_0_0 - 0.0833333333333332*G23_0_0_1_0_0_1_0 - 0.0833333333333332*G23_0_0_1_0_0_2_0 + 0.0833333333333332*G23_0_0_1_0_1_0_0 + 0.0833333333333332*G23_0_0_1_0_2_1_0 + 0.0833333333333332*G23_0_0_1_0_3_2_0 - 0.0833333333333332*G23_0_0_1_0_4_0_1 - 0.0833333333333332*G23_0_0_1_0_4_1_1 - 0.0833333333333332*G23_0_0_1_0_4_2_1 + 0.0833333333333332*G23_0_0_1_0_5_0_1 + 0.0833333333333332*G23_0_0_1_0_6_1_1 + 0.0833333333333332*G23_0_0_1_0_7_2_1 - 0.0833333333333332*G23_0_0_1_0_8_0_2 - 0.0833333333333332*G23_0_0_1_0_8_1_2 - 0.0833333333333332*G23_0_0_1_0_8_2_2 + 0.0833333333333332*G23_0_0_1_0_9_0_2 + 0.0833333333333332*G23_0_0_1_0_10_1_2 + 0.0833333333333332*G23_0_0_1_0_11_2_2 - 0.0833333333333332*G23_0_0_2_0_0_0_0 - 0.0833333333333332*G23_0_0_2_0_0_1_0 - 0.0833333333333332*G23_0_0_2_0_0_2_0 + 0.0833333333333332*G23_0_0_2_0_1_0_0 + 0.0833333333333332*G23_0_0_2_0_2_1_0 + 0.0833333333333332*G23_0_0_2_0_3_2_0 - 0.0833333333333332*G23_0_0_2_0_4_0_1 - 0.0833333333333332*G23_0_0_2_0_4_1_1 - 0.0833333333333332*G23_0_0_2_0_4_2_1 + 0.0833333333333332*G23_0_0_2_0_5_0_1 + 0.0833333333333332*G23_0_0_2_0_6_1_1 + 0.0833333333333332*G23_0_0_2_0_7_2_1 - 0.0833333333333332*G23_0_0_2_0_8_0_2 - 0.0833333333333332*G23_0_0_2_0_8_1_2 - 0.0833333333333332*G23_0_0_2_0_8_2_2 + 0.0833333333333332*G23_0_0_2_0_9_0_2 + 0.0833333333333332*G23_0_0_2_0_10_1_2 + 0.0833333333333332*G23_0_0_2_0_11_2_2;
    A[1] = 0.00833333333333331*G0_0_0 + 0.0166666666666666*G0_0_1 + 0.00833333333333331*G0_0_2 + 0.00833333333333331*G0_0_3 + 0.00833333333333331*G1_0_0 + 0.0166666666666666*G1_0_1 + 0.00833333333333331*G1_0_2 + 0.00833333333333331*G1_0_3 + 0.166666666666666*G2_0_0_0_0 + 0.166666666666666*G3_0_0_3_0 + 0.166666666666666*G4_0_0_6_0 - 0.166666666666666*G9_0_0_0 - 0.166666666666666*G10_0_3_0 - 0.166666666666666*G11_0_6_0 + 0.0416666666666666*G16_0_0_0_0 + 0.0416666666666666*G16_0_1_0_0 + 0.0416666666666666*G16_0_2_0_0 + 0.0416666666666666*G16_0_3_0_0 + 0.0416666666666666*G17_0_0_0_0_0_0 + 0.0416666666666666*G17_0_0_0_0_0_1 + 0.0416666666666666*G17_0_0_0_0_0_2 + 0.0416666666666666*G17_0_0_0_0_0_3 + 0.0416666666666666*G17_0_0_1_0_1_0 + 0.0416666666666666*G17_0_0_1_0_1_1 + 0.0416666666666666*G17_0_0_1_0_1_2 + 0.0416666666666666*G17_0_0_1_0_1_3 + 0.0416666666666666*G17_0_0_2_0_2_0 + 0.0416666666666666*G17_0_0_2_0_2_1 + 0.0416666666666666*G17_0_0_2_0_2_2 + 0.0416666666666666*G17_0_0_2_0_2_3 - 0.0416666666666666*G18_0_0_0_0_0_0 - 0.0416666666666666*G18_0_0_0_0_0_1 - 0.0416666666666666*G18_0_0_0_0_0_2 - 0.0416666666666666*G18_0_0_0_0_0_3 - 0.0416666666666666*G18_0_0_1_0_1_0 - 0.0416666666666666*G18_0_0_1_0_1_1 - 0.0416666666666666*G18_0_0_1_0_1_2 - 0.0416666666666666*G18_0_0_1_0_1_3 - 0.0416666666666666*G18_0_0_2_0_2_0 - 0.0416666666666666*G18_0_0_2_0_2_1 - 0.0416666666666666*G18_0_0_2_0_2_2 - 0.0416666666666666*G18_0_0_2_0_2_3 + 0.0833333333333332*G19_0_0_0_0_0_0_0_0_0 + 0.0833333333333332*G19_0_0_0_0_0_0_0_0_1 + 0.0833333333333332*G19_0_0_0_0_0_0_0_0_2 - 0.0833333333333332*G19_0_0_0_0_0_0_0_1_0 - 0.0833333333333332*G19_0_0_0_0_0_0_0_2_1 - 0.0833333333333332*G19_0_0_0_0_0_0_0_3_2 + 0.0833333333333332*G19_0_0_0_1_0_0_1_0_0 + 0.0833333333333332*G19_0_0_0_1_0_0_1_0_1 + 0.0833333333333332*G19_0_0_0_1_0_0_1_0_2 - 0.0833333333333332*G19_0_0_0_1_0_0_1_1_0 - 0.0833333333333332*G19_0_0_0_1_0_0_1_2_1 - 0.0833333333333332*G19_0_0_0_1_0_0_1_3_2 + 0.0833333333333332*G19_0_0_0_2_0_0_2_0_0 + 0.0833333333333332*G19_0_0_0_2_0_0_2_0_1 + 0.0833333333333332*G19_0_0_0_2_0_0_2_0_2 - 0.0833333333333332*G19_0_0_0_2_0_0_2_1_0 - 0.0833333333333332*G19_0_0_0_2_0_0_2_2_1 - 0.0833333333333332*G19_0_0_0_2_0_0_2_3_2 + 0.0833333333333332*G19_1_0_0_0_1_0_0_0_0 + 0.0833333333333332*G19_1_0_0_0_1_0_0_0_1 + 0.0833333333333332*G19_1_0_0_0_1_0_0_0_2 - 0.0833333333333332*G19_1_0_0_0_1_0_0_1_0 - 0.0833333333333332*G19_1_0_0_0_1_0_0_2_1 - 0.0833333333333332*G19_1_0_0_0_1_0_0_3_2 + 0.0833333333333332*G19_1_0_0_1_1_0_1_0_0 + 0.0833333333333332*G19_1_0_0_1_1_0_1_0_1 + 0.0833333333333332*G19_1_0_0_1_1_0_1_0_2 - 0.0833333333333332*G19_1_0_0_1_1_0_1_1_0 - 0.0833333333333332*G19_1_0_0_1_1_0_1_2_1 - 0.0833333333333332*G19_1_0_0_1_1_0_1_3_2 + 0.0833333333333332*G19_1_0_0_2_1_0_2_0_0 + 0.0833333333333332*G19_1_0_0_2_1_0_2_0_1 + 0.0833333333333332*G19_1_0_0_2_1_0_2_0_2 - 0.0833333333333332*G19_1_0_0_2_1_0_2_1_0 - 0.0833333333333332*G19_1_0_0_2_1_0_2_2_1 - 0.0833333333333332*G19_1_0_0_2_1_0_2_3_2 + 0.0833333333333332*G19_2_0_0_0_2_0_0_0_0 + 0.0833333333333332*G19_2_0_0_0_2_0_0_0_1 + 0.0833333333333332*G19_2_0_0_0_2_0_0_0_2 - 0.0833333333333332*G19_2_0_0_0_2_0_0_1_0 - 0.0833333333333332*G19_2_0_0_0_2_0_0_2_1 - 0.0833333333333332*G19_2_0_0_0_2_0_0_3_2 + 0.0833333333333332*G19_2_0_0_1_2_0_1_0_0 + 0.0833333333333332*G19_2_0_0_1_2_0_1_0_1 + 0.0833333333333332*G19_2_0_0_1_2_0_1_0_2 - 0.0833333333333332*G19_2_0_0_1_2_0_1_1_0 - 0.0833333333333332*G19_2_0_0_1_2_0_1_2_1 - 0.0833333333333332*G19_2_0_0_1_2_0_1_3_2 + 0.0833333333333332*G19_2_0_0_2_2_0_2_0_0 + 0.0833333333333332*G19_2_0_0_2_2_0_2_0_1 + 0.0833333333333332*G19_2_0_0_2_2_0_2_0_2 - 0.0833333333333332*G19_2_0_0_2_2_0_2_1_0 - 0.0833333333333332*G19_2_0_0_2_2_0_2_2_1 - 0.0833333333333332*G19_2_0_0_2_2_0_2_3_2 - 0.0833333333333332*G20_0_0_0_0_0_0_0_0_0 - 0.0833333333333332*G20_0_0_0_0_0_0_0_0_1 - 0.0833333333333332*G20_0_0_0_0_0_0_0_0_2 + 0.0833333333333332*G20_0_0_0_0_0_0_0_1_0 + 0.0833333333333332*G20_0_0_0_0_0_0_0_2_1 + 0.0833333333333332*G20_0_0_0_0_0_0_0_3_2 - 0.0833333333333332*G20_0_0_0_1_0_0_1_0_0 - 0.0833333333333332*G20_0_0_0_1_0_0_1_0_1 - 0.0833333333333332*G20_0_0_0_1_0_0_1_0_2 + 0.0833333333333332*G20_0_0_0_1_0_0_1_1_0 + 0.0833333333333332*G20_0_0_0_1_0_0_1_2_1 + 0.0833333333333332*G20_0_0_0_1_0_0_1_3_2 - 0.0833333333333332*G20_0_0_0_2_0_0_2_0_0 - 0.0833333333333332*G20_0_0_0_2_0_0_2_0_1 - 0.0833333333333332*G20_0_0_0_2_0_0_2_0_2 + 0.0833333333333332*G20_0_0_0_2_0_0_2_1_0 + 0.0833333333333332*G20_0_0_0_2_0_0_2_2_1 + 0.0833333333333332*G20_0_0_0_2_0_0_2_3_2 - 0.0833333333333332*G20_1_0_0_0_1_0_0_0_0 - 0.0833333333333332*G20_1_0_0_0_1_0_0_0_1 - 0.0833333333333332*G20_1_0_0_0_1_0_0_0_2 + 0.0833333333333332*G20_1_0_0_0_1_0_0_1_0 + 0.0833333333333332*G20_1_0_0_0_1_0_0_2_1 + 0.0833333333333332*G20_1_0_0_0_1_0_0_3_2 - 0.0833333333333332*G20_1_0_0_1_1_0_1_0_0 - 0.0833333333333332*G20_1_0_0_1_1_0_1_0_1 - 0.0833333333333332*G20_1_0_0_1_1_0_1_0_2 + 0.0833333333333332*G20_1_0_0_1_1_0_1_1_0 + 0.0833333333333332*G20_1_0_0_1_1_0_1_2_1 + 0.0833333333333332*G20_1_0_0_1_1_0_1_3_2 - 0.0833333333333332*G20_1_0_0_2_1_0_2_0_0 - 0.0833333333333332*G20_1_0_0_2_1_0_2_0_1 - 0.0833333333333332*G20_1_0_0_2_1_0_2_0_2 + 0.0833333333333332*G20_1_0_0_2_1_0_2_1_0 + 0.0833333333333332*G20_1_0_0_2_1_0_2_2_1 + 0.0833333333333332*G20_1_0_0_2_1_0_2_3_2 - 0.0833333333333332*G20_2_0_0_0_2_0_0_0_0 - 0.0833333333333332*G20_2_0_0_0_2_0_0_0_1 - 0.0833333333333332*G20_2_0_0_0_2_0_0_0_2 + 0.0833333333333332*G20_2_0_0_0_2_0_0_1_0 + 0.0833333333333332*G20_2_0_0_0_2_0_0_2_1 + 0.0833333333333332*G20_2_0_0_0_2_0_0_3_2 - 0.0833333333333332*G20_2_0_0_1_2_0_1_0_0 - 0.0833333333333332*G20_2_0_0_1_2_0_1_0_1 - 0.0833333333333332*G20_2_0_0_1_2_0_1_0_2 + 0.0833333333333332*G20_2_0_0_1_2_0_1_1_0 + 0.0833333333333332*G20_2_0_0_1_2_0_1_2_1 + 0.0833333333333332*G20_2_0_0_1_2_0_1_3_2 - 0.0833333333333332*G20_2_0_0_2_2_0_2_0_0 - 0.0833333333333332*G20_2_0_0_2_2_0_2_0_1 - 0.0833333333333332*G20_2_0_0_2_2_0_2_0_2 + 0.0833333333333332*G20_2_0_0_2_2_0_2_1_0 + 0.0833333333333332*G20_2_0_0_2_2_0_2_2_1 + 0.0833333333333332*G20_2_0_0_2_2_0_2_3_2 + 0.166666666666666*G21_0_0_0_0_0_0_0_0 + 0.166666666666666*G21_0_0_0_0_0_0_0_1 + 0.166666666666666*G21_0_0_0_0_0_0_0_2 - 0.166666666666666*G21_0_0_0_0_0_0_1_0 - 0.166666666666666*G21_0_0_0_0_0_0_2_1 - 0.166666666666666*G21_0_0_0_0_0_0_3_2 + 0.166666666666666*G21_0_0_0_1_0_1_0_0 + 0.166666666666666*G21_0_0_0_1_0_1_0_1 + 0.166666666666666*G21_0_0_0_1_0_1_0_2 - 0.166666666666666*G21_0_0_0_1_0_1_1_0 - 0.166666666666666*G21_0_0_0_1_0_1_2_1 - 0.166666666666666*G21_0_0_0_1_0_1_3_2 + 0.166666666666666*G21_0_0_0_2_0_2_0_0 + 0.166666666666666*G21_0_0_0_2_0_2_0_1 + 0.166666666666666*G21_0_0_0_2_0_2_0_2 - 0.166666666666666*G21_0_0_0_2_0_2_1_0 - 0.166666666666666*G21_0_0_0_2_0_2_2_1 - 0.166666666666666*G21_0_0_0_2_0_2_3_2 - 0.166666666666666*G22_0_0_0_0_0_0_0_0 - 0.166666666666666*G22_0_0_0_0_0_0_0_1 - 0.166666666666666*G22_0_0_0_0_0_0_0_2 + 0.166666666666666*G22_0_0_0_0_0_0_1_0 + 0.166666666666666*G22_0_0_0_0_0_0_2_1 + 0.166666666666666*G22_0_0_0_0_0_0_3_2 - 0.166666666666666*G22_0_0_0_1_0_1_0_0 - 0.166666666666666*G22_0_0_0_1_0_1_0_1 - 0.166666666666666*G22_0_0_0_1_0_1_0_2 + 0.166666666666666*G22_0_0_0_1_0_1_1_0 + 0.166666666666666*G22_0_0_0_1_0_1_2_1 + 0.166666666666666*G22_0_0_0_1_0_1_3_2 - 0.166666666666666*G22_0_0_0_2_0_2_0_0 - 0.166666666666666*G22_0_0_0_2_0_2_0_1 - 0.166666666666666*G22_0_0_0_2_0_2_0_2 + 0.166666666666666*G22_0_0_0_2_0_2_1_0 + 0.166666666666666*G22_0_0_0_2_0_2_2_1 + 0.166666666666666*G22_0_0_0_2_0_2_3_2 + 0.0833333333333332*G23_0_0_0_0_0_0_0 + 0.0833333333333332*G23_0_0_0_0_0_1_0 + 0.0833333333333332*G23_0_0_0_0_0_2_0 - 0.0833333333333332*G23_0_0_0_0_1_0_0 - 0.0833333333333332*G23_0_0_0_0_2_1_0 - 0.0833333333333332*G23_0_0_0_0_3_2_0 + 0.0833333333333332*G23_0_0_0_0_4_0_1 + 0.0833333333333332*G23_0_0_0_0_4_1_1 + 0.0833333333333332*G23_0_0_0_0_4_2_1 - 0.0833333333333332*G23_0_0_0_0_5_0_1 - 0.0833333333333332*G23_0_0_0_0_6_1_1 - 0.0833333333333332*G23_0_0_0_0_7_2_1 + 0.0833333333333332*G23_0_0_0_0_8_0_2 + 0.0833333333333332*G23_0_0_0_0_8_1_2 + 0.0833333333333332*G23_0_0_0_0_8_2_2 - 0.0833333333333332*G23_0_0_0_0_9_0_2 - 0.0833333333333332*G23_0_0_0_0_10_1_2 - 0.0833333333333332*G23_0_0_0_0_11_2_2;
    A[2] = 0.00833333333333331*G0_0_0 + 0.00833333333333331*G0_0_1 + 0.0166666666666666*G0_0_2 + 0.00833333333333331*G0_0_3 + 0.00833333333333331*G1_0_0 + 0.00833333333333331*G1_0_1 + 0.0166666666666666*G1_0_2 + 0.00833333333333331*G1_0_3 + 0.166666666666666*G2_0_0_0_1 + 0.166666666666666*G3_0_0_3_1 + 0.166666666666666*G4_0_0_6_1 - 0.166666666666666*G9_0_0_1 - 0.166666666666666*G10_0_3_1 - 0.166666666666666*G11_0_6_1 + 0.0416666666666666*G16_0_0_1_0 + 0.0416666666666666*G16_0_1_1_0 + 0.0416666666666666*G16_0_2_1_0 + 0.0416666666666666*G16_0_3_1_0 + 0.0416666666666666*G17_0_0_0_1_0_0 + 0.0416666666666666*G17_0_0_0_1_0_1 + 0.0416666666666666*G17_0_0_0_1_0_2 + 0.0416666666666666*G17_0_0_0_1_0_3 + 0.0416666666666666*G17_0_0_1_1_1_0 + 0.0416666666666666*G17_0_0_1_1_1_1 + 0.0416666666666666*G17_0_0_1_1_1_2 + 0.0416666666666666*G17_0_0_1_1_1_3 + 0.0416666666666666*G17_0_0_2_1_2_0 + 0.0416666666666666*G17_0_0_2_1_2_1 + 0.0416666666666666*G17_0_0_2_1_2_2 + 0.0416666666666666*G17_0_0_2_1_2_3 - 0.0416666666666666*G18_0_0_0_1_0_0 - 0.0416666666666666*G18_0_0_0_1_0_1 - 0.0416666666666666*G18_0_0_0_1_0_2 - 0.0416666666666666*G18_0_0_0_1_0_3 - 0.0416666666666666*G18_0_0_1_1_1_0 - 0.0416666666666666*G18_0_0_1_1_1_1 - 0.0416666666666666*G18_0_0_1_1_1_2 - 0.0416666666666666*G18_0_0_1_1_1_3 - 0.0416666666666666*G18_0_0_2_1_2_0 - 0.0416666666666666*G18_0_0_2_1_2_1 - 0.0416666666666666*G18_0_0_2_1_2_2 - 0.0416666666666666*G18_0_0_2_1_2_3 + 0.0833333333333332*G19_0_0_0_0_0_1_0_0_0 + 0.0833333333333332*G19_0_0_0_0_0_1_0_0_1 + 0.0833333333333332*G19_0_0_0_0_0_1_0_0_2 - 0.0833333333333332*G19_0_0_0_0_0_1_0_1_0 - 0.0833333333333332*G19_0_0_0_0_0_1_0_2_1 - 0.0833333333333332*G19_0_0_0_0_0_1_0_3_2 + 0.0833333333333332*G19_0_0_0_1_0_1_1_0_0 + 0.0833333333333332*G19_0_0_0_1_0_1_1_0_1 + 0.0833333333333332*G19_0_0_0_1_0_1_1_0_2 - 0.0833333333333332*G19_0_0_0_1_0_1_1_1_0 - 0.0833333333333332*G19_0_0_0_1_0_1_1_2_1 - 0.0833333333333332*G19_0_0_0_1_0_1_1_3_2 + 0.0833333333333332*G19_0_0_0_2_0_1_2_0_0 + 0.0833333333333332*G19_0_0_0_2_0_1_2_0_1 + 0.0833333333333332*G19_0_0_0_2_0_1_2_0_2 - 0.0833333333333332*G19_0_0_0_2_0_1_2_1_0 - 0.0833333333333332*G19_0_0_0_2_0_1_2_2_1 - 0.0833333333333332*G19_0_0_0_2_0_1_2_3_2 + 0.0833333333333332*G19_1_0_0_0_1_1_0_0_0 + 0.0833333333333332*G19_1_0_0_0_1_1_0_0_1 + 0.0833333333333332*G19_1_0_0_0_1_1_0_0_2 - 0.0833333333333332*G19_1_0_0_0_1_1_0_1_0 - 0.0833333333333332*G19_1_0_0_0_1_1_0_2_1 - 0.0833333333333332*G19_1_0_0_0_1_1_0_3_2 + 0.0833333333333332*G19_1_0_0_1_1_1_1_0_0 + 0.0833333333333332*G19_1_0_0_1_1_1_1_0_1 + 0.0833333333333332*G19_1_0_0_1_1_1_1_0_2 - 0.0833333333333332*G19_1_0_0_1_1_1_1_1_0 - 0.0833333333333332*G19_1_0_0_1_1_1_1_2_1 - 0.0833333333333332*G19_1_0_0_1_1_1_1_3_2 + 0.0833333333333332*G19_1_0_0_2_1_1_2_0_0 + 0.0833333333333332*G19_1_0_0_2_1_1_2_0_1 + 0.0833333333333332*G19_1_0_0_2_1_1_2_0_2 - 0.0833333333333332*G19_1_0_0_2_1_1_2_1_0 - 0.0833333333333332*G19_1_0_0_2_1_1_2_2_1 - 0.0833333333333332*G19_1_0_0_2_1_1_2_3_2 + 0.0833333333333332*G19_2_0_0_0_2_1_0_0_0 + 0.0833333333333332*G19_2_0_0_0_2_1_0_0_1 + 0.0833333333333332*G19_2_0_0_0_2_1_0_0_2 - 0.0833333333333332*G19_2_0_0_0_2_1_0_1_0 - 0.0833333333333332*G19_2_0_0_0_2_1_0_2_1 - 0.0833333333333332*G19_2_0_0_0_2_1_0_3_2 + 0.0833333333333332*G19_2_0_0_1_2_1_1_0_0 + 0.0833333333333332*G19_2_0_0_1_2_1_1_0_1 + 0.0833333333333332*G19_2_0_0_1_2_1_1_0_2 - 0.0833333333333332*G19_2_0_0_1_2_1_1_1_0 - 0.0833333333333332*G19_2_0_0_1_2_1_1_2_1 - 0.0833333333333332*G19_2_0_0_1_2_1_1_3_2 + 0.0833333333333332*G19_2_0_0_2_2_1_2_0_0 + 0.0833333333333332*G19_2_0_0_2_2_1_2_0_1 + 0.0833333333333332*G19_2_0_0_2_2_1_2_0_2 - 0.0833333333333332*G19_2_0_0_2_2_1_2_1_0 - 0.0833333333333332*G19_2_0_0_2_2_1_2_2_1 - 0.0833333333333332*G19_2_0_0_2_2_1_2_3_2 - 0.0833333333333332*G20_0_0_0_0_0_1_0_0_0 - 0.0833333333333332*G20_0_0_0_0_0_1_0_0_1 - 0.0833333333333332*G20_0_0_0_0_0_1_0_0_2 + 0.0833333333333332*G20_0_0_0_0_0_1_0_1_0 + 0.0833333333333332*G20_0_0_0_0_0_1_0_2_1 + 0.0833333333333332*G20_0_0_0_0_0_1_0_3_2 - 0.0833333333333332*G20_0_0_0_1_0_1_1_0_0 - 0.0833333333333332*G20_0_0_0_1_0_1_1_0_1 - 0.0833333333333332*G20_0_0_0_1_0_1_1_0_2 + 0.0833333333333332*G20_0_0_0_1_0_1_1_1_0 + 0.0833333333333332*G20_0_0_0_1_0_1_1_2_1 + 0.0833333333333332*G20_0_0_0_1_0_1_1_3_2 - 0.0833333333333332*G20_0_0_0_2_0_1_2_0_0 - 0.0833333333333332*G20_0_0_0_2_0_1_2_0_1 - 0.0833333333333332*G20_0_0_0_2_0_1_2_0_2 + 0.0833333333333332*G20_0_0_0_2_0_1_2_1_0 + 0.0833333333333332*G20_0_0_0_2_0_1_2_2_1 + 0.0833333333333332*G20_0_0_0_2_0_1_2_3_2 - 0.0833333333333332*G20_1_0_0_0_1_1_0_0_0 - 0.0833333333333332*G20_1_0_0_0_1_1_0_0_1 - 0.0833333333333332*G20_1_0_0_0_1_1_0_0_2 + 0.0833333333333332*G20_1_0_0_0_1_1_0_1_0 + 0.0833333333333332*G20_1_0_0_0_1_1_0_2_1 + 0.0833333333333332*G20_1_0_0_0_1_1_0_3_2 - 0.0833333333333332*G20_1_0_0_1_1_1_1_0_0 - 0.0833333333333332*G20_1_0_0_1_1_1_1_0_1 - 0.0833333333333332*G20_1_0_0_1_1_1_1_0_2 + 0.0833333333333332*G20_1_0_0_1_1_1_1_1_0 + 0.0833333333333332*G20_1_0_0_1_1_1_1_2_1 + 0.0833333333333332*G20_1_0_0_1_1_1_1_3_2 - 0.0833333333333332*G20_1_0_0_2_1_1_2_0_0 - 0.0833333333333332*G20_1_0_0_2_1_1_2_0_1 - 0.0833333333333332*G20_1_0_0_2_1_1_2_0_2 + 0.0833333333333332*G20_1_0_0_2_1_1_2_1_0 + 0.0833333333333332*G20_1_0_0_2_1_1_2_2_1 + 0.0833333333333332*G20_1_0_0_2_1_1_2_3_2 - 0.0833333333333332*G20_2_0_0_0_2_1_0_0_0 - 0.0833333333333332*G20_2_0_0_0_2_1_0_0_1 - 0.0833333333333332*G20_2_0_0_0_2_1_0_0_2 + 0.0833333333333332*G20_2_0_0_0_2_1_0_1_0 + 0.0833333333333332*G20_2_0_0_0_2_1_0_2_1 + 0.0833333333333332*G20_2_0_0_0_2_1_0_3_2 - 0.0833333333333332*G20_2_0_0_1_2_1_1_0_0 - 0.0833333333333332*G20_2_0_0_1_2_1_1_0_1 - 0.0833333333333332*G20_2_0_0_1_2_1_1_0_2 + 0.0833333333333332*G20_2_0_0_1_2_1_1_1_0 + 0.0833333333333332*G20_2_0_0_1_2_1_1_2_1 + 0.0833333333333332*G20_2_0_0_1_2_1_1_3_2 - 0.0833333333333332*G20_2_0_0_2_2_1_2_0_0 - 0.0833333333333332*G20_2_0_0_2_2_1_2_0_1 - 0.0833333333333332*G20_2_0_0_2_2_1_2_0_2 + 0.0833333333333332*G20_2_0_0_2_2_1_2_1_0 + 0.0833333333333332*G20_2_0_0_2_2_1_2_2_1 + 0.0833333333333332*G20_2_0_0_2_2_1_2_3_2 + 0.166666666666666*G21_0_0_0_0_1_0_0_0 + 0.166666666666666*G21_0_0_0_0_1_0_0_1 + 0.166666666666666*G21_0_0_0_0_1_0_0_2 - 0.166666666666666*G21_0_0_0_0_1_0_1_0 - 0.166666666666666*G21_0_0_0_0_1_0_2_1 - 0.166666666666666*G21_0_0_0_0_1_0_3_2 + 0.166666666666666*G21_0_0_0_1_1_1_0_0 + 0.166666666666666*G21_0_0_0_1_1_1_0_1 + 0.166666666666666*G21_0_0_0_1_1_1_0_2 - 0.166666666666666*G21_0_0_0_1_1_1_1_0 - 0.166666666666666*G21_0_0_0_1_1_1_2_1 - 0.166666666666666*G21_0_0_0_1_1_1_3_2 + 0.166666666666666*G21_0_0_0_2_1_2_0_0 + 0.166666666666666*G21_0_0_0_2_1_2_0_1 + 0.166666666666666*G21_0_0_0_2_1_2_0_2 - 0.166666666666666*G21_0_0_0_2_1_2_1_0 - 0.166666666666666*G21_0_0_0_2_1_2_2_1 - 0.166666666666666*G21_0_0_0_2_1_2_3_2 - 0.166666666666666*G22_0_0_0_0_1_0_0_0 - 0.166666666666666*G22_0_0_0_0_1_0_0_1 - 0.166666666666666*G22_0_0_0_0_1_0_0_2 + 0.166666666666666*G22_0_0_0_0_1_0_1_0 + 0.166666666666666*G22_0_0_0_0_1_0_2_1 + 0.166666666666666*G22_0_0_0_0_1_0_3_2 - 0.166666666666666*G22_0_0_0_1_1_1_0_0 - 0.166666666666666*G22_0_0_0_1_1_1_0_1 - 0.166666666666666*G22_0_0_0_1_1_1_0_2 + 0.166666666666666*G22_0_0_0_1_1_1_1_0 + 0.166666666666666*G22_0_0_0_1_1_1_2_1 + 0.166666666666666*G22_0_0_0_1_1_1_3_2 - 0.166666666666666*G22_0_0_0_2_1_2_0_0 - 0.166666666666666*G22_0_0_0_2_1_2_0_1 - 0.166666666666666*G22_0_0_0_2_1_2_0_2 + 0.166666666666666*G22_0_0_0_2_1_2_1_0 + 0.166666666666666*G22_0_0_0_2_1_2_2_1 + 0.166666666666666*G22_0_0_0_2_1_2_3_2 + 0.0833333333333332*G23_0_0_1_0_0_0_0 + 0.0833333333333332*G23_0_0_1_0_0_1_0 + 0.0833333333333332*G23_0_0_1_0_0_2_0 - 0.0833333333333332*G23_0_0_1_0_1_0_0 - 0.0833333333333332*G23_0_0_1_0_2_1_0 - 0.0833333333333332*G23_0_0_1_0_3_2_0 + 0.0833333333333332*G23_0_0_1_0_4_0_1 + 0.0833333333333332*G23_0_0_1_0_4_1_1 + 0.0833333333333332*G23_0_0_1_0_4_2_1 - 0.0833333333333332*G23_0_0_1_0_5_0_1 - 0.0833333333333332*G23_0_0_1_0_6_1_1 - 0.0833333333333332*G23_0_0_1_0_7_2_1 + 0.0833333333333332*G23_0_0_1_0_8_0_2 + 0.0833333333333332*G23_0_0_1_0_8_1_2 + 0.0833333333333332*G23_0_0_1_0_8_2_2 - 0.0833333333333332*G23_0_0_1_0_9_0_2 - 0.0833333333333332*G23_0_0_1_0_10_1_2 - 0.0833333333333332*G23_0_0_1_0_11_2_2;
    A[3] = 0.00833333333333331*G0_0_0 + 0.00833333333333331*G0_0_1 + 0.00833333333333331*G0_0_2 + 0.0166666666666666*G0_0_3 + 0.00833333333333331*G1_0_0 + 0.00833333333333331*G1_0_1 + 0.00833333333333331*G1_0_2 + 0.0166666666666666*G1_0_3 + 0.166666666666666*G2_0_0_0_2 + 0.166666666666666*G3_0_0_3_2 + 0.166666666666666*G4_0_0_6_2 - 0.166666666666666*G9_0_0_2 - 0.166666666666666*G10_0_3_2 - 0.166666666666666*G11_0_6_2 + 0.0416666666666666*G16_0_0_2_0 + 0.0416666666666666*G16_0_1_2_0 + 0.0416666666666666*G16_0_2_2_0 + 0.0416666666666666*G16_0_3_2_0 + 0.0416666666666666*G17_0_0_0_2_0_0 + 0.0416666666666666*G17_0_0_0_2_0_1 + 0.0416666666666666*G17_0_0_0_2_0_2 + 0.0416666666666666*G17_0_0_0_2_0_3 + 0.0416666666666666*G17_0_0_1_2_1_0 + 0.0416666666666666*G17_0_0_1_2_1_1 + 0.0416666666666666*G17_0_0_1_2_1_2 + 0.0416666666666666*G17_0_0_1_2_1_3 + 0.0416666666666666*G17_0_0_2_2_2_0 + 0.0416666666666666*G17_0_0_2_2_2_1 + 0.0416666666666666*G17_0_0_2_2_2_2 + 0.0416666666666666*G17_0_0_2_2_2_3 - 0.0416666666666666*G18_0_0_0_2_0_0 - 0.0416666666666666*G18_0_0_0_2_0_1 - 0.0416666666666666*G18_0_0_0_2_0_2 - 0.0416666666666666*G18_0_0_0_2_0_3 - 0.0416666666666666*G18_0_0_1_2_1_0 - 0.0416666666666666*G18_0_0_1_2_1_1 - 0.0416666666666666*G18_0_0_1_2_1_2 - 0.0416666666666666*G18_0_0_1_2_1_3 - 0.0416666666666666*G18_0_0_2_2_2_0 - 0.0416666666666666*G18_0_0_2_2_2_1 - 0.0416666666666666*G18_0_0_2_2_2_2 - 0.0416666666666666*G18_0_0_2_2_2_3 + 0.0833333333333332*G19_0_0_0_0_0_2_0_0_0 + 0.0833333333333332*G19_0_0_0_0_0_2_0_0_1 + 0.0833333333333332*G19_0_0_0_0_0_2_0_0_2 - 0.0833333333333332*G19_0_0_0_0_0_2_0_1_0 - 0.0833333333333332*G19_0_0_0_0_0_2_0_2_1 - 0.0833333333333332*G19_0_0_0_0_0_2_0_3_2 + 0.0833333333333332*G19_0_0_0_1_0_2_1_0_0 + 0.0833333333333332*G19_0_0_0_1_0_2_1_0_1 + 0.0833333333333332*G19_0_0_0_1_0_2_1_0_2 - 0.0833333333333332*G19_0_0_0_1_0_2_1_1_0 - 0.0833333333333332*G19_0_0_0_1_0_2_1_2_1 - 0.0833333333333332*G19_0_0_0_1_0_2_1_3_2 + 0.0833333333333332*G19_0_0_0_2_0_2_2_0_0 + 0.0833333333333332*G19_0_0_0_2_0_2_2_0_1 + 0.0833333333333332*G19_0_0_0_2_0_2_2_0_2 - 0.0833333333333332*G19_0_0_0_2_0_2_2_1_0 - 0.0833333333333332*G19_0_0_0_2_0_2_2_2_1 - 0.0833333333333332*G19_0_0_0_2_0_2_2_3_2 + 0.0833333333333332*G19_1_0_0_0_1_2_0_0_0 + 0.0833333333333332*G19_1_0_0_0_1_2_0_0_1 + 0.0833333333333332*G19_1_0_0_0_1_2_0_0_2 - 0.0833333333333332*G19_1_0_0_0_1_2_0_1_0 - 0.0833333333333332*G19_1_0_0_0_1_2_0_2_1 - 0.0833333333333332*G19_1_0_0_0_1_2_0_3_2 + 0.0833333333333332*G19_1_0_0_1_1_2_1_0_0 + 0.0833333333333332*G19_1_0_0_1_1_2_1_0_1 + 0.0833333333333332*G19_1_0_0_1_1_2_1_0_2 - 0.0833333333333332*G19_1_0_0_1_1_2_1_1_0 - 0.0833333333333332*G19_1_0_0_1_1_2_1_2_1 - 0.0833333333333332*G19_1_0_0_1_1_2_1_3_2 + 0.0833333333333332*G19_1_0_0_2_1_2_2_0_0 + 0.0833333333333332*G19_1_0_0_2_1_2_2_0_1 + 0.0833333333333332*G19_1_0_0_2_1_2_2_0_2 - 0.0833333333333332*G19_1_0_0_2_1_2_2_1_0 - 0.0833333333333332*G19_1_0_0_2_1_2_2_2_1 - 0.0833333333333332*G19_1_0_0_2_1_2_2_3_2 + 0.0833333333333332*G19_2_0_0_0_2_2_0_0_0 + 0.0833333333333332*G19_2_0_0_0_2_2_0_0_1 + 0.0833333333333332*G19_2_0_0_0_2_2_0_0_2 - 0.0833333333333332*G19_2_0_0_0_2_2_0_1_0 - 0.0833333333333332*G19_2_0_0_0_2_2_0_2_1 - 0.0833333333333332*G19_2_0_0_0_2_2_0_3_2 + 0.0833333333333332*G19_2_0_0_1_2_2_1_0_0 + 0.0833333333333332*G19_2_0_0_1_2_2_1_0_1 + 0.0833333333333332*G19_2_0_0_1_2_2_1_0_2 - 0.0833333333333332*G19_2_0_0_1_2_2_1_1_0 - 0.0833333333333332*G19_2_0_0_1_2_2_1_2_1 - 0.0833333333333332*G19_2_0_0_1_2_2_1_3_2 + 0.0833333333333332*G19_2_0_0_2_2_2_2_0_0 + 0.0833333333333332*G19_2_0_0_2_2_2_2_0_1 + 0.0833333333333332*G19_2_0_0_2_2_2_2_0_2 - 0.0833333333333332*G19_2_0_0_2_2_2_2_1_0 - 0.0833333333333332*G19_2_0_0_2_2_2_2_2_1 - 0.0833333333333332*G19_2_0_0_2_2_2_2_3_2 - 0.0833333333333332*G20_0_0_0_0_0_2_0_0_0 - 0.0833333333333332*G20_0_0_0_0_0_2_0_0_1 - 0.0833333333333332*G20_0_0_0_0_0_2_0_0_2 + 0.0833333333333332*G20_0_0_0_0_0_2_0_1_0 + 0.0833333333333332*G20_0_0_0_0_0_2_0_2_1 + 0.0833333333333332*G20_0_0_0_0_0_2_0_3_2 - 0.0833333333333332*G20_0_0_0_1_0_2_1_0_0 - 0.0833333333333332*G20_0_0_0_1_0_2_1_0_1 - 0.0833333333333332*G20_0_0_0_1_0_2_1_0_2 + 0.0833333333333332*G20_0_0_0_1_0_2_1_1_0 + 0.0833333333333332*G20_0_0_0_1_0_2_1_2_1 + 0.0833333333333332*G20_0_0_0_1_0_2_1_3_2 - 0.0833333333333332*G20_0_0_0_2_0_2_2_0_0 - 0.0833333333333332*G20_0_0_0_2_0_2_2_0_1 - 0.0833333333333332*G20_0_0_0_2_0_2_2_0_2 + 0.0833333333333332*G20_0_0_0_2_0_2_2_1_0 + 0.0833333333333332*G20_0_0_0_2_0_2_2_2_1 + 0.0833333333333332*G20_0_0_0_2_0_2_2_3_2 - 0.0833333333333332*G20_1_0_0_0_1_2_0_0_0 - 0.0833333333333332*G20_1_0_0_0_1_2_0_0_1 - 0.0833333333333332*G20_1_0_0_0_1_2_0_0_2 + 0.0833333333333332*G20_1_0_0_0_1_2_0_1_0 + 0.0833333333333332*G20_1_0_0_0_1_2_0_2_1 + 0.0833333333333332*G20_1_0_0_0_1_2_0_3_2 - 0.0833333333333332*G20_1_0_0_1_1_2_1_0_0 - 0.0833333333333332*G20_1_0_0_1_1_2_1_0_1 - 0.0833333333333332*G20_1_0_0_1_1_2_1_0_2 + 0.0833333333333332*G20_1_0_0_1_1_2_1_1_0 + 0.0833333333333332*G20_1_0_0_1_1_2_1_2_1 + 0.0833333333333332*G20_1_0_0_1_1_2_1_3_2 - 0.0833333333333332*G20_1_0_0_2_1_2_2_0_0 - 0.0833333333333332*G20_1_0_0_2_1_2_2_0_1 - 0.0833333333333332*G20_1_0_0_2_1_2_2_0_2 + 0.0833333333333332*G20_1_0_0_2_1_2_2_1_0 + 0.0833333333333332*G20_1_0_0_2_1_2_2_2_1 + 0.0833333333333332*G20_1_0_0_2_1_2_2_3_2 - 0.0833333333333332*G20_2_0_0_0_2_2_0_0_0 - 0.0833333333333332*G20_2_0_0_0_2_2_0_0_1 - 0.0833333333333332*G20_2_0_0_0_2_2_0_0_2 + 0.0833333333333332*G20_2_0_0_0_2_2_0_1_0 + 0.0833333333333332*G20_2_0_0_0_2_2_0_2_1 + 0.0833333333333332*G20_2_0_0_0_2_2_0_3_2 - 0.0833333333333332*G20_2_0_0_1_2_2_1_0_0 - 0.0833333333333332*G20_2_0_0_1_2_2_1_0_1 - 0.0833333333333332*G20_2_0_0_1_2_2_1_0_2 + 0.0833333333333332*G20_2_0_0_1_2_2_1_1_0 + 0.0833333333333332*G20_2_0_0_1_2_2_1_2_1 + 0.0833333333333332*G20_2_0_0_1_2_2_1_3_2 - 0.0833333333333332*G20_2_0_0_2_2_2_2_0_0 - 0.0833333333333332*G20_2_0_0_2_2_2_2_0_1 - 0.0833333333333332*G20_2_0_0_2_2_2_2_0_2 + 0.0833333333333332*G20_2_0_0_2_2_2_2_1_0 + 0.0833333333333332*G20_2_0_0_2_2_2_2_2_1 + 0.0833333333333332*G20_2_0_0_2_2_2_2_3_2 + 0.166666666666666*G21_0_0_0_0_2_0_0_0 + 0.166666666666666*G21_0_0_0_0_2_0_0_1 + 0.166666666666666*G21_0_0_0_0_2_0_0_2 - 0.166666666666666*G21_0_0_0_0_2_0_1_0 - 0.166666666666666*G21_0_0_0_0_2_0_2_1 - 0.166666666666666*G21_0_0_0_0_2_0_3_2 + 0.166666666666666*G21_0_0_0_1_2_1_0_0 + 0.166666666666666*G21_0_0_0_1_2_1_0_1 + 0.166666666666666*G21_0_0_0_1_2_1_0_2 - 0.166666666666666*G21_0_0_0_1_2_1_1_0 - 0.166666666666666*G21_0_0_0_1_2_1_2_1 - 0.166666666666666*G21_0_0_0_1_2_1_3_2 + 0.166666666666666*G21_0_0_0_2_2_2_0_0 + 0.166666666666666*G21_0_0_0_2_2_2_0_1 + 0.166666666666666*G21_0_0_0_2_2_2_0_2 - 0.166666666666666*G21_0_0_0_2_2_2_1_0 - 0.166666666666666*G21_0_0_0_2_2_2_2_1 - 0.166666666666666*G21_0_0_0_2_2_2_3_2 - 0.166666666666666*G22_0_0_0_0_2_0_0_0 - 0.166666666666666*G22_0_0_0_0_2_0_0_1 - 0.166666666666666*G22_0_0_0_0_2_0_0_2 + 0.166666666666666*G22_0_0_0_0_2_0_1_0 + 0.166666666666666*G22_0_0_0_0_2_0_2_1 + 0.166666666666666*G22_0_0_0_0_2_0_3_2 - 0.166666666666666*G22_0_0_0_1_2_1_0_0 - 0.166666666666666*G22_0_0_0_1_2_1_0_1 - 0.166666666666666*G22_0_0_0_1_2_1_0_2 + 0.166666666666666*G22_0_0_0_1_2_1_1_0 + 0.166666666666666*G22_0_0_0_1_2_1_2_1 + 0.166666666666666*G22_0_0_0_1_2_1_3_2 - 0.166666666666666*G22_0_0_0_2_2_2_0_0 - 0.166666666666666*G22_0_0_0_2_2_2_0_1 - 0.166666666666666*G22_0_0_0_2_2_2_0_2 + 0.166666666666666*G22_0_0_0_2_2_2_1_0 + 0.166666666666666*G22_0_0_0_2_2_2_2_1 + 0.166666666666666*G22_0_0_0_2_2_2_3_2 + 0.0833333333333332*G23_0_0_2_0_0_0_0 + 0.0833333333333332*G23_0_0_2_0_0_1_0 + 0.0833333333333332*G23_0_0_2_0_0_2_0 - 0.0833333333333332*G23_0_0_2_0_1_0_0 - 0.0833333333333332*G23_0_0_2_0_2_1_0 - 0.0833333333333332*G23_0_0_2_0_3_2_0 + 0.0833333333333332*G23_0_0_2_0_4_0_1 + 0.0833333333333332*G23_0_0_2_0_4_1_1 + 0.0833333333333332*G23_0_0_2_0_4_2_1 - 0.0833333333333332*G23_0_0_2_0_5_0_1 - 0.0833333333333332*G23_0_0_2_0_6_1_1 - 0.0833333333333332*G23_0_0_2_0_7_2_1 + 0.0833333333333332*G23_0_0_2_0_8_0_2 + 0.0833333333333332*G23_0_0_2_0_8_1_2 + 0.0833333333333332*G23_0_0_2_0_8_2_2 - 0.0833333333333332*G23_0_0_2_0_9_0_2 - 0.0833333333333332*G23_0_0_2_0_10_1_2 - 0.0833333333333332*G23_0_0_2_0_11_2_2;
    A[4] = 0.0166666666666666*G0_0_4 + 0.00833333333333331*G0_0_5 + 0.00833333333333331*G0_0_6 + 0.00833333333333331*G0_0_7 + 0.0166666666666666*G1_0_4 + 0.00833333333333331*G1_0_5 + 0.00833333333333331*G1_0_6 + 0.00833333333333331*G1_0_7 - 0.166666666666666*G2_0_0_1_0 - 0.166666666666666*G2_0_0_1_1 - 0.166666666666666*G2_0_0_1_2 - 0.166666666666666*G5_0_0_4_0 - 0.166666666666666*G5_0_0_4_1 - 0.166666666666666*G5_0_0_4_2 - 0.166666666666666*G6_0_0_7_0 - 0.166666666666666*G6_0_0_7_1 - 0.166666666666666*G6_0_0_7_2 + 0.166666666666666*G9_0_1_0 + 0.166666666666666*G9_0_1_1 + 0.166666666666666*G9_0_1_2 + 0.166666666666666*G12_0_4_0 + 0.166666666666666*G12_0_4_1 + 0.166666666666666*G12_0_4_2 + 0.166666666666666*G13_0_7_0 + 0.166666666666666*G13_0_7_1 + 0.166666666666666*G13_0_7_2 - 0.0416666666666666*G16_0_0_0_1 - 0.0416666666666666*G16_0_0_1_1 - 0.0416666666666666*G16_0_0_2_1 - 0.0416666666666666*G16_0_1_0_1 - 0.0416666666666666*G16_0_1_1_1 - 0.0416666666666666*G16_0_1_2_1 - 0.0416666666666666*G16_0_2_0_1 - 0.0416666666666666*G16_0_2_1_1 - 0.0416666666666666*G16_0_2_2_1 - 0.0416666666666666*G16_0_3_0_1 - 0.0416666666666666*G16_0_3_1_1 - 0.0416666666666666*G16_0_3_2_1 - 0.0416666666666666*G17_0_0_0_0_0_4 - 0.0416666666666666*G17_0_0_0_0_0_5 - 0.0416666666666666*G17_0_0_0_0_0_6 - 0.0416666666666666*G17_0_0_0_0_0_7 - 0.0416666666666666*G17_0_0_0_1_0_4 - 0.0416666666666666*G17_0_0_0_1_0_5 - 0.0416666666666666*G17_0_0_0_1_0_6 - 0.0416666666666666*G17_0_0_0_1_0_7 - 0.0416666666666666*G17_0_0_0_2_0_4 - 0.0416666666666666*G17_0_0_0_2_0_5 - 0.0416666666666666*G17_0_0_0_2_0_6 - 0.0416666666666666*G17_0_0_0_2_0_7 - 0.0416666666666666*G17_0_0_1_0_1_4 - 0.0416666666666666*G17_0_0_1_0_1_5 - 0.0416666666666666*G17_0_0_1_0_1_6 - 0.0416666666666666*G17_0_0_1_0_1_7 - 0.0416666666666666*G17_0_0_1_1_1_4 - 0.0416666666666666*G17_0_0_1_1_1_5 - 0.0416666666666666*G17_0_0_1_1_1_6 - 0.0416666666666666*G17_0_0_1_1_1_7 - 0.0416666666666666*G17_0_0_1_2_1_4 - 0.0416666666666666*G17_0_0_1_2_1_5 - 0.0416666666666666*G17_0_0_1_2_1_6 - 0.0416666666666666*G17_0_0_1_2_1_7 - 0.0416666666666666*G17_0_0_2_0_2_4 - 0.0416666666666666*G17_0_0_2_0_2_5 - 0.0416666666666666*G17_0_0_2_0_2_6 - 0.0416666666666666*G17_0_0_2_0_2_7 - 0.0416666666666666*G17_0_0_2_1_2_4 - 0.0416666666666666*G17_0_0_2_1_2_5 - 0.0416666666666666*G17_0_0_2_1_2_6 - 0.0416666666666666*G17_0_0_2_1_2_7 - 0.0416666666666666*G17_0_0_2_2_2_4 - 0.0416666666666666*G17_0_0_2_2_2_5 - 0.0416666666666666*G17_0_0_2_2_2_6 - 0.0416666666666666*G17_0_0_2_2_2_7 + 0.0416666666666666*G18_0_0_0_0_0_4 + 0.0416666666666666*G18_0_0_0_0_0_5 + 0.0416666666666666*G18_0_0_0_0_0_6 + 0.0416666666666666*G18_0_0_0_0_0_7 + 0.0416666666666666*G18_0_0_0_1_0_4 + 0.0416666666666666*G18_0_0_0_1_0_5 + 0.0416666666666666*G18_0_0_0_1_0_6 + 0.0416666666666666*G18_0_0_0_1_0_7 + 0.0416666666666666*G18_0_0_0_2_0_4 + 0.0416666666666666*G18_0_0_0_2_0_5 + 0.0416666666666666*G18_0_0_0_2_0_6 + 0.0416666666666666*G18_0_0_0_2_0_7 + 0.0416666666666666*G18_0_0_1_0_1_4 + 0.0416666666666666*G18_0_0_1_0_1_5 + 0.0416666666666666*G18_0_0_1_0_1_6 + 0.0416666666666666*G18_0_0_1_0_1_7 + 0.0416666666666666*G18_0_0_1_1_1_4 + 0.0416666666666666*G18_0_0_1_1_1_5 + 0.0416666666666666*G18_0_0_1_1_1_6 + 0.0416666666666666*G18_0_0_1_1_1_7 + 0.0416666666666666*G18_0_0_1_2_1_4 + 0.0416666666666666*G18_0_0_1_2_1_5 + 0.0416666666666666*G18_0_0_1_2_1_6 + 0.0416666666666666*G18_0_0_1_2_1_7 + 0.0416666666666666*G18_0_0_2_0_2_4 + 0.0416666666666666*G18_0_0_2_0_2_5 + 0.0416666666666666*G18_0_0_2_0_2_6 + 0.0416666666666666*G18_0_0_2_0_2_7 + 0.0416666666666666*G18_0_0_2_1_2_4 + 0.0416666666666666*G18_0_0_2_1_2_5 + 0.0416666666666666*G18_0_0_2_1_2_6 + 0.0416666666666666*G18_0_0_2_1_2_7 + 0.0416666666666666*G18_0_0_2_2_2_4 + 0.0416666666666666*G18_0_0_2_2_2_5 + 0.0416666666666666*G18_0_0_2_2_2_6 + 0.0416666666666666*G18_0_0_2_2_2_7 - 0.0833333333333332*G19_0_0_0_0_0_0_0_4_0 - 0.0833333333333332*G19_0_0_0_0_0_0_0_4_1 - 0.0833333333333332*G19_0_0_0_0_0_0_0_4_2 + 0.0833333333333332*G19_0_0_0_0_0_0_0_5_0 + 0.0833333333333332*G19_0_0_0_0_0_0_0_6_1 + 0.0833333333333332*G19_0_0_0_0_0_0_0_7_2 - 0.0833333333333332*G19_0_0_0_0_0_1_0_4_0 - 0.0833333333333332*G19_0_0_0_0_0_1_0_4_1 - 0.0833333333333332*G19_0_0_0_0_0_1_0_4_2 + 0.0833333333333332*G19_0_0_0_0_0_1_0_5_0 + 0.0833333333333332*G19_0_0_0_0_0_1_0_6_1 + 0.0833333333333332*G19_0_0_0_0_0_1_0_7_2 - 0.0833333333333332*G19_0_0_0_0_0_2_0_4_0 - 0.0833333333333332*G19_0_0_0_0_0_2_0_4_1 - 0.0833333333333332*G19_0_0_0_0_0_2_0_4_2 + 0.0833333333333332*G19_0_0_0_0_0_2_0_5_0 + 0.0833333333333332*G19_0_0_0_0_0_2_0_6_1 + 0.0833333333333332*G19_0_0_0_0_0_2_0_7_2 - 0.0833333333333332*G19_0_0_0_1_0_0_1_4_0 - 0.0833333333333332*G19_0_0_0_1_0_0_1_4_1 - 0.0833333333333332*G19_0_0_0_1_0_0_1_4_2 + 0.0833333333333332*G19_0_0_0_1_0_0_1_5_0 + 0.0833333333333332*G19_0_0_0_1_0_0_1_6_1 + 0.0833333333333332*G19_0_0_0_1_0_0_1_7_2 - 0.0833333333333332*G19_0_0_0_1_0_1_1_4_0 - 0.0833333333333332*G19_0_0_0_1_0_1_1_4_1 - 0.0833333333333332*G19_0_0_0_1_0_1_1_4_2 + 0.0833333333333332*G19_0_0_0_1_0_1_1_5_0 + 0.0833333333333332*G19_0_0_0_1_0_1_1_6_1 + 0.0833333333333332*G19_0_0_0_1_0_1_1_7_2 - 0.0833333333333332*G19_0_0_0_1_0_2_1_4_0 - 0.0833333333333332*G19_0_0_0_1_0_2_1_4_1 - 0.0833333333333332*G19_0_0_0_1_0_2_1_4_2 + 0.0833333333333332*G19_0_0_0_1_0_2_1_5_0 + 0.0833333333333332*G19_0_0_0_1_0_2_1_6_1 + 0.0833333333333332*G19_0_0_0_1_0_2_1_7_2 - 0.0833333333333332*G19_0_0_0_2_0_0_2_4_0 - 0.0833333333333332*G19_0_0_0_2_0_0_2_4_1 - 0.0833333333333332*G19_0_0_0_2_0_0_2_4_2 + 0.0833333333333332*G19_0_0_0_2_0_0_2_5_0 + 0.0833333333333332*G19_0_0_0_2_0_0_2_6_1 + 0.0833333333333332*G19_0_0_0_2_0_0_2_7_2 - 0.0833333333333332*G19_0_0_0_2_0_1_2_4_0 - 0.0833333333333332*G19_0_0_0_2_0_1_2_4_1 - 0.0833333333333332*G19_0_0_0_2_0_1_2_4_2 + 0.0833333333333332*G19_0_0_0_2_0_1_2_5_0 + 0.0833333333333332*G19_0_0_0_2_0_1_2_6_1 + 0.0833333333333332*G19_0_0_0_2_0_1_2_7_2 - 0.0833333333333332*G19_0_0_0_2_0_2_2_4_0 - 0.0833333333333332*G19_0_0_0_2_0_2_2_4_1 - 0.0833333333333332*G19_0_0_0_2_0_2_2_4_2 + 0.0833333333333332*G19_0_0_0_2_0_2_2_5_0 + 0.0833333333333332*G19_0_0_0_2_0_2_2_6_1 + 0.0833333333333332*G19_0_0_0_2_0_2_2_7_2 - 0.0833333333333332*G19_1_0_0_0_1_0_0_4_0 - 0.0833333333333332*G19_1_0_0_0_1_0_0_4_1 - 0.0833333333333332*G19_1_0_0_0_1_0_0_4_2 + 0.0833333333333332*G19_1_0_0_0_1_0_0_5_0 + 0.0833333333333332*G19_1_0_0_0_1_0_0_6_1 + 0.0833333333333332*G19_1_0_0_0_1_0_0_7_2 - 0.0833333333333332*G19_1_0_0_0_1_1_0_4_0 - 0.0833333333333332*G19_1_0_0_0_1_1_0_4_1 - 0.0833333333333332*G19_1_0_0_0_1_1_0_4_2 + 0.0833333333333332*G19_1_0_0_0_1_1_0_5_0 + 0.0833333333333332*G19_1_0_0_0_1_1_0_6_1 + 0.0833333333333332*G19_1_0_0_0_1_1_0_7_2 - 0.0833333333333332*G19_1_0_0_0_1_2_0_4_0 - 0.0833333333333332*G19_1_0_0_0_1_2_0_4_1 - 0.0833333333333332*G19_1_0_0_0_1_2_0_4_2 + 0.0833333333333332*G19_1_0_0_0_1_2_0_5_0 + 0.0833333333333332*G19_1_0_0_0_1_2_0_6_1 + 0.0833333333333332*G19_1_0_0_0_1_2_0_7_2 - 0.0833333333333332*G19_1_0_0_1_1_0_1_4_0 - 0.0833333333333332*G19_1_0_0_1_1_0_1_4_1 - 0.0833333333333332*G19_1_0_0_1_1_0_1_4_2 + 0.0833333333333332*G19_1_0_0_1_1_0_1_5_0 + 0.0833333333333332*G19_1_0_0_1_1_0_1_6_1 + 0.0833333333333332*G19_1_0_0_1_1_0_1_7_2 - 0.0833333333333332*G19_1_0_0_1_1_1_1_4_0 - 0.0833333333333332*G19_1_0_0_1_1_1_1_4_1 - 0.0833333333333332*G19_1_0_0_1_1_1_1_4_2 + 0.0833333333333332*G19_1_0_0_1_1_1_1_5_0 + 0.0833333333333332*G19_1_0_0_1_1_1_1_6_1 + 0.0833333333333332*G19_1_0_0_1_1_1_1_7_2 - 0.0833333333333332*G19_1_0_0_1_1_2_1_4_0 - 0.0833333333333332*G19_1_0_0_1_1_2_1_4_1 - 0.0833333333333332*G19_1_0_0_1_1_2_1_4_2 + 0.0833333333333332*G19_1_0_0_1_1_2_1_5_0 + 0.0833333333333332*G19_1_0_0_1_1_2_1_6_1 + 0.0833333333333332*G19_1_0_0_1_1_2_1_7_2 - 0.0833333333333332*G19_1_0_0_2_1_0_2_4_0 - 0.0833333333333332*G19_1_0_0_2_1_0_2_4_1 - 0.0833333333333332*G19_1_0_0_2_1_0_2_4_2 + 0.0833333333333332*G19_1_0_0_2_1_0_2_5_0 + 0.0833333333333332*G19_1_0_0_2_1_0_2_6_1 + 0.0833333333333332*G19_1_0_0_2_1_0_2_7_2 - 0.0833333333333332*G19_1_0_0_2_1_1_2_4_0 - 0.0833333333333332*G19_1_0_0_2_1_1_2_4_1 - 0.0833333333333332*G19_1_0_0_2_1_1_2_4_2 + 0.0833333333333332*G19_1_0_0_2_1_1_2_5_0 + 0.0833333333333332*G19_1_0_0_2_1_1_2_6_1 + 0.0833333333333332*G19_1_0_0_2_1_1_2_7_2 - 0.0833333333333332*G19_1_0_0_2_1_2_2_4_0 - 0.0833333333333332*G19_1_0_0_2_1_2_2_4_1 - 0.0833333333333332*G19_1_0_0_2_1_2_2_4_2 + 0.0833333333333332*G19_1_0_0_2_1_2_2_5_0 + 0.0833333333333332*G19_1_0_0_2_1_2_2_6_1 + 0.0833333333333332*G19_1_0_0_2_1_2_2_7_2 - 0.0833333333333332*G19_2_0_0_0_2_0_0_4_0 - 0.0833333333333332*G19_2_0_0_0_2_0_0_4_1 - 0.0833333333333332*G19_2_0_0_0_2_0_0_4_2 + 0.0833333333333332*G19_2_0_0_0_2_0_0_5_0 + 0.0833333333333332*G19_2_0_0_0_2_0_0_6_1 + 0.0833333333333332*G19_2_0_0_0_2_0_0_7_2 - 0.0833333333333332*G19_2_0_0_0_2_1_0_4_0 - 0.0833333333333332*G19_2_0_0_0_2_1_0_4_1 - 0.0833333333333332*G19_2_0_0_0_2_1_0_4_2 + 0.0833333333333332*G19_2_0_0_0_2_1_0_5_0 + 0.0833333333333332*G19_2_0_0_0_2_1_0_6_1 + 0.0833333333333332*G19_2_0_0_0_2_1_0_7_2 - 0.0833333333333332*G19_2_0_0_0_2_2_0_4_0 - 0.0833333333333332*G19_2_0_0_0_2_2_0_4_1 - 0.0833333333333332*G19_2_0_0_0_2_2_0_4_2 + 0.0833333333333332*G19_2_0_0_0_2_2_0_5_0 + 0.0833333333333332*G19_2_0_0_0_2_2_0_6_1 + 0.0833333333333332*G19_2_0_0_0_2_2_0_7_2 - 0.0833333333333332*G19_2_0_0_1_2_0_1_4_0 - 0.0833333333333332*G19_2_0_0_1_2_0_1_4_1 - 0.0833333333333332*G19_2_0_0_1_2_0_1_4_2 + 0.0833333333333332*G19_2_0_0_1_2_0_1_5_0 + 0.0833333333333332*G19_2_0_0_1_2_0_1_6_1 + 0.0833333333333332*G19_2_0_0_1_2_0_1_7_2 - 0.0833333333333332*G19_2_0_0_1_2_1_1_4_0 - 0.0833333333333332*G19_2_0_0_1_2_1_1_4_1 - 0.0833333333333332*G19_2_0_0_1_2_1_1_4_2 + 0.0833333333333332*G19_2_0_0_1_2_1_1_5_0 + 0.0833333333333332*G19_2_0_0_1_2_1_1_6_1 + 0.0833333333333332*G19_2_0_0_1_2_1_1_7_2 - 0.0833333333333332*G19_2_0_0_1_2_2_1_4_0 - 0.0833333333333332*G19_2_0_0_1_2_2_1_4_1 - 0.0833333333333332*G19_2_0_0_1_2_2_1_4_2 + 0.0833333333333332*G19_2_0_0_1_2_2_1_5_0 + 0.0833333333333332*G19_2_0_0_1_2_2_1_6_1 + 0.0833333333333332*G19_2_0_0_1_2_2_1_7_2 - 0.0833333333333332*G19_2_0_0_2_2_0_2_4_0 - 0.0833333333333332*G19_2_0_0_2_2_0_2_4_1 - 0.0833333333333332*G19_2_0_0_2_2_0_2_4_2 + 0.0833333333333332*G19_2_0_0_2_2_0_2_5_0 + 0.0833333333333332*G19_2_0_0_2_2_0_2_6_1 + 0.0833333333333332*G19_2_0_0_2_2_0_2_7_2 - 0.0833333333333332*G19_2_0_0_2_2_1_2_4_0 - 0.0833333333333332*G19_2_0_0_2_2_1_2_4_1 - 0.0833333333333332*G19_2_0_0_2_2_1_2_4_2 + 0.0833333333333332*G19_2_0_0_2_2_1_2_5_0 + 0.0833333333333332*G19_2_0_0_2_2_1_2_6_1 + 0.0833333333333332*G19_2_0_0_2_2_1_2_7_2 - 0.0833333333333332*G19_2_0_0_2_2_2_2_4_0 - 0.0833333333333332*G19_2_0_0_2_2_2_2_4_1 - 0.0833333333333332*G19_2_0_0_2_2_2_2_4_2 + 0.0833333333333332*G19_2_0_0_2_2_2_2_5_0 + 0.0833333333333332*G19_2_0_0_2_2_2_2_6_1 + 0.0833333333333332*G19_2_0_0_2_2_2_2_7_2 + 0.0833333333333332*G20_0_0_0_0_0_0_0_4_0 + 0.0833333333333332*G20_0_0_0_0_0_0_0_4_1 + 0.0833333333333332*G20_0_0_0_0_0_0_0_4_2 - 0.0833333333333332*G20_0_0_0_0_0_0_0_5_0 - 0.0833333333333332*G20_0_0_0_0_0_0_0_6_1 - 0.0833333333333332*G20_0_0_0_0_0_0_0_7_2 + 0.0833333333333332*G20_0_0_0_0_0_1_0_4_0 + 0.0833333333333332*G20_0_0_0_0_0_1_0_4_1 + 0.0833333333333332*G20_0_0_0_0_0_1_0_4_2 - 0.0833333333333332*G20_0_0_0_0_0_1_0_5_0 - 0.0833333333333332*G20_0_0_0_0_0_1_0_6_1 - 0.0833333333333332*G20_0_0_0_0_0_1_0_7_2 + 0.0833333333333332*G20_0_0_0_0_0_2_0_4_0 + 0.0833333333333332*G20_0_0_0_0_0_2_0_4_1 + 0.0833333333333332*G20_0_0_0_0_0_2_0_4_2 - 0.0833333333333332*G20_0_0_0_0_0_2_0_5_0 - 0.0833333333333332*G20_0_0_0_0_0_2_0_6_1 - 0.0833333333333332*G20_0_0_0_0_0_2_0_7_2 + 0.0833333333333332*G20_0_0_0_1_0_0_1_4_0 + 0.0833333333333332*G20_0_0_0_1_0_0_1_4_1 + 0.0833333333333332*G20_0_0_0_1_0_0_1_4_2 - 0.0833333333333332*G20_0_0_0_1_0_0_1_5_0 - 0.0833333333333332*G20_0_0_0_1_0_0_1_6_1 - 0.0833333333333332*G20_0_0_0_1_0_0_1_7_2 + 0.0833333333333332*G20_0_0_0_1_0_1_1_4_0 + 0.0833333333333332*G20_0_0_0_1_0_1_1_4_1 + 0.0833333333333332*G20_0_0_0_1_0_1_1_4_2 - 0.0833333333333332*G20_0_0_0_1_0_1_1_5_0 - 0.0833333333333332*G20_0_0_0_1_0_1_1_6_1 - 0.0833333333333332*G20_0_0_0_1_0_1_1_7_2 + 0.0833333333333332*G20_0_0_0_1_0_2_1_4_0 + 0.0833333333333332*G20_0_0_0_1_0_2_1_4_1 + 0.0833333333333332*G20_0_0_0_1_0_2_1_4_2 - 0.0833333333333332*G20_0_0_0_1_0_2_1_5_0 - 0.0833333333333332*G20_0_0_0_1_0_2_1_6_1 - 0.0833333333333332*G20_0_0_0_1_0_2_1_7_2 + 0.0833333333333332*G20_0_0_0_2_0_0_2_4_0 + 0.0833333333333332*G20_0_0_0_2_0_0_2_4_1 + 0.0833333333333332*G20_0_0_0_2_0_0_2_4_2 - 0.0833333333333332*G20_0_0_0_2_0_0_2_5_0 - 0.0833333333333332*G20_0_0_0_2_0_0_2_6_1 - 0.0833333333333332*G20_0_0_0_2_0_0_2_7_2 + 0.0833333333333332*G20_0_0_0_2_0_1_2_4_0 + 0.0833333333333332*G20_0_0_0_2_0_1_2_4_1 + 0.0833333333333332*G20_0_0_0_2_0_1_2_4_2 - 0.0833333333333332*G20_0_0_0_2_0_1_2_5_0 - 0.0833333333333332*G20_0_0_0_2_0_1_2_6_1 - 0.0833333333333332*G20_0_0_0_2_0_1_2_7_2 + 0.0833333333333332*G20_0_0_0_2_0_2_2_4_0 + 0.0833333333333332*G20_0_0_0_2_0_2_2_4_1 + 0.0833333333333332*G20_0_0_0_2_0_2_2_4_2 - 0.0833333333333332*G20_0_0_0_2_0_2_2_5_0 - 0.0833333333333332*G20_0_0_0_2_0_2_2_6_1 - 0.0833333333333332*G20_0_0_0_2_0_2_2_7_2 + 0.0833333333333332*G20_1_0_0_0_1_0_0_4_0 + 0.0833333333333332*G20_1_0_0_0_1_0_0_4_1 + 0.0833333333333332*G20_1_0_0_0_1_0_0_4_2 - 0.0833333333333332*G20_1_0_0_0_1_0_0_5_0 - 0.0833333333333332*G20_1_0_0_0_1_0_0_6_1 - 0.0833333333333332*G20_1_0_0_0_1_0_0_7_2 + 0.0833333333333332*G20_1_0_0_0_1_1_0_4_0 + 0.0833333333333332*G20_1_0_0_0_1_1_0_4_1 + 0.0833333333333332*G20_1_0_0_0_1_1_0_4_2 - 0.0833333333333332*G20_1_0_0_0_1_1_0_5_0 - 0.0833333333333332*G20_1_0_0_0_1_1_0_6_1 - 0.0833333333333332*G20_1_0_0_0_1_1_0_7_2 + 0.0833333333333332*G20_1_0_0_0_1_2_0_4_0 + 0.0833333333333332*G20_1_0_0_0_1_2_0_4_1 + 0.0833333333333332*G20_1_0_0_0_1_2_0_4_2 - 0.0833333333333332*G20_1_0_0_0_1_2_0_5_0 - 0.0833333333333332*G20_1_0_0_0_1_2_0_6_1 - 0.0833333333333332*G20_1_0_0_0_1_2_0_7_2 + 0.0833333333333332*G20_1_0_0_1_1_0_1_4_0 + 0.0833333333333332*G20_1_0_0_1_1_0_1_4_1 + 0.0833333333333332*G20_1_0_0_1_1_0_1_4_2 - 0.0833333333333332*G20_1_0_0_1_1_0_1_5_0 - 0.0833333333333332*G20_1_0_0_1_1_0_1_6_1 - 0.0833333333333332*G20_1_0_0_1_1_0_1_7_2 + 0.0833333333333332*G20_1_0_0_1_1_1_1_4_0 + 0.0833333333333332*G20_1_0_0_1_1_1_1_4_1 + 0.0833333333333332*G20_1_0_0_1_1_1_1_4_2 - 0.0833333333333332*G20_1_0_0_1_1_1_1_5_0 - 0.0833333333333332*G20_1_0_0_1_1_1_1_6_1 - 0.0833333333333332*G20_1_0_0_1_1_1_1_7_2 + 0.0833333333333332*G20_1_0_0_1_1_2_1_4_0 + 0.0833333333333332*G20_1_0_0_1_1_2_1_4_1 + 0.0833333333333332*G20_1_0_0_1_1_2_1_4_2 - 0.0833333333333332*G20_1_0_0_1_1_2_1_5_0 - 0.0833333333333332*G20_1_0_0_1_1_2_1_6_1 - 0.0833333333333332*G20_1_0_0_1_1_2_1_7_2 + 0.0833333333333332*G20_1_0_0_2_1_0_2_4_0 + 0.0833333333333332*G20_1_0_0_2_1_0_2_4_1 + 0.0833333333333332*G20_1_0_0_2_1_0_2_4_2 - 0.0833333333333332*G20_1_0_0_2_1_0_2_5_0 - 0.0833333333333332*G20_1_0_0_2_1_0_2_6_1 - 0.0833333333333332*G20_1_0_0_2_1_0_2_7_2 + 0.0833333333333332*G20_1_0_0_2_1_1_2_4_0 + 0.0833333333333332*G20_1_0_0_2_1_1_2_4_1 + 0.0833333333333332*G20_1_0_0_2_1_1_2_4_2 - 0.0833333333333332*G20_1_0_0_2_1_1_2_5_0 - 0.0833333333333332*G20_1_0_0_2_1_1_2_6_1 - 0.0833333333333332*G20_1_0_0_2_1_1_2_7_2 + 0.0833333333333332*G20_1_0_0_2_1_2_2_4_0 + 0.0833333333333332*G20_1_0_0_2_1_2_2_4_1 + 0.0833333333333332*G20_1_0_0_2_1_2_2_4_2 - 0.0833333333333332*G20_1_0_0_2_1_2_2_5_0 - 0.0833333333333332*G20_1_0_0_2_1_2_2_6_1 - 0.0833333333333332*G20_1_0_0_2_1_2_2_7_2 + 0.0833333333333332*G20_2_0_0_0_2_0_0_4_0 + 0.0833333333333332*G20_2_0_0_0_2_0_0_4_1 + 0.0833333333333332*G20_2_0_0_0_2_0_0_4_2 - 0.0833333333333332*G20_2_0_0_0_2_0_0_5_0 - 0.0833333333333332*G20_2_0_0_0_2_0_0_6_1 - 0.0833333333333332*G20_2_0_0_0_2_0_0_7_2 + 0.0833333333333332*G20_2_0_0_0_2_1_0_4_0 + 0.0833333333333332*G20_2_0_0_0_2_1_0_4_1 + 0.0833333333333332*G20_2_0_0_0_2_1_0_4_2 - 0.0833333333333332*G20_2_0_0_0_2_1_0_5_0 - 0.0833333333333332*G20_2_0_0_0_2_1_0_6_1 - 0.0833333333333332*G20_2_0_0_0_2_1_0_7_2 + 0.0833333333333332*G20_2_0_0_0_2_2_0_4_0 + 0.0833333333333332*G20_2_0_0_0_2_2_0_4_1 + 0.0833333333333332*G20_2_0_0_0_2_2_0_4_2 - 0.0833333333333332*G20_2_0_0_0_2_2_0_5_0 - 0.0833333333333332*G20_2_0_0_0_2_2_0_6_1 - 0.0833333333333332*G20_2_0_0_0_2_2_0_7_2 + 0.0833333333333332*G20_2_0_0_1_2_0_1_4_0 + 0.0833333333333332*G20_2_0_0_1_2_0_1_4_1 + 0.0833333333333332*G20_2_0_0_1_2_0_1_4_2 - 0.0833333333333332*G20_2_0_0_1_2_0_1_5_0 - 0.0833333333333332*G20_2_0_0_1_2_0_1_6_1 - 0.0833333333333332*G20_2_0_0_1_2_0_1_7_2 + 0.0833333333333332*G20_2_0_0_1_2_1_1_4_0 + 0.0833333333333332*G20_2_0_0_1_2_1_1_4_1 + 0.0833333333333332*G20_2_0_0_1_2_1_1_4_2 - 0.0833333333333332*G20_2_0_0_1_2_1_1_5_0 - 0.0833333333333332*G20_2_0_0_1_2_1_1_6_1 - 0.0833333333333332*G20_2_0_0_1_2_1_1_7_2 + 0.0833333333333332*G20_2_0_0_1_2_2_1_4_0 + 0.0833333333333332*G20_2_0_0_1_2_2_1_4_1 + 0.0833333333333332*G20_2_0_0_1_2_2_1_4_2 - 0.0833333333333332*G20_2_0_0_1_2_2_1_5_0 - 0.0833333333333332*G20_2_0_0_1_2_2_1_6_1 - 0.0833333333333332*G20_2_0_0_1_2_2_1_7_2 + 0.0833333333333332*G20_2_0_0_2_2_0_2_4_0 + 0.0833333333333332*G20_2_0_0_2_2_0_2_4_1 + 0.0833333333333332*G20_2_0_0_2_2_0_2_4_2 - 0.0833333333333332*G20_2_0_0_2_2_0_2_5_0 - 0.0833333333333332*G20_2_0_0_2_2_0_2_6_1 - 0.0833333333333332*G20_2_0_0_2_2_0_2_7_2 + 0.0833333333333332*G20_2_0_0_2_2_1_2_4_0 + 0.0833333333333332*G20_2_0_0_2_2_1_2_4_1 + 0.0833333333333332*G20_2_0_0_2_2_1_2_4_2 - 0.0833333333333332*G20_2_0_0_2_2_1_2_5_0 - 0.0833333333333332*G20_2_0_0_2_2_1_2_6_1 - 0.0833333333333332*G20_2_0_0_2_2_1_2_7_2 + 0.0833333333333332*G20_2_0_0_2_2_2_2_4_0 + 0.0833333333333332*G20_2_0_0_2_2_2_2_4_1 + 0.0833333333333332*G20_2_0_0_2_2_2_2_4_2 - 0.0833333333333332*G20_2_0_0_2_2_2_2_5_0 - 0.0833333333333332*G20_2_0_0_2_2_2_2_6_1 - 0.0833333333333332*G20_2_0_0_2_2_2_2_7_2 - 0.166666666666666*G21_1_0_0_0_0_0_0_0 - 0.166666666666666*G21_1_0_0_0_0_0_0_1 - 0.166666666666666*G21_1_0_0_0_0_0_0_2 + 0.166666666666666*G21_1_0_0_0_0_0_1_0 + 0.166666666666666*G21_1_0_0_0_0_0_2_1 + 0.166666666666666*G21_1_0_0_0_0_0_3_2 - 0.166666666666666*G21_1_0_0_0_1_0_0_0 - 0.166666666666666*G21_1_0_0_0_1_0_0_1 - 0.166666666666666*G21_1_0_0_0_1_0_0_2 + 0.166666666666666*G21_1_0_0_0_1_0_1_0 + 0.166666666666666*G21_1_0_0_0_1_0_2_1 + 0.166666666666666*G21_1_0_0_0_1_0_3_2 - 0.166666666666666*G21_1_0_0_0_2_0_0_0 - 0.166666666666666*G21_1_0_0_0_2_0_0_1 - 0.166666666666666*G21_1_0_0_0_2_0_0_2 + 0.166666666666666*G21_1_0_0_0_2_0_1_0 + 0.166666666666666*G21_1_0_0_0_2_0_2_1 + 0.166666666666666*G21_1_0_0_0_2_0_3_2 - 0.166666666666666*G21_1_0_0_1_0_1_0_0 - 0.166666666666666*G21_1_0_0_1_0_1_0_1 - 0.166666666666666*G21_1_0_0_1_0_1_0_2 + 0.166666666666666*G21_1_0_0_1_0_1_1_0 + 0.166666666666666*G21_1_0_0_1_0_1_2_1 + 0.166666666666666*G21_1_0_0_1_0_1_3_2 - 0.166666666666666*G21_1_0_0_1_1_1_0_0 - 0.166666666666666*G21_1_0_0_1_1_1_0_1 - 0.166666666666666*G21_1_0_0_1_1_1_0_2 + 0.166666666666666*G21_1_0_0_1_1_1_1_0 + 0.166666666666666*G21_1_0_0_1_1_1_2_1 + 0.166666666666666*G21_1_0_0_1_1_1_3_2 - 0.166666666666666*G21_1_0_0_1_2_1_0_0 - 0.166666666666666*G21_1_0_0_1_2_1_0_1 - 0.166666666666666*G21_1_0_0_1_2_1_0_2 + 0.166666666666666*G21_1_0_0_1_2_1_1_0 + 0.166666666666666*G21_1_0_0_1_2_1_2_1 + 0.166666666666666*G21_1_0_0_1_2_1_3_2 - 0.166666666666666*G21_1_0_0_2_0_2_0_0 - 0.166666666666666*G21_1_0_0_2_0_2_0_1 - 0.166666666666666*G21_1_0_0_2_0_2_0_2 + 0.166666666666666*G21_1_0_0_2_0_2_1_0 + 0.166666666666666*G21_1_0_0_2_0_2_2_1 + 0.166666666666666*G21_1_0_0_2_0_2_3_2 - 0.166666666666666*G21_1_0_0_2_1_2_0_0 - 0.166666666666666*G21_1_0_0_2_1_2_0_1 - 0.166666666666666*G21_1_0_0_2_1_2_0_2 + 0.166666666666666*G21_1_0_0_2_1_2_1_0 + 0.166666666666666*G21_1_0_0_2_1_2_2_1 + 0.166666666666666*G21_1_0_0_2_1_2_3_2 - 0.166666666666666*G21_1_0_0_2_2_2_0_0 - 0.166666666666666*G21_1_0_0_2_2_2_0_1 - 0.166666666666666*G21_1_0_0_2_2_2_0_2 + 0.166666666666666*G21_1_0_0_2_2_2_1_0 + 0.166666666666666*G21_1_0_0_2_2_2_2_1 + 0.166666666666666*G21_1_0_0_2_2_2_3_2 + 0.166666666666666*G22_1_0_0_0_0_0_0_0 + 0.166666666666666*G22_1_0_0_0_0_0_0_1 + 0.166666666666666*G22_1_0_0_0_0_0_0_2 - 0.166666666666666*G22_1_0_0_0_0_0_1_0 - 0.166666666666666*G22_1_0_0_0_0_0_2_1 - 0.166666666666666*G22_1_0_0_0_0_0_3_2 + 0.166666666666666*G22_1_0_0_0_1_0_0_0 + 0.166666666666666*G22_1_0_0_0_1_0_0_1 + 0.166666666666666*G22_1_0_0_0_1_0_0_2 - 0.166666666666666*G22_1_0_0_0_1_0_1_0 - 0.166666666666666*G22_1_0_0_0_1_0_2_1 - 0.166666666666666*G22_1_0_0_0_1_0_3_2 + 0.166666666666666*G22_1_0_0_0_2_0_0_0 + 0.166666666666666*G22_1_0_0_0_2_0_0_1 + 0.166666666666666*G22_1_0_0_0_2_0_0_2 - 0.166666666666666*G22_1_0_0_0_2_0_1_0 - 0.166666666666666*G22_1_0_0_0_2_0_2_1 - 0.166666666666666*G22_1_0_0_0_2_0_3_2 + 0.166666666666666*G22_1_0_0_1_0_1_0_0 + 0.166666666666666*G22_1_0_0_1_0_1_0_1 + 0.166666666666666*G22_1_0_0_1_0_1_0_2 - 0.166666666666666*G22_1_0_0_1_0_1_1_0 - 0.166666666666666*G22_1_0_0_1_0_1_2_1 - 0.166666666666666*G22_1_0_0_1_0_1_3_2 + 0.166666666666666*G22_1_0_0_1_1_1_0_0 + 0.166666666666666*G22_1_0_0_1_1_1_0_1 + 0.166666666666666*G22_1_0_0_1_1_1_0_2 - 0.166666666666666*G22_1_0_0_1_1_1_1_0 - 0.166666666666666*G22_1_0_0_1_1_1_2_1 - 0.166666666666666*G22_1_0_0_1_1_1_3_2 + 0.166666666666666*G22_1_0_0_1_2_1_0_0 + 0.166666666666666*G22_1_0_0_1_2_1_0_1 + 0.166666666666666*G22_1_0_0_1_2_1_0_2 - 0.166666666666666*G22_1_0_0_1_2_1_1_0 - 0.166666666666666*G22_1_0_0_1_2_1_2_1 - 0.166666666666666*G22_1_0_0_1_2_1_3_2 + 0.166666666666666*G22_1_0_0_2_0_2_0_0 + 0.166666666666666*G22_1_0_0_2_0_2_0_1 + 0.166666666666666*G22_1_0_0_2_0_2_0_2 - 0.166666666666666*G22_1_0_0_2_0_2_1_0 - 0.166666666666666*G22_1_0_0_2_0_2_2_1 - 0.166666666666666*G22_1_0_0_2_0_2_3_2 + 0.166666666666666*G22_1_0_0_2_1_2_0_0 + 0.166666666666666*G22_1_0_0_2_1_2_0_1 + 0.166666666666666*G22_1_0_0_2_1_2_0_2 - 0.166666666666666*G22_1_0_0_2_1_2_1_0 - 0.166666666666666*G22_1_0_0_2_1_2_2_1 - 0.166666666666666*G22_1_0_0_2_1_2_3_2 + 0.166666666666666*G22_1_0_0_2_2_2_0_0 + 0.166666666666666*G22_1_0_0_2_2_2_0_1 + 0.166666666666666*G22_1_0_0_2_2_2_0_2 - 0.166666666666666*G22_1_0_0_2_2_2_1_0 - 0.166666666666666*G22_1_0_0_2_2_2_2_1 - 0.166666666666666*G22_1_0_0_2_2_2_3_2 - 0.0833333333333332*G23_0_0_0_1_0_0_0 - 0.0833333333333332*G23_0_0_0_1_0_1_0 - 0.0833333333333332*G23_0_0_0_1_0_2_0 + 0.0833333333333332*G23_0_0_0_1_1_0_0 + 0.0833333333333332*G23_0_0_0_1_2_1_0 + 0.0833333333333332*G23_0_0_0_1_3_2_0 - 0.0833333333333332*G23_0_0_0_1_4_0_1 - 0.0833333333333332*G23_0_0_0_1_4_1_1 - 0.0833333333333332*G23_0_0_0_1_4_2_1 + 0.0833333333333332*G23_0_0_0_1_5_0_1 + 0.0833333333333332*G23_0_0_0_1_6_1_1 + 0.0833333333333332*G23_0_0_0_1_7_2_1 - 0.0833333333333332*G23_0_0_0_1_8_0_2 - 0.0833333333333332*G23_0_0_0_1_8_1_2 - 0.0833333333333332*G23_0_0_0_1_8_2_2 + 0.0833333333333332*G23_0_0_0_1_9_0_2 + 0.0833333333333332*G23_0_0_0_1_10_1_2 + 0.0833333333333332*G23_0_0_0_1_11_2_2 - 0.0833333333333332*G23_0_0_1_1_0_0_0 - 0.0833333333333332*G23_0_0_1_1_0_1_0 - 0.0833333333333332*G23_0_0_1_1_0_2_0 + 0.0833333333333332*G23_0_0_1_1_1_0_0 + 0.0833333333333332*G23_0_0_1_1_2_1_0 + 0.0833333333333332*G23_0_0_1_1_3_2_0 - 0.0833333333333332*G23_0_0_1_1_4_0_1 - 0.0833333333333332*G23_0_0_1_1_4_1_1 - 0.0833333333333332*G23_0_0_1_1_4_2_1 + 0.0833333333333332*G23_0_0_1_1_5_0_1 + 0.0833333333333332*G23_0_0_1_1_6_1_1 + 0.0833333333333332*G23_0_0_1_1_7_2_1 - 0.0833333333333332*G23_0_0_1_1_8_0_2 - 0.0833333333333332*G23_0_0_1_1_8_1_2 - 0.0833333333333332*G23_0_0_1_1_8_2_2 + 0.0833333333333332*G23_0_0_1_1_9_0_2 + 0.0833333333333332*G23_0_0_1_1_10_1_2 + 0.0833333333333332*G23_0_0_1_1_11_2_2 - 0.0833333333333332*G23_0_0_2_1_0_0_0 - 0.0833333333333332*G23_0_0_2_1_0_1_0 - 0.0833333333333332*G23_0_0_2_1_0_2_0 + 0.0833333333333332*G23_0_0_2_1_1_0_0 + 0.0833333333333332*G23_0_0_2_1_2_1_0 + 0.0833333333333332*G23_0_0_2_1_3_2_0 - 0.0833333333333332*G23_0_0_2_1_4_0_1 - 0.0833333333333332*G23_0_0_2_1_4_1_1 - 0.0833333333333332*G23_0_0_2_1_4_2_1 + 0.0833333333333332*G23_0_0_2_1_5_0_1 + 0.0833333333333332*G23_0_0_2_1_6_1_1 + 0.0833333333333332*G23_0_0_2_1_7_2_1 - 0.0833333333333332*G23_0_0_2_1_8_0_2 - 0.0833333333333332*G23_0_0_2_1_8_1_2 - 0.0833333333333332*G23_0_0_2_1_8_2_2 + 0.0833333333333332*G23_0_0_2_1_9_0_2 + 0.0833333333333332*G23_0_0_2_1_10_1_2 + 0.0833333333333332*G23_0_0_2_1_11_2_2;
    A[5] = 0.00833333333333331*G0_0_4 + 0.0166666666666666*G0_0_5 + 0.00833333333333331*G0_0_6 + 0.00833333333333331*G0_0_7 + 0.00833333333333331*G1_0_4 + 0.0166666666666666*G1_0_5 + 0.00833333333333331*G1_0_6 + 0.00833333333333331*G1_0_7 + 0.166666666666666*G2_0_0_1_0 + 0.166666666666666*G5_0_0_4_0 + 0.166666666666666*G6_0_0_7_0 - 0.166666666666666*G9_0_1_0 - 0.166666666666666*G12_0_4_0 - 0.166666666666666*G13_0_7_0 + 0.0416666666666666*G16_0_0_0_1 + 0.0416666666666666*G16_0_1_0_1 + 0.0416666666666666*G16_0_2_0_1 + 0.0416666666666666*G16_0_3_0_1 + 0.0416666666666666*G17_0_0_0_0_0_4 + 0.0416666666666666*G17_0_0_0_0_0_5 + 0.0416666666666666*G17_0_0_0_0_0_6 + 0.0416666666666666*G17_0_0_0_0_0_7 + 0.0416666666666666*G17_0_0_1_0_1_4 + 0.0416666666666666*G17_0_0_1_0_1_5 + 0.0416666666666666*G17_0_0_1_0_1_6 + 0.0416666666666666*G17_0_0_1_0_1_7 + 0.0416666666666666*G17_0_0_2_0_2_4 + 0.0416666666666666*G17_0_0_2_0_2_5 + 0.0416666666666666*G17_0_0_2_0_2_6 + 0.0416666666666666*G17_0_0_2_0_2_7 - 0.0416666666666666*G18_0_0_0_0_0_4 - 0.0416666666666666*G18_0_0_0_0_0_5 - 0.0416666666666666*G18_0_0_0_0_0_6 - 0.0416666666666666*G18_0_0_0_0_0_7 - 0.0416666666666666*G18_0_0_1_0_1_4 - 0.0416666666666666*G18_0_0_1_0_1_5 - 0.0416666666666666*G18_0_0_1_0_1_6 - 0.0416666666666666*G18_0_0_1_0_1_7 - 0.0416666666666666*G18_0_0_2_0_2_4 - 0.0416666666666666*G18_0_0_2_0_2_5 - 0.0416666666666666*G18_0_0_2_0_2_6 - 0.0416666666666666*G18_0_0_2_0_2_7 + 0.0833333333333332*G19_0_0_0_0_0_0_0_4_0 + 0.0833333333333332*G19_0_0_0_0_0_0_0_4_1 + 0.0833333333333332*G19_0_0_0_0_0_0_0_4_2 - 0.0833333333333332*G19_0_0_0_0_0_0_0_5_0 - 0.0833333333333332*G19_0_0_0_0_0_0_0_6_1 - 0.0833333333333332*G19_0_0_0_0_0_0_0_7_2 + 0.0833333333333332*G19_0_0_0_1_0_0_1_4_0 + 0.0833333333333332*G19_0_0_0_1_0_0_1_4_1 + 0.0833333333333332*G19_0_0_0_1_0_0_1_4_2 - 0.0833333333333332*G19_0_0_0_1_0_0_1_5_0 - 0.0833333333333332*G19_0_0_0_1_0_0_1_6_1 - 0.0833333333333332*G19_0_0_0_1_0_0_1_7_2 + 0.0833333333333332*G19_0_0_0_2_0_0_2_4_0 + 0.0833333333333332*G19_0_0_0_2_0_0_2_4_1 + 0.0833333333333332*G19_0_0_0_2_0_0_2_4_2 - 0.0833333333333332*G19_0_0_0_2_0_0_2_5_0 - 0.0833333333333332*G19_0_0_0_2_0_0_2_6_1 - 0.0833333333333332*G19_0_0_0_2_0_0_2_7_2 + 0.0833333333333332*G19_1_0_0_0_1_0_0_4_0 + 0.0833333333333332*G19_1_0_0_0_1_0_0_4_1 + 0.0833333333333332*G19_1_0_0_0_1_0_0_4_2 - 0.0833333333333332*G19_1_0_0_0_1_0_0_5_0 - 0.0833333333333332*G19_1_0_0_0_1_0_0_6_1 - 0.0833333333333332*G19_1_0_0_0_1_0_0_7_2 + 0.0833333333333332*G19_1_0_0_1_1_0_1_4_0 + 0.0833333333333332*G19_1_0_0_1_1_0_1_4_1 + 0.0833333333333332*G19_1_0_0_1_1_0_1_4_2 - 0.0833333333333332*G19_1_0_0_1_1_0_1_5_0 - 0.0833333333333332*G19_1_0_0_1_1_0_1_6_1 - 0.0833333333333332*G19_1_0_0_1_1_0_1_7_2 + 0.0833333333333332*G19_1_0_0_2_1_0_2_4_0 + 0.0833333333333332*G19_1_0_0_2_1_0_2_4_1 + 0.0833333333333332*G19_1_0_0_2_1_0_2_4_2 - 0.0833333333333332*G19_1_0_0_2_1_0_2_5_0 - 0.0833333333333332*G19_1_0_0_2_1_0_2_6_1 - 0.0833333333333332*G19_1_0_0_2_1_0_2_7_2 + 0.0833333333333332*G19_2_0_0_0_2_0_0_4_0 + 0.0833333333333332*G19_2_0_0_0_2_0_0_4_1 + 0.0833333333333332*G19_2_0_0_0_2_0_0_4_2 - 0.0833333333333332*G19_2_0_0_0_2_0_0_5_0 - 0.0833333333333332*G19_2_0_0_0_2_0_0_6_1 - 0.0833333333333332*G19_2_0_0_0_2_0_0_7_2 + 0.0833333333333332*G19_2_0_0_1_2_0_1_4_0 + 0.0833333333333332*G19_2_0_0_1_2_0_1_4_1 + 0.0833333333333332*G19_2_0_0_1_2_0_1_4_2 - 0.0833333333333332*G19_2_0_0_1_2_0_1_5_0 - 0.0833333333333332*G19_2_0_0_1_2_0_1_6_1 - 0.0833333333333332*G19_2_0_0_1_2_0_1_7_2 + 0.0833333333333332*G19_2_0_0_2_2_0_2_4_0 + 0.0833333333333332*G19_2_0_0_2_2_0_2_4_1 + 0.0833333333333332*G19_2_0_0_2_2_0_2_4_2 - 0.0833333333333332*G19_2_0_0_2_2_0_2_5_0 - 0.0833333333333332*G19_2_0_0_2_2_0_2_6_1 - 0.0833333333333332*G19_2_0_0_2_2_0_2_7_2 - 0.0833333333333332*G20_0_0_0_0_0_0_0_4_0 - 0.0833333333333332*G20_0_0_0_0_0_0_0_4_1 - 0.0833333333333332*G20_0_0_0_0_0_0_0_4_2 + 0.0833333333333332*G20_0_0_0_0_0_0_0_5_0 + 0.0833333333333332*G20_0_0_0_0_0_0_0_6_1 + 0.0833333333333332*G20_0_0_0_0_0_0_0_7_2 - 0.0833333333333332*G20_0_0_0_1_0_0_1_4_0 - 0.0833333333333332*G20_0_0_0_1_0_0_1_4_1 - 0.0833333333333332*G20_0_0_0_1_0_0_1_4_2 + 0.0833333333333332*G20_0_0_0_1_0_0_1_5_0 + 0.0833333333333332*G20_0_0_0_1_0_0_1_6_1 + 0.0833333333333332*G20_0_0_0_1_0_0_1_7_2 - 0.0833333333333332*G20_0_0_0_2_0_0_2_4_0 - 0.0833333333333332*G20_0_0_0_2_0_0_2_4_1 - 0.0833333333333332*G20_0_0_0_2_0_0_2_4_2 + 0.0833333333333332*G20_0_0_0_2_0_0_2_5_0 + 0.0833333333333332*G20_0_0_0_2_0_0_2_6_1 + 0.0833333333333332*G20_0_0_0_2_0_0_2_7_2 - 0.0833333333333332*G20_1_0_0_0_1_0_0_4_0 - 0.0833333333333332*G20_1_0_0_0_1_0_0_4_1 - 0.0833333333333332*G20_1_0_0_0_1_0_0_4_2 + 0.0833333333333332*G20_1_0_0_0_1_0_0_5_0 + 0.0833333333333332*G20_1_0_0_0_1_0_0_6_1 + 0.0833333333333332*G20_1_0_0_0_1_0_0_7_2 - 0.0833333333333332*G20_1_0_0_1_1_0_1_4_0 - 0.0833333333333332*G20_1_0_0_1_1_0_1_4_1 - 0.0833333333333332*G20_1_0_0_1_1_0_1_4_2 + 0.0833333333333332*G20_1_0_0_1_1_0_1_5_0 + 0.0833333333333332*G20_1_0_0_1_1_0_1_6_1 + 0.0833333333333332*G20_1_0_0_1_1_0_1_7_2 - 0.0833333333333332*G20_1_0_0_2_1_0_2_4_0 - 0.0833333333333332*G20_1_0_0_2_1_0_2_4_1 - 0.0833333333333332*G20_1_0_0_2_1_0_2_4_2 + 0.0833333333333332*G20_1_0_0_2_1_0_2_5_0 + 0.0833333333333332*G20_1_0_0_2_1_0_2_6_1 + 0.0833333333333332*G20_1_0_0_2_1_0_2_7_2 - 0.0833333333333332*G20_2_0_0_0_2_0_0_4_0 - 0.0833333333333332*G20_2_0_0_0_2_0_0_4_1 - 0.0833333333333332*G20_2_0_0_0_2_0_0_4_2 + 0.0833333333333332*G20_2_0_0_0_2_0_0_5_0 + 0.0833333333333332*G20_2_0_0_0_2_0_0_6_1 + 0.0833333333333332*G20_2_0_0_0_2_0_0_7_2 - 0.0833333333333332*G20_2_0_0_1_2_0_1_4_0 - 0.0833333333333332*G20_2_0_0_1_2_0_1_4_1 - 0.0833333333333332*G20_2_0_0_1_2_0_1_4_2 + 0.0833333333333332*G20_2_0_0_1_2_0_1_5_0 + 0.0833333333333332*G20_2_0_0_1_2_0_1_6_1 + 0.0833333333333332*G20_2_0_0_1_2_0_1_7_2 - 0.0833333333333332*G20_2_0_0_2_2_0_2_4_0 - 0.0833333333333332*G20_2_0_0_2_2_0_2_4_1 - 0.0833333333333332*G20_2_0_0_2_2_0_2_4_2 + 0.0833333333333332*G20_2_0_0_2_2_0_2_5_0 + 0.0833333333333332*G20_2_0_0_2_2_0_2_6_1 + 0.0833333333333332*G20_2_0_0_2_2_0_2_7_2 + 0.166666666666666*G21_1_0_0_0_0_0_0_0 + 0.166666666666666*G21_1_0_0_0_0_0_0_1 + 0.166666666666666*G21_1_0_0_0_0_0_0_2 - 0.166666666666666*G21_1_0_0_0_0_0_1_0 - 0.166666666666666*G21_1_0_0_0_0_0_2_1 - 0.166666666666666*G21_1_0_0_0_0_0_3_2 + 0.166666666666666*G21_1_0_0_1_0_1_0_0 + 0.166666666666666*G21_1_0_0_1_0_1_0_1 + 0.166666666666666*G21_1_0_0_1_0_1_0_2 - 0.166666666666666*G21_1_0_0_1_0_1_1_0 - 0.166666666666666*G21_1_0_0_1_0_1_2_1 - 0.166666666666666*G21_1_0_0_1_0_1_3_2 + 0.166666666666666*G21_1_0_0_2_0_2_0_0 + 0.166666666666666*G21_1_0_0_2_0_2_0_1 + 0.166666666666666*G21_1_0_0_2_0_2_0_2 - 0.166666666666666*G21_1_0_0_2_0_2_1_0 - 0.166666666666666*G21_1_0_0_2_0_2_2_1 - 0.166666666666666*G21_1_0_0_2_0_2_3_2 - 0.166666666666666*G22_1_0_0_0_0_0_0_0 - 0.166666666666666*G22_1_0_0_0_0_0_0_1 - 0.166666666666666*G22_1_0_0_0_0_0_0_2 + 0.166666666666666*G22_1_0_0_0_0_0_1_0 + 0.166666666666666*G22_1_0_0_0_0_0_2_1 + 0.166666666666666*G22_1_0_0_0_0_0_3_2 - 0.166666666666666*G22_1_0_0_1_0_1_0_0 - 0.166666666666666*G22_1_0_0_1_0_1_0_1 - 0.166666666666666*G22_1_0_0_1_0_1_0_2 + 0.166666666666666*G22_1_0_0_1_0_1_1_0 + 0.166666666666666*G22_1_0_0_1_0_1_2_1 + 0.166666666666666*G22_1_0_0_1_0_1_3_2 - 0.166666666666666*G22_1_0_0_2_0_2_0_0 - 0.166666666666666*G22_1_0_0_2_0_2_0_1 - 0.166666666666666*G22_1_0_0_2_0_2_0_2 + 0.166666666666666*G22_1_0_0_2_0_2_1_0 + 0.166666666666666*G22_1_0_0_2_0_2_2_1 + 0.166666666666666*G22_1_0_0_2_0_2_3_2 + 0.0833333333333332*G23_0_0_0_1_0_0_0 + 0.0833333333333332*G23_0_0_0_1_0_1_0 + 0.0833333333333332*G23_0_0_0_1_0_2_0 - 0.0833333333333332*G23_0_0_0_1_1_0_0 - 0.0833333333333332*G23_0_0_0_1_2_1_0 - 0.0833333333333332*G23_0_0_0_1_3_2_0 + 0.0833333333333332*G23_0_0_0_1_4_0_1 + 0.0833333333333332*G23_0_0_0_1_4_1_1 + 0.0833333333333332*G23_0_0_0_1_4_2_1 - 0.0833333333333332*G23_0_0_0_1_5_0_1 - 0.0833333333333332*G23_0_0_0_1_6_1_1 - 0.0833333333333332*G23_0_0_0_1_7_2_1 + 0.0833333333333332*G23_0_0_0_1_8_0_2 + 0.0833333333333332*G23_0_0_0_1_8_1_2 + 0.0833333333333332*G23_0_0_0_1_8_2_2 - 0.0833333333333332*G23_0_0_0_1_9_0_2 - 0.0833333333333332*G23_0_0_0_1_10_1_2 - 0.0833333333333332*G23_0_0_0_1_11_2_2;
    A[6] = 0.00833333333333331*G0_0_4 + 0.00833333333333331*G0_0_5 + 0.0166666666666666*G0_0_6 + 0.00833333333333331*G0_0_7 + 0.00833333333333331*G1_0_4 + 0.00833333333333331*G1_0_5 + 0.0166666666666666*G1_0_6 + 0.00833333333333331*G1_0_7 + 0.166666666666666*G2_0_0_1_1 + 0.166666666666666*G5_0_0_4_1 + 0.166666666666666*G6_0_0_7_1 - 0.166666666666666*G9_0_1_1 - 0.166666666666666*G12_0_4_1 - 0.166666666666666*G13_0_7_1 + 0.0416666666666666*G16_0_0_1_1 + 0.0416666666666666*G16_0_1_1_1 + 0.0416666666666666*G16_0_2_1_1 + 0.0416666666666666*G16_0_3_1_1 + 0.0416666666666666*G17_0_0_0_1_0_4 + 0.0416666666666666*G17_0_0_0_1_0_5 + 0.0416666666666666*G17_0_0_0_1_0_6 + 0.0416666666666666*G17_0_0_0_1_0_7 + 0.0416666666666666*G17_0_0_1_1_1_4 + 0.0416666666666666*G17_0_0_1_1_1_5 + 0.0416666666666666*G17_0_0_1_1_1_6 + 0.0416666666666666*G17_0_0_1_1_1_7 + 0.0416666666666666*G17_0_0_2_1_2_4 + 0.0416666666666666*G17_0_0_2_1_2_5 + 0.0416666666666666*G17_0_0_2_1_2_6 + 0.0416666666666666*G17_0_0_2_1_2_7 - 0.0416666666666666*G18_0_0_0_1_0_4 - 0.0416666666666666*G18_0_0_0_1_0_5 - 0.0416666666666666*G18_0_0_0_1_0_6 - 0.0416666666666666*G18_0_0_0_1_0_7 - 0.0416666666666666*G18_0_0_1_1_1_4 - 0.0416666666666666*G18_0_0_1_1_1_5 - 0.0416666666666666*G18_0_0_1_1_1_6 - 0.0416666666666666*G18_0_0_1_1_1_7 - 0.0416666666666666*G18_0_0_2_1_2_4 - 0.0416666666666666*G18_0_0_2_1_2_5 - 0.0416666666666666*G18_0_0_2_1_2_6 - 0.0416666666666666*G18_0_0_2_1_2_7 + 0.0833333333333332*G19_0_0_0_0_0_1_0_4_0 + 0.0833333333333332*G19_0_0_0_0_0_1_0_4_1 + 0.0833333333333332*G19_0_0_0_0_0_1_0_4_2 - 0.0833333333333332*G19_0_0_0_0_0_1_0_5_0 - 0.0833333333333332*G19_0_0_0_0_0_1_0_6_1 - 0.0833333333333332*G19_0_0_0_0_0_1_0_7_2 + 0.0833333333333332*G19_0_0_0_1_0_1_1_4_0 + 0.0833333333333332*G19_0_0_0_1_0_1_1_4_1 + 0.0833333333333332*G19_0_0_0_1_0_1_1_4_2 - 0.0833333333333332*G19_0_0_0_1_0_1_1_5_0 - 0.0833333333333332*G19_0_0_0_1_0_1_1_6_1 - 0.0833333333333332*G19_0_0_0_1_0_1_1_7_2 + 0.0833333333333332*G19_0_0_0_2_0_1_2_4_0 + 0.0833333333333332*G19_0_0_0_2_0_1_2_4_1 + 0.0833333333333332*G19_0_0_0_2_0_1_2_4_2 - 0.0833333333333332*G19_0_0_0_2_0_1_2_5_0 - 0.0833333333333332*G19_0_0_0_2_0_1_2_6_1 - 0.0833333333333332*G19_0_0_0_2_0_1_2_7_2 + 0.0833333333333332*G19_1_0_0_0_1_1_0_4_0 + 0.0833333333333332*G19_1_0_0_0_1_1_0_4_1 + 0.0833333333333332*G19_1_0_0_0_1_1_0_4_2 - 0.0833333333333332*G19_1_0_0_0_1_1_0_5_0 - 0.0833333333333332*G19_1_0_0_0_1_1_0_6_1 - 0.0833333333333332*G19_1_0_0_0_1_1_0_7_2 + 0.0833333333333332*G19_1_0_0_1_1_1_1_4_0 + 0.0833333333333332*G19_1_0_0_1_1_1_1_4_1 + 0.0833333333333332*G19_1_0_0_1_1_1_1_4_2 - 0.0833333333333332*G19_1_0_0_1_1_1_1_5_0 - 0.0833333333333332*G19_1_0_0_1_1_1_1_6_1 - 0.0833333333333332*G19_1_0_0_1_1_1_1_7_2 + 0.0833333333333332*G19_1_0_0_2_1_1_2_4_0 + 0.0833333333333332*G19_1_0_0_2_1_1_2_4_1 + 0.0833333333333332*G19_1_0_0_2_1_1_2_4_2 - 0.0833333333333332*G19_1_0_0_2_1_1_2_5_0 - 0.0833333333333332*G19_1_0_0_2_1_1_2_6_1 - 0.0833333333333332*G19_1_0_0_2_1_1_2_7_2 + 0.0833333333333332*G19_2_0_0_0_2_1_0_4_0 + 0.0833333333333332*G19_2_0_0_0_2_1_0_4_1 + 0.0833333333333332*G19_2_0_0_0_2_1_0_4_2 - 0.0833333333333332*G19_2_0_0_0_2_1_0_5_0 - 0.0833333333333332*G19_2_0_0_0_2_1_0_6_1 - 0.0833333333333332*G19_2_0_0_0_2_1_0_7_2 + 0.0833333333333332*G19_2_0_0_1_2_1_1_4_0 + 0.0833333333333332*G19_2_0_0_1_2_1_1_4_1 + 0.0833333333333332*G19_2_0_0_1_2_1_1_4_2 - 0.0833333333333332*G19_2_0_0_1_2_1_1_5_0 - 0.0833333333333332*G19_2_0_0_1_2_1_1_6_1 - 0.0833333333333332*G19_2_0_0_1_2_1_1_7_2 + 0.0833333333333332*G19_2_0_0_2_2_1_2_4_0 + 0.0833333333333332*G19_2_0_0_2_2_1_2_4_1 + 0.0833333333333332*G19_2_0_0_2_2_1_2_4_2 - 0.0833333333333332*G19_2_0_0_2_2_1_2_5_0 - 0.0833333333333332*G19_2_0_0_2_2_1_2_6_1 - 0.0833333333333332*G19_2_0_0_2_2_1_2_7_2 - 0.0833333333333332*G20_0_0_0_0_0_1_0_4_0 - 0.0833333333333332*G20_0_0_0_0_0_1_0_4_1 - 0.0833333333333332*G20_0_0_0_0_0_1_0_4_2 + 0.0833333333333332*G20_0_0_0_0_0_1_0_5_0 + 0.0833333333333332*G20_0_0_0_0_0_1_0_6_1 + 0.0833333333333332*G20_0_0_0_0_0_1_0_7_2 - 0.0833333333333332*G20_0_0_0_1_0_1_1_4_0 - 0.0833333333333332*G20_0_0_0_1_0_1_1_4_1 - 0.0833333333333332*G20_0_0_0_1_0_1_1_4_2 + 0.0833333333333332*G20_0_0_0_1_0_1_1_5_0 + 0.0833333333333332*G20_0_0_0_1_0_1_1_6_1 + 0.0833333333333332*G20_0_0_0_1_0_1_1_7_2 - 0.0833333333333332*G20_0_0_0_2_0_1_2_4_0 - 0.0833333333333332*G20_0_0_0_2_0_1_2_4_1 - 0.0833333333333332*G20_0_0_0_2_0_1_2_4_2 + 0.0833333333333332*G20_0_0_0_2_0_1_2_5_0 + 0.0833333333333332*G20_0_0_0_2_0_1_2_6_1 + 0.0833333333333332*G20_0_0_0_2_0_1_2_7_2 - 0.0833333333333332*G20_1_0_0_0_1_1_0_4_0 - 0.0833333333333332*G20_1_0_0_0_1_1_0_4_1 - 0.0833333333333332*G20_1_0_0_0_1_1_0_4_2 + 0.0833333333333332*G20_1_0_0_0_1_1_0_5_0 + 0.0833333333333332*G20_1_0_0_0_1_1_0_6_1 + 0.0833333333333332*G20_1_0_0_0_1_1_0_7_2 - 0.0833333333333332*G20_1_0_0_1_1_1_1_4_0 - 0.0833333333333332*G20_1_0_0_1_1_1_1_4_1 - 0.0833333333333332*G20_1_0_0_1_1_1_1_4_2 + 0.0833333333333332*G20_1_0_0_1_1_1_1_5_0 + 0.0833333333333332*G20_1_0_0_1_1_1_1_6_1 + 0.0833333333333332*G20_1_0_0_1_1_1_1_7_2 - 0.0833333333333332*G20_1_0_0_2_1_1_2_4_0 - 0.0833333333333332*G20_1_0_0_2_1_1_2_4_1 - 0.0833333333333332*G20_1_0_0_2_1_1_2_4_2 + 0.0833333333333332*G20_1_0_0_2_1_1_2_5_0 + 0.0833333333333332*G20_1_0_0_2_1_1_2_6_1 + 0.0833333333333332*G20_1_0_0_2_1_1_2_7_2 - 0.0833333333333332*G20_2_0_0_0_2_1_0_4_0 - 0.0833333333333332*G20_2_0_0_0_2_1_0_4_1 - 0.0833333333333332*G20_2_0_0_0_2_1_0_4_2 + 0.0833333333333332*G20_2_0_0_0_2_1_0_5_0 + 0.0833333333333332*G20_2_0_0_0_2_1_0_6_1 + 0.0833333333333332*G20_2_0_0_0_2_1_0_7_2 - 0.0833333333333332*G20_2_0_0_1_2_1_1_4_0 - 0.0833333333333332*G20_2_0_0_1_2_1_1_4_1 - 0.0833333333333332*G20_2_0_0_1_2_1_1_4_2 + 0.0833333333333332*G20_2_0_0_1_2_1_1_5_0 + 0.0833333333333332*G20_2_0_0_1_2_1_1_6_1 + 0.0833333333333332*G20_2_0_0_1_2_1_1_7_2 - 0.0833333333333332*G20_2_0_0_2_2_1_2_4_0 - 0.0833333333333332*G20_2_0_0_2_2_1_2_4_1 - 0.0833333333333332*G20_2_0_0_2_2_1_2_4_2 + 0.0833333333333332*G20_2_0_0_2_2_1_2_5_0 + 0.0833333333333332*G20_2_0_0_2_2_1_2_6_1 + 0.0833333333333332*G20_2_0_0_2_2_1_2_7_2 + 0.166666666666666*G21_1_0_0_0_1_0_0_0 + 0.166666666666666*G21_1_0_0_0_1_0_0_1 + 0.166666666666666*G21_1_0_0_0_1_0_0_2 - 0.166666666666666*G21_1_0_0_0_1_0_1_0 - 0.166666666666666*G21_1_0_0_0_1_0_2_1 - 0.166666666666666*G21_1_0_0_0_1_0_3_2 + 0.166666666666666*G21_1_0_0_1_1_1_0_0 + 0.166666666666666*G21_1_0_0_1_1_1_0_1 + 0.166666666666666*G21_1_0_0_1_1_1_0_2 - 0.166666666666666*G21_1_0_0_1_1_1_1_0 - 0.166666666666666*G21_1_0_0_1_1_1_2_1 - 0.166666666666666*G21_1_0_0_1_1_1_3_2 + 0.166666666666666*G21_1_0_0_2_1_2_0_0 + 0.166666666666666*G21_1_0_0_2_1_2_0_1 + 0.166666666666666*G21_1_0_0_2_1_2_0_2 - 0.166666666666666*G21_1_0_0_2_1_2_1_0 - 0.166666666666666*G21_1_0_0_2_1_2_2_1 - 0.166666666666666*G21_1_0_0_2_1_2_3_2 - 0.166666666666666*G22_1_0_0_0_1_0_0_0 - 0.166666666666666*G22_1_0_0_0_1_0_0_1 - 0.166666666666666*G22_1_0_0_0_1_0_0_2 + 0.166666666666666*G22_1_0_0_0_1_0_1_0 + 0.166666666666666*G22_1_0_0_0_1_0_2_1 + 0.166666666666666*G22_1_0_0_0_1_0_3_2 - 0.166666666666666*G22_1_0_0_1_1_1_0_0 - 0.166666666666666*G22_1_0_0_1_1_1_0_1 - 0.166666666666666*G22_1_0_0_1_1_1_0_2 + 0.166666666666666*G22_1_0_0_1_1_1_1_0 + 0.166666666666666*G22_1_0_0_1_1_1_2_1 + 0.166666666666666*G22_1_0_0_1_1_1_3_2 - 0.166666666666666*G22_1_0_0_2_1_2_0_0 - 0.166666666666666*G22_1_0_0_2_1_2_0_1 - 0.166666666666666*G22_1_0_0_2_1_2_0_2 + 0.166666666666666*G22_1_0_0_2_1_2_1_0 + 0.166666666666666*G22_1_0_0_2_1_2_2_1 + 0.166666666666666*G22_1_0_0_2_1_2_3_2 + 0.0833333333333332*G23_0_0_1_1_0_0_0 + 0.0833333333333332*G23_0_0_1_1_0_1_0 + 0.0833333333333332*G23_0_0_1_1_0_2_0 - 0.0833333333333332*G23_0_0_1_1_1_0_0 - 0.0833333333333332*G23_0_0_1_1_2_1_0 - 0.0833333333333332*G23_0_0_1_1_3_2_0 + 0.0833333333333332*G23_0_0_1_1_4_0_1 + 0.0833333333333332*G23_0_0_1_1_4_1_1 + 0.0833333333333332*G23_0_0_1_1_4_2_1 - 0.0833333333333332*G23_0_0_1_1_5_0_1 - 0.0833333333333332*G23_0_0_1_1_6_1_1 - 0.0833333333333332*G23_0_0_1_1_7_2_1 + 0.0833333333333332*G23_0_0_1_1_8_0_2 + 0.0833333333333332*G23_0_0_1_1_8_1_2 + 0.0833333333333332*G23_0_0_1_1_8_2_2 - 0.0833333333333332*G23_0_0_1_1_9_0_2 - 0.0833333333333332*G23_0_0_1_1_10_1_2 - 0.0833333333333332*G23_0_0_1_1_11_2_2;
    A[7] = 0.00833333333333331*G0_0_4 + 0.00833333333333331*G0_0_5 + 0.00833333333333331*G0_0_6 + 0.0166666666666666*G0_0_7 + 0.00833333333333331*G1_0_4 + 0.00833333333333331*G1_0_5 + 0.00833333333333331*G1_0_6 + 0.0166666666666666*G1_0_7 + 0.166666666666666*G2_0_0_1_2 + 0.166666666666666*G5_0_0_4_2 + 0.166666666666666*G6_0_0_7_2 - 0.166666666666666*G9_0_1_2 - 0.166666666666666*G12_0_4_2 - 0.166666666666666*G13_0_7_2 + 0.0416666666666666*G16_0_0_2_1 + 0.0416666666666666*G16_0_1_2_1 + 0.0416666666666666*G16_0_2_2_1 + 0.0416666666666666*G16_0_3_2_1 + 0.0416666666666666*G17_0_0_0_2_0_4 + 0.0416666666666666*G17_0_0_0_2_0_5 + 0.0416666666666666*G17_0_0_0_2_0_6 + 0.0416666666666666*G17_0_0_0_2_0_7 + 0.0416666666666666*G17_0_0_1_2_1_4 + 0.0416666666666666*G17_0_0_1_2_1_5 + 0.0416666666666666*G17_0_0_1_2_1_6 + 0.0416666666666666*G17_0_0_1_2_1_7 + 0.0416666666666666*G17_0_0_2_2_2_4 + 0.0416666666666666*G17_0_0_2_2_2_5 + 0.0416666666666666*G17_0_0_2_2_2_6 + 0.0416666666666666*G17_0_0_2_2_2_7 - 0.0416666666666666*G18_0_0_0_2_0_4 - 0.0416666666666666*G18_0_0_0_2_0_5 - 0.0416666666666666*G18_0_0_0_2_0_6 - 0.0416666666666666*G18_0_0_0_2_0_7 - 0.0416666666666666*G18_0_0_1_2_1_4 - 0.0416666666666666*G18_0_0_1_2_1_5 - 0.0416666666666666*G18_0_0_1_2_1_6 - 0.0416666666666666*G18_0_0_1_2_1_7 - 0.0416666666666666*G18_0_0_2_2_2_4 - 0.0416666666666666*G18_0_0_2_2_2_5 - 0.0416666666666666*G18_0_0_2_2_2_6 - 0.0416666666666666*G18_0_0_2_2_2_7 + 0.0833333333333332*G19_0_0_0_0_0_2_0_4_0 + 0.0833333333333332*G19_0_0_0_0_0_2_0_4_1 + 0.0833333333333332*G19_0_0_0_0_0_2_0_4_2 - 0.0833333333333332*G19_0_0_0_0_0_2_0_5_0 - 0.0833333333333332*G19_0_0_0_0_0_2_0_6_1 - 0.0833333333333332*G19_0_0_0_0_0_2_0_7_2 + 0.0833333333333332*G19_0_0_0_1_0_2_1_4_0 + 0.0833333333333332*G19_0_0_0_1_0_2_1_4_1 + 0.0833333333333332*G19_0_0_0_1_0_2_1_4_2 - 0.0833333333333332*G19_0_0_0_1_0_2_1_5_0 - 0.0833333333333332*G19_0_0_0_1_0_2_1_6_1 - 0.0833333333333332*G19_0_0_0_1_0_2_1_7_2 + 0.0833333333333332*G19_0_0_0_2_0_2_2_4_0 + 0.0833333333333332*G19_0_0_0_2_0_2_2_4_1 + 0.0833333333333332*G19_0_0_0_2_0_2_2_4_2 - 0.0833333333333332*G19_0_0_0_2_0_2_2_5_0 - 0.0833333333333332*G19_0_0_0_2_0_2_2_6_1 - 0.0833333333333332*G19_0_0_0_2_0_2_2_7_2 + 0.0833333333333332*G19_1_0_0_0_1_2_0_4_0 + 0.0833333333333332*G19_1_0_0_0_1_2_0_4_1 + 0.0833333333333332*G19_1_0_0_0_1_2_0_4_2 - 0.0833333333333332*G19_1_0_0_0_1_2_0_5_0 - 0.0833333333333332*G19_1_0_0_0_1_2_0_6_1 - 0.0833333333333332*G19_1_0_0_0_1_2_0_7_2 + 0.0833333333333332*G19_1_0_0_1_1_2_1_4_0 + 0.0833333333333332*G19_1_0_0_1_1_2_1_4_1 + 0.0833333333333332*G19_1_0_0_1_1_2_1_4_2 - 0.0833333333333332*G19_1_0_0_1_1_2_1_5_0 - 0.0833333333333332*G19_1_0_0_1_1_2_1_6_1 - 0.0833333333333332*G19_1_0_0_1_1_2_1_7_2 + 0.0833333333333332*G19_1_0_0_2_1_2_2_4_0 + 0.0833333333333332*G19_1_0_0_2_1_2_2_4_1 + 0.0833333333333332*G19_1_0_0_2_1_2_2_4_2 - 0.0833333333333332*G19_1_0_0_2_1_2_2_5_0 - 0.0833333333333332*G19_1_0_0_2_1_2_2_6_1 - 0.0833333333333332*G19_1_0_0_2_1_2_2_7_2 + 0.0833333333333332*G19_2_0_0_0_2_2_0_4_0 + 0.0833333333333332*G19_2_0_0_0_2_2_0_4_1 + 0.0833333333333332*G19_2_0_0_0_2_2_0_4_2 - 0.0833333333333332*G19_2_0_0_0_2_2_0_5_0 - 0.0833333333333332*G19_2_0_0_0_2_2_0_6_1 - 0.0833333333333332*G19_2_0_0_0_2_2_0_7_2 + 0.0833333333333332*G19_2_0_0_1_2_2_1_4_0 + 0.0833333333333332*G19_2_0_0_1_2_2_1_4_1 + 0.0833333333333332*G19_2_0_0_1_2_2_1_4_2 - 0.0833333333333332*G19_2_0_0_1_2_2_1_5_0 - 0.0833333333333332*G19_2_0_0_1_2_2_1_6_1 - 0.0833333333333332*G19_2_0_0_1_2_2_1_7_2 + 0.0833333333333332*G19_2_0_0_2_2_2_2_4_0 + 0.0833333333333332*G19_2_0_0_2_2_2_2_4_1 + 0.0833333333333332*G19_2_0_0_2_2_2_2_4_2 - 0.0833333333333332*G19_2_0_0_2_2_2_2_5_0 - 0.0833333333333332*G19_2_0_0_2_2_2_2_6_1 - 0.0833333333333332*G19_2_0_0_2_2_2_2_7_2 - 0.0833333333333332*G20_0_0_0_0_0_2_0_4_0 - 0.0833333333333332*G20_0_0_0_0_0_2_0_4_1 - 0.0833333333333332*G20_0_0_0_0_0_2_0_4_2 + 0.0833333333333332*G20_0_0_0_0_0_2_0_5_0 + 0.0833333333333332*G20_0_0_0_0_0_2_0_6_1 + 0.0833333333333332*G20_0_0_0_0_0_2_0_7_2 - 0.0833333333333332*G20_0_0_0_1_0_2_1_4_0 - 0.0833333333333332*G20_0_0_0_1_0_2_1_4_1 - 0.0833333333333332*G20_0_0_0_1_0_2_1_4_2 + 0.0833333333333332*G20_0_0_0_1_0_2_1_5_0 + 0.0833333333333332*G20_0_0_0_1_0_2_1_6_1 + 0.0833333333333332*G20_0_0_0_1_0_2_1_7_2 - 0.0833333333333332*G20_0_0_0_2_0_2_2_4_0 - 0.0833333333333332*G20_0_0_0_2_0_2_2_4_1 - 0.0833333333333332*G20_0_0_0_2_0_2_2_4_2 + 0.0833333333333332*G20_0_0_0_2_0_2_2_5_0 + 0.0833333333333332*G20_0_0_0_2_0_2_2_6_1 + 0.0833333333333332*G20_0_0_0_2_0_2_2_7_2 - 0.0833333333333332*G20_1_0_0_0_1_2_0_4_0 - 0.0833333333333332*G20_1_0_0_0_1_2_0_4_1 - 0.0833333333333332*G20_1_0_0_0_1_2_0_4_2 + 0.0833333333333332*G20_1_0_0_0_1_2_0_5_0 + 0.0833333333333332*G20_1_0_0_0_1_2_0_6_1 + 0.0833333333333332*G20_1_0_0_0_1_2_0_7_2 - 0.0833333333333332*G20_1_0_0_1_1_2_1_4_0 - 0.0833333333333332*G20_1_0_0_1_1_2_1_4_1 - 0.0833333333333332*G20_1_0_0_1_1_2_1_4_2 + 0.0833333333333332*G20_1_0_0_1_1_2_1_5_0 + 0.0833333333333332*G20_1_0_0_1_1_2_1_6_1 + 0.0833333333333332*G20_1_0_0_1_1_2_1_7_2 - 0.0833333333333332*G20_1_0_0_2_1_2_2_4_0 - 0.0833333333333332*G20_1_0_0_2_1_2_2_4_1 - 0.0833333333333332*G20_1_0_0_2_1_2_2_4_2 + 0.0833333333333332*G20_1_0_0_2_1_2_2_5_0 + 0.0833333333333332*G20_1_0_0_2_1_2_2_6_1 + 0.0833333333333332*G20_1_0_0_2_1_2_2_7_2 - 0.0833333333333332*G20_2_0_0_0_2_2_0_4_0 - 0.0833333333333332*G20_2_0_0_0_2_2_0_4_1 - 0.0833333333333332*G20_2_0_0_0_2_2_0_4_2 + 0.0833333333333332*G20_2_0_0_0_2_2_0_5_0 + 0.0833333333333332*G20_2_0_0_0_2_2_0_6_1 + 0.0833333333333332*G20_2_0_0_0_2_2_0_7_2 - 0.0833333333333332*G20_2_0_0_1_2_2_1_4_0 - 0.0833333333333332*G20_2_0_0_1_2_2_1_4_1 - 0.0833333333333332*G20_2_0_0_1_2_2_1_4_2 + 0.0833333333333332*G20_2_0_0_1_2_2_1_5_0 + 0.0833333333333332*G20_2_0_0_1_2_2_1_6_1 + 0.0833333333333332*G20_2_0_0_1_2_2_1_7_2 - 0.0833333333333332*G20_2_0_0_2_2_2_2_4_0 - 0.0833333333333332*G20_2_0_0_2_2_2_2_4_1 - 0.0833333333333332*G20_2_0_0_2_2_2_2_4_2 + 0.0833333333333332*G20_2_0_0_2_2_2_2_5_0 + 0.0833333333333332*G20_2_0_0_2_2_2_2_6_1 + 0.0833333333333332*G20_2_0_0_2_2_2_2_7_2 + 0.166666666666666*G21_1_0_0_0_2_0_0_0 + 0.166666666666666*G21_1_0_0_0_2_0_0_1 + 0.166666666666666*G21_1_0_0_0_2_0_0_2 - 0.166666666666666*G21_1_0_0_0_2_0_1_0 - 0.166666666666666*G21_1_0_0_0_2_0_2_1 - 0.166666666666666*G21_1_0_0_0_2_0_3_2 + 0.166666666666666*G21_1_0_0_1_2_1_0_0 + 0.166666666666666*G21_1_0_0_1_2_1_0_1 + 0.166666666666666*G21_1_0_0_1_2_1_0_2 - 0.166666666666666*G21_1_0_0_1_2_1_1_0 - 0.166666666666666*G21_1_0_0_1_2_1_2_1 - 0.166666666666666*G21_1_0_0_1_2_1_3_2 + 0.166666666666666*G21_1_0_0_2_2_2_0_0 + 0.166666666666666*G21_1_0_0_2_2_2_0_1 + 0.166666666666666*G21_1_0_0_2_2_2_0_2 - 0.166666666666666*G21_1_0_0_2_2_2_1_0 - 0.166666666666666*G21_1_0_0_2_2_2_2_1 - 0.166666666666666*G21_1_0_0_2_2_2_3_2 - 0.166666666666666*G22_1_0_0_0_2_0_0_0 - 0.166666666666666*G22_1_0_0_0_2_0_0_1 - 0.166666666666666*G22_1_0_0_0_2_0_0_2 + 0.166666666666666*G22_1_0_0_0_2_0_1_0 + 0.166666666666666*G22_1_0_0_0_2_0_2_1 + 0.166666666666666*G22_1_0_0_0_2_0_3_2 - 0.166666666666666*G22_1_0_0_1_2_1_0_0 - 0.166666666666666*G22_1_0_0_1_2_1_0_1 - 0.166666666666666*G22_1_0_0_1_2_1_0_2 + 0.166666666666666*G22_1_0_0_1_2_1_1_0 + 0.166666666666666*G22_1_0_0_1_2_1_2_1 + 0.166666666666666*G22_1_0_0_1_2_1_3_2 - 0.166666666666666*G22_1_0_0_2_2_2_0_0 - 0.166666666666666*G22_1_0_0_2_2_2_0_1 - 0.166666666666666*G22_1_0_0_2_2_2_0_2 + 0.166666666666666*G22_1_0_0_2_2_2_1_0 + 0.166666666666666*G22_1_0_0_2_2_2_2_1 + 0.166666666666666*G22_1_0_0_2_2_2_3_2 + 0.0833333333333332*G23_0_0_2_1_0_0_0 + 0.0833333333333332*G23_0_0_2_1_0_1_0 + 0.0833333333333332*G23_0_0_2_1_0_2_0 - 0.0833333333333332*G23_0_0_2_1_1_0_0 - 0.0833333333333332*G23_0_0_2_1_2_1_0 - 0.0833333333333332*G23_0_0_2_1_3_2_0 + 0.0833333333333332*G23_0_0_2_1_4_0_1 + 0.0833333333333332*G23_0_0_2_1_4_1_1 + 0.0833333333333332*G23_0_0_2_1_4_2_1 - 0.0833333333333332*G23_0_0_2_1_5_0_1 - 0.0833333333333332*G23_0_0_2_1_6_1_1 - 0.0833333333333332*G23_0_0_2_1_7_2_1 + 0.0833333333333332*G23_0_0_2_1_8_0_2 + 0.0833333333333332*G23_0_0_2_1_8_1_2 + 0.0833333333333332*G23_0_0_2_1_8_2_2 - 0.0833333333333332*G23_0_0_2_1_9_0_2 - 0.0833333333333332*G23_0_0_2_1_10_1_2 - 0.0833333333333332*G23_0_0_2_1_11_2_2;
    A[8] = 0.0166666666666666*G0_0_8 + 0.00833333333333331*G0_0_9 + 0.00833333333333331*G0_0_10 + 0.00833333333333331*G0_0_11 + 0.0166666666666666*G1_0_8 + 0.00833333333333331*G1_0_9 + 0.00833333333333331*G1_0_10 + 0.00833333333333331*G1_0_11 - 0.166666666666666*G2_0_0_2_0 - 0.166666666666666*G2_0_0_2_1 - 0.166666666666666*G2_0_0_2_2 - 0.166666666666666*G7_0_0_5_0 - 0.166666666666666*G7_0_0_5_1 - 0.166666666666666*G7_0_0_5_2 - 0.166666666666666*G8_0_0_8_0 - 0.166666666666666*G8_0_0_8_1 - 0.166666666666666*G8_0_0_8_2 + 0.166666666666666*G9_0_2_0 + 0.166666666666666*G9_0_2_1 + 0.166666666666666*G9_0_2_2 + 0.166666666666666*G14_0_5_0 + 0.166666666666666*G14_0_5_1 + 0.166666666666666*G14_0_5_2 + 0.166666666666666*G15_0_8_0 + 0.166666666666666*G15_0_8_1 + 0.166666666666666*G15_0_8_2 - 0.0416666666666666*G16_0_0_0_2 - 0.0416666666666666*G16_0_0_1_2 - 0.0416666666666666*G16_0_0_2_2 - 0.0416666666666666*G16_0_1_0_2 - 0.0416666666666666*G16_0_1_1_2 - 0.0416666666666666*G16_0_1_2_2 - 0.0416666666666666*G16_0_2_0_2 - 0.0416666666666666*G16_0_2_1_2 - 0.0416666666666666*G16_0_2_2_2 - 0.0416666666666666*G16_0_3_0_2 - 0.0416666666666666*G16_0_3_1_2 - 0.0416666666666666*G16_0_3_2_2 - 0.0416666666666666*G17_0_0_0_0_0_8 - 0.0416666666666666*G17_0_0_0_0_0_9 - 0.0416666666666666*G17_0_0_0_0_0_10 - 0.0416666666666666*G17_0_0_0_0_0_11 - 0.0416666666666666*G17_0_0_0_1_0_8 - 0.0416666666666666*G17_0_0_0_1_0_9 - 0.0416666666666666*G17_0_0_0_1_0_10 - 0.0416666666666666*G17_0_0_0_1_0_11 - 0.0416666666666666*G17_0_0_0_2_0_8 - 0.0416666666666666*G17_0_0_0_2_0_9 - 0.0416666666666666*G17_0_0_0_2_0_10 - 0.0416666666666666*G17_0_0_0_2_0_11 - 0.0416666666666666*G17_0_0_1_0_1_8 - 0.0416666666666666*G17_0_0_1_0_1_9 - 0.0416666666666666*G17_0_0_1_0_1_10 - 0.0416666666666666*G17_0_0_1_0_1_11 - 0.0416666666666666*G17_0_0_1_1_1_8 - 0.0416666666666666*G17_0_0_1_1_1_9 - 0.0416666666666666*G17_0_0_1_1_1_10 - 0.0416666666666666*G17_0_0_1_1_1_11 - 0.0416666666666666*G17_0_0_1_2_1_8 - 0.0416666666666666*G17_0_0_1_2_1_9 - 0.0416666666666666*G17_0_0_1_2_1_10 - 0.0416666666666666*G17_0_0_1_2_1_11 - 0.0416666666666666*G17_0_0_2_0_2_8 - 0.0416666666666666*G17_0_0_2_0_2_9 - 0.0416666666666666*G17_0_0_2_0_2_10 - 0.0416666666666666*G17_0_0_2_0_2_11 - 0.0416666666666666*G17_0_0_2_1_2_8 - 0.0416666666666666*G17_0_0_2_1_2_9 - 0.0416666666666666*G17_0_0_2_1_2_10 - 0.0416666666666666*G17_0_0_2_1_2_11 - 0.0416666666666666*G17_0_0_2_2_2_8 - 0.0416666666666666*G17_0_0_2_2_2_9 - 0.0416666666666666*G17_0_0_2_2_2_10 - 0.0416666666666666*G17_0_0_2_2_2_11 + 0.0416666666666666*G18_0_0_0_0_0_8 + 0.0416666666666666*G18_0_0_0_0_0_9 + 0.0416666666666666*G18_0_0_0_0_0_10 + 0.0416666666666666*G18_0_0_0_0_0_11 + 0.0416666666666666*G18_0_0_0_1_0_8 + 0.0416666666666666*G18_0_0_0_1_0_9 + 0.0416666666666666*G18_0_0_0_1_0_10 + 0.0416666666666666*G18_0_0_0_1_0_11 + 0.0416666666666666*G18_0_0_0_2_0_8 + 0.0416666666666666*G18_0_0_0_2_0_9 + 0.0416666666666666*G18_0_0_0_2_0_10 + 0.0416666666666666*G18_0_0_0_2_0_11 + 0.0416666666666666*G18_0_0_1_0_1_8 + 0.0416666666666666*G18_0_0_1_0_1_9 + 0.0416666666666666*G18_0_0_1_0_1_10 + 0.0416666666666666*G18_0_0_1_0_1_11 + 0.0416666666666666*G18_0_0_1_1_1_8 + 0.0416666666666666*G18_0_0_1_1_1_9 + 0.0416666666666666*G18_0_0_1_1_1_10 + 0.0416666666666666*G18_0_0_1_1_1_11 + 0.0416666666666666*G18_0_0_1_2_1_8 + 0.0416666666666666*G18_0_0_1_2_1_9 + 0.0416666666666666*G18_0_0_1_2_1_10 + 0.0416666666666666*G18_0_0_1_2_1_11 + 0.0416666666666666*G18_0_0_2_0_2_8 + 0.0416666666666666*G18_0_0_2_0_2_9 + 0.0416666666666666*G18_0_0_2_0_2_10 + 0.0416666666666666*G18_0_0_2_0_2_11 + 0.0416666666666666*G18_0_0_2_1_2_8 + 0.0416666666666666*G18_0_0_2_1_2_9 + 0.0416666666666666*G18_0_0_2_1_2_10 + 0.0416666666666666*G18_0_0_2_1_2_11 + 0.0416666666666666*G18_0_0_2_2_2_8 + 0.0416666666666666*G18_0_0_2_2_2_9 + 0.0416666666666666*G18_0_0_2_2_2_10 + 0.0416666666666666*G18_0_0_2_2_2_11 - 0.0833333333333332*G19_0_0_0_0_0_0_0_8_0 - 0.0833333333333332*G19_0_0_0_0_0_0_0_8_1 - 0.0833333333333332*G19_0_0_0_0_0_0_0_8_2 + 0.0833333333333332*G19_0_0_0_0_0_0_0_9_0 + 0.0833333333333332*G19_0_0_0_0_0_0_0_10_1 + 0.0833333333333332*G19_0_0_0_0_0_0_0_11_2 - 0.0833333333333332*G19_0_0_0_0_0_1_0_8_0 - 0.0833333333333332*G19_0_0_0_0_0_1_0_8_1 - 0.0833333333333332*G19_0_0_0_0_0_1_0_8_2 + 0.0833333333333332*G19_0_0_0_0_0_1_0_9_0 + 0.0833333333333332*G19_0_0_0_0_0_1_0_10_1 + 0.0833333333333332*G19_0_0_0_0_0_1_0_11_2 - 0.0833333333333332*G19_0_0_0_0_0_2_0_8_0 - 0.0833333333333332*G19_0_0_0_0_0_2_0_8_1 - 0.0833333333333332*G19_0_0_0_0_0_2_0_8_2 + 0.0833333333333332*G19_0_0_0_0_0_2_0_9_0 + 0.0833333333333332*G19_0_0_0_0_0_2_0_10_1 + 0.0833333333333332*G19_0_0_0_0_0_2_0_11_2 - 0.0833333333333332*G19_0_0_0_1_0_0_1_8_0 - 0.0833333333333332*G19_0_0_0_1_0_0_1_8_1 - 0.0833333333333332*G19_0_0_0_1_0_0_1_8_2 + 0.0833333333333332*G19_0_0_0_1_0_0_1_9_0 + 0.0833333333333332*G19_0_0_0_1_0_0_1_10_1 + 0.0833333333333332*G19_0_0_0_1_0_0_1_11_2 - 0.0833333333333332*G19_0_0_0_1_0_1_1_8_0 - 0.0833333333333332*G19_0_0_0_1_0_1_1_8_1 - 0.0833333333333332*G19_0_0_0_1_0_1_1_8_2 + 0.0833333333333332*G19_0_0_0_1_0_1_1_9_0 + 0.0833333333333332*G19_0_0_0_1_0_1_1_10_1 + 0.0833333333333332*G19_0_0_0_1_0_1_1_11_2 - 0.0833333333333332*G19_0_0_0_1_0_2_1_8_0 - 0.0833333333333332*G19_0_0_0_1_0_2_1_8_1 - 0.0833333333333332*G19_0_0_0_1_0_2_1_8_2 + 0.0833333333333332*G19_0_0_0_1_0_2_1_9_0 + 0.0833333333333332*G19_0_0_0_1_0_2_1_10_1 + 0.0833333333333332*G19_0_0_0_1_0_2_1_11_2 - 0.0833333333333332*G19_0_0_0_2_0_0_2_8_0 - 0.0833333333333332*G19_0_0_0_2_0_0_2_8_1 - 0.0833333333333332*G19_0_0_0_2_0_0_2_8_2 + 0.0833333333333332*G19_0_0_0_2_0_0_2_9_0 + 0.0833333333333332*G19_0_0_0_2_0_0_2_10_1 + 0.0833333333333332*G19_0_0_0_2_0_0_2_11_2 - 0.0833333333333332*G19_0_0_0_2_0_1_2_8_0 - 0.0833333333333332*G19_0_0_0_2_0_1_2_8_1 - 0.0833333333333332*G19_0_0_0_2_0_1_2_8_2 + 0.0833333333333332*G19_0_0_0_2_0_1_2_9_0 + 0.0833333333333332*G19_0_0_0_2_0_1_2_10_1 + 0.0833333333333332*G19_0_0_0_2_0_1_2_11_2 - 0.0833333333333332*G19_0_0_0_2_0_2_2_8_0 - 0.0833333333333332*G19_0_0_0_2_0_2_2_8_1 - 0.0833333333333332*G19_0_0_0_2_0_2_2_8_2 + 0.0833333333333332*G19_0_0_0_2_0_2_2_9_0 + 0.0833333333333332*G19_0_0_0_2_0_2_2_10_1 + 0.0833333333333332*G19_0_0_0_2_0_2_2_11_2 - 0.0833333333333332*G19_1_0_0_0_1_0_0_8_0 - 0.0833333333333332*G19_1_0_0_0_1_0_0_8_1 - 0.0833333333333332*G19_1_0_0_0_1_0_0_8_2 + 0.0833333333333332*G19_1_0_0_0_1_0_0_9_0 + 0.0833333333333332*G19_1_0_0_0_1_0_0_10_1 + 0.0833333333333332*G19_1_0_0_0_1_0_0_11_2 - 0.0833333333333332*G19_1_0_0_0_1_1_0_8_0 - 0.0833333333333332*G19_1_0_0_0_1_1_0_8_1 - 0.0833333333333332*G19_1_0_0_0_1_1_0_8_2 + 0.0833333333333332*G19_1_0_0_0_1_1_0_9_0 + 0.0833333333333332*G19_1_0_0_0_1_1_0_10_1 + 0.0833333333333332*G19_1_0_0_0_1_1_0_11_2 - 0.0833333333333332*G19_1_0_0_0_1_2_0_8_0 - 0.0833333333333332*G19_1_0_0_0_1_2_0_8_1 - 0.0833333333333332*G19_1_0_0_0_1_2_0_8_2 + 0.0833333333333332*G19_1_0_0_0_1_2_0_9_0 + 0.0833333333333332*G19_1_0_0_0_1_2_0_10_1 + 0.0833333333333332*G19_1_0_0_0_1_2_0_11_2 - 0.0833333333333332*G19_1_0_0_1_1_0_1_8_0 - 0.0833333333333332*G19_1_0_0_1_1_0_1_8_1 - 0.0833333333333332*G19_1_0_0_1_1_0_1_8_2 + 0.0833333333333332*G19_1_0_0_1_1_0_1_9_0 + 0.0833333333333332*G19_1_0_0_1_1_0_1_10_1 + 0.0833333333333332*G19_1_0_0_1_1_0_1_11_2 - 0.0833333333333332*G19_1_0_0_1_1_1_1_8_0 - 0.0833333333333332*G19_1_0_0_1_1_1_1_8_1 - 0.0833333333333332*G19_1_0_0_1_1_1_1_8_2 + 0.0833333333333332*G19_1_0_0_1_1_1_1_9_0 + 0.0833333333333332*G19_1_0_0_1_1_1_1_10_1 + 0.0833333333333332*G19_1_0_0_1_1_1_1_11_2 - 0.0833333333333332*G19_1_0_0_1_1_2_1_8_0 - 0.0833333333333332*G19_1_0_0_1_1_2_1_8_1 - 0.0833333333333332*G19_1_0_0_1_1_2_1_8_2 + 0.0833333333333332*G19_1_0_0_1_1_2_1_9_0 + 0.0833333333333332*G19_1_0_0_1_1_2_1_10_1 + 0.0833333333333332*G19_1_0_0_1_1_2_1_11_2 - 0.0833333333333332*G19_1_0_0_2_1_0_2_8_0 - 0.0833333333333332*G19_1_0_0_2_1_0_2_8_1 - 0.0833333333333332*G19_1_0_0_2_1_0_2_8_2 + 0.0833333333333332*G19_1_0_0_2_1_0_2_9_0 + 0.0833333333333332*G19_1_0_0_2_1_0_2_10_1 + 0.0833333333333332*G19_1_0_0_2_1_0_2_11_2 - 0.0833333333333332*G19_1_0_0_2_1_1_2_8_0 - 0.0833333333333332*G19_1_0_0_2_1_1_2_8_1 - 0.0833333333333332*G19_1_0_0_2_1_1_2_8_2 + 0.0833333333333332*G19_1_0_0_2_1_1_2_9_0 + 0.0833333333333332*G19_1_0_0_2_1_1_2_10_1 + 0.0833333333333332*G19_1_0_0_2_1_1_2_11_2 - 0.0833333333333332*G19_1_0_0_2_1_2_2_8_0 - 0.0833333333333332*G19_1_0_0_2_1_2_2_8_1 - 0.0833333333333332*G19_1_0_0_2_1_2_2_8_2 + 0.0833333333333332*G19_1_0_0_2_1_2_2_9_0 + 0.0833333333333332*G19_1_0_0_2_1_2_2_10_1 + 0.0833333333333332*G19_1_0_0_2_1_2_2_11_2 - 0.0833333333333332*G19_2_0_0_0_2_0_0_8_0 - 0.0833333333333332*G19_2_0_0_0_2_0_0_8_1 - 0.0833333333333332*G19_2_0_0_0_2_0_0_8_2 + 0.0833333333333332*G19_2_0_0_0_2_0_0_9_0 + 0.0833333333333332*G19_2_0_0_0_2_0_0_10_1 + 0.0833333333333332*G19_2_0_0_0_2_0_0_11_2 - 0.0833333333333332*G19_2_0_0_0_2_1_0_8_0 - 0.0833333333333332*G19_2_0_0_0_2_1_0_8_1 - 0.0833333333333332*G19_2_0_0_0_2_1_0_8_2 + 0.0833333333333332*G19_2_0_0_0_2_1_0_9_0 + 0.0833333333333332*G19_2_0_0_0_2_1_0_10_1 + 0.0833333333333332*G19_2_0_0_0_2_1_0_11_2 - 0.0833333333333332*G19_2_0_0_0_2_2_0_8_0 - 0.0833333333333332*G19_2_0_0_0_2_2_0_8_1 - 0.0833333333333332*G19_2_0_0_0_2_2_0_8_2 + 0.0833333333333332*G19_2_0_0_0_2_2_0_9_0 + 0.0833333333333332*G19_2_0_0_0_2_2_0_10_1 + 0.0833333333333332*G19_2_0_0_0_2_2_0_11_2 - 0.0833333333333332*G19_2_0_0_1_2_0_1_8_0 - 0.0833333333333332*G19_2_0_0_1_2_0_1_8_1 - 0.0833333333333332*G19_2_0_0_1_2_0_1_8_2 + 0.0833333333333332*G19_2_0_0_1_2_0_1_9_0 + 0.0833333333333332*G19_2_0_0_1_2_0_1_10_1 + 0.0833333333333332*G19_2_0_0_1_2_0_1_11_2 - 0.0833333333333332*G19_2_0_0_1_2_1_1_8_0 - 0.0833333333333332*G19_2_0_0_1_2_1_1_8_1 - 0.0833333333333332*G19_2_0_0_1_2_1_1_8_2 + 0.0833333333333332*G19_2_0_0_1_2_1_1_9_0 + 0.0833333333333332*G19_2_0_0_1_2_1_1_10_1 + 0.0833333333333332*G19_2_0_0_1_2_1_1_11_2 - 0.0833333333333332*G19_2_0_0_1_2_2_1_8_0 - 0.0833333333333332*G19_2_0_0_1_2_2_1_8_1 - 0.0833333333333332*G19_2_0_0_1_2_2_1_8_2 + 0.0833333333333332*G19_2_0_0_1_2_2_1_9_0 + 0.0833333333333332*G19_2_0_0_1_2_2_1_10_1 + 0.0833333333333332*G19_2_0_0_1_2_2_1_11_2 - 0.0833333333333332*G19_2_0_0_2_2_0_2_8_0 - 0.0833333333333332*G19_2_0_0_2_2_0_2_8_1 - 0.0833333333333332*G19_2_0_0_2_2_0_2_8_2 + 0.0833333333333332*G19_2_0_0_2_2_0_2_9_0 + 0.0833333333333332*G19_2_0_0_2_2_0_2_10_1 + 0.0833333333333332*G19_2_0_0_2_2_0_2_11_2 - 0.0833333333333332*G19_2_0_0_2_2_1_2_8_0 - 0.0833333333333332*G19_2_0_0_2_2_1_2_8_1 - 0.0833333333333332*G19_2_0_0_2_2_1_2_8_2 + 0.0833333333333332*G19_2_0_0_2_2_1_2_9_0 + 0.0833333333333332*G19_2_0_0_2_2_1_2_10_1 + 0.0833333333333332*G19_2_0_0_2_2_1_2_11_2 - 0.0833333333333332*G19_2_0_0_2_2_2_2_8_0 - 0.0833333333333332*G19_2_0_0_2_2_2_2_8_1 - 0.0833333333333332*G19_2_0_0_2_2_2_2_8_2 + 0.0833333333333332*G19_2_0_0_2_2_2_2_9_0 + 0.0833333333333332*G19_2_0_0_2_2_2_2_10_1 + 0.0833333333333332*G19_2_0_0_2_2_2_2_11_2 + 0.0833333333333332*G20_0_0_0_0_0_0_0_8_0 + 0.0833333333333332*G20_0_0_0_0_0_0_0_8_1 + 0.0833333333333332*G20_0_0_0_0_0_0_0_8_2 - 0.0833333333333332*G20_0_0_0_0_0_0_0_9_0 - 0.0833333333333332*G20_0_0_0_0_0_0_0_10_1 - 0.0833333333333332*G20_0_0_0_0_0_0_0_11_2 + 0.0833333333333332*G20_0_0_0_0_0_1_0_8_0 + 0.0833333333333332*G20_0_0_0_0_0_1_0_8_1 + 0.0833333333333332*G20_0_0_0_0_0_1_0_8_2 - 0.0833333333333332*G20_0_0_0_0_0_1_0_9_0 - 0.0833333333333332*G20_0_0_0_0_0_1_0_10_1 - 0.0833333333333332*G20_0_0_0_0_0_1_0_11_2 + 0.0833333333333332*G20_0_0_0_0_0_2_0_8_0 + 0.0833333333333332*G20_0_0_0_0_0_2_0_8_1 + 0.0833333333333332*G20_0_0_0_0_0_2_0_8_2 - 0.0833333333333332*G20_0_0_0_0_0_2_0_9_0 - 0.0833333333333332*G20_0_0_0_0_0_2_0_10_1 - 0.0833333333333332*G20_0_0_0_0_0_2_0_11_2 + 0.0833333333333332*G20_0_0_0_1_0_0_1_8_0 + 0.0833333333333332*G20_0_0_0_1_0_0_1_8_1 + 0.0833333333333332*G20_0_0_0_1_0_0_1_8_2 - 0.0833333333333332*G20_0_0_0_1_0_0_1_9_0 - 0.0833333333333332*G20_0_0_0_1_0_0_1_10_1 - 0.0833333333333332*G20_0_0_0_1_0_0_1_11_2 + 0.0833333333333332*G20_0_0_0_1_0_1_1_8_0 + 0.0833333333333332*G20_0_0_0_1_0_1_1_8_1 + 0.0833333333333332*G20_0_0_0_1_0_1_1_8_2 - 0.0833333333333332*G20_0_0_0_1_0_1_1_9_0 - 0.0833333333333332*G20_0_0_0_1_0_1_1_10_1 - 0.0833333333333332*G20_0_0_0_1_0_1_1_11_2 + 0.0833333333333332*G20_0_0_0_1_0_2_1_8_0 + 0.0833333333333332*G20_0_0_0_1_0_2_1_8_1 + 0.0833333333333332*G20_0_0_0_1_0_2_1_8_2 - 0.0833333333333332*G20_0_0_0_1_0_2_1_9_0 - 0.0833333333333332*G20_0_0_0_1_0_2_1_10_1 - 0.0833333333333332*G20_0_0_0_1_0_2_1_11_2 + 0.0833333333333332*G20_0_0_0_2_0_0_2_8_0 + 0.0833333333333332*G20_0_0_0_2_0_0_2_8_1 + 0.0833333333333332*G20_0_0_0_2_0_0_2_8_2 - 0.0833333333333332*G20_0_0_0_2_0_0_2_9_0 - 0.0833333333333332*G20_0_0_0_2_0_0_2_10_1 - 0.0833333333333332*G20_0_0_0_2_0_0_2_11_2 + 0.0833333333333332*G20_0_0_0_2_0_1_2_8_0 + 0.0833333333333332*G20_0_0_0_2_0_1_2_8_1 + 0.0833333333333332*G20_0_0_0_2_0_1_2_8_2 - 0.0833333333333332*G20_0_0_0_2_0_1_2_9_0 - 0.0833333333333332*G20_0_0_0_2_0_1_2_10_1 - 0.0833333333333332*G20_0_0_0_2_0_1_2_11_2 + 0.0833333333333332*G20_0_0_0_2_0_2_2_8_0 + 0.0833333333333332*G20_0_0_0_2_0_2_2_8_1 + 0.0833333333333332*G20_0_0_0_2_0_2_2_8_2 - 0.0833333333333332*G20_0_0_0_2_0_2_2_9_0 - 0.0833333333333332*G20_0_0_0_2_0_2_2_10_1 - 0.0833333333333332*G20_0_0_0_2_0_2_2_11_2 + 0.0833333333333332*G20_1_0_0_0_1_0_0_8_0 + 0.0833333333333332*G20_1_0_0_0_1_0_0_8_1 + 0.0833333333333332*G20_1_0_0_0_1_0_0_8_2 - 0.0833333333333332*G20_1_0_0_0_1_0_0_9_0 - 0.0833333333333332*G20_1_0_0_0_1_0_0_10_1 - 0.0833333333333332*G20_1_0_0_0_1_0_0_11_2 + 0.0833333333333332*G20_1_0_0_0_1_1_0_8_0 + 0.0833333333333332*G20_1_0_0_0_1_1_0_8_1 + 0.0833333333333332*G20_1_0_0_0_1_1_0_8_2 - 0.0833333333333332*G20_1_0_0_0_1_1_0_9_0 - 0.0833333333333332*G20_1_0_0_0_1_1_0_10_1 - 0.0833333333333332*G20_1_0_0_0_1_1_0_11_2 + 0.0833333333333332*G20_1_0_0_0_1_2_0_8_0 + 0.0833333333333332*G20_1_0_0_0_1_2_0_8_1 + 0.0833333333333332*G20_1_0_0_0_1_2_0_8_2 - 0.0833333333333332*G20_1_0_0_0_1_2_0_9_0 - 0.0833333333333332*G20_1_0_0_0_1_2_0_10_1 - 0.0833333333333332*G20_1_0_0_0_1_2_0_11_2 + 0.0833333333333332*G20_1_0_0_1_1_0_1_8_0 + 0.0833333333333332*G20_1_0_0_1_1_0_1_8_1 + 0.0833333333333332*G20_1_0_0_1_1_0_1_8_2 - 0.0833333333333332*G20_1_0_0_1_1_0_1_9_0 - 0.0833333333333332*G20_1_0_0_1_1_0_1_10_1 - 0.0833333333333332*G20_1_0_0_1_1_0_1_11_2 + 0.0833333333333332*G20_1_0_0_1_1_1_1_8_0 + 0.0833333333333332*G20_1_0_0_1_1_1_1_8_1 + 0.0833333333333332*G20_1_0_0_1_1_1_1_8_2 - 0.0833333333333332*G20_1_0_0_1_1_1_1_9_0 - 0.0833333333333332*G20_1_0_0_1_1_1_1_10_1 - 0.0833333333333332*G20_1_0_0_1_1_1_1_11_2 + 0.0833333333333332*G20_1_0_0_1_1_2_1_8_0 + 0.0833333333333332*G20_1_0_0_1_1_2_1_8_1 + 0.0833333333333332*G20_1_0_0_1_1_2_1_8_2 - 0.0833333333333332*G20_1_0_0_1_1_2_1_9_0 - 0.0833333333333332*G20_1_0_0_1_1_2_1_10_1 - 0.0833333333333332*G20_1_0_0_1_1_2_1_11_2 + 0.0833333333333332*G20_1_0_0_2_1_0_2_8_0 + 0.0833333333333332*G20_1_0_0_2_1_0_2_8_1 + 0.0833333333333332*G20_1_0_0_2_1_0_2_8_2 - 0.0833333333333332*G20_1_0_0_2_1_0_2_9_0 - 0.0833333333333332*G20_1_0_0_2_1_0_2_10_1 - 0.0833333333333332*G20_1_0_0_2_1_0_2_11_2 + 0.0833333333333332*G20_1_0_0_2_1_1_2_8_0 + 0.0833333333333332*G20_1_0_0_2_1_1_2_8_1 + 0.0833333333333332*G20_1_0_0_2_1_1_2_8_2 - 0.0833333333333332*G20_1_0_0_2_1_1_2_9_0 - 0.0833333333333332*G20_1_0_0_2_1_1_2_10_1 - 0.0833333333333332*G20_1_0_0_2_1_1_2_11_2 + 0.0833333333333332*G20_1_0_0_2_1_2_2_8_0 + 0.0833333333333332*G20_1_0_0_2_1_2_2_8_1 + 0.0833333333333332*G20_1_0_0_2_1_2_2_8_2 - 0.0833333333333332*G20_1_0_0_2_1_2_2_9_0 - 0.0833333333333332*G20_1_0_0_2_1_2_2_10_1 - 0.0833333333333332*G20_1_0_0_2_1_2_2_11_2 + 0.0833333333333332*G20_2_0_0_0_2_0_0_8_0 + 0.0833333333333332*G20_2_0_0_0_2_0_0_8_1 + 0.0833333333333332*G20_2_0_0_0_2_0_0_8_2 - 0.0833333333333332*G20_2_0_0_0_2_0_0_9_0 - 0.0833333333333332*G20_2_0_0_0_2_0_0_10_1 - 0.0833333333333332*G20_2_0_0_0_2_0_0_11_2 + 0.0833333333333332*G20_2_0_0_0_2_1_0_8_0 + 0.0833333333333332*G20_2_0_0_0_2_1_0_8_1 + 0.0833333333333332*G20_2_0_0_0_2_1_0_8_2 - 0.0833333333333332*G20_2_0_0_0_2_1_0_9_0 - 0.0833333333333332*G20_2_0_0_0_2_1_0_10_1 - 0.0833333333333332*G20_2_0_0_0_2_1_0_11_2 + 0.0833333333333332*G20_2_0_0_0_2_2_0_8_0 + 0.0833333333333332*G20_2_0_0_0_2_2_0_8_1 + 0.0833333333333332*G20_2_0_0_0_2_2_0_8_2 - 0.0833333333333332*G20_2_0_0_0_2_2_0_9_0 - 0.0833333333333332*G20_2_0_0_0_2_2_0_10_1 - 0.0833333333333332*G20_2_0_0_0_2_2_0_11_2 + 0.0833333333333332*G20_2_0_0_1_2_0_1_8_0 + 0.0833333333333332*G20_2_0_0_1_2_0_1_8_1 + 0.0833333333333332*G20_2_0_0_1_2_0_1_8_2 - 0.0833333333333332*G20_2_0_0_1_2_0_1_9_0 - 0.0833333333333332*G20_2_0_0_1_2_0_1_10_1 - 0.0833333333333332*G20_2_0_0_1_2_0_1_11_2 + 0.0833333333333332*G20_2_0_0_1_2_1_1_8_0 + 0.0833333333333332*G20_2_0_0_1_2_1_1_8_1 + 0.0833333333333332*G20_2_0_0_1_2_1_1_8_2 - 0.0833333333333332*G20_2_0_0_1_2_1_1_9_0 - 0.0833333333333332*G20_2_0_0_1_2_1_1_10_1 - 0.0833333333333332*G20_2_0_0_1_2_1_1_11_2 + 0.0833333333333332*G20_2_0_0_1_2_2_1_8_0 + 0.0833333333333332*G20_2_0_0_1_2_2_1_8_1 + 0.0833333333333332*G20_2_0_0_1_2_2_1_8_2 - 0.0833333333333332*G20_2_0_0_1_2_2_1_9_0 - 0.0833333333333332*G20_2_0_0_1_2_2_1_10_1 - 0.0833333333333332*G20_2_0_0_1_2_2_1_11_2 + 0.0833333333333332*G20_2_0_0_2_2_0_2_8_0 + 0.0833333333333332*G20_2_0_0_2_2_0_2_8_1 + 0.0833333333333332*G20_2_0_0_2_2_0_2_8_2 - 0.0833333333333332*G20_2_0_0_2_2_0_2_9_0 - 0.0833333333333332*G20_2_0_0_2_2_0_2_10_1 - 0.0833333333333332*G20_2_0_0_2_2_0_2_11_2 + 0.0833333333333332*G20_2_0_0_2_2_1_2_8_0 + 0.0833333333333332*G20_2_0_0_2_2_1_2_8_1 + 0.0833333333333332*G20_2_0_0_2_2_1_2_8_2 - 0.0833333333333332*G20_2_0_0_2_2_1_2_9_0 - 0.0833333333333332*G20_2_0_0_2_2_1_2_10_1 - 0.0833333333333332*G20_2_0_0_2_2_1_2_11_2 + 0.0833333333333332*G20_2_0_0_2_2_2_2_8_0 + 0.0833333333333332*G20_2_0_0_2_2_2_2_8_1 + 0.0833333333333332*G20_2_0_0_2_2_2_2_8_2 - 0.0833333333333332*G20_2_0_0_2_2_2_2_9_0 - 0.0833333333333332*G20_2_0_0_2_2_2_2_10_1 - 0.0833333333333332*G20_2_0_0_2_2_2_2_11_2 - 0.166666666666666*G21_2_0_0_0_0_0_0_0 - 0.166666666666666*G21_2_0_0_0_0_0_0_1 - 0.166666666666666*G21_2_0_0_0_0_0_0_2 + 0.166666666666666*G21_2_0_0_0_0_0_1_0 + 0.166666666666666*G21_2_0_0_0_0_0_2_1 + 0.166666666666666*G21_2_0_0_0_0_0_3_2 - 0.166666666666666*G21_2_0_0_0_1_0_0_0 - 0.166666666666666*G21_2_0_0_0_1_0_0_1 - 0.166666666666666*G21_2_0_0_0_1_0_0_2 + 0.166666666666666*G21_2_0_0_0_1_0_1_0 + 0.166666666666666*G21_2_0_0_0_1_0_2_1 + 0.166666666666666*G21_2_0_0_0_1_0_3_2 - 0.166666666666666*G21_2_0_0_0_2_0_0_0 - 0.166666666666666*G21_2_0_0_0_2_0_0_1 - 0.166666666666666*G21_2_0_0_0_2_0_0_2 + 0.166666666666666*G21_2_0_0_0_2_0_1_0 + 0.166666666666666*G21_2_0_0_0_2_0_2_1 + 0.166666666666666*G21_2_0_0_0_2_0_3_2 - 0.166666666666666*G21_2_0_0_1_0_1_0_0 - 0.166666666666666*G21_2_0_0_1_0_1_0_1 - 0.166666666666666*G21_2_0_0_1_0_1_0_2 + 0.166666666666666*G21_2_0_0_1_0_1_1_0 + 0.166666666666666*G21_2_0_0_1_0_1_2_1 + 0.166666666666666*G21_2_0_0_1_0_1_3_2 - 0.166666666666666*G21_2_0_0_1_1_1_0_0 - 0.166666666666666*G21_2_0_0_1_1_1_0_1 - 0.166666666666666*G21_2_0_0_1_1_1_0_2 + 0.166666666666666*G21_2_0_0_1_1_1_1_0 + 0.166666666666666*G21_2_0_0_1_1_1_2_1 + 0.166666666666666*G21_2_0_0_1_1_1_3_2 - 0.166666666666666*G21_2_0_0_1_2_1_0_0 - 0.166666666666666*G21_2_0_0_1_2_1_0_1 - 0.166666666666666*G21_2_0_0_1_2_1_0_2 + 0.166666666666666*G21_2_0_0_1_2_1_1_0 + 0.166666666666666*G21_2_0_0_1_2_1_2_1 + 0.166666666666666*G21_2_0_0_1_2_1_3_2 - 0.166666666666666*G21_2_0_0_2_0_2_0_0 - 0.166666666666666*G21_2_0_0_2_0_2_0_1 - 0.166666666666666*G21_2_0_0_2_0_2_0_2 + 0.166666666666666*G21_2_0_0_2_0_2_1_0 + 0.166666666666666*G21_2_0_0_2_0_2_2_1 + 0.166666666666666*G21_2_0_0_2_0_2_3_2 - 0.166666666666666*G21_2_0_0_2_1_2_0_0 - 0.166666666666666*G21_2_0_0_2_1_2_0_1 - 0.166666666666666*G21_2_0_0_2_1_2_0_2 + 0.166666666666666*G21_2_0_0_2_1_2_1_0 + 0.166666666666666*G21_2_0_0_2_1_2_2_1 + 0.166666666666666*G21_2_0_0_2_1_2_3_2 - 0.166666666666666*G21_2_0_0_2_2_2_0_0 - 0.166666666666666*G21_2_0_0_2_2_2_0_1 - 0.166666666666666*G21_2_0_0_2_2_2_0_2 + 0.166666666666666*G21_2_0_0_2_2_2_1_0 + 0.166666666666666*G21_2_0_0_2_2_2_2_1 + 0.166666666666666*G21_2_0_0_2_2_2_3_2 + 0.166666666666666*G22_2_0_0_0_0_0_0_0 + 0.166666666666666*G22_2_0_0_0_0_0_0_1 + 0.166666666666666*G22_2_0_0_0_0_0_0_2 - 0.166666666666666*G22_2_0_0_0_0_0_1_0 - 0.166666666666666*G22_2_0_0_0_0_0_2_1 - 0.166666666666666*G22_2_0_0_0_0_0_3_2 + 0.166666666666666*G22_2_0_0_0_1_0_0_0 + 0.166666666666666*G22_2_0_0_0_1_0_0_1 + 0.166666666666666*G22_2_0_0_0_1_0_0_2 - 0.166666666666666*G22_2_0_0_0_1_0_1_0 - 0.166666666666666*G22_2_0_0_0_1_0_2_1 - 0.166666666666666*G22_2_0_0_0_1_0_3_2 + 0.166666666666666*G22_2_0_0_0_2_0_0_0 + 0.166666666666666*G22_2_0_0_0_2_0_0_1 + 0.166666666666666*G22_2_0_0_0_2_0_0_2 - 0.166666666666666*G22_2_0_0_0_2_0_1_0 - 0.166666666666666*G22_2_0_0_0_2_0_2_1 - 0.166666666666666*G22_2_0_0_0_2_0_3_2 + 0.166666666666666*G22_2_0_0_1_0_1_0_0 + 0.166666666666666*G22_2_0_0_1_0_1_0_1 + 0.166666666666666*G22_2_0_0_1_0_1_0_2 - 0.166666666666666*G22_2_0_0_1_0_1_1_0 - 0.166666666666666*G22_2_0_0_1_0_1_2_1 - 0.166666666666666*G22_2_0_0_1_0_1_3_2 + 0.166666666666666*G22_2_0_0_1_1_1_0_0 + 0.166666666666666*G22_2_0_0_1_1_1_0_1 + 0.166666666666666*G22_2_0_0_1_1_1_0_2 - 0.166666666666666*G22_2_0_0_1_1_1_1_0 - 0.166666666666666*G22_2_0_0_1_1_1_2_1 - 0.166666666666666*G22_2_0_0_1_1_1_3_2 + 0.166666666666666*G22_2_0_0_1_2_1_0_0 + 0.166666666666666*G22_2_0_0_1_2_1_0_1 + 0.166666666666666*G22_2_0_0_1_2_1_0_2 - 0.166666666666666*G22_2_0_0_1_2_1_1_0 - 0.166666666666666*G22_2_0_0_1_2_1_2_1 - 0.166666666666666*G22_2_0_0_1_2_1_3_2 + 0.166666666666666*G22_2_0_0_2_0_2_0_0 + 0.166666666666666*G22_2_0_0_2_0_2_0_1 + 0.166666666666666*G22_2_0_0_2_0_2_0_2 - 0.166666666666666*G22_2_0_0_2_0_2_1_0 - 0.166666666666666*G22_2_0_0_2_0_2_2_1 - 0.166666666666666*G22_2_0_0_2_0_2_3_2 + 0.166666666666666*G22_2_0_0_2_1_2_0_0 + 0.166666666666666*G22_2_0_0_2_1_2_0_1 + 0.166666666666666*G22_2_0_0_2_1_2_0_2 - 0.166666666666666*G22_2_0_0_2_1_2_1_0 - 0.166666666666666*G22_2_0_0_2_1_2_2_1 - 0.166666666666666*G22_2_0_0_2_1_2_3_2 + 0.166666666666666*G22_2_0_0_2_2_2_0_0 + 0.166666666666666*G22_2_0_0_2_2_2_0_1 + 0.166666666666666*G22_2_0_0_2_2_2_0_2 - 0.166666666666666*G22_2_0_0_2_2_2_1_0 - 0.166666666666666*G22_2_0_0_2_2_2_2_1 - 0.166666666666666*G22_2_0_0_2_2_2_3_2 - 0.0833333333333332*G23_0_0_0_2_0_0_0 - 0.0833333333333332*G23_0_0_0_2_0_1_0 - 0.0833333333333332*G23_0_0_0_2_0_2_0 + 0.0833333333333332*G23_0_0_0_2_1_0_0 + 0.0833333333333332*G23_0_0_0_2_2_1_0 + 0.0833333333333332*G23_0_0_0_2_3_2_0 - 0.0833333333333332*G23_0_0_0_2_4_0_1 - 0.0833333333333332*G23_0_0_0_2_4_1_1 - 0.0833333333333332*G23_0_0_0_2_4_2_1 + 0.0833333333333332*G23_0_0_0_2_5_0_1 + 0.0833333333333332*G23_0_0_0_2_6_1_1 + 0.0833333333333332*G23_0_0_0_2_7_2_1 - 0.0833333333333332*G23_0_0_0_2_8_0_2 - 0.0833333333333332*G23_0_0_0_2_8_1_2 - 0.0833333333333332*G23_0_0_0_2_8_2_2 + 0.0833333333333332*G23_0_0_0_2_9_0_2 + 0.0833333333333332*G23_0_0_0_2_10_1_2 + 0.0833333333333332*G23_0_0_0_2_11_2_2 - 0.0833333333333332*G23_0_0_1_2_0_0_0 - 0.0833333333333332*G23_0_0_1_2_0_1_0 - 0.0833333333333332*G23_0_0_1_2_0_2_0 + 0.0833333333333332*G23_0_0_1_2_1_0_0 + 0.0833333333333332*G23_0_0_1_2_2_1_0 + 0.0833333333333332*G23_0_0_1_2_3_2_0 - 0.0833333333333332*G23_0_0_1_2_4_0_1 - 0.0833333333333332*G23_0_0_1_2_4_1_1 - 0.0833333333333332*G23_0_0_1_2_4_2_1 + 0.0833333333333332*G23_0_0_1_2_5_0_1 + 0.0833333333333332*G23_0_0_1_2_6_1_1 + 0.0833333333333332*G23_0_0_1_2_7_2_1 - 0.0833333333333332*G23_0_0_1_2_8_0_2 - 0.0833333333333332*G23_0_0_1_2_8_1_2 - 0.0833333333333332*G23_0_0_1_2_8_2_2 + 0.0833333333333332*G23_0_0_1_2_9_0_2 + 0.0833333333333332*G23_0_0_1_2_10_1_2 + 0.0833333333333332*G23_0_0_1_2_11_2_2 - 0.0833333333333332*G23_0_0_2_2_0_0_0 - 0.0833333333333332*G23_0_0_2_2_0_1_0 - 0.0833333333333332*G23_0_0_2_2_0_2_0 + 0.0833333333333332*G23_0_0_2_2_1_0_0 + 0.0833333333333332*G23_0_0_2_2_2_1_0 + 0.0833333333333332*G23_0_0_2_2_3_2_0 - 0.0833333333333332*G23_0_0_2_2_4_0_1 - 0.0833333333333332*G23_0_0_2_2_4_1_1 - 0.0833333333333332*G23_0_0_2_2_4_2_1 + 0.0833333333333332*G23_0_0_2_2_5_0_1 + 0.0833333333333332*G23_0_0_2_2_6_1_1 + 0.0833333333333332*G23_0_0_2_2_7_2_1 - 0.0833333333333332*G23_0_0_2_2_8_0_2 - 0.0833333333333332*G23_0_0_2_2_8_1_2 - 0.0833333333333332*G23_0_0_2_2_8_2_2 + 0.0833333333333332*G23_0_0_2_2_9_0_2 + 0.0833333333333332*G23_0_0_2_2_10_1_2 + 0.0833333333333332*G23_0_0_2_2_11_2_2;
    A[9] = 0.00833333333333331*G0_0_8 + 0.0166666666666666*G0_0_9 + 0.00833333333333331*G0_0_10 + 0.00833333333333331*G0_0_11 + 0.00833333333333331*G1_0_8 + 0.0166666666666666*G1_0_9 + 0.00833333333333331*G1_0_10 + 0.00833333333333331*G1_0_11 + 0.166666666666666*G2_0_0_2_0 + 0.166666666666666*G7_0_0_5_0 + 0.166666666666666*G8_0_0_8_0 - 0.166666666666666*G9_0_2_0 - 0.166666666666666*G14_0_5_0 - 0.166666666666666*G15_0_8_0 + 0.0416666666666666*G16_0_0_0_2 + 0.0416666666666666*G16_0_1_0_2 + 0.0416666666666666*G16_0_2_0_2 + 0.0416666666666666*G16_0_3_0_2 + 0.0416666666666666*G17_0_0_0_0_0_8 + 0.0416666666666666*G17_0_0_0_0_0_9 + 0.0416666666666666*G17_0_0_0_0_0_10 + 0.0416666666666666*G17_0_0_0_0_0_11 + 0.0416666666666666*G17_0_0_1_0_1_8 + 0.0416666666666666*G17_0_0_1_0_1_9 + 0.0416666666666666*G17_0_0_1_0_1_10 + 0.0416666666666666*G17_0_0_1_0_1_11 + 0.0416666666666666*G17_0_0_2_0_2_8 + 0.0416666666666666*G17_0_0_2_0_2_9 + 0.0416666666666666*G17_0_0_2_0_2_10 + 0.0416666666666666*G17_0_0_2_0_2_11 - 0.0416666666666666*G18_0_0_0_0_0_8 - 0.0416666666666666*G18_0_0_0_0_0_9 - 0.0416666666666666*G18_0_0_0_0_0_10 - 0.0416666666666666*G18_0_0_0_0_0_11 - 0.0416666666666666*G18_0_0_1_0_1_8 - 0.0416666666666666*G18_0_0_1_0_1_9 - 0.0416666666666666*G18_0_0_1_0_1_10 - 0.0416666666666666*G18_0_0_1_0_1_11 - 0.0416666666666666*G18_0_0_2_0_2_8 - 0.0416666666666666*G18_0_0_2_0_2_9 - 0.0416666666666666*G18_0_0_2_0_2_10 - 0.0416666666666666*G18_0_0_2_0_2_11 + 0.0833333333333332*G19_0_0_0_0_0_0_0_8_0 + 0.0833333333333332*G19_0_0_0_0_0_0_0_8_1 + 0.0833333333333332*G19_0_0_0_0_0_0_0_8_2 - 0.0833333333333332*G19_0_0_0_0_0_0_0_9_0 - 0.0833333333333332*G19_0_0_0_0_0_0_0_10_1 - 0.0833333333333332*G19_0_0_0_0_0_0_0_11_2 + 0.0833333333333332*G19_0_0_0_1_0_0_1_8_0 + 0.0833333333333332*G19_0_0_0_1_0_0_1_8_1 + 0.0833333333333332*G19_0_0_0_1_0_0_1_8_2 - 0.0833333333333332*G19_0_0_0_1_0_0_1_9_0 - 0.0833333333333332*G19_0_0_0_1_0_0_1_10_1 - 0.0833333333333332*G19_0_0_0_1_0_0_1_11_2 + 0.0833333333333332*G19_0_0_0_2_0_0_2_8_0 + 0.0833333333333332*G19_0_0_0_2_0_0_2_8_1 + 0.0833333333333332*G19_0_0_0_2_0_0_2_8_2 - 0.0833333333333332*G19_0_0_0_2_0_0_2_9_0 - 0.0833333333333332*G19_0_0_0_2_0_0_2_10_1 - 0.0833333333333332*G19_0_0_0_2_0_0_2_11_2 + 0.0833333333333332*G19_1_0_0_0_1_0_0_8_0 + 0.0833333333333332*G19_1_0_0_0_1_0_0_8_1 + 0.0833333333333332*G19_1_0_0_0_1_0_0_8_2 - 0.0833333333333332*G19_1_0_0_0_1_0_0_9_0 - 0.0833333333333332*G19_1_0_0_0_1_0_0_10_1 - 0.0833333333333332*G19_1_0_0_0_1_0_0_11_2 + 0.0833333333333332*G19_1_0_0_1_1_0_1_8_0 + 0.0833333333333332*G19_1_0_0_1_1_0_1_8_1 + 0.0833333333333332*G19_1_0_0_1_1_0_1_8_2 - 0.0833333333333332*G19_1_0_0_1_1_0_1_9_0 - 0.0833333333333332*G19_1_0_0_1_1_0_1_10_1 - 0.0833333333333332*G19_1_0_0_1_1_0_1_11_2 + 0.0833333333333332*G19_1_0_0_2_1_0_2_8_0 + 0.0833333333333332*G19_1_0_0_2_1_0_2_8_1 + 0.0833333333333332*G19_1_0_0_2_1_0_2_8_2 - 0.0833333333333332*G19_1_0_0_2_1_0_2_9_0 - 0.0833333333333332*G19_1_0_0_2_1_0_2_10_1 - 0.0833333333333332*G19_1_0_0_2_1_0_2_11_2 + 0.0833333333333332*G19_2_0_0_0_2_0_0_8_0 + 0.0833333333333332*G19_2_0_0_0_2_0_0_8_1 + 0.0833333333333332*G19_2_0_0_0_2_0_0_8_2 - 0.0833333333333332*G19_2_0_0_0_2_0_0_9_0 - 0.0833333333333332*G19_2_0_0_0_2_0_0_10_1 - 0.0833333333333332*G19_2_0_0_0_2_0_0_11_2 + 0.0833333333333332*G19_2_0_0_1_2_0_1_8_0 + 0.0833333333333332*G19_2_0_0_1_2_0_1_8_1 + 0.0833333333333332*G19_2_0_0_1_2_0_1_8_2 - 0.0833333333333332*G19_2_0_0_1_2_0_1_9_0 - 0.0833333333333332*G19_2_0_0_1_2_0_1_10_1 - 0.0833333333333332*G19_2_0_0_1_2_0_1_11_2 + 0.0833333333333332*G19_2_0_0_2_2_0_2_8_0 + 0.0833333333333332*G19_2_0_0_2_2_0_2_8_1 + 0.0833333333333332*G19_2_0_0_2_2_0_2_8_2 - 0.0833333333333332*G19_2_0_0_2_2_0_2_9_0 - 0.0833333333333332*G19_2_0_0_2_2_0_2_10_1 - 0.0833333333333332*G19_2_0_0_2_2_0_2_11_2 - 0.0833333333333332*G20_0_0_0_0_0_0_0_8_0 - 0.0833333333333332*G20_0_0_0_0_0_0_0_8_1 - 0.0833333333333332*G20_0_0_0_0_0_0_0_8_2 + 0.0833333333333332*G20_0_0_0_0_0_0_0_9_0 + 0.0833333333333332*G20_0_0_0_0_0_0_0_10_1 + 0.0833333333333332*G20_0_0_0_0_0_0_0_11_2 - 0.0833333333333332*G20_0_0_0_1_0_0_1_8_0 - 0.0833333333333332*G20_0_0_0_1_0_0_1_8_1 - 0.0833333333333332*G20_0_0_0_1_0_0_1_8_2 + 0.0833333333333332*G20_0_0_0_1_0_0_1_9_0 + 0.0833333333333332*G20_0_0_0_1_0_0_1_10_1 + 0.0833333333333332*G20_0_0_0_1_0_0_1_11_2 - 0.0833333333333332*G20_0_0_0_2_0_0_2_8_0 - 0.0833333333333332*G20_0_0_0_2_0_0_2_8_1 - 0.0833333333333332*G20_0_0_0_2_0_0_2_8_2 + 0.0833333333333332*G20_0_0_0_2_0_0_2_9_0 + 0.0833333333333332*G20_0_0_0_2_0_0_2_10_1 + 0.0833333333333332*G20_0_0_0_2_0_0_2_11_2 - 0.0833333333333332*G20_1_0_0_0_1_0_0_8_0 - 0.0833333333333332*G20_1_0_0_0_1_0_0_8_1 - 0.0833333333333332*G20_1_0_0_0_1_0_0_8_2 + 0.0833333333333332*G20_1_0_0_0_1_0_0_9_0 + 0.0833333333333332*G20_1_0_0_0_1_0_0_10_1 + 0.0833333333333332*G20_1_0_0_0_1_0_0_11_2 - 0.0833333333333332*G20_1_0_0_1_1_0_1_8_0 - 0.0833333333333332*G20_1_0_0_1_1_0_1_8_1 - 0.0833333333333332*G20_1_0_0_1_1_0_1_8_2 + 0.0833333333333332*G20_1_0_0_1_1_0_1_9_0 + 0.0833333333333332*G20_1_0_0_1_1_0_1_10_1 + 0.0833333333333332*G20_1_0_0_1_1_0_1_11_2 - 0.0833333333333332*G20_1_0_0_2_1_0_2_8_0 - 0.0833333333333332*G20_1_0_0_2_1_0_2_8_1 - 0.0833333333333332*G20_1_0_0_2_1_0_2_8_2 + 0.0833333333333332*G20_1_0_0_2_1_0_2_9_0 + 0.0833333333333332*G20_1_0_0_2_1_0_2_10_1 + 0.0833333333333332*G20_1_0_0_2_1_0_2_11_2 - 0.0833333333333332*G20_2_0_0_0_2_0_0_8_0 - 0.0833333333333332*G20_2_0_0_0_2_0_0_8_1 - 0.0833333333333332*G20_2_0_0_0_2_0_0_8_2 + 0.0833333333333332*G20_2_0_0_0_2_0_0_9_0 + 0.0833333333333332*G20_2_0_0_0_2_0_0_10_1 + 0.0833333333333332*G20_2_0_0_0_2_0_0_11_2 - 0.0833333333333332*G20_2_0_0_1_2_0_1_8_0 - 0.0833333333333332*G20_2_0_0_1_2_0_1_8_1 - 0.0833333333333332*G20_2_0_0_1_2_0_1_8_2 + 0.0833333333333332*G20_2_0_0_1_2_0_1_9_0 + 0.0833333333333332*G20_2_0_0_1_2_0_1_10_1 + 0.0833333333333332*G20_2_0_0_1_2_0_1_11_2 - 0.0833333333333332*G20_2_0_0_2_2_0_2_8_0 - 0.0833333333333332*G20_2_0_0_2_2_0_2_8_1 - 0.0833333333333332*G20_2_0_0_2_2_0_2_8_2 + 0.0833333333333332*G20_2_0_0_2_2_0_2_9_0 + 0.0833333333333332*G20_2_0_0_2_2_0_2_10_1 + 0.0833333333333332*G20_2_0_0_2_2_0_2_11_2 + 0.166666666666666*G21_2_0_0_0_0_0_0_0 + 0.166666666666666*G21_2_0_0_0_0_0_0_1 + 0.166666666666666*G21_2_0_0_0_0_0_0_2 - 0.166666666666666*G21_2_0_0_0_0_0_1_0 - 0.166666666666666*G21_2_0_0_0_0_0_2_1 - 0.166666666666666*G21_2_0_0_0_0_0_3_2 + 0.166666666666666*G21_2_0_0_1_0_1_0_0 + 0.166666666666666*G21_2_0_0_1_0_1_0_1 + 0.166666666666666*G21_2_0_0_1_0_1_0_2 - 0.166666666666666*G21_2_0_0_1_0_1_1_0 - 0.166666666666666*G21_2_0_0_1_0_1_2_1 - 0.166666666666666*G21_2_0_0_1_0_1_3_2 + 0.166666666666666*G21_2_0_0_2_0_2_0_0 + 0.166666666666666*G21_2_0_0_2_0_2_0_1 + 0.166666666666666*G21_2_0_0_2_0_2_0_2 - 0.166666666666666*G21_2_0_0_2_0_2_1_0 - 0.166666666666666*G21_2_0_0_2_0_2_2_1 - 0.166666666666666*G21_2_0_0_2_0_2_3_2 - 0.166666666666666*G22_2_0_0_0_0_0_0_0 - 0.166666666666666*G22_2_0_0_0_0_0_0_1 - 0.166666666666666*G22_2_0_0_0_0_0_0_2 + 0.166666666666666*G22_2_0_0_0_0_0_1_0 + 0.166666666666666*G22_2_0_0_0_0_0_2_1 + 0.166666666666666*G22_2_0_0_0_0_0_3_2 - 0.166666666666666*G22_2_0_0_1_0_1_0_0 - 0.166666666666666*G22_2_0_0_1_0_1_0_1 - 0.166666666666666*G22_2_0_0_1_0_1_0_2 + 0.166666666666666*G22_2_0_0_1_0_1_1_0 + 0.166666666666666*G22_2_0_0_1_0_1_2_1 + 0.166666666666666*G22_2_0_0_1_0_1_3_2 - 0.166666666666666*G22_2_0_0_2_0_2_0_0 - 0.166666666666666*G22_2_0_0_2_0_2_0_1 - 0.166666666666666*G22_2_0_0_2_0_2_0_2 + 0.166666666666666*G22_2_0_0_2_0_2_1_0 + 0.166666666666666*G22_2_0_0_2_0_2_2_1 + 0.166666666666666*G22_2_0_0_2_0_2_3_2 + 0.0833333333333332*G23_0_0_0_2_0_0_0 + 0.0833333333333332*G23_0_0_0_2_0_1_0 + 0.0833333333333332*G23_0_0_0_2_0_2_0 - 0.0833333333333332*G23_0_0_0_2_1_0_0 - 0.0833333333333332*G23_0_0_0_2_2_1_0 - 0.0833333333333332*G23_0_0_0_2_3_2_0 + 0.0833333333333332*G23_0_0_0_2_4_0_1 + 0.0833333333333332*G23_0_0_0_2_4_1_1 + 0.0833333333333332*G23_0_0_0_2_4_2_1 - 0.0833333333333332*G23_0_0_0_2_5_0_1 - 0.0833333333333332*G23_0_0_0_2_6_1_1 - 0.0833333333333332*G23_0_0_0_2_7_2_1 + 0.0833333333333332*G23_0_0_0_2_8_0_2 + 0.0833333333333332*G23_0_0_0_2_8_1_2 + 0.0833333333333332*G23_0_0_0_2_8_2_2 - 0.0833333333333332*G23_0_0_0_2_9_0_2 - 0.0833333333333332*G23_0_0_0_2_10_1_2 - 0.0833333333333332*G23_0_0_0_2_11_2_2;
    A[10] = 0.00833333333333331*G0_0_8 + 0.00833333333333331*G0_0_9 + 0.0166666666666666*G0_0_10 + 0.00833333333333331*G0_0_11 + 0.00833333333333331*G1_0_8 + 0.00833333333333331*G1_0_9 + 0.0166666666666666*G1_0_10 + 0.00833333333333331*G1_0_11 + 0.166666666666666*G2_0_0_2_1 + 0.166666666666666*G7_0_0_5_1 + 0.166666666666666*G8_0_0_8_1 - 0.166666666666666*G9_0_2_1 - 0.166666666666666*G14_0_5_1 - 0.166666666666666*G15_0_8_1 + 0.0416666666666666*G16_0_0_1_2 + 0.0416666666666666*G16_0_1_1_2 + 0.0416666666666666*G16_0_2_1_2 + 0.0416666666666666*G16_0_3_1_2 + 0.0416666666666666*G17_0_0_0_1_0_8 + 0.0416666666666666*G17_0_0_0_1_0_9 + 0.0416666666666666*G17_0_0_0_1_0_10 + 0.0416666666666666*G17_0_0_0_1_0_11 + 0.0416666666666666*G17_0_0_1_1_1_8 + 0.0416666666666666*G17_0_0_1_1_1_9 + 0.0416666666666666*G17_0_0_1_1_1_10 + 0.0416666666666666*G17_0_0_1_1_1_11 + 0.0416666666666666*G17_0_0_2_1_2_8 + 0.0416666666666666*G17_0_0_2_1_2_9 + 0.0416666666666666*G17_0_0_2_1_2_10 + 0.0416666666666666*G17_0_0_2_1_2_11 - 0.0416666666666666*G18_0_0_0_1_0_8 - 0.0416666666666666*G18_0_0_0_1_0_9 - 0.0416666666666666*G18_0_0_0_1_0_10 - 0.0416666666666666*G18_0_0_0_1_0_11 - 0.0416666666666666*G18_0_0_1_1_1_8 - 0.0416666666666666*G18_0_0_1_1_1_9 - 0.0416666666666666*G18_0_0_1_1_1_10 - 0.0416666666666666*G18_0_0_1_1_1_11 - 0.0416666666666666*G18_0_0_2_1_2_8 - 0.0416666666666666*G18_0_0_2_1_2_9 - 0.0416666666666666*G18_0_0_2_1_2_10 - 0.0416666666666666*G18_0_0_2_1_2_11 + 0.0833333333333332*G19_0_0_0_0_0_1_0_8_0 + 0.0833333333333332*G19_0_0_0_0_0_1_0_8_1 + 0.0833333333333332*G19_0_0_0_0_0_1_0_8_2 - 0.0833333333333332*G19_0_0_0_0_0_1_0_9_0 - 0.0833333333333332*G19_0_0_0_0_0_1_0_10_1 - 0.0833333333333332*G19_0_0_0_0_0_1_0_11_2 + 0.0833333333333332*G19_0_0_0_1_0_1_1_8_0 + 0.0833333333333332*G19_0_0_0_1_0_1_1_8_1 + 0.0833333333333332*G19_0_0_0_1_0_1_1_8_2 - 0.0833333333333332*G19_0_0_0_1_0_1_1_9_0 - 0.0833333333333332*G19_0_0_0_1_0_1_1_10_1 - 0.0833333333333332*G19_0_0_0_1_0_1_1_11_2 + 0.0833333333333332*G19_0_0_0_2_0_1_2_8_0 + 0.0833333333333332*G19_0_0_0_2_0_1_2_8_1 + 0.0833333333333332*G19_0_0_0_2_0_1_2_8_2 - 0.0833333333333332*G19_0_0_0_2_0_1_2_9_0 - 0.0833333333333332*G19_0_0_0_2_0_1_2_10_1 - 0.0833333333333332*G19_0_0_0_2_0_1_2_11_2 + 0.0833333333333332*G19_1_0_0_0_1_1_0_8_0 + 0.0833333333333332*G19_1_0_0_0_1_1_0_8_1 + 0.0833333333333332*G19_1_0_0_0_1_1_0_8_2 - 0.0833333333333332*G19_1_0_0_0_1_1_0_9_0 - 0.0833333333333332*G19_1_0_0_0_1_1_0_10_1 - 0.0833333333333332*G19_1_0_0_0_1_1_0_11_2 + 0.0833333333333332*G19_1_0_0_1_1_1_1_8_0 + 0.0833333333333332*G19_1_0_0_1_1_1_1_8_1 + 0.0833333333333332*G19_1_0_0_1_1_1_1_8_2 - 0.0833333333333332*G19_1_0_0_1_1_1_1_9_0 - 0.0833333333333332*G19_1_0_0_1_1_1_1_10_1 - 0.0833333333333332*G19_1_0_0_1_1_1_1_11_2 + 0.0833333333333332*G19_1_0_0_2_1_1_2_8_0 + 0.0833333333333332*G19_1_0_0_2_1_1_2_8_1 + 0.0833333333333332*G19_1_0_0_2_1_1_2_8_2 - 0.0833333333333332*G19_1_0_0_2_1_1_2_9_0 - 0.0833333333333332*G19_1_0_0_2_1_1_2_10_1 - 0.0833333333333332*G19_1_0_0_2_1_1_2_11_2 + 0.0833333333333332*G19_2_0_0_0_2_1_0_8_0 + 0.0833333333333332*G19_2_0_0_0_2_1_0_8_1 + 0.0833333333333332*G19_2_0_0_0_2_1_0_8_2 - 0.0833333333333332*G19_2_0_0_0_2_1_0_9_0 - 0.0833333333333332*G19_2_0_0_0_2_1_0_10_1 - 0.0833333333333332*G19_2_0_0_0_2_1_0_11_2 + 0.0833333333333332*G19_2_0_0_1_2_1_1_8_0 + 0.0833333333333332*G19_2_0_0_1_2_1_1_8_1 + 0.0833333333333332*G19_2_0_0_1_2_1_1_8_2 - 0.0833333333333332*G19_2_0_0_1_2_1_1_9_0 - 0.0833333333333332*G19_2_0_0_1_2_1_1_10_1 - 0.0833333333333332*G19_2_0_0_1_2_1_1_11_2 + 0.0833333333333332*G19_2_0_0_2_2_1_2_8_0 + 0.0833333333333332*G19_2_0_0_2_2_1_2_8_1 + 0.0833333333333332*G19_2_0_0_2_2_1_2_8_2 - 0.0833333333333332*G19_2_0_0_2_2_1_2_9_0 - 0.0833333333333332*G19_2_0_0_2_2_1_2_10_1 - 0.0833333333333332*G19_2_0_0_2_2_1_2_11_2 - 0.0833333333333332*G20_0_0_0_0_0_1_0_8_0 - 0.0833333333333332*G20_0_0_0_0_0_1_0_8_1 - 0.0833333333333332*G20_0_0_0_0_0_1_0_8_2 + 0.0833333333333332*G20_0_0_0_0_0_1_0_9_0 + 0.0833333333333332*G20_0_0_0_0_0_1_0_10_1 + 0.0833333333333332*G20_0_0_0_0_0_1_0_11_2 - 0.0833333333333332*G20_0_0_0_1_0_1_1_8_0 - 0.0833333333333332*G20_0_0_0_1_0_1_1_8_1 - 0.0833333333333332*G20_0_0_0_1_0_1_1_8_2 + 0.0833333333333332*G20_0_0_0_1_0_1_1_9_0 + 0.0833333333333332*G20_0_0_0_1_0_1_1_10_1 + 0.0833333333333332*G20_0_0_0_1_0_1_1_11_2 - 0.0833333333333332*G20_0_0_0_2_0_1_2_8_0 - 0.0833333333333332*G20_0_0_0_2_0_1_2_8_1 - 0.0833333333333332*G20_0_0_0_2_0_1_2_8_2 + 0.0833333333333332*G20_0_0_0_2_0_1_2_9_0 + 0.0833333333333332*G20_0_0_0_2_0_1_2_10_1 + 0.0833333333333332*G20_0_0_0_2_0_1_2_11_2 - 0.0833333333333332*G20_1_0_0_0_1_1_0_8_0 - 0.0833333333333332*G20_1_0_0_0_1_1_0_8_1 - 0.0833333333333332*G20_1_0_0_0_1_1_0_8_2 + 0.0833333333333332*G20_1_0_0_0_1_1_0_9_0 + 0.0833333333333332*G20_1_0_0_0_1_1_0_10_1 + 0.0833333333333332*G20_1_0_0_0_1_1_0_11_2 - 0.0833333333333332*G20_1_0_0_1_1_1_1_8_0 - 0.0833333333333332*G20_1_0_0_1_1_1_1_8_1 - 0.0833333333333332*G20_1_0_0_1_1_1_1_8_2 + 0.0833333333333332*G20_1_0_0_1_1_1_1_9_0 + 0.0833333333333332*G20_1_0_0_1_1_1_1_10_1 + 0.0833333333333332*G20_1_0_0_1_1_1_1_11_2 - 0.0833333333333332*G20_1_0_0_2_1_1_2_8_0 - 0.0833333333333332*G20_1_0_0_2_1_1_2_8_1 - 0.0833333333333332*G20_1_0_0_2_1_1_2_8_2 + 0.0833333333333332*G20_1_0_0_2_1_1_2_9_0 + 0.0833333333333332*G20_1_0_0_2_1_1_2_10_1 + 0.0833333333333332*G20_1_0_0_2_1_1_2_11_2 - 0.0833333333333332*G20_2_0_0_0_2_1_0_8_0 - 0.0833333333333332*G20_2_0_0_0_2_1_0_8_1 - 0.0833333333333332*G20_2_0_0_0_2_1_0_8_2 + 0.0833333333333332*G20_2_0_0_0_2_1_0_9_0 + 0.0833333333333332*G20_2_0_0_0_2_1_0_10_1 + 0.0833333333333332*G20_2_0_0_0_2_1_0_11_2 - 0.0833333333333332*G20_2_0_0_1_2_1_1_8_0 - 0.0833333333333332*G20_2_0_0_1_2_1_1_8_1 - 0.0833333333333332*G20_2_0_0_1_2_1_1_8_2 + 0.0833333333333332*G20_2_0_0_1_2_1_1_9_0 + 0.0833333333333332*G20_2_0_0_1_2_1_1_10_1 + 0.0833333333333332*G20_2_0_0_1_2_1_1_11_2 - 0.0833333333333332*G20_2_0_0_2_2_1_2_8_0 - 0.0833333333333332*G20_2_0_0_2_2_1_2_8_1 - 0.0833333333333332*G20_2_0_0_2_2_1_2_8_2 + 0.0833333333333332*G20_2_0_0_2_2_1_2_9_0 + 0.0833333333333332*G20_2_0_0_2_2_1_2_10_1 + 0.0833333333333332*G20_2_0_0_2_2_1_2_11_2 + 0.166666666666666*G21_2_0_0_0_1_0_0_0 + 0.166666666666666*G21_2_0_0_0_1_0_0_1 + 0.166666666666666*G21_2_0_0_0_1_0_0_2 - 0.166666666666666*G21_2_0_0_0_1_0_1_0 - 0.166666666666666*G21_2_0_0_0_1_0_2_1 - 0.166666666666666*G21_2_0_0_0_1_0_3_2 + 0.166666666666666*G21_2_0_0_1_1_1_0_0 + 0.166666666666666*G21_2_0_0_1_1_1_0_1 + 0.166666666666666*G21_2_0_0_1_1_1_0_2 - 0.166666666666666*G21_2_0_0_1_1_1_1_0 - 0.166666666666666*G21_2_0_0_1_1_1_2_1 - 0.166666666666666*G21_2_0_0_1_1_1_3_2 + 0.166666666666666*G21_2_0_0_2_1_2_0_0 + 0.166666666666666*G21_2_0_0_2_1_2_0_1 + 0.166666666666666*G21_2_0_0_2_1_2_0_2 - 0.166666666666666*G21_2_0_0_2_1_2_1_0 - 0.166666666666666*G21_2_0_0_2_1_2_2_1 - 0.166666666666666*G21_2_0_0_2_1_2_3_2 - 0.166666666666666*G22_2_0_0_0_1_0_0_0 - 0.166666666666666*G22_2_0_0_0_1_0_0_1 - 0.166666666666666*G22_2_0_0_0_1_0_0_2 + 0.166666666666666*G22_2_0_0_0_1_0_1_0 + 0.166666666666666*G22_2_0_0_0_1_0_2_1 + 0.166666666666666*G22_2_0_0_0_1_0_3_2 - 0.166666666666666*G22_2_0_0_1_1_1_0_0 - 0.166666666666666*G22_2_0_0_1_1_1_0_1 - 0.166666666666666*G22_2_0_0_1_1_1_0_2 + 0.166666666666666*G22_2_0_0_1_1_1_1_0 + 0.166666666666666*G22_2_0_0_1_1_1_2_1 + 0.166666666666666*G22_2_0_0_1_1_1_3_2 - 0.166666666666666*G22_2_0_0_2_1_2_0_0 - 0.166666666666666*G22_2_0_0_2_1_2_0_1 - 0.166666666666666*G22_2_0_0_2_1_2_0_2 + 0.166666666666666*G22_2_0_0_2_1_2_1_0 + 0.166666666666666*G22_2_0_0_2_1_2_2_1 + 0.166666666666666*G22_2_0_0_2_1_2_3_2 + 0.0833333333333332*G23_0_0_1_2_0_0_0 + 0.0833333333333332*G23_0_0_1_2_0_1_0 + 0.0833333333333332*G23_0_0_1_2_0_2_0 - 0.0833333333333332*G23_0_0_1_2_1_0_0 - 0.0833333333333332*G23_0_0_1_2_2_1_0 - 0.0833333333333332*G23_0_0_1_2_3_2_0 + 0.0833333333333332*G23_0_0_1_2_4_0_1 + 0.0833333333333332*G23_0_0_1_2_4_1_1 + 0.0833333333333332*G23_0_0_1_2_4_2_1 - 0.0833333333333332*G23_0_0_1_2_5_0_1 - 0.0833333333333332*G23_0_0_1_2_6_1_1 - 0.0833333333333332*G23_0_0_1_2_7_2_1 + 0.0833333333333332*G23_0_0_1_2_8_0_2 + 0.0833333333333332*G23_0_0_1_2_8_1_2 + 0.0833333333333332*G23_0_0_1_2_8_2_2 - 0.0833333333333332*G23_0_0_1_2_9_0_2 - 0.0833333333333332*G23_0_0_1_2_10_1_2 - 0.0833333333333332*G23_0_0_1_2_11_2_2;
    A[11] = 0.00833333333333331*G0_0_8 + 0.00833333333333331*G0_0_9 + 0.00833333333333331*G0_0_10 + 0.0166666666666666*G0_0_11 + 0.00833333333333331*G1_0_8 + 0.00833333333333331*G1_0_9 + 0.00833333333333331*G1_0_10 + 0.0166666666666666*G1_0_11 + 0.166666666666666*G2_0_0_2_2 + 0.166666666666666*G7_0_0_5_2 + 0.166666666666666*G8_0_0_8_2 - 0.166666666666666*G9_0_2_2 - 0.166666666666666*G14_0_5_2 - 0.166666666666666*G15_0_8_2 + 0.0416666666666666*G16_0_0_2_2 + 0.0416666666666666*G16_0_1_2_2 + 0.0416666666666666*G16_0_2_2_2 + 0.0416666666666666*G16_0_3_2_2 + 0.0416666666666666*G17_0_0_0_2_0_8 + 0.0416666666666666*G17_0_0_0_2_0_9 + 0.0416666666666666*G17_0_0_0_2_0_10 + 0.0416666666666666*G17_0_0_0_2_0_11 + 0.0416666666666666*G17_0_0_1_2_1_8 + 0.0416666666666666*G17_0_0_1_2_1_9 + 0.0416666666666666*G17_0_0_1_2_1_10 + 0.0416666666666666*G17_0_0_1_2_1_11 + 0.0416666666666666*G17_0_0_2_2_2_8 + 0.0416666666666666*G17_0_0_2_2_2_9 + 0.0416666666666666*G17_0_0_2_2_2_10 + 0.0416666666666666*G17_0_0_2_2_2_11 - 0.0416666666666666*G18_0_0_0_2_0_8 - 0.0416666666666666*G18_0_0_0_2_0_9 - 0.0416666666666666*G18_0_0_0_2_0_10 - 0.0416666666666666*G18_0_0_0_2_0_11 - 0.0416666666666666*G18_0_0_1_2_1_8 - 0.0416666666666666*G18_0_0_1_2_1_9 - 0.0416666666666666*G18_0_0_1_2_1_10 - 0.0416666666666666*G18_0_0_1_2_1_11 - 0.0416666666666666*G18_0_0_2_2_2_8 - 0.0416666666666666*G18_0_0_2_2_2_9 - 0.0416666666666666*G18_0_0_2_2_2_10 - 0.0416666666666666*G18_0_0_2_2_2_11 + 0.0833333333333332*G19_0_0_0_0_0_2_0_8_0 + 0.0833333333333332*G19_0_0_0_0_0_2_0_8_1 + 0.0833333333333332*G19_0_0_0_0_0_2_0_8_2 - 0.0833333333333332*G19_0_0_0_0_0_2_0_9_0 - 0.0833333333333332*G19_0_0_0_0_0_2_0_10_1 - 0.0833333333333332*G19_0_0_0_0_0_2_0_11_2 + 0.0833333333333332*G19_0_0_0_1_0_2_1_8_0 + 0.0833333333333332*G19_0_0_0_1_0_2_1_8_1 + 0.0833333333333332*G19_0_0_0_1_0_2_1_8_2 - 0.0833333333333332*G19_0_0_0_1_0_2_1_9_0 - 0.0833333333333332*G19_0_0_0_1_0_2_1_10_1 - 0.0833333333333332*G19_0_0_0_1_0_2_1_11_2 + 0.0833333333333332*G19_0_0_0_2_0_2_2_8_0 + 0.0833333333333332*G19_0_0_0_2_0_2_2_8_1 + 0.0833333333333332*G19_0_0_0_2_0_2_2_8_2 - 0.0833333333333332*G19_0_0_0_2_0_2_2_9_0 - 0.0833333333333332*G19_0_0_0_2_0_2_2_10_1 - 0.0833333333333332*G19_0_0_0_2_0_2_2_11_2 + 0.0833333333333332*G19_1_0_0_0_1_2_0_8_0 + 0.0833333333333332*G19_1_0_0_0_1_2_0_8_1 + 0.0833333333333332*G19_1_0_0_0_1_2_0_8_2 - 0.0833333333333332*G19_1_0_0_0_1_2_0_9_0 - 0.0833333333333332*G19_1_0_0_0_1_2_0_10_1 - 0.0833333333333332*G19_1_0_0_0_1_2_0_11_2 + 0.0833333333333332*G19_1_0_0_1_1_2_1_8_0 + 0.0833333333333332*G19_1_0_0_1_1_2_1_8_1 + 0.0833333333333332*G19_1_0_0_1_1_2_1_8_2 - 0.0833333333333332*G19_1_0_0_1_1_2_1_9_0 - 0.0833333333333332*G19_1_0_0_1_1_2_1_10_1 - 0.0833333333333332*G19_1_0_0_1_1_2_1_11_2 + 0.0833333333333332*G19_1_0_0_2_1_2_2_8_0 + 0.0833333333333332*G19_1_0_0_2_1_2_2_8_1 + 0.0833333333333332*G19_1_0_0_2_1_2_2_8_2 - 0.0833333333333332*G19_1_0_0_2_1_2_2_9_0 - 0.0833333333333332*G19_1_0_0_2_1_2_2_10_1 - 0.0833333333333332*G19_1_0_0_2_1_2_2_11_2 + 0.0833333333333332*G19_2_0_0_0_2_2_0_8_0 + 0.0833333333333332*G19_2_0_0_0_2_2_0_8_1 + 0.0833333333333332*G19_2_0_0_0_2_2_0_8_2 - 0.0833333333333332*G19_2_0_0_0_2_2_0_9_0 - 0.0833333333333332*G19_2_0_0_0_2_2_0_10_1 - 0.0833333333333332*G19_2_0_0_0_2_2_0_11_2 + 0.0833333333333332*G19_2_0_0_1_2_2_1_8_0 + 0.0833333333333332*G19_2_0_0_1_2_2_1_8_1 + 0.0833333333333332*G19_2_0_0_1_2_2_1_8_2 - 0.0833333333333332*G19_2_0_0_1_2_2_1_9_0 - 0.0833333333333332*G19_2_0_0_1_2_2_1_10_1 - 0.0833333333333332*G19_2_0_0_1_2_2_1_11_2 + 0.0833333333333332*G19_2_0_0_2_2_2_2_8_0 + 0.0833333333333332*G19_2_0_0_2_2_2_2_8_1 + 0.0833333333333332*G19_2_0_0_2_2_2_2_8_2 - 0.0833333333333332*G19_2_0_0_2_2_2_2_9_0 - 0.0833333333333332*G19_2_0_0_2_2_2_2_10_1 - 0.0833333333333332*G19_2_0_0_2_2_2_2_11_2 - 0.0833333333333332*G20_0_0_0_0_0_2_0_8_0 - 0.0833333333333332*G20_0_0_0_0_0_2_0_8_1 - 0.0833333333333332*G20_0_0_0_0_0_2_0_8_2 + 0.0833333333333332*G20_0_0_0_0_0_2_0_9_0 + 0.0833333333333332*G20_0_0_0_0_0_2_0_10_1 + 0.0833333333333332*G20_0_0_0_0_0_2_0_11_2 - 0.0833333333333332*G20_0_0_0_1_0_2_1_8_0 - 0.0833333333333332*G20_0_0_0_1_0_2_1_8_1 - 0.0833333333333332*G20_0_0_0_1_0_2_1_8_2 + 0.0833333333333332*G20_0_0_0_1_0_2_1_9_0 + 0.0833333333333332*G20_0_0_0_1_0_2_1_10_1 + 0.0833333333333332*G20_0_0_0_1_0_2_1_11_2 - 0.0833333333333332*G20_0_0_0_2_0_2_2_8_0 - 0.0833333333333332*G20_0_0_0_2_0_2_2_8_1 - 0.0833333333333332*G20_0_0_0_2_0_2_2_8_2 + 0.0833333333333332*G20_0_0_0_2_0_2_2_9_0 + 0.0833333333333332*G20_0_0_0_2_0_2_2_10_1 + 0.0833333333333332*G20_0_0_0_2_0_2_2_11_2 - 0.0833333333333332*G20_1_0_0_0_1_2_0_8_0 - 0.0833333333333332*G20_1_0_0_0_1_2_0_8_1 - 0.0833333333333332*G20_1_0_0_0_1_2_0_8_2 + 0.0833333333333332*G20_1_0_0_0_1_2_0_9_0 + 0.0833333333333332*G20_1_0_0_0_1_2_0_10_1 + 0.0833333333333332*G20_1_0_0_0_1_2_0_11_2 - 0.0833333333333332*G20_1_0_0_1_1_2_1_8_0 - 0.0833333333333332*G20_1_0_0_1_1_2_1_8_1 - 0.0833333333333332*G20_1_0_0_1_1_2_1_8_2 + 0.0833333333333332*G20_1_0_0_1_1_2_1_9_0 + 0.0833333333333332*G20_1_0_0_1_1_2_1_10_1 + 0.0833333333333332*G20_1_0_0_1_1_2_1_11_2 - 0.0833333333333332*G20_1_0_0_2_1_2_2_8_0 - 0.0833333333333332*G20_1_0_0_2_1_2_2_8_1 - 0.0833333333333332*G20_1_0_0_2_1_2_2_8_2 + 0.0833333333333332*G20_1_0_0_2_1_2_2_9_0 + 0.0833333333333332*G20_1_0_0_2_1_2_2_10_1 + 0.0833333333333332*G20_1_0_0_2_1_2_2_11_2 - 0.0833333333333332*G20_2_0_0_0_2_2_0_8_0 - 0.0833333333333332*G20_2_0_0_0_2_2_0_8_1 - 0.0833333333333332*G20_2_0_0_0_2_2_0_8_2 + 0.0833333333333332*G20_2_0_0_0_2_2_0_9_0 + 0.0833333333333332*G20_2_0_0_0_2_2_0_10_1 + 0.0833333333333332*G20_2_0_0_0_2_2_0_11_2 - 0.0833333333333332*G20_2_0_0_1_2_2_1_8_0 - 0.0833333333333332*G20_2_0_0_1_2_2_1_8_1 - 0.0833333333333332*G20_2_0_0_1_2_2_1_8_2 + 0.0833333333333332*G20_2_0_0_1_2_2_1_9_0 + 0.0833333333333332*G20_2_0_0_1_2_2_1_10_1 + 0.0833333333333332*G20_2_0_0_1_2_2_1_11_2 - 0.0833333333333332*G20_2_0_0_2_2_2_2_8_0 - 0.0833333333333332*G20_2_0_0_2_2_2_2_8_1 - 0.0833333333333332*G20_2_0_0_2_2_2_2_8_2 + 0.0833333333333332*G20_2_0_0_2_2_2_2_9_0 + 0.0833333333333332*G20_2_0_0_2_2_2_2_10_1 + 0.0833333333333332*G20_2_0_0_2_2_2_2_11_2 + 0.166666666666666*G21_2_0_0_0_2_0_0_0 + 0.166666666666666*G21_2_0_0_0_2_0_0_1 + 0.166666666666666*G21_2_0_0_0_2_0_0_2 - 0.166666666666666*G21_2_0_0_0_2_0_1_0 - 0.166666666666666*G21_2_0_0_0_2_0_2_1 - 0.166666666666666*G21_2_0_0_0_2_0_3_2 + 0.166666666666666*G21_2_0_0_1_2_1_0_0 + 0.166666666666666*G21_2_0_0_1_2_1_0_1 + 0.166666666666666*G21_2_0_0_1_2_1_0_2 - 0.166666666666666*G21_2_0_0_1_2_1_1_0 - 0.166666666666666*G21_2_0_0_1_2_1_2_1 - 0.166666666666666*G21_2_0_0_1_2_1_3_2 + 0.166666666666666*G21_2_0_0_2_2_2_0_0 + 0.166666666666666*G21_2_0_0_2_2_2_0_1 + 0.166666666666666*G21_2_0_0_2_2_2_0_2 - 0.166666666666666*G21_2_0_0_2_2_2_1_0 - 0.166666666666666*G21_2_0_0_2_2_2_2_1 - 0.166666666666666*G21_2_0_0_2_2_2_3_2 - 0.166666666666666*G22_2_0_0_0_2_0_0_0 - 0.166666666666666*G22_2_0_0_0_2_0_0_1 - 0.166666666666666*G22_2_0_0_0_2_0_0_2 + 0.166666666666666*G22_2_0_0_0_2_0_1_0 + 0.166666666666666*G22_2_0_0_0_2_0_2_1 + 0.166666666666666*G22_2_0_0_0_2_0_3_2 - 0.166666666666666*G22_2_0_0_1_2_1_0_0 - 0.166666666666666*G22_2_0_0_1_2_1_0_1 - 0.166666666666666*G22_2_0_0_1_2_1_0_2 + 0.166666666666666*G22_2_0_0_1_2_1_1_0 + 0.166666666666666*G22_2_0_0_1_2_1_2_1 + 0.166666666666666*G22_2_0_0_1_2_1_3_2 - 0.166666666666666*G22_2_0_0_2_2_2_0_0 - 0.166666666666666*G22_2_0_0_2_2_2_0_1 - 0.166666666666666*G22_2_0_0_2_2_2_0_2 + 0.166666666666666*G22_2_0_0_2_2_2_1_0 + 0.166666666666666*G22_2_0_0_2_2_2_2_1 + 0.166666666666666*G22_2_0_0_2_2_2_3_2 + 0.0833333333333332*G23_0_0_2_2_0_0_0 + 0.0833333333333332*G23_0_0_2_2_0_1_0 + 0.0833333333333332*G23_0_0_2_2_0_2_0 - 0.0833333333333332*G23_0_0_2_2_1_0_0 - 0.0833333333333332*G23_0_0_2_2_2_1_0 - 0.0833333333333332*G23_0_0_2_2_3_2_0 + 0.0833333333333332*G23_0_0_2_2_4_0_1 + 0.0833333333333332*G23_0_0_2_2_4_1_1 + 0.0833333333333332*G23_0_0_2_2_4_2_1 - 0.0833333333333332*G23_0_0_2_2_5_0_1 - 0.0833333333333332*G23_0_0_2_2_6_1_1 - 0.0833333333333332*G23_0_0_2_2_7_2_1 + 0.0833333333333332*G23_0_0_2_2_8_0_2 + 0.0833333333333332*G23_0_0_2_2_8_1_2 + 0.0833333333333332*G23_0_0_2_2_8_2_2 - 0.0833333333333332*G23_0_0_2_2_9_0_2 - 0.0833333333333332*G23_0_0_2_2_10_1_2 - 0.0833333333333332*G23_0_0_2_2_11_2_2;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class UFC_FSIMomentum3DLinearForm: public ufc::form
{
public:

  /// Constructor
  UFC_FSIMomentum3DLinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_FSIMomentum3DLinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w8_a0w1_a1 | va0*va1[b0]*vi0[b0]*dX(0) + w10_a0w2_a1 | va0*va1[b0]*vi0[b0]*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx0) | va0*va1*va2[b0]*((d/dXa3)vi0[b0])*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx1) | va0*va1*va2[3]*((d/dXa3)vi0[0])*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx2) | va0*va1*va2[6]*((d/dXa3)vi0[0])*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx1) | va0*va1*va2[4]*((d/dXa3)vi0[1])*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx2) | va0*va1*va2[7]*((d/dXa3)vi0[1])*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx1) | va0*va1*va2[5]*((d/dXa3)vi0[2])*dX(0) + w10_a0w7_a1w9_a2(dXa3/dx2) | va0*va1*va2[8]*((d/dXa3)vi0[2])*dX(0) + -w10_a0w9_a1(dXa2/dx0) | va0*va1[b0]*((d/dXa2)vi0[b0])*dX(0) + -w10_a0w9_a1(dXa2/dx1) | va0*va1[3]*((d/dXa2)vi0[0])*dX(0) + -w10_a0w9_a1(dXa2/dx2) | va0*va1[6]*((d/dXa2)vi0[0])*dX(0) + -w10_a0w9_a1(dXa2/dx1) | va0*va1[4]*((d/dXa2)vi0[1])*dX(0) + -w10_a0w9_a1(dXa2/dx2) | va0*va1[7]*((d/dXa2)vi0[1])*dX(0) + -w10_a0w9_a1(dXa2/dx1) | va0*va1[5]*((d/dXa2)vi0[2])*dX(0) + -w10_a0w9_a1(dXa2/dx2) | va0*va1[8]*((d/dXa2)vi0[2])*dX(0) + w10_a0w3_a1(dXa2/dxa3) | va0*va1*((d/dXa2)vi0[a3])*dX(0) + w10_a0w4_a1w0_a2w2_a5(dXa3/dxa4) | va0*va1*va2[a4]*((d/dXa3)vi0[b0])*va5[b0]*dX(0) + -w10_a0w4_a1w6_a2w2_a5(dXa3/dxa4) | va0*va1*va2[a4]*((d/dXa3)vi0[b0])*va5[b0]*dX(0) + -0.5w10_a1w4_a2w0_a3w0_a4w1_a7(dXa5/dxa6)(dXa8/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa8)va7[b0])*dX(0) + 0.5w10_a1w4_a2w0_a3w6_a4w1_a7(dXa5/dxa6)(dXa8/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa8)va7[b0])*dX(0) + 0.5w10_a1w4_a2w6_a3w0_a4w1_a7(dXa5/dxa6)(dXa8/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa8)va7[b0])*dX(0) + -0.5w10_a1w4_a2w6_a3w6_a4w1_a7(dXa5/dxa6)(dXa8/dxa0) | va1*va2*va3[a6]*((d/dXa5)vi0[b0])*va4[a0]*((d/dXa8)va7[b0])*dX(0) + -w10_a1w4_a2w0_a3w3_a6(dXa4/dxa5)(dXa7/dxa0) | va1*va2*va3[a5]*((d/dXa4)vi0[a0])*((d/dXa7)va6)*dX(0) + w10_a1w4_a2w6_a3w3_a6(dXa4/dxa5)(dXa7/dxa0) | va1*va2*va3[a5]*((d/dXa4)vi0[a0])*((d/dXa7)va6)*dX(0) + -0.5w10_a0w5_a1w1_a4(dXa2/dxa3)(dXa5/dxa6) | va0*va1*((d/dXa2)vi0[a3])*((d/dXa5)va4[a6])*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 11;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_finite_element_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_finite_element_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_finite_element_2();
      break;
    case 3:
      return new UFC_FSIMomentum3DLinearForm_finite_element_3();
      break;
    case 4:
      return new UFC_FSIMomentum3DLinearForm_finite_element_4();
      break;
    case 5:
      return new UFC_FSIMomentum3DLinearForm_finite_element_5();
      break;
    case 6:
      return new UFC_FSIMomentum3DLinearForm_finite_element_6();
      break;
    case 7:
      return new UFC_FSIMomentum3DLinearForm_finite_element_7();
      break;
    case 8:
      return new UFC_FSIMomentum3DLinearForm_finite_element_8();
      break;
    case 9:
      return new UFC_FSIMomentum3DLinearForm_finite_element_9();
      break;
    case 10:
      return new UFC_FSIMomentum3DLinearForm_finite_element_10();
      break;
    case 11:
      return new UFC_FSIMomentum3DLinearForm_finite_element_11();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_FSIMomentum3DLinearForm_dof_map_0();
      break;
    case 1:
      return new UFC_FSIMomentum3DLinearForm_dof_map_1();
      break;
    case 2:
      return new UFC_FSIMomentum3DLinearForm_dof_map_2();
      break;
    case 3:
      return new UFC_FSIMomentum3DLinearForm_dof_map_3();
      break;
    case 4:
      return new UFC_FSIMomentum3DLinearForm_dof_map_4();
      break;
    case 5:
      return new UFC_FSIMomentum3DLinearForm_dof_map_5();
      break;
    case 6:
      return new UFC_FSIMomentum3DLinearForm_dof_map_6();
      break;
    case 7:
      return new UFC_FSIMomentum3DLinearForm_dof_map_7();
      break;
    case 8:
      return new UFC_FSIMomentum3DLinearForm_dof_map_8();
      break;
    case 9:
      return new UFC_FSIMomentum3DLinearForm_dof_map_9();
      break;
    case 10:
      return new UFC_FSIMomentum3DLinearForm_dof_map_10();
      break;
    case 11:
      return new UFC_FSIMomentum3DLinearForm_dof_map_11();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new UFC_FSIMomentum3DLinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

#include <dolfin/Form.h>

class FSIMomentum3DBilinearForm : public dolfin::Form
{
public:

  FSIMomentum3DBilinearForm(dolfin::Function& w0, dolfin::Function& w1, dolfin::Function& w2, dolfin::Function& w3, dolfin::Function& w4, dolfin::Function& w5, dolfin::Function& w6, dolfin::Function& w7, dolfin::Function& w8, dolfin::Function& w9) : dolfin::Form()
  {
    __coefficients.push_back(&w0);
    __coefficients.push_back(&w1);
    __coefficients.push_back(&w2);
    __coefficients.push_back(&w3);
    __coefficients.push_back(&w4);
    __coefficients.push_back(&w5);
    __coefficients.push_back(&w6);
    __coefficients.push_back(&w7);
    __coefficients.push_back(&w8);
    __coefficients.push_back(&w9);
  }

  /// Return UFC form
  virtual const ufc::form& form() const
  {
    return __form;
  }
  
  /// Return array of coefficients
  virtual const dolfin::Array<dolfin::Function*>& coefficients() const
  {
    return __coefficients;
  }

private:

  // UFC form
  UFC_FSIMomentum3DBilinearForm __form;

  /// Array of coefficients
  dolfin::Array<dolfin::Function*> __coefficients;

};

class FSIMomentum3DLinearForm : public dolfin::Form
{
public:

  FSIMomentum3DLinearForm(dolfin::Function& w0, dolfin::Function& w1, dolfin::Function& w2, dolfin::Function& w3, dolfin::Function& w4, dolfin::Function& w5, dolfin::Function& w6, dolfin::Function& w7, dolfin::Function& w8, dolfin::Function& w9, dolfin::Function& w10) : dolfin::Form()
  {
    __coefficients.push_back(&w0);
    __coefficients.push_back(&w1);
    __coefficients.push_back(&w2);
    __coefficients.push_back(&w3);
    __coefficients.push_back(&w4);
    __coefficients.push_back(&w5);
    __coefficients.push_back(&w6);
    __coefficients.push_back(&w7);
    __coefficients.push_back(&w8);
    __coefficients.push_back(&w9);
    __coefficients.push_back(&w10);
  }

  /// Return UFC form
  virtual const ufc::form& form() const
  {
    return __form;
  }
  
  /// Return array of coefficients
  virtual const dolfin::Array<dolfin::Function*>& coefficients() const
  {
    return __coefficients;
  }

private:

  // UFC form
  UFC_FSIMomentum3DLinearForm __form;

  /// Array of coefficients
  dolfin::Array<dolfin::Function*> __coefficients;

};

#endif
